@@ -1,12 +1,14 @@
+#ifndef _WINGSP_H_
+#define _WINGSP_H_
+
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 
 
 #include "WINGs.h"
-#include "WUtil.h"
 
-#if WINGS_H_VERSION < 980730
+#if WINGS_H_VERSION < 990516
 #error There_is_an_old_WINGs.h_file_somewhere_in_your_system._Please_remove_it.
 #endif
 
@@ -16,6 +18,10 @@
 #include <string.h>
 #include <stdio.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #define DOUBLE_BUFFER
 
 
@@ -27,7 +33,6 @@
 #define SCROLLER_WIDTH	 20
 
 /* internal messages */
-#define WM_RADIO_PRESS	129
 #define WM_UPDATE_COLORWELL	130
 
 
@@ -53,7 +58,8 @@ typedef struct W_Font {
     short height;
     short y;
     short refCount;
-    unsigned int nonLocalized:1;
+    char *name;
+    unsigned int notFontSet:1;
 } W_Font;
 
 
@@ -86,6 +92,7 @@ typedef struct W_FocusInfo {
     struct W_FocusInfo *next;
 } W_FocusInfo;
 
+
 typedef struct W_Screen {
     Display *display;
     int screen;
@@ -98,7 +105,7 @@ typedef struct W_Screen {
     Time lastEventTime;
     
     Window rootWin;
-    
+
     struct W_View *rootView;
     
     RContext *rcontext;
@@ -113,19 +120,23 @@ typedef struct W_Screen {
 
     Window groupLeader;		       /* the leader of the application */
     				       /* also used for other things */
-    
+
+    struct W_SelectionHandlers *selectionHandlerList;
+
     struct {
 	unsigned int hasAppIcon:1;
 	unsigned int simpleApplication:1;
     } aflags;
 
-    
     WMOpenPanel *sharedOpenPanel;
+    WMSavePanel *sharedSavePanel;
 
     struct W_FontPanel *sharedFontPanel;
 
     struct W_ColorPanel *sharedColorPanel;
-    
+
+    Pixmap stipple;
+
     /* colors */
     W_Color *white;
     W_Color *black;
@@ -144,10 +155,17 @@ typedef struct W_Screen {
     GC ixorGC;			       /* IncludeInferiors XOR */
 
     GC textFieldGC;
-    
+
     W_Font *normalFont;
 
     W_Font *boldFont;
+
+    WMHashTable *fontCache;
+
+    Bool useMultiByte;
+
+    struct W_Balloon *balloon;
+
     
     struct W_Pixmap *checkButtonImageOn;
     struct W_Pixmap *checkButtonImageOff;
@@ -172,9 +190,36 @@ typedef struct W_Screen {
 
     struct W_Pixmap *pullDownIndicator;
     struct W_Pixmap *popUpIndicator;
-    
+
+    struct W_Pixmap *checkMark;
+
     struct W_Pixmap *homeIcon;
-        
+    struct W_Pixmap *altHomeIcon;
+
+    struct W_Pixmap *trashcanIcon;
+    struct W_Pixmap *altTrashcanIcon;
+
+    struct W_Pixmap *disketteIcon;
+    struct W_Pixmap *altDisketteIcon;
+
+    struct W_Pixmap *magnifyIcon;
+/*    struct W_Pixmap *altMagnifyIcon;*/
+    struct W_Pixmap *wheelIcon;
+    struct W_Pixmap *grayIcon;
+    struct W_Pixmap *rgbIcon;
+    struct W_Pixmap *cmykIcon;
+    struct W_Pixmap *hsbIcon;
+    struct W_Pixmap *customPaletteIcon;
+    struct W_Pixmap *colorListIcon;
+	
+    struct W_Pixmap *defaultObjectIcon;
+
+    Cursor defaultCursor;
+    
+    Cursor textCursor;
+
+    Cursor invisibleCursor;
+
     Atom internalMessage;	       /* for ClientMessage */
     
     Atom attribsAtom;		       /* GNUstepWindowAttributes */
@@ -182,22 +227,44 @@ typedef struct W_Screen {
     Atom deleteWindowAtom;	       /* WM_DELETE_WINDOW */
     
     Atom protocolsAtom;		       /* _XA_WM_PROTOCOLS */
-        
+    
+    Atom clipboardAtom;		       /* CLIPBOARD */
+
 
     /* stuff for detecting double-clicks */
     Time lastClickTime;		       /* time of last mousedown event */
     Window lastClickWindow;	       /* window of the last mousedown */
-        
+
+    struct W_View *modalView;
+    unsigned modal:1;
+    unsigned ignoreNextDoubleClick:1;
 } W_Screen;
 
 
 
+typedef struct W_ViewDelegate {
+    void *data;
+
+    void (*didMove)(struct W_ViewDelegate*, WMView*);
+
+    void (*didResize)(struct W_ViewDelegate*, WMView*);
+
+    void (*willMove)(struct W_ViewDelegate*, WMView*, int*, int*);
+
+    void (*willResize)(struct W_ViewDelegate*, WMView*, 
+		       unsigned int*, unsigned int*);
+} W_ViewDelegate;
+
+
+
 typedef struct W_View {
     struct W_Screen *screen;
 
     WMWidget *self;		       /* must point to the widget the
 					* view belongs to */
-    
+
+    W_ViewDelegate *delegate;
+
     Window window;
 
     WMSize size;
@@ -220,7 +287,13 @@ typedef struct W_View {
     
     void *hangedData;		       /* data holder for user program */
 
+    WMColor *backColor;
+
+#if 0
+    struct W_DragSourceProcs *dragSourceProcs;
+    struct W_DragDestinationProcs *dragDestinationProcs;
     int helpContext;
+#endif
 
     struct {
 	unsigned int realized:1;
@@ -234,8 +307,15 @@ typedef struct W_View {
 
 	unsigned int dontCompressMotion:1;   /* motion notify event compress */
 	unsigned int notifySizeChanged:1;
-	unsigned int dontCompressExpose:1;   /* will compress all expose 
+	unsigned int dontCompressExpose:1;   /* will compress all expose
 					      events into one */
+	/* toplevel only */
+	unsigned int worksWhenModal:1;
+	unsigned int pendingRelease1:1;
+	unsigned int pendingRelease2:1;
+	unsigned int pendingRelease3:1;
+	unsigned int pendingRelease4:1;
+	unsigned int pendingRelease5:1;
     } flags;
     
     int refCount;
@@ -244,27 +324,21 @@ typedef struct W_View {
 
 typedef struct W_EventHandler {
     unsigned long eventMask;
-    
+
     WMEventProc *proc;
-    
+
     void *clientData;
-    
+
     struct W_EventHandler *nextHandler;
 } W_EventHandler;
 
 
 
-typedef struct W_ViewProcedureTable {
-    void (*setBackgroundColor)(WMWidget*, WMColor *color);
-    void (*resize)(WMWidget*, unsigned int, unsigned int);
-    void (*move)(WMWidget*, int, int);
-} W_ViewProcedureTable;
-
-
 
 typedef struct _WINGsConfiguration {
     char *systemFont;
     char *boldSystemFont;
+    Bool useMultiByte;
     unsigned doubleClickDelay;
 } _WINGsConfiguration;
 
@@ -282,10 +356,13 @@ _WINGsConfiguration WINGsConfiguration;
 #define W_VIEW_MAPPED(view)	(view)->flags.mapped
 
 #define W_PIXEL(c)		(c)->color.pixel
-#define W_GC(c)			(c)->gc
 
 #define W_FONTID(f)		(f)->font->fid
 
+#define W_DRAWABLE(scr)		(scr)->rcontext->drawable
+
+
+
 W_View *W_GetViewForXWindow(Display *display, Window window);
 
 W_View *W_CreateView(W_View *parent);
@@ -299,12 +376,14 @@ void W_DestroyView(W_View *view);
 
 void W_RealizeView(W_View *view);
 
-void W_ReparentView(W_View *view, W_View *newParent);
+void W_ReparentView(W_View *view, W_View *newParent, int x, int y);
 
 void W_MapView(W_View *view);
 
 void W_MapSubviews(W_View *view);
 
+void W_UnmapSubviews(W_View *view);
+
 W_View *W_TopLevelOfView(W_View *view);
 
 void W_UnmapView(W_View *view);
@@ -324,7 +403,7 @@ void W_CleanUpEvents(W_View *view);
 void W_CallDestroyHandlers(W_View *view);
 
 void W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
-			 WMReliefType relief, char *text, int textHeight,
+			 WMReliefType relief, char *text,
 			 WMAlignment alignment, W_Pixmap *image, 
 			 WMImagePosition position, GC backGC, int ofs);
 
@@ -358,8 +437,28 @@ void W_InitApplication(WMScreen *scr);
 
 void W_InitNotificationCenter(void);
 
-W_Class W_RegisterUserWidget(W_ViewProcedureTable *procTable);
+W_Class W_RegisterUserWidget(void);
 
 void W_RedisplayView(WMView *view);
 
 Bool W_ApplicationInitialized(void);
+
+char *W_GetTextSelection(WMScreen *scr, Atom selection);
+
+void W_HandleSelectionEvent(XEvent *event);
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
+#endif /* _WINGSP_H_ */
+
+void W_FlushASAPNotificationQueue();
+
+void W_FlushIdleNotificationQueue();
+
+struct W_Balloon *W_CreateBalloon(WMScreen *scr);
+
+void W_BalloonHandleEnterView(WMView *view);
+
+void W_BalloonHandleLeaveView(WMView *view);
