@@ -1,5 +1,5 @@
 /*
- *  WindowMaker miscelaneous function library
+ *  Window Maker miscelaneous function library
  * 
  *  Copyright (c) 1997 Alfredo K. Kojima
  * 
@@ -28,6 +28,8 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
+#include <assert.h>
+#include <signal.h>
 
 #ifndef False
 # define False 0
@@ -36,7 +38,33 @@
 # define True 1
 #endif
 
-extern void wAbort(int);
+
+static void
+defaultHandler(int bla)
+{
+    if (bla)
+	raise(SIGABRT);
+    else
+	exit(1);
+}
+
+
+static waborthandler *aborthandler = (waborthandler*)defaultHandler;
+
+#define wAbort(a) (*aborthandler)(a)
+
+
+waborthandler*
+wsetabort(waborthandler *handler)
+{
+    waborthandler *old = aborthandler;
+
+    aborthandler = handler;
+
+    return old;
+}
+
+
 
 static int Aborting=0; /* if we're in the middle of an emergency exit */
 
@@ -143,9 +171,26 @@ wrelease(void *ptr)
 char*
 wstrdup(char *str)
 {
-    if (!str)
-	return NULL;
+    assert(str!=NULL);
 
     return strcpy(wmalloc(strlen(str)+1), str);
 }
 
+
+char*
+wstrappend(char *dst, char *src)
+{
+    char *str;
+    
+    if (!dst)
+	return wstrdup(src);
+    else if (!src)
+	return wstrdup(dst);
+    
+    str = wmalloc(strlen(dst)+strlen(src)+1);
+    strcpy(str, dst);
+    strcat(str, src);
+
+    return str;
+}
+
