@@ -3,10 +3,9 @@
 
 
 #include "WINGsP.h"
+#include <math.h> /* for : double rint (double) */
 
 
-char *WMBrowserDidScrollNotification = "WMBrowserDidScrollNotification";
-
 
 typedef struct W_Browser {
     W_Class widgetClass;
@@ -18,33 +17,33 @@ typedef struct W_Browser {
     short columnCount;
     short usedColumnCount;	       /* columns actually being used */
     short minColumnWidth;
- 
+
     short maxVisibleColumns;
     short firstVisibleColumn;
 
     short titleHeight;
-    
+
     short selectedColumn;
 
     WMSize columnSize;
-    
+
 
     void *clientData;
     WMAction *action;
     void *doubleClientData;
     WMAction *doubleAction;
 
-    WMBrowserFillColumnProc *fillColumn;
-    
+    WMBrowserDelegate *delegate;
+
     WMScroller *scroller;
 
     char *pathSeparator;
-    
+
     struct {
 	unsigned int isTitled:1;
 	unsigned int allowMultipleSelection:1;
 	unsigned int hasScroller:1;
-	
+
 	/* */
 	unsigned int loaded:1;
 	unsigned int loadingColumn:1;
@@ -55,11 +54,17 @@ typedef struct W_Browser {
 #define COLUMN_SPACING 	4
 #define TITLE_SPACING 2
 
-#define DEFAULT_WIDTH		305
-#define DEFAULT_HEIGHT		200
-#define DEFAULT_HAS_SCROLLER	True
+#define DEFAULT_WIDTH                 305
+#define DEFAULT_HEIGHT                200
+#define DEFAULT_HAS_SCROLLER          True
+#define DEFAULT_TITLE_HEIGHT          20
+#define DEFAULT_IS_TITLED             True
+#define DEFAULT_MAX_VISIBLE_COLUMNS   2
+#define DEFAULT_SEPARATOR             "/"
+
+#define MIN_VISIBLE_COLUMNS           1
+#define MAX_VISIBLE_COLUMNS           32
 
-#define DEFAULT_SEPARATOR	"/"
 
 #define COLUMN_IS_VISIBLE(b, c)	((c) >= (b)->firstVisibleColumn \
 				&& (c) < (b)->firstVisibleColumn + (b)->maxVisibleColumns)
@@ -70,20 +75,27 @@ static void destroyBrowser(WMBrowser *bPtr);
 
 static void setupScroller(WMBrowser *bPtr);
 
-static void scrollToColumn(WMBrowser *bPtr, int column);
+static void scrollToColumn(WMBrowser *bPtr, int column, Bool updateScroller);
 
-static void paintItem(WMList *lPtr, Drawable d, char *text, int state, 
-		      WMRect *rect);
+static void paintItem(WMList *lPtr, int index, Drawable d, char *text,
+		      int state, WMRect *rect);
 
 static void loadColumn(WMBrowser *bPtr, int column);
 
+static void removeColumn(WMBrowser *bPtr, int column);
 
-static void resizeBrowser(WMWidget*, unsigned int, unsigned int);
+static char*
+createTruncatedString(WMFont *font, char *text, int *textLen, int width);
 
-W_ViewProcedureTable _BrowserViewProcedures = {
+static void willResizeBrowser(W_ViewDelegate*, WMView*, 
+			      unsigned int*, unsigned int*);
+
+W_ViewDelegate _BrowserViewDelegate = {
     NULL,
-	resizeBrowser,
-	NULL
+	NULL,
+	NULL,
+	NULL,
+	willResizeBrowser
 };
 
 
@@ -94,11 +106,13 @@ WMCreateBrowser(WMWidget *parent)
     WMBrowser *bPtr;
     int i;
 
+    wassertrv(parent, NULL);
+
     bPtr = wmalloc(sizeof(WMBrowser));
     memset(bPtr, 0, sizeof(WMBrowser));
 
     bPtr->widgetClass = WC_Browser;
-    
+
     bPtr->view = W_CreateView(W_VIEW(parent));
     if (!bPtr->view) {
 	free(bPtr);
@@ -106,18 +120,20 @@ WMCreateBrowser(WMWidget *parent)
     }
     bPtr->view->self = bPtr;
 
+    bPtr->view->delegate = &_BrowserViewDelegate;
+
     WMCreateEventHandler(bPtr->view, ExposureMask|StructureNotifyMask
 			 |ClientMessageMask, handleEvents, bPtr);
-	
+
     /* default configuration */
     bPtr->flags.hasScroller = DEFAULT_HAS_SCROLLER;
 
-    bPtr->titleHeight = 20;
-    bPtr->flags.isTitled = 1;
-    bPtr->maxVisibleColumns = 2;
+    bPtr->titleHeight = DEFAULT_TITLE_HEIGHT;
+    bPtr->flags.isTitled = DEFAULT_IS_TITLED;
+    bPtr->maxVisibleColumns = DEFAULT_MAX_VISIBLE_COLUMNS;
+
+    WMResizeWidget(bPtr, DEFAULT_WIDTH, DEFAULT_HEIGHT);
 
-    resizeBrowser(bPtr, DEFAULT_WIDTH, DEFAULT_HEIGHT);
-    
     bPtr->pathSeparator = wstrdup(DEFAULT_SEPARATOR);
 
     if (bPtr->flags.hasScroller)
@@ -127,14 +143,79 @@ WMCreateBrowser(WMWidget *parent)
 	WMAddBrowserColumn(bPtr);
     }
     bPtr->usedColumnCount = 0;
-    
+
     bPtr->selectedColumn = -1;
 
     return bPtr;
 }
 
 
-int 
+int
+WMGetBrowserMaxVisibleColumns(WMBrowser *bPtr)
+{
+    return bPtr->maxVisibleColumns;
+}
+
+
+void
+WMSetBrowserMaxVisibleColumns(WMBrowser *bPtr, int columns)
+{
+    int curMaxVisibleColumns;
+    int newFirstVisibleColumn = 0;
+
+    assert ((int) bPtr);
+
+    columns = (columns < MIN_VISIBLE_COLUMNS) ? MIN_VISIBLE_COLUMNS : columns;
+    columns = (columns > MAX_VISIBLE_COLUMNS) ? MAX_VISIBLE_COLUMNS : columns;
+    if (columns == bPtr->maxVisibleColumns) {
+    	return;
+    }
+    curMaxVisibleColumns = bPtr->maxVisibleColumns;
+    bPtr->maxVisibleColumns = columns;
+    /* browser not loaded */
+    if (!bPtr->flags.loaded) {
+    	if ((columns > curMaxVisibleColumns) && (columns > bPtr->columnCount)) {
+	    int i = columns - bPtr->columnCount;
+	    bPtr->usedColumnCount = bPtr->columnCount;
+	    while (i--) {
+	        WMAddBrowserColumn(bPtr);
+	    }
+	    bPtr->usedColumnCount = 0;
+	}
+    /* browser loaded and columns > curMaxVisibleColumns */
+    } else if (columns > curMaxVisibleColumns) {
+	if (bPtr->usedColumnCount > columns) {
+	    newFirstVisibleColumn = bPtr->usedColumnCount - columns;
+	}
+	if (newFirstVisibleColumn > bPtr->firstVisibleColumn) {
+	    newFirstVisibleColumn = bPtr->firstVisibleColumn;
+	}
+	if (columns > bPtr->columnCount) {
+	    int i = columns - bPtr->columnCount;
+	    int curUsedColumnCount = bPtr->usedColumnCount;
+	    bPtr->usedColumnCount = bPtr->columnCount;
+	    while (i--) {
+		WMAddBrowserColumn(bPtr);
+	    }
+	    bPtr->usedColumnCount = curUsedColumnCount;
+	}
+    /* browser loaded and columns < curMaxVisibleColumns */
+    } else {
+	newFirstVisibleColumn = bPtr->firstVisibleColumn;
+	if (newFirstVisibleColumn + columns >= bPtr->usedColumnCount) {
+	    removeColumn(bPtr, newFirstVisibleColumn + columns);
+	}
+    }
+    WMResizeWidget(bPtr, bPtr->view->size.width, bPtr->view->size.height);
+    if (bPtr->flags.loaded) {
+	XClearArea(bPtr->view->screen->display, bPtr->view->window, 0, 0,
+		   bPtr->view->size.width, bPtr->titleHeight, False);
+	scrollToColumn (bPtr, newFirstVisibleColumn, True);
+    }
+}
+
+
+int
 WMGetBrowserNumberOfColumns(WMBrowser *bPtr)
 {
     return bPtr->usedColumnCount;
@@ -155,35 +236,34 @@ drawTitleOfColumn(WMBrowser *bPtr, int column)
 {
     WMScreen *scr = bPtr->view->screen;
     int x;
-    
+
     x=(column-bPtr->firstVisibleColumn)*(bPtr->columnSize.width+COLUMN_SPACING);
 
-    XFillRectangle(scr->display, bPtr->view->window, W_GC(scr->darkGray), x, 0,
+    XFillRectangle(scr->display, bPtr->view->window, WMColorGC(scr->darkGray), x, 0,
 		   bPtr->columnSize.width, bPtr->titleHeight);
     W_DrawRelief(scr, bPtr->view->window, x, 0,
 		 bPtr->columnSize.width, bPtr->titleHeight, WRSunken);
 
-    if (column < bPtr->usedColumnCount && bPtr->titles[column])
-	W_PaintText(bPtr->view, bPtr->view->window, scr->boldFont, x, 
-		    (bPtr->titleHeight-scr->boldFont->height)/2,
-		    bPtr->columnSize.width, WACenter, W_GC(scr->white),
-		    False, bPtr->titles[column], strlen(bPtr->titles[column]));
-}
+    if (column < bPtr->usedColumnCount && bPtr->titles[column]) {
+	int titleLen = strlen(bPtr->titles[column]);
+	int widthC = bPtr->columnSize.width-8;
 
-
-void
-WMSetBrowserColumnTitle(WMBrowser *bPtr, int column, char *title)
-{
-    assert(column >= 0);
-    assert(column < bPtr->usedColumnCount);
-
-    if (bPtr->titles[column])
-	free(bPtr->titles[column]);
-
-    bPtr->titles[column] = wstrdup(title);
-   
-    if (COLUMN_IS_VISIBLE(bPtr, column) && bPtr->flags.isTitled) {
-	drawTitleOfColumn(bPtr, column);
+	if (WMWidthOfString(scr->boldFont, bPtr->titles[column], titleLen)
+	    > widthC) {
+	    char *titleBuf = createTruncatedString(scr->boldFont,
+						   bPtr->titles[column],
+						   &titleLen, widthC);
+	    W_PaintText(bPtr->view, bPtr->view->window, scr->boldFont, x,
+			(bPtr->titleHeight-WMFontHeight(scr->boldFont))/2,
+			bPtr->columnSize.width, WACenter, WMColorGC(scr->white),
+			False, titleBuf, titleLen);
+	    free (titleBuf);
+	} else {
+	    W_PaintText(bPtr->view, bPtr->view->window, scr->boldFont, x,
+			(bPtr->titleHeight-WMFontHeight(scr->boldFont))/2,
+			bPtr->columnSize.width, WACenter, WMColorGC(scr->white),
+			False, bPtr->titles[column], titleLen);
+	}
     }
 }
 
@@ -193,19 +273,19 @@ WMGetBrowserListInColumn(WMBrowser *bPtr, int column)
 {
     if (column < 0 || column >= bPtr->usedColumnCount)
 	return NULL;
-    
+
     return bPtr->columns[column];
 }
 
 
-void 
-WMSetBrowserFillColumnProc(WMBrowser *bPtr, WMBrowserFillColumnProc *proc)
+void
+WMSetBrowserDelegate(WMBrowser *bPtr, WMBrowserDelegate *delegate)
 {
-    bPtr->fillColumn = proc;
+    bPtr->delegate = delegate;
 }
 
 
-int 
+int
 WMGetBrowserFirstVisibleColumn(WMBrowser *bPtr)
 {
     return bPtr->firstVisibleColumn;
@@ -215,50 +295,50 @@ WMGetBrowserFirstVisibleColumn(WMBrowser *bPtr)
 static void
 removeColumn(WMBrowser *bPtr, int column)
 {
-    int i;
+    int i, clearEnd, destroyEnd;
     WMList **clist;
     char **tlist;
 
-    if (column >= bPtr->usedColumnCount)
-	return;
+    assert ((int) bPtr);
 
-    if (column < bPtr->maxVisibleColumns) {
-	int tmp;
-	for (i=column; i < bPtr->maxVisibleColumns; i++) {
-	    if (bPtr->titles[i])
-		free(bPtr->titles[i]);
-	    bPtr->titles[i] = NULL;
-	    
-	    WMClearList(bPtr->columns[i]);
-	    bPtr->usedColumnCount--;
-	}
-	tmp = bPtr->columnCount;
-	for (i=bPtr->maxVisibleColumns; i < tmp; i++) {
-	    if (bPtr->titles[i])
-		free(bPtr->titles[i]);
-	    bPtr->titles[i] = NULL;
-	    
-	    WMDestroyWidget(bPtr->columns[i]);
-	    bPtr->columns[i] = NULL;
-	    bPtr->columnCount--;
-	    bPtr->usedColumnCount--;
-	}
-    } else {
-	for (i=column; i < bPtr->columnCount; i++) {
-	    if (bPtr->titles[i])
-		free(bPtr->titles[i]);
-	    bPtr->titles[i] = NULL;
-	    
-	    WMDestroyWidget(bPtr->columns[i]);
-	    bPtr->columns[i] = NULL;
-	    bPtr->columnCount--;
-	    bPtr->usedColumnCount--;
-	}	
+    column = (column < 0) ? 0 : column;
+    if (column >= bPtr->columnCount) {
+	return;
     }
-    clist = wmalloc(sizeof(WMList*)*bPtr->columnCount);
-    tlist = wmalloc(sizeof(char*)*bPtr->columnCount);
-    memcpy(clist, bPtr->columns, sizeof(WMList*)*bPtr->columnCount);
-    memcpy(tlist, bPtr->titles, sizeof(char*)*bPtr->columnCount);
+    if (column < bPtr->maxVisibleColumns) {
+	clearEnd = bPtr->maxVisibleColumns;
+	destroyEnd = bPtr->columnCount;
+	bPtr->columnCount = bPtr->maxVisibleColumns;
+    } else {
+	clearEnd = column;
+	destroyEnd = bPtr->columnCount;
+	bPtr->columnCount = column;
+    }
+    if (column < bPtr->usedColumnCount) {
+	bPtr->usedColumnCount = column;
+    }
+    for (i=column; i < clearEnd; i++) {
+	if (bPtr->titles[i]) {
+	    free(bPtr->titles[i]);
+	    bPtr->titles[i] = NULL;
+	}
+	WMClearList(bPtr->columns[i]);
+    }
+    for (;i < destroyEnd; i++) {
+	if (bPtr->titles[i]) {
+	    free(bPtr->titles[i]);
+	    bPtr->titles[i] = NULL;
+	}
+        WMRemoveNotificationObserverWithName(bPtr,
+                                             WMListSelectionDidChangeNotification,
+					     bPtr->columns[i]);
+	WMDestroyWidget(bPtr->columns[i]);
+	bPtr->columns[i] = NULL;
+    }
+    clist = wmalloc(sizeof(WMList*) * (bPtr->columnCount));
+    tlist = wmalloc(sizeof(char*) * (bPtr->columnCount));
+    memcpy(clist, bPtr->columns, sizeof(WMList*) * (bPtr->columnCount));
+    memcpy(tlist, bPtr->titles, sizeof(char*) * (bPtr->columnCount));
     free(bPtr->titles);
     free(bPtr->columns);
     bPtr->titles = tlist;
@@ -266,11 +346,10 @@ removeColumn(WMBrowser *bPtr, int column)
 }
 
 
-
 WMListItem*
 WMGetBrowserSelectedItemInColumn(WMBrowser *bPtr, int column)
 {
-    if ((column < 0) || (column > bPtr->columnCount))
+    if ((column < 0) || (column >= bPtr->usedColumnCount))
 	return NULL;
 
     return WMGetListSelectedItem(bPtr->columns[column]);
@@ -293,7 +372,24 @@ WMGetBrowserSelectedRowInColumn(WMBrowser *bPtr, int column)
     } else {
 	return -1;
     }
-} 
+}
+
+
+void
+WMSetBrowserColumnTitle(WMBrowser *bPtr, int column, char *title)
+{
+    assert(column >= 0);
+    assert(column < bPtr->usedColumnCount);
+
+    if (bPtr->titles[column])
+	free(bPtr->titles[column]);
+
+    bPtr->titles[column] = wstrdup(title);
+
+    if (COLUMN_IS_VISIBLE(bPtr, column) && bPtr->flags.isTitled) {
+	drawTitleOfColumn(bPtr, column);
+    }
+}
 
 
 void
@@ -306,7 +402,7 @@ WMSetBrowserTitled(WMBrowser *bPtr, Bool flag)
 	return;
 
     columnX = 0;
-    
+
     if (!bPtr->flags.isTitled) {
 	columnY = TITLE_SPACING + bPtr->titleHeight;
 
@@ -332,7 +428,7 @@ WMSetBrowserTitled(WMBrowser *bPtr, Bool flag)
 	    WMMoveWidget(bPtr->columns[i], columnX, 0);
 	}
     }
-    
+
     bPtr->flags.isTitled = flag;
 }
 
@@ -344,7 +440,7 @@ WMAddSortedBrowserItem(WMBrowser *bPtr, int column, char *text, Bool isBranch)
 
     if (column < 0 || column >= bPtr->columnCount)
 	return NULL;
-	
+
     item = WMAddSortedListItem(bPtr->columns[column], text);
     item->isBranch = isBranch;
 
@@ -371,33 +467,34 @@ WMInsertBrowserItem(WMBrowser *bPtr, int column, int row, char *text,
 
 
 
-static void 
-resizeBrowser(WMWidget *w, unsigned int width, unsigned int height)
+static void
+willResizeBrowser(W_ViewDelegate *self, WMView *view, 
+		  unsigned int *width, unsigned int *height)
 {
-    WMBrowser *bPtr = (WMBrowser*)w;
+    WMBrowser *bPtr = (WMBrowser*)view->self;
     int cols = bPtr->maxVisibleColumns;
     int colX, colY;
     int i;
 
-    assert(width > 0);
-    assert(height > 0);
-    
-    bPtr->columnSize.width = (width-(cols-1)*COLUMN_SPACING) / cols;
-    bPtr->columnSize.height = height;
-    
+    assert(*width > 0);
+    assert(*height > 0);
+
+    bPtr->columnSize.width = (*width-(cols-1)*COLUMN_SPACING) / cols;
+    bPtr->columnSize.height = *height;
+
     if (bPtr->flags.isTitled) {
-	bPtr->columnSize.height -= TITLE_SPACING + bPtr->titleHeight;
 	colY = TITLE_SPACING + bPtr->titleHeight;
+	bPtr->columnSize.height -= colY;
     } else {
 	colY = 0;
     }
-    
+
     if (bPtr->flags.hasScroller) {
 	bPtr->columnSize.height -= SCROLLER_WIDTH + 4;
 
 	if (bPtr->scroller) {
-	    WMResizeWidget(bPtr->scroller, width-2, 1);
-	    WMMoveWidget(bPtr->scroller, 1, height-SCROLLER_WIDTH-1);
+	    WMResizeWidget(bPtr->scroller, *width-2, 1);
+	    WMMoveWidget(bPtr->scroller, 1, *height-SCROLLER_WIDTH-1);
 	}
     }
 
@@ -405,20 +502,19 @@ resizeBrowser(WMWidget *w, unsigned int width, unsigned int height)
     for (i = 0; i < bPtr->columnCount; i++) {
 	WMResizeWidget(bPtr->columns[i], bPtr->columnSize.width,
 		       bPtr->columnSize.height);
-	
+
 	WMMoveWidget(bPtr->columns[i], colX, colY);
-	
+
 	if (COLUMN_IS_VISIBLE(bPtr, i)) {
 	    colX += bPtr->columnSize.width+COLUMN_SPACING;
 	}
     }
-
-    W_ResizeView(bPtr->view, width, height);
 }
 
 
 static void
-paintItem(WMList *lPtr, Drawable d, char *text, int state, WMRect *rect)
+paintItem(WMList *lPtr, int index, Drawable d, char *text, int state,
+	  WMRect *rect)
 {
     WMView *view = W_VIEW(lPtr);
     W_Screen *scr = view->screen;
@@ -430,24 +526,37 @@ paintItem(WMList *lPtr, Drawable d, char *text, int state, WMRect *rect)
     y = rect->pos.y;
 
     if (state & WLDSSelected)
-	XFillRectangle(scr->display, d, W_GC(scr->white), x, y,
+	XFillRectangle(scr->display, d, WMColorGC(scr->white), x, y,
 		       width, height);
-    else 
+    else
 	XClearArea(scr->display, d, x, y, width, height, False);
-	
-    W_PaintText(view, d, scr->normalFont,  x+2, y, width,
-		WALeft, W_GC(scr->black), False, text, strlen(text));
-    
+
+    if (text) {
+	/* Avoid overlaping... */
+	int textLen = strlen(text);
+	int widthC = (state & WLDSIsBranch) ? width-20 : width-8;
+	if (WMWidthOfString(scr->normalFont, text, textLen) > widthC) {
+	    char *textBuf = createTruncatedString(scr->normalFont,
+		                                  text, &textLen, widthC);
+            W_PaintText(view, d, scr->normalFont,  x+4, y, widthC,
+		    	WALeft, WMColorGC(scr->black), False, textBuf, textLen);
+	    free(textBuf);
+	} else {
+      	    W_PaintText(view, d, scr->normalFont,  x+4, y, widthC,
+		    	WALeft, WMColorGC(scr->black), False, text, textLen);
+	}
+    }
+
     if (state & WLDSIsBranch) {
-	XDrawLine(scr->display, d, W_GC(scr->darkGray), x+width-11, y+3,
+	XDrawLine(scr->display, d, WMColorGC(scr->darkGray), x+width-11, y+3,
 		  x+width-6, y+height/2);
 	if (state & WLDSSelected)
-	    XDrawLine(scr->display, d,W_GC(scr->gray), x+width-11, y+height-5,
+	    XDrawLine(scr->display, d,WMColorGC(scr->gray), x+width-11, y+height-5,
 		      x+width-6, y+height/2);
 	else
-	    XDrawLine(scr->display, d,W_GC(scr->white), x+width-11, y+height-5,
+	    XDrawLine(scr->display, d,WMColorGC(scr->white), x+width-11, y+height-5,
 		      x+width-6, y+height/2);
-	XDrawLine(scr->display, d, W_GC(scr->black), x+width-12, y+3,
+	XDrawLine(scr->display, d, WMColorGC(scr->black), x+width-12, y+3,
 		  x+width-12, y+height-5);
     }
 }
@@ -464,52 +573,53 @@ scrollCallback(WMWidget *scroller, void *self)
     switch (WMGetScrollerHitPart(sPtr)) {
      case WSDecrementLine:
 	if (bPtr->firstVisibleColumn > 0) {
-	    scrollToColumn(bPtr, bPtr->firstVisibleColumn-1);
+	    scrollToColumn(bPtr, bPtr->firstVisibleColumn-1, True);
 	}
 	break;
-	
+
      case WSDecrementPage:
 	if (bPtr->firstVisibleColumn > 0) {
 	    newFirst = bPtr->firstVisibleColumn - bPtr->maxVisibleColumns;
 
-	    scrollToColumn(bPtr, newFirst);
+	    scrollToColumn(bPtr, newFirst, True);
 	}
 	break;
 
-	
+
      case WSIncrementLine:
-	if (LAST_VISIBLE_COLUMN < bPtr->columnCount) {
-	    scrollToColumn(bPtr, bPtr->firstVisibleColumn+1);
+	if (LAST_VISIBLE_COLUMN < bPtr->usedColumnCount) {
+	    scrollToColumn(bPtr, bPtr->firstVisibleColumn+1, True);
 	}
 	break;
-	
+
      case WSIncrementPage:
-	if (LAST_VISIBLE_COLUMN < bPtr->columnCount) {
+	if (LAST_VISIBLE_COLUMN < bPtr->usedColumnCount) {
 	    newFirst = bPtr->firstVisibleColumn + bPtr->maxVisibleColumns;
 
 	    if (newFirst+bPtr->maxVisibleColumns >= bPtr->columnCount)
 		newFirst = bPtr->columnCount - bPtr->maxVisibleColumns;
 
-	    scrollToColumn(bPtr, newFirst);
+	    scrollToColumn(bPtr, newFirst, True);
 	}
 	break;
-	
+
      case WSKnob:
 	{
-	    float floatValue;
-	    float value = bPtr->columnCount - bPtr->maxVisibleColumns;
-	    
+	    double floatValue;
+	    double value = bPtr->columnCount - bPtr->maxVisibleColumns;
+
 	    floatValue = WMGetScrollerValue(bPtr->scroller);
-	    
+
 	    floatValue = (floatValue*value)/value;
-	    
-	    newFirst = floatValue*(float)(bPtr->columnCount - bPtr->maxVisibleColumns);
+
+	    newFirst = rint(floatValue*(float)(bPtr->columnCount - bPtr->maxVisibleColumns));
 
 	    if (bPtr->firstVisibleColumn != newFirst)
-		scrollToColumn(bPtr, newFirst);
-	    else
+		scrollToColumn(bPtr, newFirst, False);
+/*	    else
 		WMSetScrollerParameters(bPtr->scroller, floatValue,
 					bPtr->maxVisibleColumns/(float)bPtr->columnCount);
+ */
 
 	}
 	break;
@@ -530,14 +640,14 @@ setupScroller(WMBrowser *bPtr)
     int y;
 
     y = bPtr->view->size.height - SCROLLER_WIDTH - 1;
-    
+
     sPtr = WMCreateScroller(bPtr);
     WMSetScrollerAction(sPtr, scrollCallback, bPtr);
     WMMoveWidget(sPtr, 1, y);
     WMResizeWidget(sPtr, bPtr->view->size.width-2, SCROLLER_WIDTH);
-    
+
     bPtr->scroller = sPtr;
-    
+
     WMMapWidget(sPtr);
 }
 
@@ -550,6 +660,14 @@ WMSetBrowserAction(WMBrowser *bPtr, WMAction *action, void *clientData)
 }
 
 
+void
+WMSetBrowserDoubleAction(WMBrowser *bPtr, WMAction *action, void *clientData)
+{
+    bPtr->doubleAction = action;
+    bPtr->doubleClientData = clientData;
+}
+
+
 void
 WMSetBrowserHasScroller(WMBrowser *bPtr, int hasScroller)
 {
@@ -558,42 +676,72 @@ WMSetBrowserHasScroller(WMBrowser *bPtr, int hasScroller)
 
 
 
-Bool
+char*
 WMSetBrowserPath(WMBrowser *bPtr, char *path)
 {
     int i;
     char *str = wstrdup(path);
-    char *tmp;
+    char *tmp, *retPtr = NULL;
     int item;
-    Bool ok = True;
+    WMListItem *listItem;
+
+    /* WMLoadBrowserColumnZero must be call first */
+    if (!bPtr->flags.loaded) {
+	return False;
+    }
 
     removeColumn(bPtr, 1);
 
+    WMSelectListItem(bPtr->columns[0], -1);
+    WMSetListPosition(bPtr->columns[0], 0);
+
     i = 0;
     tmp = strtok(str, bPtr->pathSeparator);
     while (tmp) {
 	/* select it in the column */
-	item = WMFindRowOfListItemWithTitle(bPtr->columns[i], tmp);
+        item = WMFindRowOfListItemWithTitle(bPtr->columns[i], tmp);
 	if (item<0) {
-	    ok = False;
+            retPtr = &path[(int)(tmp - str)];
 	    break;
 	}
 	WMSelectListItem(bPtr->columns[i], item);
-	
+	WMSetListPosition(bPtr->columns[i], item);
+
+	listItem = WMGetListItem(bPtr->columns[i], item);
+	if (!listItem || !listItem->isBranch) {
+	    break;
+	}
+
 	/* load next column */
 	WMAddBrowserColumn(bPtr);
 
 	loadColumn(bPtr, i+1);
-	
+
 	tmp = strtok(NULL, bPtr->pathSeparator);
 
 	i++;
     }
     free(str);
 
-    scrollToColumn(bPtr, bPtr->columnCount-bPtr->maxVisibleColumns);
+    for (i = bPtr->usedColumnCount - 1;
+    	 (i > -1) && !WMGetListSelectedItem(bPtr->columns[i]);
+	 i--);
 
-    return ok;
+    bPtr->selectedColumn = i;
+
+    if (bPtr->columnCount < bPtr->maxVisibleColumns) {
+	int i = bPtr->maxVisibleColumns - bPtr->columnCount;
+	int curUsedColumnCount = bPtr->usedColumnCount;
+	bPtr->usedColumnCount = bPtr->columnCount;
+	while (i--) {
+	    WMAddBrowserColumn(bPtr);
+	}
+	bPtr->usedColumnCount = curUsedColumnCount;
+    }
+
+    scrollToColumn(bPtr, bPtr->columnCount - bPtr->maxVisibleColumns, True);
+
+    return retPtr;
 }
 
 
@@ -604,16 +752,21 @@ WMGetBrowserPath(WMBrowser *bPtr)
 }
 
 
+
 char*
 WMGetBrowserPathToColumn(WMBrowser *bPtr, int column)
 {
     int i, size;
     char *path;
     WMListItem *item;
-    
+
     if (column >= bPtr->usedColumnCount)
 	column = bPtr->usedColumnCount-1;
-    
+
+    if (column < 0) {
+        return wstrdup(bPtr->pathSeparator);
+    }
+
     /* calculate size of buffer */
     size = 0;
     for (i = 0; i <= column; i++) {
@@ -622,7 +775,7 @@ WMGetBrowserPathToColumn(WMBrowser *bPtr, int column)
 	    break;
 	size += strlen(item->text);
     }
-    
+
     /* get the path */
     path = wmalloc(size+(column+1)*strlen(bPtr->pathSeparator)+1);
     /* ignore first / */
@@ -642,11 +795,28 @@ WMGetBrowserPathToColumn(WMBrowser *bPtr, int column)
 static void
 loadColumn(WMBrowser *bPtr, int column)
 {
-    assert(bPtr->fillColumn);
-    
+    assert(bPtr->delegate);
+    assert(bPtr->delegate->createRowsForColumn);
+
     bPtr->flags.loadingColumn = 1;
-    (*bPtr->fillColumn)(bPtr, column);
+    (*bPtr->delegate->createRowsForColumn)(bPtr->delegate, bPtr, column,
+					   bPtr->columns[column]);
     bPtr->flags.loadingColumn = 0;
+
+    if (bPtr->delegate->titleOfColumn) {
+	char *title;
+
+	title = (*bPtr->delegate->titleOfColumn)(bPtr->delegate, bPtr, column);
+
+	if (bPtr->titles[column])
+	    free(bPtr->titles[column]);
+
+	bPtr->titles[column] = wstrdup(title);
+
+        if (COLUMN_IS_VISIBLE(bPtr, column) && bPtr->flags.isTitled) {
+            drawTitleOfColumn(bPtr, column);
+        }
+    }
 }
 
 
@@ -658,10 +828,10 @@ paintBrowser(WMBrowser *bPtr)
     if (!bPtr->view->flags.mapped)
 	return;
 
-    W_DrawRelief(bPtr->view->screen, bPtr->view->window, 0, 
+    W_DrawRelief(bPtr->view->screen, bPtr->view->window, 0,
 		 bPtr->view->size.height-SCROLLER_WIDTH-2,
 		 bPtr->view->size.width, 22, WRSunken);
-    
+
     if (bPtr->flags.isTitled) {
 	for (i=0; i<bPtr->maxVisibleColumns; i++) {
 	    drawTitleOfColumn(bPtr, i+bPtr->firstVisibleColumn);
@@ -682,62 +852,70 @@ handleEvents(XEvent *event, void *data)
      case Expose:
 	paintBrowser(bPtr);
 	break;
-	
+
      case DestroyNotify:
 	destroyBrowser(bPtr);
 	break;
-	
+
     }
 }
 
 
 
 static void
-scrollToColumn(WMBrowser *bPtr, int column)
+scrollToColumn(WMBrowser *bPtr, int column, Bool updateScroller)
 {
     int i;
     int x;
     int notify = 0;
 
-    
-    if (column != bPtr->firstVisibleColumn)
+
+    if (column != bPtr->firstVisibleColumn) {
 	notify = 1;
+    }
 
     if (column < 0)
 	column = 0;
 
+    if (notify && bPtr->delegate && bPtr->delegate->willScroll) {
+	(*bPtr->delegate->willScroll)(bPtr->delegate, bPtr);
+    }
+
     x = 0;
     bPtr->firstVisibleColumn = column;
-    for (i = 0; i < bPtr->usedColumnCount; i++) {
+    for (i = 0; i < bPtr->columnCount; i++) {
 	if (COLUMN_IS_VISIBLE(bPtr, i)) {
 	    WMMoveWidget(bPtr->columns[i], x,
 			 WMWidgetView(bPtr->columns[i])->pos.y);
 	    if (!WMWidgetView(bPtr->columns[i])->flags.realized)
 		WMRealizeWidget(bPtr->columns[i]);
 	    WMMapWidget(bPtr->columns[i]);
-	    x += bPtr->columnSize.width+COLUMN_SPACING;
+	    x += bPtr->columnSize.width + COLUMN_SPACING;
 	} else {
 	    WMUnmapWidget(bPtr->columns[i]);
 	}
     }
 
     /* update the scroller */
-    if (bPtr->columnCount > bPtr->maxVisibleColumns) {
-	float value, proportion;
+    if (updateScroller) {
+	if (bPtr->columnCount > bPtr->maxVisibleColumns) {
+	    float value, proportion;
 
-	value = bPtr->firstVisibleColumn
-	    /(float)(bPtr->columnCount-bPtr->maxVisibleColumns);
-	proportion = bPtr->maxVisibleColumns/(float)bPtr->columnCount;
-	WMSetScrollerParameters(bPtr->scroller, value, proportion);
-    } else {
-	WMSetScrollerParameters(bPtr->scroller, 0, 1);
+	    value = bPtr->firstVisibleColumn
+		/(float)(bPtr->columnCount-bPtr->maxVisibleColumns);
+	    proportion = bPtr->maxVisibleColumns/(float)bPtr->columnCount;
+	    WMSetScrollerParameters(bPtr->scroller, value, proportion);
+	} else {
+	    WMSetScrollerParameters(bPtr->scroller, 0, 1);
+	}
     }
-    
+
     if (bPtr->view->flags.mapped)
 	paintBrowser(bPtr);
 
-    if (notify)
-	WMPostNotificationName(WMBrowserDidScrollNotification, bPtr, NULL);
+    if (notify && bPtr->delegate && bPtr->delegate->didScroll) {
+	(*bPtr->delegate->didScroll)(bPtr->delegate, bPtr);
+    }
 }
 
 
@@ -747,36 +925,63 @@ listCallback(void *self, void *clientData)
     WMBrowser *bPtr = (WMBrowser*)clientData;
     WMList *lPtr = (WMList*)self;
     WMListItem *item;
+    static WMListItem *oldItem = NULL;
     int i;
-    
-    item = WMGetListSelectedItem(lPtr);
-    if (!item)
-	return;
-    
-    for (i=0; i<bPtr->columnCount; i++) {
-	if (lPtr == bPtr->columns[i])
-	    break;
-    }
-    assert(i<bPtr->columnCount);
-    
-    bPtr->selectedColumn = i;
 
-    /* columns at right must be cleared */
-    removeColumn(bPtr, i+1);
-    /* open directory */
-    if (item->isBranch) {	
-	WMAddBrowserColumn(bPtr);
-	loadColumn(bPtr, bPtr->usedColumnCount-1);
+    item = WMGetListSelectedItem(lPtr);
+    if (!item) {
+        oldItem = item;
+        return;
     }
-    if (bPtr->usedColumnCount < bPtr->maxVisibleColumns)
-	i = 0;
-    else
-	i = bPtr->usedColumnCount-bPtr->maxVisibleColumns;
-    scrollToColumn(bPtr, i);
+
+    if (oldItem != item) {
+        for (i=0; i<bPtr->columnCount; i++) {
+            if (lPtr == bPtr->columns[i])
+                break;
+        }
+        assert(i<bPtr->columnCount);
+
+	bPtr->selectedColumn = i;
+
+        /* columns at right must be cleared */
+        removeColumn(bPtr, i+1);
+        /* open directory */
+        if (item->isBranch) {
+            WMAddBrowserColumn(bPtr);
+        }
+        if (bPtr->usedColumnCount < bPtr->maxVisibleColumns)
+            i = 0;
+        else
+            i = bPtr->usedColumnCount-bPtr->maxVisibleColumns;
+        scrollToColumn(bPtr, i, True);
+	if (item->isBranch) {
+            loadColumn(bPtr, bPtr->usedColumnCount-1);
+	}
+    }
+
 
     /* call callback for click */
     if (bPtr->action)
 	(*bPtr->action)(bPtr, bPtr->clientData);
+
+    oldItem = item;
+}
+
+
+static void
+listDoubleCallback(void *self, void *clientData)
+{
+    WMBrowser *bPtr = (WMBrowser*)clientData;
+    WMList *lPtr = (WMList*)self;
+    WMListItem *item;
+
+    item = WMGetListSelectedItem(lPtr);
+    if (!item)
+	return;
+
+    /* call callback for double click */
+    if (bPtr->doubleAction)
+	(*bPtr->doubleAction)(bPtr, bPtr->doubleClientData);
 }
 
 
@@ -788,12 +993,12 @@ WMLoadBrowserColumnZero(WMBrowser *bPtr)
 	WMAddBrowserColumn(bPtr);
 
 	loadColumn(bPtr, 0);
-	    
+
 	/* make column 0 visible */
-	scrollToColumn(bPtr, 0);
+	scrollToColumn(bPtr, 0, True);
 
 	bPtr->flags.loaded = 1;
-    } 
+    }
 }
 
 
@@ -801,25 +1006,50 @@ void
 WMRemoveBrowserItem(WMBrowser *bPtr, int column, int row)
 {
     WMList *list;
-    
+
     if (column < 0 || column >= bPtr->usedColumnCount)
 	return;
-    
+
     list = WMGetBrowserListInColumn(bPtr, column);
-    
+
     if (row < 0 || row >= WMGetListNumberOfRows(list))
 	return;
-    
+
     removeColumn(bPtr, column+1);
     if (bPtr->usedColumnCount < bPtr->maxVisibleColumns)
-	scrollToColumn(bPtr, 0);
+	scrollToColumn(bPtr, 0, True);
     else
-	scrollToColumn(bPtr, bPtr->usedColumnCount-bPtr->maxVisibleColumns);
-    
+	scrollToColumn(bPtr, bPtr->usedColumnCount-bPtr->maxVisibleColumns,
+		       True);
+
     WMRemoveListItem(list, row);
 }
 
 
+static void
+listSelectionObserver(void *observerData, WMNotification *notification)
+{
+    WMBrowser *bPtr = (WMBrowser*)observerData;
+    int column, item = (int)WMGetNotificationClientData(notification);
+    WMList *lPtr = (WMList*)WMGetNotificationObject(notification);
+
+    for (column = 0; column < bPtr->usedColumnCount; column++)
+        if (bPtr->columns[column] == lPtr)
+            break;
+
+    /* this can happen when a list is being cleared with WMClearList
+     * after the column was removed */
+    if (column >= bPtr->usedColumnCount) {
+	return;
+    }
+
+    if (item < 0)
+        column--;
+
+    bPtr->selectedColumn = column;
+}
+
+
 int
 WMAddBrowserColumn(WMBrowser *bPtr)
 {
@@ -829,7 +1059,7 @@ WMAddBrowserColumn(WMBrowser *bPtr)
     int colY;
     int index;
 
-    
+
     if (bPtr->usedColumnCount < bPtr->columnCount) {
 	return bPtr->usedColumnCount++;
     }
@@ -859,12 +1089,16 @@ WMAddBrowserColumn(WMBrowser *bPtr)
 
     list = WMCreateList(bPtr);
     WMSetListAction(list, listCallback, bPtr);
+    WMSetListDoubleAction(list, listDoubleCallback, bPtr);
     WMSetListUserDrawProc(list, paintItem);
+    WMAddNotificationObserver(listSelectionObserver, bPtr,
+			      WMListSelectionDidChangeNotification, list);
+
     bPtr->columns[index] = list;
 
     WMResizeWidget(list, bPtr->columnSize.width, bPtr->columnSize.height);
     WMMoveWidget(list, (bPtr->columnSize.width+COLUMN_SPACING)*index, colY);
-    if (COLUMN_IS_VISIBLE(bPtr, index)) 
+    if (COLUMN_IS_VISIBLE(bPtr, index))
 	WMMapWidget(list);
 
     /* update the scroller */
@@ -887,15 +1121,45 @@ destroyBrowser(WMBrowser *bPtr)
 {
     int i;
 
-    for (i=0; i<bPtr->columnCount; i++) {
+    for (i = 0; i < bPtr->columnCount; i++) {
 	if (bPtr->titles[i])
 	    free(bPtr->titles[i]);
     }
     free(bPtr->titles);
-    
+
     free(bPtr->pathSeparator);
-    
+
+    WMRemoveNotificationObserver(bPtr);
+
     free(bPtr);
 }
 
 
+static char*
+createTruncatedString(WMFont *font, char *text, int *textLen, int width)
+{
+    int dLen = WMWidthOfString(font, ".", 1);
+    char *textBuf = (char*)wmalloc((*textLen)+4);
+
+    if (width >= 3*dLen) {
+	int dddLen = 3*dLen;
+	int tmpTextLen = *textLen;
+
+	strcpy(textBuf, text);
+	while (tmpTextLen
+	       && (WMWidthOfString(font, textBuf, tmpTextLen)+dddLen > width))
+	    tmpTextLen--;
+	strcpy(textBuf+tmpTextLen, "...");
+	*textLen = tmpTextLen+3;
+    } else if (width >= 2*dLen) {
+	strcpy(textBuf, "..");
+	*textLen = 2;
+    } else if (width >= dLen) {
+	strcpy(textBuf, ".");
+	*textLen = 1;
+    } else {
+	*textBuf = '\0';
+	*textLen = 0;
+    }
+    return textBuf;
+}
