@@ -9,16 +9,16 @@ typedef struct W_Button {
     WMView *view;
 
     char *caption;
-    int captionHeight;
     
     char *altCaption;
-    int altCaptionHeight;
     
     WMFont *font;
-    
+
     W_Pixmap *image;
     W_Pixmap *altImage;
 
+    W_Pixmap *dimage;
+
     void *clientData;
     WMAction *action;
 
@@ -65,6 +65,8 @@ typedef struct W_Button {
 	unsigned int wasPushed:1;
 
 	unsigned int redrawPending:1;
+	
+	unsigned int addedObserver:1;
     } flags;
 } Button;
 
@@ -96,13 +98,8 @@ static void paintButton(Button *bPtr);
 static void handleEvents(XEvent *event, void *data);
 static void handleActionEvents(XEvent *event, void *data);
 
-static void resizeButton();
 
-W_ViewProcedureTable _ButtonViewProcedures = {
-    NULL,
-	resizeButton,
-	NULL
-};
+static char *WMPushedRadioNotification="WMPushedRadioNotification";
 
 
 #define NFONT(b) (b)->view->screen->normalFont
@@ -142,8 +139,8 @@ WMCreateCustomButton(WMWidget *parent, int behaviourMask)
     bPtr->flags.enabled = 1;
 
 
-    WMCreateEventHandler(bPtr->view, ExposureMask|StructureNotifyMask
-			 |ClientMessageMask, handleEvents, bPtr);
+    WMCreateEventHandler(bPtr->view, ExposureMask|StructureNotifyMask,
+			 handleEvents, bPtr);
 
     WMCreateEventHandler(bPtr->view, ButtonPressMask|ButtonReleaseMask
 			 |EnterWindowMask|LeaveWindowMask, 
@@ -229,14 +226,65 @@ WMCreateButton(WMWidget *parent, WMButtonType type)
 }
 
 
+static void
+updateDisabledMask(WMButton *bPtr)
+{
+    WMScreen *scr = WMWidgetScreen(bPtr);
+    Display *dpy = scr->display;
+
+    if (bPtr->image) {
+	XGCValues gcv;
+
+	bPtr->dimage->mask = XCreatePixmap(dpy, scr->stipple,
+					   bPtr->dimage->width, 
+					   bPtr->dimage->height, 1);
+
+	XSetForeground(dpy, scr->monoGC, 0);
+	XFillRectangle(dpy, bPtr->dimage->mask, scr->monoGC, 0, 0,
+		       bPtr->dimage->width, bPtr->dimage->height);
+
+	gcv.foreground = 1;
+	gcv.background = 0;
+	gcv.stipple = scr->stipple;
+	gcv.fill_style = FillStippled;
+	gcv.clip_mask = bPtr->image->mask;
+	gcv.clip_x_origin = 0;
+	gcv.clip_y_origin = 0;
+	
+	XChangeGC(dpy, scr->monoGC, GCForeground|GCBackground|GCStipple
+		  |GCFillStyle|GCClipMask|GCClipXOrigin|GCClipYOrigin, &gcv);
+
+	XFillRectangle(dpy, bPtr->dimage->mask, scr->monoGC, 0, 0,
+		       bPtr->dimage->width, bPtr->dimage->height);
+
+	gcv.fill_style = FillSolid;
+	gcv.clip_mask = None;
+	XChangeGC(dpy, scr->monoGC, GCFillStyle|GCClipMask, &gcv);
+    }
+}
+
+
 void
 WMSetButtonImage(WMButton *bPtr, WMPixmap *image)
 {
     if (bPtr->image!=NULL)
 	WMReleasePixmap(bPtr->image);
     bPtr->image = WMRetainPixmap(image);
-    
-    
+
+    if (bPtr->dimage) {
+	bPtr->dimage->pixmap = None;
+	WMReleasePixmap(bPtr->dimage);
+	bPtr->dimage = NULL;
+    }
+
+    if (image) {
+	bPtr->dimage = WMCreatePixmapFromXPixmaps(WMWidgetScreen(bPtr), 
+						  image->pixmap, None,
+						  image->width, image->height,
+						  image->depth);
+	updateDisabledMask(bPtr);
+    }
+
     if (bPtr->view->flags.realized) {
 	paintButton(bPtr);
     }
@@ -290,12 +338,8 @@ WMSetButtonText(WMButton *bPtr, char *text)
 
     if (text!=NULL) {
 	bPtr->caption = wstrdup(text);
-	bPtr->captionHeight =
-	    W_GetTextHeight(bPtr->font ? bPtr->font : NFONT(bPtr), text,
-			    bPtr->view->size.width, True);
     } else {
 	bPtr->caption = NULL;
-	bPtr->captionHeight = 0;
     }
     
     
@@ -313,12 +357,8 @@ WMSetButtonAltText(WMButton *bPtr, char *text)
     
     if (text!=NULL) {
 	bPtr->altCaption = wstrdup(text);
-	bPtr->altCaptionHeight =
-	    W_GetTextHeight(bPtr->font ? bPtr->font : NFONT(bPtr), text,
-			    bPtr->view->size.width, True);
     } else {
 	bPtr->altCaption = NULL;
-	bPtr->altCaptionHeight = 0;
     }
     
     if (bPtr->view->flags.realized) {
@@ -365,15 +405,6 @@ WMSetButtonFont(WMButton *bPtr, WMFont *font)
 	WMReleaseFont(bPtr->font);
 
     bPtr->font = WMRetainFont(font);
-    
-    if (bPtr->caption) {
-	bPtr->captionHeight = W_GetTextHeight(font, bPtr->caption, 
-					      bPtr->view->size.width, True);
-    }
-    if (bPtr->altCaption) {
-	bPtr->altCaptionHeight = W_GetTextHeight(font, bPtr->altCaption, 
-						 bPtr->view->size.width, True);
-    }
 }
 
 
@@ -401,6 +432,9 @@ WMPerformButtonClick(WMButton *bPtr)
 {
     CHECK_CLASS(bPtr, WC_Button);
   
+    if (!bPtr->flags.enabled)
+	return;
+
     bPtr->flags.pushed = 1;
     bPtr->flags.selected = 1;
     
@@ -409,7 +443,11 @@ WMPerformButtonClick(WMButton *bPtr)
 	XFlush(WMScreenDisplay(WMWidgetScreen(bPtr)));
 	wusleep(20000);
     }
-    
+  
+    if (bPtr->groupIndex>0) {
+	WMPostNotificationName(WMPushedRadioNotification, bPtr, NULL);
+    }
+
     if (bPtr->action)
 	(*bPtr->action)(bPtr, bPtr->clientData);
      
@@ -432,6 +470,25 @@ WMSetButtonAction(WMButton *bPtr, WMAction *action, void *clientData)
 }
 
 
+
+
+static void
+radioPushObserver(void *observerData, WMNotification *notification)
+{
+    WMButton *bPtr = (WMButton*)observerData;
+    WMButton *pushedButton = (WMButton*)WMGetNotificationObject(notification);
+
+    if (bPtr!=pushedButton && pushedButton->groupIndex == bPtr->groupIndex
+	&& bPtr->groupIndex!=0) {
+	if (bPtr->flags.selected) {
+	    bPtr->flags.selected = 0;
+	    paintButton(bPtr);
+	}
+    }
+}
+
+
+
 void
 WMGroupButtons(WMButton *bPtr, WMButton *newMember)
 {
@@ -439,6 +496,17 @@ WMGroupButtons(WMButton *bPtr, WMButton *newMember)
     
     CHECK_CLASS(bPtr, WC_Button);
     CHECK_CLASS(newMember, WC_Button);
+
+    if (!bPtr->flags.addedObserver) {
+	WMAddNotificationObserver(radioPushObserver, bPtr,
+				  WMPushedRadioNotification, NULL);
+	bPtr->flags.addedObserver = 1;
+    }
+    if (!newMember->flags.addedObserver) {
+	WMAddNotificationObserver(radioPushObserver, newMember,
+				  WMPushedRadioNotification, NULL);
+	newMember->flags.addedObserver = 1;
+    }
     
     if (bPtr->groupIndex==0) {
 	bPtr->groupIndex = ++tagIndex;
@@ -466,27 +534,6 @@ WMSetButtonPeriodicDelay(WMButton *bPtr, float delay, float interval)
 }
 
 
-static void
-resizeButton(WMButton *bPtr, unsigned int width, unsigned int height)
-{
-    W_ResizeView(bPtr->view, width, height);
-    
-    if (bPtr->caption) {
-	bPtr->captionHeight =
-	    W_GetTextHeight(bPtr->font ? bPtr->font : NFONT(bPtr),
-			    bPtr->caption, bPtr->view->size.width, True);
-    } else {
-	bPtr->captionHeight = 0;
-    }
-    if (bPtr->altCaption) {
-	bPtr->altCaptionHeight =
-	    W_GetTextHeight(bPtr->font ? bPtr->font : NFONT(bPtr), 
-			    bPtr->altCaption, bPtr->view->size.width, True);
-    } else {
-	bPtr->altCaptionHeight = 0;
-    }
-}
-
 
 static void
 paintButton(Button *bPtr)
@@ -496,14 +543,15 @@ paintButton(Button *bPtr)
     WMReliefType relief;
     int offset;
     char *caption;
-    int captionHeight;
     WMPixmap *image;
     GC textGC;
 
     gc = NULL;
     caption = bPtr->caption;
-    captionHeight = bPtr->captionHeight;
-    image = bPtr->image;
+    if (bPtr->flags.enabled || !bPtr->dimage)
+	image = bPtr->image;
+    else
+	image = bPtr->dimage;
     offset = 0;
     if (bPtr->flags.bordered)
 	relief = WRRaised;
@@ -512,35 +560,33 @@ paintButton(Button *bPtr)
 
     if (bPtr->flags.selected) {
 	if (bPtr->flags.stateLight)
-	    gc = W_GC(scrPtr->white);
+	    gc = WMColorGC(scrPtr->white);
 
 	if (bPtr->flags.stateChange) {
 	    if (bPtr->altCaption) {
 		caption = bPtr->altCaption;
-		captionHeight = bPtr->altCaptionHeight;
 	    }
 	    if (bPtr->altImage)
 		image = bPtr->altImage;
 	}
-	
+
 	if (bPtr->flags.statePush && bPtr->flags.bordered) {
 	    relief = WRSunken;
 	    offset = 1;
 	}
     }
-        
+
     if (bPtr->flags.pushed) {
 	if (bPtr->flags.pushIn) {
-	    relief = WRSunken;
+	    relief = WRPushed;
 	    offset = 1;
 	}
 	if (bPtr->flags.pushLight)
-	    gc = W_GC(scrPtr->white);
-	
+	    gc = WMColorGC(scrPtr->white);
+
 	if (bPtr->flags.pushChange) {
 	    if (bPtr->altCaption) {
 		caption = bPtr->altCaption;
-		captionHeight = bPtr->altCaptionHeight;
 	    }
 	    if (bPtr->altImage)
 		image = bPtr->altImage;
@@ -549,14 +595,13 @@ paintButton(Button *bPtr)
     
 
     if (bPtr->flags.enabled)
-	textGC = W_GC(scrPtr->black);
+	textGC = WMColorGC(scrPtr->black);
     else
-	textGC = W_GC(scrPtr->darkGray);
+	textGC = WMColorGC(scrPtr->darkGray);
 
-    W_PaintTextAndImage(bPtr->view, False, textGC,
+    W_PaintTextAndImage(bPtr->view, True, textGC,
 			(bPtr->font!=NULL ? bPtr->font : scrPtr->normalFont),
-			relief, caption, captionHeight,
-			bPtr->flags.alignment, image, 
+			relief, caption, bPtr->flags.alignment, image, 
 			bPtr->flags.imagePosition, gc, offset);
 }
 
@@ -580,19 +625,6 @@ handleEvents(XEvent *event, void *data)
      case DestroyNotify:
 	destroyButton(bPtr);
 	break;
-	
-     case ClientMessage:
-	if (W_CheckInternalMessage(bPtr->view->screen, &(event->xclient),
-				   WM_RADIO_PRESS)
-	    && event->xclient.data.l[2]==bPtr->groupIndex) {
-		/* some other button was pushed down */
-	    if (bPtr->flags.selected 
-		&& event->xclient.data.l[0]!=bPtr->view->window) {
-		bPtr->flags.selected = 0;
-		paintButton(bPtr);
-	    }
-	}
-	break;
     }
 }
 
@@ -689,19 +721,12 @@ handleActionEvents(XEvent *event, void *data)
 	paintButton(bPtr);
     
     if (doclick) {
-	XEvent ev;
-
+	if (bPtr->flags.selected && bPtr->groupIndex>0) {
+	    WMPostNotificationName(WMPushedRadioNotification, bPtr, NULL);
+	}
+	
 	if (bPtr->action)
 	    (*bPtr->action)(bPtr, bPtr->clientData);
-
-	if (bPtr->flags.selected && bPtr->groupIndex>0) {
-	    SETUP_INTERNAL_MESSAGE(ev, bPtr->view->screen);
-	    ev.xclient.data.l[0]=bPtr->view->window;
-	    ev.xclient.data.l[1]=WM_RADIO_PRESS;
-	    ev.xclient.data.l[2]=bPtr->groupIndex;
-
-	    W_BroadcastMessage(bPtr->view->parent, &ev);
-	}
     }
 }
 
@@ -710,6 +735,10 @@ handleActionEvents(XEvent *event, void *data)
 static void
 destroyButton(Button *bPtr)
 {
+    if (bPtr->flags.addedObserver) {
+	WMRemoveNotificationObserver(bPtr);
+    }
+
     if (bPtr->timer)
 	WMDeleteTimerHandler(bPtr->timer);
     
@@ -724,7 +753,13 @@ destroyButton(Button *bPtr)
     
     if (bPtr->image)
 	WMReleasePixmap(bPtr->image);
-    
+
+    if (bPtr->dimage) {
+	/* yuck.. kluge */
+	bPtr->dimage->pixmap = None;
+
+	WMReleasePixmap(bPtr->dimage);
+    }
     if (bPtr->altImage)
 	WMReleasePixmap(bPtr->altImage);
 
