@@ -11,18 +11,23 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#ifdef HAVE_POLL_H
+#include <poll.h>
+#endif
+
+
+#include <X11/Xos.h>
+
 #ifdef HAVE_SYS_SELECT_H
 # include <sys/select.h>
 #endif
 
-#ifdef HAVE_GETTIMEOFDAY
-# include <sys/time.h>
-# ifdef TIME_WITH_SYS_TIME
-#  include <time.h>
-# endif
-#else /* ! HAVE_GETTIMEOFDAY */
-# include <time.h>
-#endif /* ! HAVE_GETTIMEOFDAY */
+#include <time.h>
+
+#ifndef X_GETTIMEOFDAY
+#define X_GETTIMEOFDAY(t) gettimeofday(t, (struct timezone*)0)
+#endif
+
 
 
 extern _WINGsConfiguration WINGsConfiguration;
@@ -31,7 +36,7 @@ extern _WINGsConfiguration WINGsConfiguration;
 
 typedef struct TimerHandler {
     WMCallback		*callback;	       /* procedure to call */
-    unsigned long 	msec;		       /* when to call the callback */
+    struct timeval	when;		       /* when to call the callback */
     void 		*clientData;
     struct TimerHandler *next;
 } TimerHandler;
@@ -44,6 +49,14 @@ typedef struct IdleHandler {
 } IdleHandler;
 
 
+typedef struct InputHandler {
+    WMInputProc		*callback;
+    void		*clientData;
+    int			fd;
+    int			mask;
+    struct InputHandler *next;
+} InputHandler;
+
 
 /* table to map event types to event masks */
 static unsigned long eventMasks[] = {
@@ -94,6 +107,7 @@ static TimerHandler *timerHandler=NULL;
 
 static IdleHandler *idleHandler=NULL;
 
+static InputHandler *inputHandler=NULL;
 
 /* hook for other toolkits or wmaker process their events */
 static WMEventHook *extraEventHandler=NULL;
@@ -105,20 +119,25 @@ static WMEventHook *extraEventHandler=NULL;
 #define idlePending()	(idleHandler)
 
 
+static void
+rightNow(struct timeval *tv) {
+    X_GETTIMEOFDAY(tv);
+}
 
-/* return current time in milliseconds */
-#ifdef HAVE_GETTIMEOFDAY
-static unsigned long 
-rightNow(void) {
-    struct timeval tv;
+/* is t1 after t2 ? */
+#define IS_AFTER(t1, t2)	(((t1).tv_sec > (t2).tv_sec) || \
+				 (((t1).tv_sec == (t2).tv_sec) \
+				  && ((t1).tv_usec > (t2).tv_usec)))
 
-    gettimeofday(&tv, NULL);
 
-    return 1000L*(unsigned long)tv.tv_sec + (unsigned long)tv.tv_usec/1000L;
+static void
+addmillisecs(struct timeval *tv, int milliseconds)
+{
+    tv->tv_usec += milliseconds*1000;
+
+    tv->tv_sec += tv->tv_usec/1000000;
+    tv->tv_usec = tv->tv_usec%1000000;
 }
-#else /* !HAVE_GETTIMEOFDAY */
-# define rightNow()	(1000*(unsigned long)time(NULL))
-#endif /* !HAVE_GETTIMEOFDAY */
 
 
 WMHandlerID
@@ -130,17 +149,18 @@ WMAddTimerHandler(int milliseconds, WMCallback *callback, void *cdata)
     if (!handler)
       return NULL;
     
-    handler->msec = rightNow()+milliseconds;
+    rightNow(&handler->when);
+    addmillisecs(&handler->when, milliseconds);
     handler->callback = callback;
     handler->clientData = cdata;
     /* insert callback in queue, sorted by time left */
-    if (!timerHandler || timerHandler->msec >= handler->msec) {
+    if (!timerHandler || !IS_AFTER(handler->when, timerHandler->when)) {
 	/* first in the queue */
 	handler->next = timerHandler;
 	timerHandler = handler;
     } else {
 	tmp = timerHandler;
-	while (tmp->next && tmp->next->msec < handler->msec) {
+	while (tmp->next && IS_AFTER(handler->when, tmp->next->when)) {
 	    tmp = tmp->next;
 	}
 	handler->next = tmp->next;
@@ -258,13 +278,60 @@ WMDeleteIdleHandler(WMHandlerID handlerID)
 
 
 
+WMHandlerID
+WMAddInputHandler(int fd, int condition, WMInputProc *proc, void *clientData)
+{
+    InputHandler *handler;
+    
+    handler = wmalloc(sizeof(InputHandler));
+    
+    handler->fd = fd;
+    handler->mask = condition;
+    handler->callback = proc;
+    handler->clientData = clientData;
+    
+    handler->next = inputHandler;
+    
+    inputHandler = handler;
+
+    return handler;
+}
+
+
+void
+WMDeleteInputHandler(WMHandlerID handlerID)
+{
+    InputHandler *tmp, *handler = (InputHandler*)handlerID;
+
+    if (!handler || !inputHandler)
+	return;
+
+    tmp = inputHandler;
+    if (tmp == handler) {
+	inputHandler = handler->next;
+	free(handler);
+    } else {
+	while (tmp->next) {
+	    if (tmp->next == handler) {
+		tmp->next = handler->next;
+		free(handler);
+		break;
+	    }
+	    tmp = tmp->next;
+	}
+    }    
+}
+
+
 static void
 checkIdleHandlers()
 {
     IdleHandler *handler, *tmp;
 
-    if (!idleHandler)
-      return;
+    if (!idleHandler) {
+	W_FlushIdleNotificationQueue();
+	return;
+    }
 
     handler = idleHandler;
     
@@ -279,6 +346,7 @@ checkIdleHandlers()
 	
 	handler = tmp;
     }
+    W_FlushIdleNotificationQueue();
 }
 
 
@@ -287,38 +355,47 @@ static void
 checkTimerHandlers()
 {
     TimerHandler *handler;
-    unsigned long now = rightNow();
+    struct timeval now;
 
-    if (!timerHandler || (timerHandler->msec > now))
-      return;
+    rightNow(&now);
 
-    while (timerHandler && timerHandler->msec <= now) {
+    while (timerHandler && IS_AFTER(now, timerHandler->when)) {
 	handler = timerHandler;
 	timerHandler = timerHandler->next;
 	handler->next = NULL;
 	(*handler->callback)(handler->clientData);
 	free(handler);
     }
+
+    W_FlushASAPNotificationQueue();
 }
 
 
 
-static unsigned long
-msToNextTimerEvent()
+static void
+delayUntilNextTimerEvent(struct timeval *delay)
 {
-    unsigned long now;
+    struct timeval now;
 
     if (!timerHandler) {
         /* The return value of this function is only valid if there _are_
          timers active. */
-        return 0;
+	delay->tv_sec = 0;
+	delay->tv_usec = 0;
+	return;
     }
 
-    now = rightNow();
-    if (timerHandler->msec < now) {
-        return 0;
+    rightNow(&now);
+    if (IS_AFTER(now, timerHandler->when)) {
+	delay->tv_sec = 0;
+	delay->tv_usec = 0;
     } else {
-        return timerHandler->msec - now;
+	delay->tv_sec = timerHandler->when.tv_sec - now.tv_sec;
+	delay->tv_usec = timerHandler->when.tv_usec - now.tv_usec;
+	if (delay->tv_usec < 0) {
+	    delay->tv_usec += 1000000;
+	    delay->tv_sec--;
+	}
     }
 }
 
@@ -412,6 +489,7 @@ WMDeleteEventHandler(WMView *view, unsigned long mask, WMEventProc *eventProc,
 }
 
 
+
 void
 W_CleanUpEvents(WMView *view)
 {
@@ -432,21 +510,28 @@ static Time
 getEventTime(WMScreen *screen, XEvent *event)
 {
     switch (event->type) {
-        case ButtonPress:
-        case ButtonRelease:
-            return event->xbutton.time;
-        case KeyPress:
-        case KeyRelease:
-            return event->xkey.time;
-        case MotionNotify:
-            return event->xmotion.time;
-        case EnterNotify:
-        case LeaveNotify:
-            return event->xcrossing.time;
-        case PropertyNotify:
-            return event->xproperty.time;
+     case ButtonPress:
+     case ButtonRelease:
+	return event->xbutton.time;
+     case KeyPress:
+     case KeyRelease:
+	return event->xkey.time;
+     case MotionNotify:
+	return event->xmotion.time;
+     case EnterNotify:
+     case LeaveNotify:
+	return event->xcrossing.time;
+     case PropertyNotify:
+	return event->xproperty.time;
+     case SelectionClear:
+	return event->xselectionclear.time;
+     case SelectionRequest:
+	return event->xselectionrequest.time;
+     case SelectionNotify:
+	return event->xselection.time;
+     default:
+	return screen->lastEventTime;	
     }
-    return screen->lastEventTime;
 }
 
 
@@ -501,11 +586,18 @@ WMHandleEvent(XEvent *event)
 
 	return False;
     }
-        
+
     view->screen->lastEventTime = getEventTime(view->screen, event);
 
     toplevel = W_TopLevelOfView(view);
 
+    if (event->type == SelectionNotify || event->type == SelectionClear
+	|| event->type == SelectionRequest) {
+	puts("REQREWQ");
+	/* handle selection related events */
+	W_HandleSelectionEvent(event);
+    }
+
     /* if it's a key event, redispatch it to the focused control */
     if (mask & (KeyPressMask|KeyReleaseMask)) {
 	W_View *focused = W_FocusedViewOfToplevel(toplevel);
@@ -536,6 +628,22 @@ WMHandleEvent(XEvent *event)
     }
 
 
+    if (view->screen->modal && toplevel!=view->screen->modalView
+	&& !toplevel->flags.worksWhenModal) {
+	if (event->type == KeyPress || event->type == KeyRelease
+	    || event->type == MotionNotify || event->type == ButtonPress
+	    || event->type == ButtonRelease
+	    || event->type == FocusIn || event->type == FocusOut) {
+	    return True;
+	}
+    }
+
+    /* do balloon stuffs */
+    if (event->type == EnterNotify)
+	W_BalloonHandleEnterView(view);
+    else if (event->type == LeaveNotify)
+	W_BalloonHandleLeaveView(view);
+
     /* This is a hack. It will make the panel be secure while
      * the event handlers are handled, as some event handler
      * might destroy the widget. */
@@ -573,9 +681,13 @@ WMHandleEvent(XEvent *event)
     }
 
     /* save button click info to track double-clicks */
-    if (event->type == ButtonPress) {
-	view->screen->lastClickWindow = event->xbutton.window;
-	view->screen->lastClickTime = event->xbutton.time;
+    if (view->screen->ignoreNextDoubleClick) {
+	view->screen->ignoreNextDoubleClick = 0;
+    } else {
+	if (event->type == ButtonPress) {
+	    view->screen->lastClickWindow = event->xbutton.window;
+	    view->screen->lastClickTime = event->xbutton.time;
+	}
     }
     
     W_ReleaseView(toplevel);
@@ -601,22 +713,203 @@ WMIsDoubleClick(XEvent *event)
 	return False;
     
     if (event->xbutton.time - view->screen->lastClickTime 
-	< WINGsConfiguration.doubleClickDelay)
+	< WINGsConfiguration.doubleClickDelay) {
+	view->screen->lastClickTime = 0;
+	view->screen->lastClickWindow = None;
+	view->screen->ignoreNextDoubleClick = 1;
 	return True;
-    else
+    } else
 	return False;
 }
 
 
+Bool
+W_WaitForEvent(Display *dpy, unsigned long xeventmask)
+{
+#if defined(HAVE_POLL) && defined(HAVE_POLL_H) && !defined(HAVE_SELECT)
+    struct pollfd *fds;
+    InputHandler *handler;
+    int count, timeout, nfds, k, retval;
+
+    for (nfds = 1, handler = inputHandler; 
+         handler != 0; handler = handler->next) nfds++;
+    
+    fds = wmalloc(nfds * sizeof(struct pollfd));
+    fds[0].fd = ConnectionNumber(dpy);
+    fds[0].events = POLLIN;
+
+    for (k = 1, handler = inputHandler; 
+         handler; 
+         handler = handler->next, k++) {
+        fds[k].fd = handler->fd;
+        fds[k].events = 0;
+	if (handler->mask & WIReadMask)
+            fds[k].events |= POLLIN;
+
+	if (handler->mask & WIWriteMask)
+            fds[k].events |= POLLOUT;
+
+#if 0 /* FIXME */
+	if (handler->mask & WIExceptMask)
+	    FD_SET(handler->fd, &eset);
+#endif
+    }
+
+    /*
+     * Setup the select() timeout to the estimated time until the
+     * next timer expires.
+     */
+    if (timerPending()) {
+        struct timeval tv;
+	delayUntilNextTimerEvent(&tv);
+        timeout = tv.tv_sec * 1000 + tv.tv_usec / 1000;
+    } else {
+        timeout = -1;
+    }
+
+    if (xeventmask==0) {
+	if (XPending(dpy))      
+	    return True;
+    } else {
+	XEvent ev;
+	if (XCheckMaskEvent(dpy, xeventmask, &ev)) {
+	    XPutBackEvent(dpy, &ev);
+	    return True;
+	}
+    }
+    
+    count = poll(fds, nfds, timeout);
+
+    if (count > 0) {
+	handler = inputHandler;
+        k = 1;
+	while (handler) {
+	    int mask;
+
+	    mask = 0;
+
+	    if (fds[k].revents & (POLLIN|POLLRDNORM|POLLRDBAND|POLLPRI))
+		mask |= WIReadMask;
+	    
+	    if (fds[k].revents & (POLLOUT | POLLWRBAND))
+		mask |= WIWriteMask;
+	    
+	    if (fds[k].revents & (POLLHUP | POLLNVAL | POLLERR))
+		mask |= WIExceptMask;
+	    
+	    if (mask!=0 && handler->callback) {
+		(*handler->callback)(handler->fd, mask, 
+				     handler->clientData);
+	    }
+
+	    handler = handler->next;
+            k++;
+	}
+    }
+
+    retval = fds[0].revents & (POLLIN|POLLRDNORM|POLLRDBAND|POLLPRI);
+    free(fds);
+
+    W_FlushASAPNotificationQueue();
+
+    return retval;
+#else /* not HAVE_POLL */
+#ifdef HAVE_SELECT
+    struct timeval timeout;
+    struct timeval *timeoutPtr;
+    fd_set rset, wset, eset;
+    int maxfd;
+    int count;
+    InputHandler *handler = inputHandler;
+
+    FD_ZERO(&rset);
+    FD_ZERO(&wset);
+    FD_ZERO(&eset);
+
+    FD_SET(ConnectionNumber(dpy), &rset);
+    maxfd = ConnectionNumber(dpy);
+
+    while (handler) {
+	if (handler->mask & WIReadMask)
+	    FD_SET(handler->fd, &rset);
+
+	if (handler->mask & WIWriteMask)
+	    FD_SET(handler->fd, &wset);
+
+	if (handler->mask & WIExceptMask)
+	    FD_SET(handler->fd, &eset);
+
+	if (maxfd < handler->fd)
+	    maxfd = handler->fd;
+
+	handler = handler->next;
+    }
+
+
+    /*
+     * Setup the select() timeout to the estimated time until the
+     * next timer expires.
+     */
+    if (timerPending()) {
+	delayUntilNextTimerEvent(&timeout);
+	timeoutPtr = &timeout;
+    } else {
+	timeoutPtr = (struct timeval*)0;
+    }
+
+    XSync(dpy, False);
+    if (xeventmask==0) {
+	if (XPending(dpy))
+	    return True;
+    } else {
+	XEvent ev;
+	if (XCheckMaskEvent(dpy, xeventmask, &ev)) {
+	    XPutBackEvent(dpy, &ev);
+	    return True;
+	}
+    }
+    
+    count = select(1 + maxfd, &rset, &wset, &eset, timeoutPtr);
+
+    if (count > 0) {
+	handler = inputHandler;
+
+	while (handler) {
+	    int mask;
+
+	    mask = 0;
+
+	    if (FD_ISSET(handler->fd, &rset))
+		mask |= WIReadMask;
+	    
+	    if (FD_ISSET(handler->fd, &wset))
+		mask |= WIWriteMask;
+	    
+	    if (FD_ISSET(handler->fd, &eset))
+		mask |= WIExceptMask;
+	    
+	    if (mask!=0 && handler->callback) {
+		(*handler->callback)(handler->fd, mask, 
+				     handler->clientData);
+	    }
+
+	    handler = handler->next;
+	}
+    }
+
+    W_FlushASAPNotificationQueue();
+
+    return FD_ISSET(ConnectionNumber(dpy), &rset);
+#else /* not HAVE_SELECT, not HAVE_POLL */
+Neither select nor poll. You lose.
+#endif /* HAVE_SELECT */
+#endif /* HAVE_POLL */
+}
+
+
 void
 WMNextEvent(Display *dpy, XEvent *event)
 { 
-#if defined(HAVE_SELECT)
-    unsigned long milliseconds;
-    struct timeval timeout;
-    struct timeval *timeoutOrInfty;
-    fd_set readset;
-
     /* Check any expired timers */
     if (timerPending()) {
 	checkTimerHandlers();
@@ -633,31 +926,13 @@ WMNextEvent(Display *dpy, XEvent *event)
 		checkTimerHandlers();
 	}
 
-        /* 
-         * Setup the select() timeout to the estimated time until the
-         * next timer expires.
-         */
-        if (timerPending()) {
-            milliseconds = msToNextTimerEvent();
-            timeout.tv_sec = milliseconds / 1000;
-            timeout.tv_usec = (milliseconds % 1000) * 1000;
-            timeoutOrInfty = &timeout;
-        } else {
-            timeoutOrInfty = (struct timeval*)0;
-        }
-
 	/*
 	 * Make sure that new events did not arrive while we were doing
 	 * timer/idle stuff. Or we might block forever waiting for
 	 * an event that already arrived. 
 	 */
-	if (XPending(dpy)==0) {
-	    /* Wait for input on the X connection socket */
-	    FD_ZERO(&readset);
-	    FD_SET(ConnectionNumber(dpy), &readset);
-	    select(1 + ConnectionNumber(dpy), &readset, (fd_set*)0, (fd_set*)0,
-		   timeoutOrInfty);
-	}
+	/* wait to something happen */
+	W_WaitForEvent(dpy, 0);
 	
         /* Check any expired timers */
         if (timerPending()) {
@@ -666,34 +941,12 @@ WMNextEvent(Display *dpy, XEvent *event)
     }
 
     XNextEvent(dpy, event);
-#else /* not HAVE_SELECT */
-    while (1) {
-	/* Do idle and timer stuff while there are no timer or X events */
-	while (!XPending(dpy) && idlePending()) {
-	    if (idlePending())
-		checkIdleHandlers();
-	    /* dispatch timer events */
-	    if (timerPending())
-		checkTimerHandlers();
-	}
-	
-	/* check if there are events waiting */
-	if (XPending(dpy) > 0) {
-	    break;
-	}
-
-	wmsleep(5000);
-    }
-    XNextEvent(dpy, event);
-#endif /* HAVE_SELECT */
 }
 
-
-
+#if 0
 void
 WMMaskEvent(Display *dpy, long mask, XEvent *event)
 { 
-#if defined(HAVE_SELECT)
     unsigned long milliseconds;
     struct timeval timeout;
     struct timeval *timeoutOrInfty;
@@ -712,47 +965,57 @@ WMMaskEvent(Display *dpy, long mask, XEvent *event)
          * next timer expires.
          */
         if (timerPending()) {
-            milliseconds = msToNextTimerEvent();
-            timeout.tv_sec = milliseconds / 1000;
-            timeout.tv_usec = (milliseconds % 1000) * 1000;
+	    delayUntilNextTimerEvent(&timeout);
             timeoutOrInfty = &timeout;
         } else {
             timeoutOrInfty = (struct timeval*)0;
         }
 
+	if (XCheckMaskEvent(dpy, mask, event))
+	    return;
+
         /* Wait for input on the X connection socket */
-/*	if (XPending(dpy)==0) {*/
-	    FD_ZERO(&readset);
-	    FD_SET(ConnectionNumber(dpy), &readset);
-	    select(1 + ConnectionNumber(dpy), &readset, (fd_set*)0, (fd_set*)0,
-		   timeoutOrInfty);
-/*	}*/
+	FD_ZERO(&readset);
+	FD_SET(ConnectionNumber(dpy), &readset);
+	select(1 + ConnectionNumber(dpy), &readset, (fd_set*)0, (fd_set*)0,
+	       timeoutOrInfty);
 
         /* Check any expired timers */
         if (timerPending()) {
             checkTimerHandlers();
         }
     }
-#else /* not HAVE_SELECT */
+}
+#endif
+#if 1
+/*
+ * Cant use this because XPending() will make W_WaitForEvent
+ * return even if the event in the queue is not what we want,
+ * and if we block until some new event arrives from the
+ * server, other events already in the queue (like Expose)
+ * will be deferred.
+ */
+void
+WMMaskEvent(Display *dpy, long mask, XEvent *event)
+{ 
     while (!XCheckMaskEvent(dpy, mask, event)) {
 	/* Do idle stuff while there are no timer or X events */
 	while (idlePending()) {
 	    checkIdleHandlers();
-
 	    if (XCheckMaskEvent(dpy, mask, event))
 		return;
 	}
 
-	/* dispatch timer events */
-	if (timerPending()) {
-	    checkTimerHandlers();
-	}
+        /* Wait for input on the X connection socket */
+	W_WaitForEvent(dpy, mask);
 
-	wmsleep(5000);
+        /* Check any expired timers */
+        if (timerPending()) {
+            checkTimerHandlers();
+        }
     }
-#endif /* HAVE_SELECT */
 }
-
+#endif
 
 Bool 
 WMScreenPending(WMScreen *scr)
