@@ -1,11 +1,15 @@
 
 #include "WINGsP.h"
 
+
 #include <wraster.h>
+#include <assert.h>
+
+static char *makeFontSetOfSize(char *fontset, int size);
 
 
 WMFont*
-WMCreateFont(WMScreen *scrPtr, char *fontName)
+WMCreateFontSet(WMScreen *scrPtr, char *fontName)
 {
     WMFont *font;
     Display *display = scrPtr->display;
@@ -13,13 +17,20 @@ WMCreateFont(WMScreen *scrPtr, char *fontName)
     int nmissing = 0;
     char *defaultString;
     XFontSetExtents *extents;
-    
+
+    font = WMHashGet(scrPtr->fontCache, fontName);
+    if (font) {
+	WMRetainFont(font);
+	return font;
+    }
+
     font = malloc(sizeof(WMFont));
     if (!font)
       return NULL;
-    
-    font->nonLocalized = 0;
-    
+    memset(font, 0, sizeof(WMFont));
+
+    font->notFontSet = 0;
+
     font->screen = scrPtr;
 
     font->font.set = XCreateFontSet(display, fontName, &missing, &nmissing,
@@ -46,28 +57,52 @@ WMCreateFont(WMScreen *scrPtr, char *fontName)
     
     font->height = extents->max_logical_extent.height;
     font->y = font->height - (font->height + extents->max_logical_extent.y);
-    
+
     font->refCount = 1;
-    
+
+    font->name = wstrdup(fontName);
+
+    assert(WMHashInsert(scrPtr->fontCache, font->name, font)==NULL);
+
     return font;
 }
 
 
+
 WMFont*
-WMCreateFontInDefaultEncoding(WMScreen *scrPtr, char *fontName)
+WMCreateNormalFont(WMScreen *scrPtr, char *fontName)
 {
     WMFont *font;
     Display *display = scrPtr->display;
-    
+    char *fname, *ptr;
+
+    if ((ptr = strchr(fontName, ','))) {
+	fname = wmalloc(ptr - fontName + 1);
+	strncpy(fname, fontName, ptr - fontName);
+	fname[ptr - fontName] = 0;
+    } else {
+	fname = wstrdup(fontName);
+    }
+
+    font = WMHashGet(scrPtr->fontCache, fname);
+    if (font) {
+	WMRetainFont(font);
+	free(fname);
+	return font;
+    }
+
     font = malloc(sizeof(WMFont));
-    if (!font)
-      return NULL;
+    if (!font) {
+	free(fname);
+	return NULL;
+    }
+    memset(font, 0, sizeof(WMFont));
     
-    font->nonLocalized = 1;
+    font->notFontSet = 1;
     
     font->screen = scrPtr;
-
-    font->font.normal = XLoadQueryFont(display, fontName);
+    
+    font->font.normal = XLoadQueryFont(display, fname);
     if (!font->font.normal) {
 	free(font);
 	return NULL;
@@ -75,10 +110,25 @@ WMCreateFontInDefaultEncoding(WMScreen *scrPtr, char *fontName)
 
     font->height = font->font.normal->ascent+font->font.normal->descent;
     font->y = font->font.normal->ascent;
-    
+
     font->refCount = 1;
 
-    return font;    
+    font->name = fname;
+
+    assert(WMHashInsert(scrPtr->fontCache, font->name, font)==NULL);
+
+    return font;
+}
+
+
+
+WMFont*
+WMCreateFont(WMScreen *scrPtr, char *fontName)
+{
+    if (scrPtr->useMultiByte)
+	return WMCreateFontSet(scrPtr, fontName);
+    else
+	return WMCreateNormalFont(scrPtr, fontName);
 }
 
 
@@ -86,7 +136,7 @@ WMCreateFontInDefaultEncoding(WMScreen *scrPtr, char *fontName)
 WMFont*
 WMRetainFont(WMFont *font)
 {
-    assert(font!=NULL);
+    wassertrv(font!=NULL, NULL);
 
     font->refCount++;
 
@@ -97,13 +147,19 @@ WMRetainFont(WMFont *font)
 void 
 WMReleaseFont(WMFont *font)
 {
-    assert(font!=NULL);
+    wassertr(font!=NULL);
+
     font->refCount--;
     if (font->refCount < 1) {
-	if (font->nonLocalized)
+	if (font->notFontSet)
 	    XFreeFont(font->screen->display, font->font.normal);
-	else
+	else {
 	    XFreeFontSet(font->screen->display, font->font.set);
+	}
+	if (font->name) {
+	    WMHashRemove(font->screen->fontCache, font->name);
+	    free(font->name);
+	}
 	free(font);
     }
 }
@@ -113,35 +169,44 @@ WMReleaseFont(WMFont *font)
 unsigned int
 WMFontHeight(WMFont *font)
 {
-    assert(font!=NULL);
-    
+    wassertrv(font!=NULL, 0);
+
     return font->height;
 }
 
 
 
-
 WMFont*
 WMSystemFontOfSize(WMScreen *scrPtr, int size)
 {
     WMFont *font;
     char *fontSpec;
-    
-    fontSpec = wmalloc(strlen(WINGsConfiguration.systemFont)+16);
-	
-    sprintf(fontSpec, WINGsConfiguration.systemFont, size);
 
-    font = WMCreateFont(scrPtr, fontSpec);
-    free(fontSpec);
-    
+    fontSpec = makeFontSetOfSize(WINGsConfiguration.systemFont, size);
+
+    if (scrPtr->useMultiByte)
+	font = WMCreateFontSet(scrPtr, fontSpec);
+    else
+	font = WMCreateNormalFont(scrPtr, fontSpec);
+
     if (!font) {
-	wwarning("could not load font set %s. Trying fixed.", fontSpec);
-	font = WMCreateFont(scrPtr, "fixed");
+	if (scrPtr->useMultiByte) {
+	    wwarning("could not load font set %s. Trying fixed.", fontSpec);
+	    font = WMCreateFontSet(scrPtr, "fixed");
+	    if (!font) {
+		font = WMCreateFontSet(scrPtr, "-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*");
+	    }
+	} else {
+	    wwarning("could not load font %s. Trying fixed.", fontSpec);
+	    font = WMCreateNormalFont(scrPtr, "fixed");
+	}
 	if (!font) {
 	    wwarning("could not load fixed font!");
+	    free(fontSpec);
 	    return NULL;
 	}
     }
+    free(fontSpec);
 
     return font;
 }
@@ -152,22 +217,32 @@ WMBoldSystemFontOfSize(WMScreen *scrPtr, int size)
 {
     WMFont *font;
     char *fontSpec;
-    
-    fontSpec = wmalloc(strlen(WINGsConfiguration.boldSystemFont)+16);
-    
-    sprintf(fontSpec, WINGsConfiguration.boldSystemFont, size);
-    
-    font = WMCreateFont(scrPtr, fontSpec);
-    free(fontSpec);
-    
+
+    fontSpec = makeFontSetOfSize(WINGsConfiguration.boldSystemFont, size);
+
+    if (scrPtr->useMultiByte)
+	font = WMCreateFontSet(scrPtr, fontSpec);
+    else
+	font = WMCreateNormalFont(scrPtr, fontSpec);
+
     if (!font) {
-	wwarning("could not load font set %s. Trying fixed.", fontSpec);
-	font = WMCreateFont(scrPtr, "fixed");
+	if (scrPtr->useMultiByte) {
+	    wwarning("could not load font set %s. Trying fixed.", fontSpec);
+	    font = WMCreateFontSet(scrPtr, "fixed");
+	    if (!font) {
+		font = WMCreateFontSet(scrPtr, "-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*");
+	    }
+	} else {
+	    wwarning("could not load font %s. Trying fixed.", fontSpec);
+	    font = WMCreateNormalFont(scrPtr, "fixed");
+	}
 	if (!font) {
 	    wwarning("could not load fixed font!");
+	    free(fontSpec);
 	    return NULL;
 	}
     }
+    free(fontSpec);
     
     return font;
 }
@@ -176,7 +251,9 @@ WMBoldSystemFontOfSize(WMScreen *scrPtr, int size)
 XFontSet
 WMGetFontFontSet(WMFont *font)
 {
-    if (font->nonLocalized)
+    wassertrv(font!=NULL, NULL);
+
+    if (font->notFontSet)
 	return NULL;
     else
 	return font->font.set;
@@ -186,13 +263,19 @@ WMGetFontFontSet(WMFont *font)
 int
 WMWidthOfString(WMFont *font, char *text, int length)
 {
-    assert(font!=NULL);
-    assert(text!=NULL);
+    wassertrv(font!=NULL, 0);
+    wassertrv(text!=NULL, 0);
     
-    if (font->nonLocalized)
+    if (font->notFontSet)
 	return XTextWidth(font->font.normal, text, length);
-    else
-	return XmbTextEscapement(font->font.set, text, length); 
+    else {
+	XRectangle rect;
+	XRectangle AIXsucks;
+	
+	XmbTextExtents(font->font.set, text, length, &AIXsucks, &rect);
+	
+	return rect.width;
+    }
 }
 
 
@@ -201,7 +284,9 @@ void
 WMDrawString(WMScreen *scr, Drawable d, GC gc, WMFont *font, int x, int y,
 	     char *text, int length)
 {
-    if (font->nonLocalized) {
+    wassertr(font!=NULL);
+
+    if (font->notFontSet) {
 	XSetFont(scr->display, gc, font->font.normal->fid);
 	XDrawString(scr->display, d, gc, x, y + font->y, text, length);
     } else {
@@ -215,7 +300,9 @@ void
 WMDrawImageString(WMScreen *scr, Drawable d, GC gc, WMFont *font, int x, int y,
 		  char *text, int length)
 {
-    if (font->nonLocalized) {
+    wassertr(font != NULL);
+
+    if (font->notFontSet) {
 	XSetFont(scr->display, gc, font->font.normal->fid);
 	XDrawImageString(scr->display, d, gc, x, y + font->y, text, length);
     } else {
@@ -224,3 +311,40 @@ WMDrawImageString(WMScreen *scr, Drawable d, GC gc, WMFont *font, int x, int y,
     }
 }
 
+
+
+
+static char*
+makeFontSetOfSize(char *fontset, int size)
+{
+    char font[300];
+    char *newfs = NULL;
+    char *ptr;
+    char *tmp;
+
+    do {
+	int hold = ' ';
+	
+	ptr = strchr(fontset, ',');
+	if (ptr) {
+	    hold = *(ptr+1);
+	    *(ptr+1) = 0;
+	}
+	if (strlen(fontset)>255) {
+	    wwarning("font description %s is too large.", fontset);
+	} else {
+	    sprintf(font, fontset, size);
+	    tmp = wstrappend(newfs, font);
+	    if (newfs)
+		free(newfs);
+	    newfs = tmp;
+	}
+	if (ptr) {
+	    *(ptr+1) = hold;
+	}
+	fontset = ptr+1;
+    } while (ptr!=NULL);
+
+    return newfs;
+}
+
