@@ -4,6 +4,7 @@
 
 #include <X11/Xutil.h>
 
+#include <X11/cursorfont.h>
 
 /********** data ************/
 
@@ -92,36 +93,36 @@ static char *RADIO_BUTTON_OFF[] = {
 
 
 static char *BUTTON_ARROW[] = {
-"...............",
-"....##....#### ",
-"...#.%....#... ",
-"..#..%#####... ",
-".#............ ",
-"#............. ",
-".#............ ",
-"..#..          ",
-"...#. .........",
-"....# ........."
+"..................",
+"....##....#### ...",
+"...#.%....#... ...",
+"..#..%#####... ...",
+".#............ ...",
+"#............. ...",
+".#............ ...",
+"..#..          ...",
+"...#. ............",
+"....# ............"
 };
 
-#define BUTTON_ARROW_WIDTH	15
+#define BUTTON_ARROW_WIDTH	18
 #define BUTTON_ARROW_HEIGHT	10
 
 
 static char *BUTTON_ARROW2[] = {
-"               ",
-"    ##    ####.",
-"   # %    #   .",
-"  #  %#####   .",
-" #            .",
-"#             .",
-" #            .",
-"  #  ..........",
-"   # .         ",
-"    #.         "
+"                  ",
+"    ##    ####.   ",
+"   # %    #   .   ",
+"  #  %#####   .   ",
+" #            .   ",
+"#             .   ",
+" #            .   ",
+"  #  ..........   ",
+"   # .            ",
+"    #.            "
 };
 
-#define BUTTON_ARROW2_WIDTH	15
+#define BUTTON_ARROW2_WIDTH	18
 #define BUTTON_ARROW2_HEIGHT	10
 
 
@@ -280,74 +281,53 @@ static char *PULLDOWN_INDICATOR[] = {
 #define PULLDOWN_INDICATOR_HEIGHT 	7
 
 
+#define CHECK_MARK_WIDTH 	8
+#define CHECK_MARK_HEIGHT 	10
+
+static char *CHECK_MARK[] = {
+"======== ",
+"======= #",
+"====== #%",
+"===== #%=",
+" #== #%==",
+" #% #%===",
+" % #%====",
+"  #%=====",
+" #%======",
+"#%======="};
+
+
 #define STIPPLE_WIDTH 8
 #define STIPPLE_HEIGHT 8
 static unsigned char STIPPLE_BITS[] = {
-   0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa};
+    0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa
+};
 
 
 
-extern W_ViewProcedureTable _WindowViewProcedures;
-extern W_ViewProcedureTable _FrameViewProcedures;
-extern W_ViewProcedureTable _LabelViewProcedures;
-extern W_ViewProcedureTable _ButtonViewProcedures;
-extern W_ViewProcedureTable _TextFieldViewProcedures;
-extern W_ViewProcedureTable _ScrollerViewProcedures;
-extern W_ViewProcedureTable _ScrollViewProcedures;
-extern W_ViewProcedureTable _ListViewProcedures;
-extern W_ViewProcedureTable _BrowserViewProcedures;
-extern W_ViewProcedureTable _PopUpButtonViewProcedures;
-extern W_ViewProcedureTable _ColorWellViewProcedures;
-extern W_ViewProcedureTable _ScrollViewViewProcedures;
-extern W_ViewProcedureTable _SliderViewProcedures;
-extern W_ViewProcedureTable _SplitViewViewProcedures;
 
-/*
- * All widget classes defined must have an entry here.
- */
-static W_ViewProcedureTable *procedureTables[16];
+extern void W_ReadConfigurations(void);
+
 
-static W_ViewProcedureTable **userProcedureTable = NULL;
 static int userWidgetCount=0;
 
 
 /*****  end data  ******/
 
 
-
-static void
-initProcedureTable()
-{
-    procedureTables[WC_Window] = &_WindowViewProcedures;
-    procedureTables[WC_Frame] = &_FrameViewProcedures;
-    procedureTables[WC_Label] = &_LabelViewProcedures;
-    procedureTables[WC_Button] = &_ButtonViewProcedures;
-    procedureTables[WC_TextField] = &_TextFieldViewProcedures;
-    procedureTables[WC_Scroller] = &_ScrollerViewProcedures;
-    procedureTables[WC_List] = &_ListViewProcedures;
-    procedureTables[WC_Browser] = &_BrowserViewProcedures;
-    procedureTables[WC_PopUpButton] = &_PopUpButtonViewProcedures;
-    procedureTables[WC_ColorWell] = &_ColorWellViewProcedures;
-    procedureTables[WC_ScrollView] = &_ScrollViewViewProcedures;
-    procedureTables[WC_Slider] = &_SliderViewProcedures;
-    procedureTables[WC_SplitView] = &_SplitViewViewProcedures;
-}
-
-
 static void
 renderPixmap(W_Screen *screen, Pixmap d, Pixmap mask, char **data, 
 	     int width, int height)
 {
     int x, y;
-    GC whiteGC = W_GC(screen->white);
-    GC blackGC = W_GC(screen->black);
-    GC lightGC = W_GC(screen->gray);
-    GC darkGC = W_GC(screen->darkGray);
+    GC whiteGC = WMColorGC(screen->white);
+    GC blackGC = WMColorGC(screen->black);
+    GC lightGC = WMColorGC(screen->gray);
+    GC darkGC = WMColorGC(screen->darkGray);
 
 
     if (mask)
-	XSetForeground(screen->display, screen->monoGC,
-		       W_PIXEL(screen->black));
+	XSetForeground(screen->display, screen->monoGC, 0);
 	
     for (y = 0; y < height; y++) {
 	for (x = 0; x < width; x++) {
@@ -388,12 +368,12 @@ makePixmap(W_Screen *sPtr, char **data, int width, int height, int masked)
 {
     Pixmap pixmap, mask = None;
     
-    pixmap = XCreatePixmap(sPtr->display, sPtr->rootWin, width, height, 
+    pixmap = XCreatePixmap(sPtr->display, W_DRAWABLE(sPtr), width, height, 
 			   sPtr->depth);
     
     if (masked) {
-	mask = XCreatePixmap(sPtr->display, sPtr->rootWin, width, height, 1);
-	XSetForeground(sPtr->display, sPtr->monoGC, W_PIXEL(sPtr->white));
+	mask = XCreatePixmap(sPtr->display, W_DRAWABLE(sPtr), width, height, 1);
+	XSetForeground(sPtr->display, sPtr->monoGC, 1);
 	XFillRectangle(sPtr->display, mask, sPtr->monoGC, 0, 0, width, height);
     }
 
@@ -403,39 +383,128 @@ makePixmap(W_Screen *sPtr, char **data, int width, int height, int masked)
 				      sPtr->depth);
 }
 
-#ifdef USE_TIFF
-#define WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.tiff"
-#else
-#define WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.xpm"
-#endif
+
+#define T_WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.tiff"
+#define T_DEFAULT_OBJECT_ICON_FILE RESOURCE_PATH"/defaultIcon.tiff"
+
+#define X_WINGS_IMAGES_FILE  RESOURCE_PATH"/Images.xpm"
+#define X_DEFAULT_OBJECT_ICON_FILE RESOURCE_PATH"/defaultIcon.xpm"
+
 
 static Bool
 loadPixmaps(WMScreen *scr)
 {
     RImage *image, *tmp;
-    Pixmap pixmap;
     RColor gray;
-    
-    image = RLoadImage(scr->rcontext, WINGS_IMAGES_FILE, 0);
-    if (!image) {
-	wwarning("WINGs: could not load widget images file");
-	return False;
-    }
-    /* make it have a gray background */
+    RColor white;
+
     gray.red = 0xae;
     gray.green = 0xaa;
     gray.blue = 0xae;
-    RCombineImageWithColor(image, &gray);
-    tmp = RGetSubImage(image, 0, 0, 24, 24);
-    if (!RConvertImage(scr->rcontext, tmp, &pixmap)) {
-	scr->homeIcon = NULL;
-    } else {
-	scr->homeIcon = WMCreatePixmapFromXPixmaps(scr, pixmap, None, 24, 24,
-						   scr->depth);
+
+    white.red = 0xff;
+    white.green = 0xff;
+    white.blue = 0xff;
+
+    image = RLoadImage(scr->rcontext, T_WINGS_IMAGES_FILE, 0);
+    if (!image)
+	image = RLoadImage(scr->rcontext, X_WINGS_IMAGES_FILE, 0);
+    if (!image) {
+	wwarning("WINGs: could not load widget images file: %s", 
+		 RMessageForError(RErrorCode));
+	return False;
     }
+    /* home icon */
+    /* make it have a gray background */
+    tmp = RGetSubImage(image, 0, 0, 24, 24);
+    RCombineImageWithColor(tmp, &gray);
+    scr->homeIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* make it have a white background */
+    tmp = RGetSubImage(image, 0, 0, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altHomeIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+
+    /* trash can */
+    tmp = RGetSubImage(image, 105, 0, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->trashcanIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    tmp = RGetSubImage(image, 105, 0, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altTrashcanIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* diskette */
+    tmp = RGetSubImage(image, 105, 25, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->disketteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    tmp = RGetSubImage(image, 105, 25, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altDisketteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+
+
+    tmp = RGetSubImage(image, 0, 0, 24, 24);
+    RCombineImageWithColor(tmp, &white);
+    scr->altHomeIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+
+    /* Magnifying Glass Icon for ColorPanel */
+    tmp = RGetSubImage(image, 24, 0, 40, 32);
+    RCombineImageWithColor(tmp, &gray);
+    scr->magnifyIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /*
+    tmp = RGetSubImage(image, 24, 0, 40, 32);
+    RCombineImageWithColor(tmp, &white);
+    scr->altMagnifyIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+     */
+    /* ColorWheel Icon for ColorPanel */
+    tmp = RGetSubImage(image, 0, 25, 24, 24);
+    scr->wheelIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* GrayScale Icon for ColorPanel */
+    tmp = RGetSubImage(image, 65, 0, 40, 24);
+    scr->grayIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* RGB Icon for ColorPanel */
+    tmp = RGetSubImage(image, 25, 33, 40, 24);
+    scr->rgbIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* CMYK Icon for ColorPanel */
+    tmp = RGetSubImage(image, 65, 25, 40, 24);
+    scr->cmykIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* HSB Icon for ColorPanel */
+    tmp = RGetSubImage(image, 0, 57, 40, 24);
+    scr->hsbIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* CustomColorPalette Icon for ColorPanel */
+    tmp = RGetSubImage(image, 81, 57, 40, 24);
+    scr->customPaletteIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
+    RDestroyImage(tmp);
+    /* ColorList Icon for ColorPanel */
+    tmp = RGetSubImage(image, 41, 57, 40, 24);
+    scr->colorListIcon = WMCreatePixmapFromRImage(scr, tmp, 128);
     RDestroyImage(tmp);
     
     RDestroyImage(image);
+
+#if 0
+    scr->defaultObjectIcon =
+	WMCreatePixmapFromFile(scr, T_DEFAULT_OBJECT_ICON_FILE);
+    if (!scr->defaultObjectIcon) { 
+	scr->defaultObjectIcon =
+	    WMCreatePixmapFromFile(scr, X_DEFAULT_OBJECT_ICON_FILE);
+    }
+    if (!scr->defaultObjectIcon) {
+	wwarning("WINGs: could not load default icon file");
+	return False;
+    }
+#endif
     return True;
 }
 
@@ -475,8 +544,8 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     if (!initialized) {
 
 	initialized = 1;
-	
-	initProcedureTable();
+
+	W_ReadConfigurations();
 
 	assert(W_ApplicationInitialized());
     }
@@ -492,15 +561,16 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     scrPtr->screen = screen;
     scrPtr->rcontext = context;
 
-    scrPtr->depth = DefaultDepth(display, screen);
+    scrPtr->depth = context->depth;
 
-    scrPtr->visual = DefaultVisual(display, screen);
+    scrPtr->visual = context->visual;
     scrPtr->lastEventTime = 0;
 
-    scrPtr->colormap = DefaultColormap(display, screen);
+    scrPtr->colormap = context->cmap;
 
     scrPtr->rootWin = RootWindow(display, screen);
 
+    scrPtr->fontCache = WMCreateHashTable(WMStringPointerHashCallbacks);
 
     /* initially allocate some colors */
     WMWhiteColor(scrPtr);
@@ -512,49 +582,61 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     
     gcv.function = GXxor;
     gcv.foreground = W_PIXEL(scrPtr->white);
-    scrPtr->xorGC = XCreateGC(display, scrPtr->rootWin, GCFunction
+    if (gcv.foreground == 0) gcv.foreground = 1;
+    scrPtr->xorGC = XCreateGC(display, W_DRAWABLE(scrPtr), GCFunction
 			      |GCGraphicsExposures|GCForeground, &gcv);
 
     gcv.function = GXxor;
     gcv.foreground = W_PIXEL(scrPtr->gray);
     gcv.subwindow_mode = IncludeInferiors;
-    scrPtr->ixorGC = XCreateGC(display, scrPtr->rootWin, GCFunction
+    scrPtr->ixorGC = XCreateGC(display, W_DRAWABLE(scrPtr), GCFunction
 			       |GCGraphicsExposures|GCForeground
 			       |GCSubwindowMode, &gcv);
 
     gcv.function = GXcopy;
-    scrPtr->copyGC = XCreateGC(display, scrPtr->rootWin, GCFunction
+    scrPtr->copyGC = XCreateGC(display, W_DRAWABLE(scrPtr), GCFunction
 			       |GCGraphicsExposures, &gcv);
 
-    scrPtr->clipGC = XCreateGC(display, scrPtr->rootWin, GCFunction
+    scrPtr->clipGC = XCreateGC(display, W_DRAWABLE(scrPtr), GCFunction
 			       |GCGraphicsExposures, &gcv);
 
     
-    stipple = XCreateBitmapFromData(display, scrPtr->rootWin, 
+    stipple = XCreateBitmapFromData(display, W_DRAWABLE(scrPtr), 
 				    STIPPLE_BITS, STIPPLE_WIDTH, STIPPLE_HEIGHT);
     gcv.foreground = W_PIXEL(scrPtr->darkGray);
     gcv.background = W_PIXEL(scrPtr->gray);
     gcv.fill_style = FillStippled;
     gcv.stipple = stipple;
-    scrPtr->stippleGC = XCreateGC(display, scrPtr->rootWin, 
+    scrPtr->stippleGC = XCreateGC(display, W_DRAWABLE(scrPtr),
 				  GCForeground|GCBackground|GCStipple
 				  |GCFillStyle|GCGraphicsExposures, &gcv);
 
     gcv.foreground = W_PIXEL(scrPtr->black);
     gcv.background = W_PIXEL(scrPtr->white);
-    scrPtr->textFieldGC = XCreateGC(display, scrPtr->rootWin,
+    scrPtr->textFieldGC = XCreateGC(display, W_DRAWABLE(scrPtr),
 				    GCForeground|GCBackground, &gcv);
 
     /* we need a 1bpp drawable for the monoGC, so borrow this one */
     scrPtr->monoGC = XCreateGC(display, stipple, 0, NULL);
 
-    XFreePixmap(display, stipple);
-    
-    
+    scrPtr->stipple = stipple;
+
+    scrPtr->useMultiByte = WINGsConfiguration.useMultiByte;
+
     scrPtr->normalFont = WMSystemFontOfSize(scrPtr, 12);
 
     scrPtr->boldFont = WMBoldSystemFontOfSize(scrPtr, 12);
 
+    if (!scrPtr->boldFont)
+	scrPtr->boldFont = scrPtr->normalFont;
+
+    if (!scrPtr->normalFont) {
+	wwarning("could not load any fonts. Make sure your font installation"
+		 "and locale settings are correct.");
+
+	return NULL;
+    }
+
     scrPtr->checkButtonImageOn = makePixmap(scrPtr, CHECK_BUTTON_ON,
 					    CHECK_BUTTON_ON_WIDTH,
 					    CHECK_BUTTON_ON_HEIGHT, False);
@@ -624,8 +706,30 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     scrPtr->pullDownIndicator = makePixmap(scrPtr, PULLDOWN_INDICATOR,
 					   PULLDOWN_INDICATOR_WIDTH,
 					   PULLDOWN_INDICATOR_HEIGHT, True);
+
+    scrPtr->checkMark = makePixmap(scrPtr, CHECK_MARK,
+				   CHECK_MARK_WIDTH,
+				   CHECK_MARK_HEIGHT, True);
+
     loadPixmaps(scrPtr);
-    
+
+    scrPtr->defaultCursor = XCreateFontCursor(display, XC_left_ptr);
+
+    scrPtr->textCursor = XCreateFontCursor(display, XC_xterm);
+
+    {
+	XColor bla;
+	Pixmap blank;
+	
+	blank = XCreatePixmap(display, scrPtr->stipple, 1, 1, 1);
+	XSetForeground(display, scrPtr->monoGC, 0);
+	XFillRectangle(display, blank, scrPtr->monoGC, 0, 0, 1, 1);
+
+	scrPtr->invisibleCursor = XCreatePixmapCursor(display, blank, blank,
+						      &bla, &bla, 0, 0);
+	XFreePixmap(display, blank);
+    }
+
     scrPtr->internalMessage = XInternAtom(display, "_WINGS_MESSAGE", False);
 
     scrPtr->attribsAtom = XInternAtom(display, "_GNUSTEP_WM_ATTR", False);
@@ -633,9 +737,17 @@ WMCreateScreenWithRContext(Display *display, int screen, RContext *context)
     scrPtr->deleteWindowAtom = XInternAtom(display, "WM_DELETE_WINDOW", False);
 
     scrPtr->protocolsAtom = XInternAtom(display, "WM_PROTOCOLS", False);
-
+    
+    scrPtr->clipboardAtom = XInternAtom(display, "CLIPBOARD", False);
+    
     scrPtr->rootView = W_CreateRootView(scrPtr);
 
+
+    scrPtr->balloon = W_CreateBalloon(scrPtr);
+
+
+    W_InitApplication(scrPtr);
+    
     return scrPtr;
 }
 
@@ -688,13 +800,13 @@ WMMapWidget(WMWidget *w)
 
 
 static void
-makeChildrenAutomap(W_View *view)
+makeChildrenAutomap(W_View *view, int flag)
 {
     view = view->childrenList;
     
     while (view) {
-	view->flags.mapWhenRealized = 1;
-	makeChildrenAutomap(view);
+	view->flags.mapWhenRealized = flag;
+	makeChildrenAutomap(view, flag);
 	
 	view = view->nextSister;
     }
@@ -707,12 +819,25 @@ WMMapSubwidgets(WMWidget *w)
     /* make sure that subwidgets created after the parent was realized
      * are mapped too */
     if (!W_VIEW(w)->flags.realized) {
-	makeChildrenAutomap(W_VIEW(w));
+	makeChildrenAutomap(W_VIEW(w), True);
     } else {
 	W_MapSubviews(W_VIEW(w));
     }
 }
 
+
+void
+WMUnmapSubwidgets(WMWidget *w)
+{   
+    if (!W_VIEW(w)->flags.realized) {
+	makeChildrenAutomap(W_VIEW(w), False);
+    } else {
+	W_UnmapSubviews(W_VIEW(w));
+    }
+}
+
+
+
 void
 WMUnmapWidget(WMWidget *w)
 {   
@@ -723,76 +848,29 @@ WMUnmapWidget(WMWidget *w)
 void
 WMSetWidgetBackgroundColor(WMWidget *w, WMColor *color)
 {    
-    if (W_CLASS(w) < WC_UserWidget
-	&& procedureTables[W_CLASS(w)]->setBackgroundColor) {
-	
-	(*procedureTables[W_CLASS(w)]->setBackgroundColor)(w, color);
-	
-    } else if (W_CLASS(w) >= WC_UserWidget
-	&& userProcedureTable[W_CLASS(w)-WC_UserWidget]->setBackgroundColor) {
-	
-	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->setBackgroundColor)(w, color);
-	
-    } else {
-	W_SetViewBackgroundColor(W_VIEW(w), color);
-    }
+    W_SetViewBackgroundColor(W_VIEW(w), color);
 }
 
 
 void
 WMMoveWidget(WMWidget *w, int x, int y)
 {
-    if (W_CLASS(w) < WC_UserWidget
-	&& procedureTables[W_CLASS(w)]->move) {
-	
-	(*procedureTables[W_CLASS(w)]->move)(w, x, y);
-	
-    } else if (W_CLASS(w) >= WC_UserWidget
-	&& userProcedureTable[W_CLASS(w)-WC_UserWidget]->move) {
-
-	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->move)(w, x, y);
-
-    } else {
-	W_MoveView(W_VIEW(w), x, y);
-    }
+    W_MoveView(W_VIEW(w), x, y);
 }
 
 
 void
 WMResizeWidget(WMWidget *w, unsigned int width, unsigned int height)
 {
-    if (W_CLASS(w) < WC_UserWidget
-	&& procedureTables[W_CLASS(w)]->resize) {
-	
-	(*procedureTables[W_CLASS(w)]->resize)(w, width, height);
-	
-    } else if (W_CLASS(w) >= WC_UserWidget
-	       && userProcedureTable[W_CLASS(w)-WC_UserWidget]->resize) {
-
-	(*userProcedureTable[W_CLASS(w)-WC_UserWidget]->resize)(w, width, height);
-
-    } else {
-	W_ResizeView(W_VIEW(w), width, height);
-    }
+    W_ResizeView(W_VIEW(w), width, height);
 }
 
 
 
 W_Class
-W_RegisterUserWidget(W_ViewProcedureTable *procTable)
+W_RegisterUserWidget(void)
 {
-    W_ViewProcedureTable **newTable;
-    
     userWidgetCount++;
-    newTable = wmalloc(sizeof(W_ViewProcedureTable*)*userWidgetCount);
-    memcpy(newTable, userProcedureTable, 
-	   sizeof(W_ViewProcedureTable*)*(userWidgetCount-1));
-
-    newTable[userWidgetCount-1] = procTable;
-    
-    free(userProcedureTable);
-    
-    userProcedureTable = newTable;
 
     return userWidgetCount + WC_UserWidget - 1;
 }
@@ -856,8 +934,16 @@ WMScreenDisplay(WMScreen *scr)
 }
 
 
+int 
+WMScreenDepth(WMScreen *scr)
+{
+    return scr->depth;
+}
+
+
 void 
 WMRedisplayWidget(WMWidget *w)
 {
     W_RedisplayView(W_VIEW(w));
 }
+
