@@ -4,6 +4,8 @@
 
 #include "WINGsP.h"
 
+char *WMListDidScrollNotification = "WMListDidScrollNotification";
+char *WMListSelectionDidChangeNotification = "WMListSelectionDidChangeNotification";
 
 typedef struct W_List {
     W_Class widgetClass;
@@ -63,12 +65,15 @@ static void updateScroller(List *lPtr);
 
 static void vScrollCallBack(WMWidget *scroller, void *self);
 
-static void resizeList();
+static void updateGeometry(WMList *lPtr);
+static void didResizeList();
 
 
-W_ViewProcedureTable _ListViewProcedures = {
+W_ViewDelegate _ListViewDelegate = {
     NULL,
-	resizeList,
+	NULL,
+	didResizeList,
+	NULL,
 	NULL
 };
 
@@ -92,6 +97,8 @@ WMCreateList(WMWidget *parent)
     }
     lPtr->view->self = lPtr;
 
+    lPtr->view->delegate = &_ListViewDelegate;
+
     WMCreateEventHandler(lPtr->view, ExposureMask|StructureNotifyMask
 			 |ClientMessageMask, handleEvents, lPtr);
 
@@ -99,7 +106,7 @@ WMCreateList(WMWidget *parent)
 			 |EnterWindowMask|LeaveWindowMask|ButtonMotionMask,
 			 handleActionEvents, lPtr);
 
-    lPtr->itemHeight = scrPtr->normalFont->height + 1;
+    lPtr->itemHeight = WMFontHeight(scrPtr->normalFont) + 1;
         
     /* create the vertical scroller */
     lPtr->vScroller = WMCreateScroller(lPtr);
@@ -111,7 +118,7 @@ WMCreateList(WMWidget *parent)
     /* make the scroller map itself when it's realized */
     WMMapWidget(lPtr->vScroller);
 
-    resizeList(lPtr, DEFAULT_WIDTH, DEFAULT_HEIGHT);
+    W_ResizeView(lPtr->view, DEFAULT_WIDTH, DEFAULT_HEIGHT);
     
     lPtr->selectedItem = -1;
 
@@ -185,7 +192,9 @@ WMInsertListItem(WMList *lPtr, int row, char *text)
     memset(item, 0, sizeof(WMListItem));
     item->text = wstrdup(text);
 
-    if (lPtr->selectedItem >= row && lPtr->selectedItem >= 0)
+
+    if (lPtr->selectedItem >= row && lPtr->selectedItem >= 0
+	&& row >= 0)
 	lPtr->selectedItem++;
 
     if (lPtr->items==NULL) {
@@ -224,16 +233,20 @@ WMRemoveListItem(WMList *lPtr, int row)
 {
     WMListItem *llist;
     WMListItem *tmp;
+    int topItem = lPtr->topItem;
+    int selNotify = 0;
 
     CHECK_CLASS(lPtr, WC_List);
     
     if (row < 0 || row >= lPtr->itemCount)
 	return;
 
-    if (lPtr->selectedItem == row)
-	lPtr->selectedItem = -1;
-    else if (lPtr->selectedItem > row)
-	lPtr->selectedItem--;
+    if (lPtr->selectedItem == row) {
+        lPtr->selectedItem = -1;
+        selNotify = 1;
+    } else if (lPtr->selectedItem > row) {
+        lPtr->selectedItem--;
+    }
     
     if (row <= lPtr->topItem+lPtr->fullFitLines+lPtr->flags.dontFitAll)
 	lPtr->topItem--;
@@ -266,6 +279,11 @@ WMRemoveListItem(WMList *lPtr, int row)
     if (!lPtr->idleID) {
 	lPtr->idleID = WMAddIdleHandler((WMCallback*)updateScroller, lPtr);
     }
+    if (lPtr->topItem != topItem)
+    	WMPostNotificationName(WMListDidScrollNotification, lPtr, NULL);
+    if (selNotify)
+        WMPostNotificationName(WMListSelectionDidChangeNotification, lPtr,
+                               (void*)((int)lPtr->selectedItem));
 }
 
 
@@ -300,6 +318,8 @@ WMSetListUserDrawItemHeight(WMList *lPtr, unsigned short height)
     
     lPtr->flags.userItemHeight = 1;
     lPtr->itemHeight = height;
+    
+    updateGeometry(lPtr);
 }
 
 
@@ -307,7 +327,8 @@ void
 WMClearList(WMList *lPtr)
 {
     WMListItem *item, *tmp;
-    
+    int oldSelected = lPtr->selectedItem;
+
     item = lPtr->items;
     while (item) {
 	free(item->text);
@@ -327,6 +348,9 @@ WMClearList(WMList *lPtr)
     if (lPtr->view->flags.realized) {
 	updateScroller(lPtr);
     }
+    if (oldSelected != -1)
+        WMPostNotificationName(WMListSelectionDidChangeNotification, lPtr,
+                               (void*)-1);
 }
 
 
@@ -370,10 +394,23 @@ WMGetListSelectedItemRow(WMList *lPtr)
 }
 
 
+int
+WMGetListItemHeight(WMList *lPtr)
+{    
+    return lPtr->itemHeight;
+}
+
+
 void
 WMSetListPosition(WMList *lPtr, int row)
 {
     lPtr->topItem = row;
+    if (lPtr->topItem + lPtr->fullFitLines > lPtr->itemCount)
+	lPtr->topItem = lPtr->itemCount - lPtr->fullFitLines;
+
+    if (lPtr->topItem < 0)
+	lPtr->topItem = 0;
+    
     if (lPtr->view->flags.realized)
 	updateScroller(lPtr);
 }
@@ -411,6 +448,7 @@ vScrollCallBack(WMWidget *scroller, void *self)
     WMList *lPtr = (WMList*)self;
     WMScroller *sPtr = (WMScroller*)scroller;
     int height;
+    int topItem = lPtr->topItem;
     
     height = lPtr->view->size.height - 4;
 
@@ -470,6 +508,9 @@ vScrollCallBack(WMWidget *scroller, void *self)
 	/* do nothing */
 	break;
     }
+
+    if (lPtr->topItem != topItem)
+	WMPostNotificationName(WMListDidScrollNotification, lPtr, NULL);
 }
 
 
@@ -510,17 +551,17 @@ paintItem(List *lPtr, int index)
 	    flags |= WLDSIsBranch;
 
 	if (lPtr->draw)
-	    (*lPtr->draw)(lPtr, view->window, itemPtr->text, flags, &rect);
-
+	    (*lPtr->draw)(lPtr, index, view->window, itemPtr->text, flags, 
+			  &rect);
     } else {
 	if (itemPtr->selected)
-	    XFillRectangle(scr->display, view->window, W_GC(scr->white), x, y,
+	    XFillRectangle(scr->display, view->window, WMColorGC(scr->white), x, y,
 			   width, height);
 	else
 	    XClearArea(scr->display, view->window, x, y, width, height, False);
 	
-	W_PaintText(view, view->window, scr->normalFont,  x+2, y, width,
-		    WALeft, W_GC(scr->black), False, 
+	W_PaintText(view, view->window, scr->normalFont,  x+4, y, width,
+		    WALeft, WMColorGC(scr->black), False, 
 		    itemPtr->text, strlen(itemPtr->text));
     }
 }
@@ -555,13 +596,13 @@ paintList(List *lPtr)
 		 lPtr->view->size.height, WRSunken);
 }
 
-
+#if 0
 static void
 scrollTo(List *lPtr, int newTop)
 {
     
 }
-
+#endif
 
 static void
 updateScroller(List *lPtr)
@@ -615,13 +656,16 @@ WMFindRowOfListItemWithTitle(WMList *lPtr, char *title)
 {
     WMListItem *item;
     int i;
+    int ok = 0;
     
     for (i=0, item=lPtr->items; item!=NULL; item=item->nextPtr, i++) {
-	if (strcmp(item->text, title)==0)
+	if (strcmp(item->text, title)==0) {
+	    ok = 1;
 	    break;
+	}
     }
 
-    return i;
+    return ok ? i : -1;
 }
 
 
@@ -629,7 +673,18 @@ void
 WMSelectListItem(WMList *lPtr, int row)
 {
     WMListItem *itemPtr;
-    int i;
+    int i, notify = 0;
+
+    if (row >= lPtr->itemCount)
+	return;
+
+    /* the check below must be changed when the multiple selection is
+     * implemented. -Dan
+     */
+    if (!lPtr->flags.allowMultipleSelection && row == lPtr->selectedItem)
+        notify = 0;
+    else
+        notify = 1;
 
     if (!lPtr->flags.allowMultipleSelection) {
 	/* unselect previous selected item */
@@ -646,6 +701,16 @@ WMSelectListItem(WMList *lPtr, int row)
 	    }
 	}
     }
+    
+    if (row < 0) {
+        if (!lPtr->flags.allowMultipleSelection) {
+	    lPtr->selectedItem = -1;
+            if (notify)
+                WMPostNotificationName(WMListSelectionDidChangeNotification,
+                                       lPtr, (void*)((int)lPtr->selectedItem));
+        }
+	return;
+    }
 
     /* select item */
     itemPtr = lPtr->items;
@@ -666,6 +731,9 @@ WMSelectListItem(WMList *lPtr, int row)
 			 WRSunken);
     }
     lPtr->selectedItem = row;
+    if (notify)
+        WMPostNotificationName(WMListSelectionDidChangeNotification, lPtr,
+                               (void*)((int)lPtr->selectedItem));
 }
 
 
@@ -688,6 +756,7 @@ handleActionEvents(XEvent *event, void *data)
 {
     List *lPtr = (List*)data;
     int tmp;
+    int topItem = lPtr->topItem;
 
     CHECK_CLASS(data, WC_List);
 
@@ -695,14 +764,10 @@ handleActionEvents(XEvent *event, void *data)
      case ButtonRelease:
 	lPtr->flags.buttonPressed = 0;
 	tmp = getItemIndexAt(lPtr, event->xbutton.y);
+
 	if (tmp == lPtr->selectedItem && tmp >= 0) {
-	    if (WMIsDoubleClick(event)) {
-		if (lPtr->doubleAction)
-		    (*lPtr->doubleAction)(lPtr, lPtr->doubleClientData);
-	    } else {
-		if (lPtr->action)
-		    (*lPtr->action)(lPtr, lPtr->clientData);
-	    }
+	    if (lPtr->action)
+		(*lPtr->action)(lPtr, lPtr->clientData);
 	}
 	break;
 
@@ -710,7 +775,7 @@ handleActionEvents(XEvent *event, void *data)
 	lPtr->flags.buttonPressed = lPtr->flags.buttonWasPressed;
 	lPtr->flags.buttonWasPressed = 0;
 	break;
-	
+
      case LeaveNotify:
 	lPtr->flags.buttonWasPressed = lPtr->flags.buttonPressed;
 	lPtr->flags.buttonPressed = 0;
@@ -719,12 +784,17 @@ handleActionEvents(XEvent *event, void *data)
      case ButtonPress:
 	if (event->xbutton.x > WMWidgetWidth(lPtr->vScroller)) {
 	    tmp = getItemIndexAt(lPtr, event->xbutton.y);
-
-	    if (tmp>=0) {
-		WMSelectListItem(lPtr, tmp);
-		lPtr->selectedItem = tmp;
-	    }
 	    lPtr->flags.buttonPressed = 1;
+
+	    if (tmp >= 0) {
+		if (tmp == lPtr->selectedItem && WMIsDoubleClick(event)) {
+		    WMSelectListItem(lPtr, tmp);
+		    if (lPtr->doubleAction)
+			(*lPtr->doubleAction)(lPtr, lPtr->doubleClientData);
+		} else {
+		    WMSelectListItem(lPtr, tmp);
+		}
+	    }
 	}
 	break;
 	
@@ -733,27 +803,43 @@ handleActionEvents(XEvent *event, void *data)
 	    tmp = getItemIndexAt(lPtr, event->xmotion.y);
 	    if (tmp>=0 && tmp != lPtr->selectedItem) {
 		WMSelectListItem(lPtr, tmp);
-		lPtr->selectedItem = tmp;
 	    }
 	}
 	break;
     }
+    if (lPtr->topItem != topItem)
+    	WMPostNotificationName(WMListDidScrollNotification, lPtr, NULL);
 }
 
 
 static void
-resizeList(WMList *lPtr, unsigned int width, unsigned int height)
-{
-    W_ResizeView(lPtr->view, width, height);
-    
-    WMResizeWidget(lPtr->vScroller, 1, height-2);
-    
-    lPtr->fullFitLines = (height - 4) / lPtr->itemHeight;
-    if (lPtr->fullFitLines * lPtr->itemHeight < height-4) {
+updateGeometry(WMList *lPtr)
+{    
+    lPtr->fullFitLines = (lPtr->view->size.height - 4) / lPtr->itemHeight;
+    if (lPtr->fullFitLines * lPtr->itemHeight < lPtr->view->size.height - 4) {
 	lPtr->flags.dontFitAll = 1;
     } else {
 	lPtr->flags.dontFitAll = 0;
     }
+
+    if (lPtr->itemCount - lPtr->topItem <= lPtr->fullFitLines) {
+	lPtr->topItem = lPtr->itemCount - lPtr->fullFitLines;
+	if (lPtr->topItem < 0)
+	    lPtr->topItem = 0;
+    }
+
+    updateScroller(lPtr);
+}
+
+
+static void
+didResizeList(W_ViewDelegate *self, WMView *view)
+{    
+    WMList *lPtr = (WMList*)view->self;
+
+    WMResizeWidget(lPtr->vScroller, 1, view->size.height-2);
+
+    updateGeometry(lPtr);
 }
 
 
