@@ -14,36 +14,40 @@ W_DrawRelief(W_Screen *scr, Drawable d, int x, int y, unsigned int width,
     GC wgc;
     GC lgc;
     GC dgc;
-    
 
     switch (relief) {
      case WRSimple:
-	XDrawRectangle(dpy, d, W_GC(scr->black), x, y, width-1, height-1);
+	XDrawRectangle(dpy, d, WMColorGC(scr->black), x, y, width-1, height-1);
 	return;
 	break;
 	
      case WRRaised:
-	bgc = W_GC(scr->black);
-	dgc = W_GC(scr->darkGray);
-	wgc = W_GC(scr->white);
-	lgc = W_GC(scr->gray);
+	bgc = WMColorGC(scr->black);
+	dgc = WMColorGC(scr->darkGray);
+	wgc = WMColorGC(scr->white);
+	lgc = WMColorGC(scr->gray);
 	break;
 	
      case WRSunken:
-	wgc = W_GC(scr->darkGray);
-	lgc = W_GC(scr->black);
-	bgc = W_GC(scr->white);
-	dgc = W_GC(scr->gray);
+	wgc = WMColorGC(scr->darkGray);
+	lgc = WMColorGC(scr->black);
+	bgc = WMColorGC(scr->white);
+	dgc = WMColorGC(scr->gray);
+	break;
+	
+     case WRPushed:
+	lgc = wgc = WMColorGC(scr->black);
+	dgc = bgc = WMColorGC(scr->white);
 	break;
 	
      case WRRidge:
-	lgc = bgc = W_GC(scr->darkGray);
-	dgc = wgc = W_GC(scr->white);
+	lgc = bgc = WMColorGC(scr->darkGray);
+	dgc = wgc = WMColorGC(scr->white);
 	break;
 
      case WRGroove:
-	wgc = dgc = W_GC(scr->darkGray);
-	lgc = bgc = W_GC(scr->white);
+	wgc = dgc = WMColorGC(scr->darkGray);
+	lgc = bgc = WMColorGC(scr->white);
 	break;
 	
      default:
@@ -51,24 +55,24 @@ W_DrawRelief(W_Screen *scr, Drawable d, int x, int y, unsigned int width,
     }
     /* top left */
     XDrawLine(dpy, d, wgc, x, y, x+width-1, y);
-    if (width > 2 && relief != WRRaised) {
+    if (width > 2 && relief != WRRaised && relief!=WRPushed) {
 	XDrawLine(dpy, d, lgc, x+1, y+1, x+width-3, y+1);
     }
     
     XDrawLine(dpy, d, wgc, x, y, x, y+height-1);
-    if (height > 2 && relief != WRRaised) {
+    if (height > 2 && relief != WRRaised && relief!=WRPushed) {
 	XDrawLine(dpy, d, lgc, x+1, y+1, x+1, y+height-3);
     }
     
     /* bottom right */
     XDrawLine(dpy, d, bgc, x, y+height-1, x+width-1, y+height-1);
-    if (width > 2) {
+    if (width > 2 && relief!=WRPushed) {
 	XDrawLine(dpy, d, dgc, x+1, y+height-2, x+width-2, y+height-2);
     }
 
     XDrawLine(dpy, d, bgc, x+width-1, y, x+width-1, y+height-1);
-    if (height > 2) {
-	XDrawLine(dpy, d, dgc, x+width-2, y+1, x+width-2, y+height-3);
+    if (height > 2 && relief!=WRPushed) {
+	XDrawLine(dpy, d, dgc, x+width-2, y+1, x+width-2, y+height-2);
     }
 }
 
@@ -112,12 +116,13 @@ W_GetTextHeight(WMFont *font, char *text, int width, int wrap)
     int count;
     int length = strlen(text);
     int h;
+    int fheight = WMFontHeight(font);
 
     h = 0;
     while (length > 0) {
-	count = fitText(ptr, font, width-4, wrap);
+	count = fitText(ptr, font, width, wrap);
 
-	h += font->height;
+	h += fheight;
 
 	if (isspace(ptr[count]))
 	    count++;
@@ -138,22 +143,22 @@ W_PaintText(W_View *view, Drawable d, WMFont *font,  int x, int y,
     int line_width;
     int line_x;
     int count;
+    int fheight = WMFontHeight(font);
 
     while (length > 0) {
-	count = fitText(ptr, font, width-4, wrap);
+	count = fitText(ptr, font, width, wrap);
 
 	line_width = WMWidthOfString(font, ptr, count);
-
 	if (alignment==WALeft)
-	    line_x = x + 4;
+	    line_x = x;
 	else if (alignment==WARight)
-	    line_x = x + width - line_width - 4;
+	    line_x = x + width - line_width;
 	else
 	    line_x = x + (width - line_width) / 2;
 
 	WMDrawString(view->screen, d, gc, font, line_x, y, ptr, count);
 
-	y += font->height;
+	y += fheight;
 
 	if (isspace(ptr[count]))
 	    count++;
@@ -164,12 +169,10 @@ W_PaintText(W_View *view, Drawable d, WMFont *font,  int x, int y,
 }
 
 
-
-
 void
 W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
-		    WMReliefType relief, char *text, int textHeight,
-		    WMAlignment alignment,  W_Pixmap *image, 
+		    WMReliefType relief, char *text,
+		    WMAlignment alignment,  W_Pixmap *image,
 		    WMImagePosition position, GC backGC, int ofs)
 {
     W_Screen *screen = view->screen;
@@ -180,7 +183,7 @@ W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
     
 #ifdef DOUBLE_BUFFER
     d = XCreatePixmap(screen->display, view->window, 
-			   view->size.width, view->size.height, screen->depth);
+		      view->size.width, view->size.height, screen->depth);
 #endif
     
     /* background */
@@ -232,15 +235,15 @@ W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
 	 case WIPLeft:
 	    ix = x;
 	    iy = y + (h - image->height) / 2;
-	    x = image->width + 5;
+	    x = x + image->width + 5;
 	    y = 0;
-	    w -= x;
+	    w -= image->width + 5;
 	    break;
 	    
 	 case WIPRight:
-	    ix = view->size.width - image->width - 5;
+	    ix = view->size.width - image->width - x;
 	    iy = y + (h - image->height) / 2;
-	    w -= image->width;
+	    w -= image->width + 5;
 	    break;
 	    
 	 case WIPBelow:
@@ -261,20 +264,24 @@ W_PaintTextAndImage(W_View *view, int wrap, GC textGC, W_Font *font,
 	
 	ix += ofs;
 	iy += ofs;
-    
+
 	XSetClipOrigin(screen->display, screen->clipGC, ix, iy);
 	XSetClipMask(screen->display, screen->clipGC, image->mask);
+
 	if (image->depth==1)
 	    XCopyPlane(screen->display, image->pixmap, d, screen->clipGC,
 		       0, 0, image->width, image->height, ix, iy, 1);
 	else
-	    XCopyArea(screen->display, image->pixmap, d, screen->clipGC, 
+	    XCopyArea(screen->display, image->pixmap, d, screen->clipGC,
 		      0, 0, image->width, image->height, ix, iy);
     }
 
     /* draw text */
     if (position != WIPImageOnly && text!=NULL) {
-	W_PaintText(view, d, font, x+ofs, y+ofs + (h-textHeight)/2, w,
+	int textHeight;
+	
+	textHeight = W_GetTextHeight(font, text, w-8, wrap);
+	W_PaintText(view, d, font, x+ofs+4, y+ofs + (h-textHeight)/2, w-8,
 		    alignment, textGC, wrap, text, strlen(text));
     }
     
