@@ -34,24 +34,27 @@ handleKeyPress(XEvent *event, void *clientData)
 
 
 int 
-WMRunAlertPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultButton,
+WMRunAlertPanel(WMScreen *scrPtr, WMWindow *owner,
+		char *title, char *msg, char *defaultButton,
 		char *alternateButton, char *otherButton)
 {
     WMAlertPanel *panel;
     int tmp;
 
-    panel = WMCreateAlertPanel(scrPtr, title, msg, defaultButton,
+    panel = WMCreateAlertPanel(scrPtr, owner, title, msg, defaultButton,
 			       alternateButton, otherButton);
 
+    scrPtr->modalView = W_VIEW(panel->win);
     WMMapWidget(panel->win);
 
+    scrPtr->modal = 1;
     while (!panel->done || WMScreenPending(scrPtr)) {
 	XEvent event;
 	
 	WMNextEvent(scrPtr->display, &event);
 	WMHandleEvent(&event);
     }
-    
+    scrPtr->modal = 0;
 
     tmp = panel->result;
 
@@ -71,7 +74,8 @@ WMDestroyAlertPanel(WMAlertPanel *panel)
 
 
 WMAlertPanel* 
-WMCreateAlertPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultButton,
+WMCreateAlertPanel(WMScreen *scrPtr, WMWindow *owner, 
+		   char *title, char *msg, char *defaultButton,
 		   char *alternateButton, char *otherButton)
 {
     WMAlertPanel *panel;
@@ -83,8 +87,17 @@ WMCreateAlertPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultButton
 
         
     panel->retKey = XKeysymToKeycode(scrPtr->display, XK_Return);
-        
-    panel->win = WMCreateWindow(scrPtr, "alertPanel");
+
+    if (owner)
+	panel->win = WMCreatePanelWithStyleForWindow(owner, "alertPanel",
+						     WMTitledWindowMask);
+    else
+	panel->win = WMCreateWindowWithStyle(scrPtr, "alertPanel",
+					     WMTitledWindowMask);
+
+    WMSetWindowUPosition(panel->win, 
+	 (scrPtr->rootView->size.width - WMWidgetWidth(panel->win))/2,
+	 (scrPtr->rootView->size.height - WMWidgetHeight(panel->win))/2);
     WMSetWindowTitle(panel->win, "");
 
     if (scrPtr->applicationIcon) {		
@@ -96,15 +109,15 @@ WMCreateAlertPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultButton
 	WMSetLabelImage(panel->iLbl, scrPtr->applicationIcon);
 	WMSetLabelImagePosition(panel->iLbl, WIPImageOnly);
     }
-    
+
     if (title) {
 	WMFont *largeFont;
 	
 	largeFont = WMBoldSystemFontOfSize(scrPtr, 24);
 
 	panel->tLbl = WMCreateLabel(panel->win);
-	WMMoveWidget(panel->tLbl, 80, (80 - largeFont->height)/2);
-	WMResizeWidget(panel->tLbl, 400 - 70, largeFont->height+4);
+	WMMoveWidget(panel->tLbl, 80, (80 - WMFontHeight(largeFont))/2);
+	WMResizeWidget(panel->tLbl, 400 - 70, WMFontHeight(largeFont)+4);
 	WMSetLabelText(panel->tLbl, title);
 	WMSetLabelTextAlignment(panel->tLbl, WALeft);
 	WMSetLabelFont(panel->tLbl, largeFont);
@@ -115,8 +128,8 @@ WMCreateAlertPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultButton
 
     if (msg) {
 	panel->mLbl = WMCreateLabel(panel->win);
-	WMMoveWidget(panel->mLbl, 10, 95);
-	WMResizeWidget(panel->mLbl, 380, scrPtr->normalFont->height*3);
+	WMMoveWidget(panel->mLbl, 10, 83);
+	WMResizeWidget(panel->mLbl, 380, WMFontHeight(scrPtr->normalFont)*4);
 	WMSetLabelText(panel->mLbl, msg);
 	WMSetLabelTextAlignment(panel->mLbl, WACenter);
     }
@@ -224,15 +237,15 @@ handleKeyPress2(XEvent *event, void *clientData)
 
 
 char*
-WMRunInputPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultText, 
+WMRunInputPanel(WMScreen *scrPtr, WMWindow *owner, char *title, 
+		char *msg, char *defaultText,
 		char *okButton, char *cancelButton)
 {
     WMInputPanel *panel;
     char *tmp;
 
-    panel = WMCreateInputPanel(scrPtr, title, msg, defaultText, okButton, 
-			       cancelButton);
-    
+    panel = WMCreateInputPanel(scrPtr, owner, title, msg, defaultText, 
+			       okButton, cancelButton);
 
     WMMapWidget(panel->win);
 
@@ -278,8 +291,8 @@ endedEditingObserver(void *observerData, WMNotification *notification)
 
 
 WMInputPanel*
-WMCreateInputPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultText,
-		   char *okButton, char *cancelButton)
+WMCreateInputPanel(WMScreen *scrPtr, WMWindow *owner, char *title, char *msg,
+		   char *defaultText, char *okButton, char *cancelButton)
 {
     WMInputPanel *panel;
     int x, dw=0, aw=0, w;
@@ -289,8 +302,13 @@ WMCreateInputPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultText,
     memset(panel, 0, sizeof(WMInputPanel));
 
     panel->retKey = XKeysymToKeycode(scrPtr->display, XK_Return);
-    
-    panel->win = WMCreateWindow(scrPtr, "inputPanel");
+
+    if (owner)
+	panel->win = WMCreatePanelWithStyleForWindow(owner, "inputPanel",
+						     WMTitledWindowMask);
+    else
+	panel->win = WMCreateWindowWithStyle(scrPtr, "inputPanel",
+					     WMTitledWindowMask);
     WMSetWindowTitle(panel->win, "");
 
     WMResizeWidget(panel->win, 320, 160);
@@ -302,7 +320,7 @@ WMCreateInputPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultText,
 	
 	panel->tLbl = WMCreateLabel(panel->win);
 	WMMoveWidget(panel->tLbl, 20, 16);
-	WMResizeWidget(panel->tLbl, 320 - 40, largeFont->height+4);
+	WMResizeWidget(panel->tLbl, 320 - 40, WMFontHeight(largeFont)+4);
 	WMSetLabelText(panel->tLbl, title);
 	WMSetLabelTextAlignment(panel->tLbl, WALeft);
 	WMSetLabelFont(panel->tLbl, largeFont);
@@ -314,7 +332,8 @@ WMCreateInputPanel(WMScreen *scrPtr, char *title, char *msg, char *defaultText,
     if (msg) {
 	panel->mLbl = WMCreateLabel(panel->win);
 	WMMoveWidget(panel->mLbl, 20, 50);
-	WMResizeWidget(panel->mLbl, 320 - 40,scrPtr->normalFont->height*2);
+	WMResizeWidget(panel->mLbl, 320 - 40, 
+		       WMFontHeight(scrPtr->normalFont)*2);
 	WMSetLabelText(panel->mLbl, msg);
 	WMSetLabelTextAlignment(panel->mLbl, WALeft);
     }
