@@ -15,6 +15,9 @@ typedef struct W_ScrollView {
     WMScroller *vScroller;
     WMScroller *hScroller;
     
+    short lineScroll;
+    short pageScroll;
+    
     struct {
 	WMReliefType relief:3;
 	unsigned int hasVScroller:1;
@@ -35,9 +38,11 @@ static void handleViewportEvents(XEvent *event, void *data);
 static void resizeScrollView();
 
 
-W_ViewProcedureTable _ScrollViewViewProcedures = {
+W_ViewDelegate _ScrollViewViewDelegate = {
     NULL,
+	NULL,
 	resizeScrollView,
+	NULL,
 	NULL
 };
 
@@ -64,7 +69,10 @@ WMCreateScrollView(WMWidget *parent)
 	free(sPtr);
 	return NULL;
     }
-    
+    sPtr->view->self = sPtr;
+
+    sPtr->view->delegate = &_ScrollViewViewDelegate;
+
     sPtr->viewport->flags.mapWhenRealized = 1;
 
     WMCreateEventHandler(sPtr->view, StructureNotifyMask|ExposureMask,
@@ -72,6 +80,10 @@ WMCreateScrollView(WMWidget *parent)
     WMCreateEventHandler(sPtr->viewport, SubstructureNotifyMask, 
 			 handleViewportEvents, sPtr);
 
+    sPtr->lineScroll = 4;
+
+    sPtr->pageScroll = 0;
+
     return sPtr;
 }
 
@@ -164,11 +176,9 @@ reorganizeInterior(WMScrollView *sPtr)
 
 
 static void
-resizeScrollView(WMScrollView *sPtr, unsigned int width, unsigned int height)
+resizeScrollView(W_ViewDelegate *self, WMView *view)
 {
-    W_ResizeView(sPtr->view, width, height);
-
-    reorganizeInterior(sPtr);
+    reorganizeInterior(view->self);
 }
 
 
@@ -208,6 +218,24 @@ WMResizeScrollViewContent(WMScrollView *sPtr, unsigned int width,
 }
 
 
+void
+WMSetScrollViewLineScroll(WMScrollView *sPtr, int amount)
+{
+    assert(amount > 0);
+    
+    sPtr->lineScroll = amount;
+}
+
+
+void
+WMSetScrollViewPageScroll(WMScrollView *sPtr, int amount)
+{
+    assert(amount >= 0);
+
+    sPtr->pageScroll = amount;
+}
+
+
 static void
 doScrolling(WMWidget *self, void *data)
 {
@@ -220,17 +248,21 @@ doScrolling(WMWidget *self, void *data)
     if (sPtr->hScroller == self) {
 	pos = -sPtr->contentView->pos.x;
 	size = sPtr->contentView->size.width-sPtr->viewport->size.width;
-	vpsize = sPtr->viewport->size.width;
+	vpsize = sPtr->viewport->size.width - sPtr->pageScroll;
     } else {
 	pos = -sPtr->contentView->pos.y;
 	size = sPtr->contentView->size.height-sPtr->viewport->size.height;
-	vpsize = sPtr->viewport->size.height;
+	vpsize = sPtr->viewport->size.height - sPtr->pageScroll;
     }
-    
+    if (vpsize <= 0)
+	vpsize = 1;
+
     switch (WMGetScrollerHitPart(self)) {
      case WSDecrementLine:
 	if (pos > 0) {
-	    pos--;
+	    pos-=sPtr->lineScroll;
+	    if (pos < 0)
+		pos = 0;
 	    value = (float)pos / size;
 	    WMSetScrollerParameters(self, value,
 				    WMGetScrollerKnobProportion(self));
@@ -238,7 +270,9 @@ doScrolling(WMWidget *self, void *data)
 	break;
      case WSIncrementLine:
 	if (pos < size) {
-	    pos++;
+	    pos+=sPtr->lineScroll;
+	    if (pos > size)
+		pos = size;
 	    value = (float)pos / size;
 	    WMSetScrollerParameters(self, value,
 				    WMGetScrollerKnobProportion(self));
@@ -273,6 +307,7 @@ doScrolling(WMWidget *self, void *data)
 	
      case WSNoPart:
      case WSKnobSlot:
+	break;
     }
     
     if (sPtr->hScroller == self) {
@@ -366,7 +401,7 @@ WMSetScrollViewContentView(WMScrollView *sPtr, WMView *view)
     
     sPtr->contentView = view;
 
-    W_ReparentView(sPtr->contentView, sPtr->viewport);
+    W_ReparentView(sPtr->contentView, sPtr->viewport, 0, 0);
 
     if (sPtr->flags.hasHScroller) {
 	float prop;
