@@ -1,5 +1,5 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -37,8 +37,6 @@
 #endif
 
 
-/* max. number of distinct window levels */
-#define MAX_WINDOW_LEVELS	5
 
 
 /* class codes */
@@ -56,6 +54,18 @@ typedef enum {
 } WClassType;
 
 
+/* generic window levels (a superset of the N*XTSTEP ones) */
+enum {
+    WMDesktopLevel = 0,
+	WMSunkenLevel = 1,
+	WMNormalLevel = 2,
+	WMFloatingLevel = 3,
+	WMDockLevel = 4,
+	WMSubmenuLevel = 5,
+	WMMainMenuLevel = 6,
+	WMOuterSpaceLevel = 7
+};
+#define MAX_WINDOW_LEVELS 8
 
 /*
  * WObjDescriptor will be used by the event dispatcher to
@@ -81,19 +91,32 @@ typedef struct WObjDescriptor {
 
     WClassType parent_type;	       /* type code of the parent */
     void *parent;		       /* parent object (WWindow or WMenu) */
-    
-    Time click_timestamp;	       /* timestamp of the last click */
-    unsigned int click_button;	       /* button number of the last click */
 } WObjDescriptor;
 
 
+/* shutdown modes */
+typedef enum {
+    WSExitMode,
+	WSLogoutMode,
+	WSKillMode,
+	WSRestartPreparationMode
+} WShutdownMode;
+
+
 /* internal buttons */
 #define WBUT_CLOSE              0
 #define WBUT_BROKENCLOSE        1
 #define WBUT_ICONIFY            2
 #define WBUT_KILL		3
-
+#ifdef XKB_BUTTON_HINT
+#define WBUT_XKBGROUP1      4
+#define WBUT_XKBGROUP2      5
+#define WBUT_XKBGROUP3      6
+#define WBUT_XKBGROUP4      7
+#define PRED_BPIXMAPS		8 /* reserved for 4 groups */
+#else
 #define PRED_BPIXMAPS		4 /* count of WBUT icons */
+#endif /* XKB_BUTTON_HINT */
 
 /* cursors */
 #define WCUR_DEFAULT	0
@@ -130,13 +153,12 @@ typedef struct WObjDescriptor {
 #define WTJ_LEFT	1
 #define WTJ_RIGHT	2
 
-
-/* icon box positions */
-#define WIB_BOTTOM	0
-#define WIB_TOP		1
-#define WIB_LEFT	2
-#define WIB_RIGHT	3
-
+/* iconification styles */
+#define WIS_ZOOM        0
+#define WIS_TWIST       1
+#define WIS_FLIP        2
+#define WIS_NONE        3
+#define WIS_RANDOM	4 /* secret */
 
 /* switchmenu actions */
 #define ACTION_ADD	0
@@ -153,37 +175,111 @@ typedef struct WObjDescriptor {
 #define SPEED_SLOW	3
 #define SPEED_ULTRASLOW 4
 
-/* startup warnings */
-#define WAR_DEFAULTS_DIR	1 /* created defaults DB directory */
-
 
 /* window states */
 #define WS_FOCUSED	0
 #define WS_UNFOCUSED	1
 #define WS_PFOCUSED	2
+#ifdef TITLE_TEXT_SHADOW
+#define WS_SMENU       3
+#define WS_SFOCUSED    3
+#define WS_SUNFOCUSED  4
+#define WS_SPFOCUSED   5
+#endif /* TITLE_TEXT_SHADOW */
 
 /* clip title colors */
 #define CLIP_NORMAL    0
 #define CLIP_COLLAPSED 1
 
 
-typedef struct W2Color {
-    char *color1;
-    char *color2;
-} W2Color;
+/* icon yard position */
+#define	IY_VERT		1
+#define	IY_HORIZ	0
+#define	IY_TOP		2
+#define	IY_BOTTOM	0
+#define	IY_RIGHT	4
+#define	IY_LEFT		0
+
+
+/* menu styles */
+#define MS_NORMAL		0
+#define MS_SINGLE_TEXTURE	1
+#define MS_FLAT			2
+
+
+/* workspace display position */
+#define WD_NONE		0
+#define WD_CENTER	1
+#define WD_TOP		2
+#define WD_BOTTOM	3
+#define WD_TOPLEFT	4
+#define WD_TOPRIGHT	5
+#define WD_BOTTOMLEFT	6
+#define WD_BOTTOMRIGHT	7
+
+
+/* program states */
+#define WSTATE_NORMAL		0
+#define WSTATE_NEED_EXIT	1
+#define WSTATE_NEED_RESTART	2
+#define WSTATE_EXITING		3
+#define WSTATE_RESTARTING	4
+#define WSTATE_MODAL		5
+
+
+#define WCHECK_STATE(state)	(state == WProgramState)
+#define WCHANGE_STATE(nstate)	\
+	if (WProgramState == WSTATE_NORMAL\
+		|| nstate != WSTATE_MODAL)\
+		 WProgramState = (nstate)
+
+
+/* notifications */
+
+#ifdef MAINFILE
+#define NOTIFICATION(n) char *WN##n = #n
+#else
+#define NOTIFICATION(n) extern char *WN##n
+#endif
+
+NOTIFICATION(WindowAppearanceSettingsChanged);
+
+NOTIFICATION(IconAppearanceSettingsChanged);
+
+NOTIFICATION(IconTileSettingsChanged);
+
+NOTIFICATION(MenuAppearanceSettingsChanged);
+
+NOTIFICATION(MenuTitleAppearanceSettingsChanged);
+
+
+/* appearance settings clientdata flags */
+enum {
+    WFontSettings = 1 << 0,
+	WTextureSettings = 1 << 1,
+	WColorSettings = 1 << 2
+};
+
+
+
+typedef struct {
+    int x1, y1;
+    int x2, y2;
+} WArea;
 
 typedef struct WCoord {
     int x, y;
 } WCoord;
 
-
-
 typedef struct WPreferences {
-    char **pixmap_path;		       /* NULL terminated array of */
+    char *pixmap_path;		       /* : separate list of */
 				       /* paths to find pixmaps */
-    char **icon_path;		       /* NULL terminated array of */
+    char *icon_path;		       /* : separated list of */
 				       /* paths to find icons */
 
+    RImage *button_images;	       /* titlebar button images */
+
+    char smooth_workspace_back;
     char size_display;		       /* display type for resize geometry */
     char move_display;		       /* display type for move geometry */
     char window_placement;	       /* window placement mode */
@@ -199,8 +295,7 @@ typedef struct WPreferences {
     
     char use_saveunders;	       /* turn on SaveUnders for menus,
 					* icons etc. */ 
-    
-    char no_window_under_dock;
+    char no_window_over_dock;
 
     char no_window_over_icons;
 
@@ -214,10 +309,19 @@ typedef struct WPreferences {
 
     char ignore_focus_click;
 
-    char on_top_transients;	       /* transient windows are kept on top
-					* of their owners */
+    char open_transients_with_parent;  /* open transient window in
+					same workspace as parent */
     char title_justification;	       /* titlebar text alignment */
 
+#ifdef TITLE_TEXT_SHADOW
+    char title_shadow;
+#endif
+
+    char multi_byte_text;
+#ifdef KEEP_XKB_LOCK_STATUS
+    char modelock;
+#endif
+
     char no_dithering;		       /* use dithering or not */
     
     char no_sound;		       /* enable/disable sound */
@@ -230,16 +334,21 @@ typedef struct WPreferences {
     
     char icon_box_position;	       /* position to place icons */
     
-    char disable_root_mouse;	       /* disable mouse actions in root window */
+    char iconification_style;          /* position to place icons */
+    
+    char disable_root_mouse;	       /* disable button events in root window */
     
     char auto_focus;		       /* focus window when it's mapped */
-    
+
+
     char *icon_back_file;	       /* background image for icons */
 
     WCoord *root_menu_pos;	       /* initial position of the root menu*/
     WCoord *app_menu_pos;
 
     WCoord *win_menu_pos;
+    
+    char icon_yard;		       /* aka iconbox */
 
     int raise_delay;		       /* delay for autoraise. 0 is disabled */
 
@@ -248,18 +357,27 @@ typedef struct WPreferences {
 
     int icon_size;		       /* size of the icon */
 
+    int menu_style;		       /* menu decoration style */
+
+    int workspace_name_display_position;
+    
+    unsigned int modifier_mask;	       /* mask to use as kbd modifier */
+
+
     char ws_advance;                   /* Create new workspace and advance */
 
     char ws_cycle;                     /* Cycle existing workspaces */
 
-    unsigned int modifier_mask;	       /* mask to use as kbd modifier */
-
     char save_session_on_exit;	       /* automatically save session on exit */
 
     char sticky_icons;		       /* If miniwindows will be onmipresent */
 
     char dont_confirm_kill;	       /* do not confirm Kill application */
 
+    char disable_miniwindows;
+
+    char dont_blink;		       /* do not blink icon selection */
+    
     /* Appearance options */
     char new_style;		       /* Use newstyle buttons */
     char superfluous;		       /* Use superfluous things */
@@ -269,33 +387,45 @@ typedef struct WPreferences {
     char windowl_button;	       /* button for window list menu */
     char menu_button;		       /* button for app menu */
 
+    /* balloon text */
+    char window_balloon;
+    char miniwin_balloon;
+    char appicon_balloon;
+    char help_balloon;
+        
+#ifdef WEENDOZE_CYCLE
+    char windoze_cycling;	       /* Windoze 95 style Alt+Tabbing */
+    char popup_switchmenu;	       /* Popup the switchmenu when Alt+Tabbing */
+#endif /* WEENDOZE_CYCLE */
     /* some constants */
     int dblclick_time;		       /* double click delay time in ms */
 
-    /* some animation constants */
     /* animate menus */
-    int menu_scroll_speed;	       /* how fast menus are scrolled */
+    char menu_scroll_speed;	       /* how fast menus are scrolled */
 
     /* animate icon sliding */
-    int icon_slide_speed;	       /* icon slide animation speed */
+    char icon_slide_speed;	       /* icon slide animation speed */
 
     /* shading animation */
-    int shade_speed;
-    
+    char shade_speed;
+
+    int edge_resistance;
+
     struct {
         unsigned int nodock:1;	       /* don't display the dock */
         unsigned int noclip:1;        /* don't display the clip */
 	unsigned int nocpp:1;	       /* don't use cpp */
+	unsigned int noupdates:1;      /* don't require ~/GNUstep (-static) */
     } flags;			       /* internal flags */
 } WPreferences;
 
 
-#define VALID_MOD_MASK (ControlMask|ShiftMask|wPreferences.modifier_mask)
-
 
 /****** Global Variables  ******/
 extern Display	*dpy;
 extern char *ProgName;
+extern unsigned int ValidModMask;
+extern char WProgramState;
 
 /****** Global Functions ******/
 extern void wAbort(Bool dumpCore);
