@@ -1,9 +1,10 @@
 /* appicon.c- icon for applications (not mini-window)
  *
- *  WindowMaker window manager
- * 
+ *  Window Maker window manager
+ *
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- * 
+ *  Copyright (c) 1998       Dan Pascu
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -16,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
  *  USA.
  */
 
@@ -43,8 +44,11 @@
 #include "framewin.h"
 #include "dialog.h"
 #include "client.h"
+#ifdef XDND
+#include "xdnd.h"
+#endif
 
-/* 
+/*
  * icon_file for the dock is got from the preferences file by
  * using the classname/instancename
  */
@@ -63,7 +67,7 @@ static void iconExpose(WObjDescriptor *desc, XEvent *event);
 
 WAppIcon*
 wAppIconCreateForDock(WScreen *scr, char *command, char *wm_instance,
-		      char *wm_class)
+		      char *wm_class, int tile)
 {
     WAppIcon *dicon;
     char *path;
@@ -82,22 +86,40 @@ wAppIconCreateForDock(WScreen *scr, char *command, char *wm_instance,
     scr->app_icon_list = dicon;
 
     if (command) {
-	dicon->command = wstrdup(command);	
+	dicon->command = wstrdup(command);
     }
     if (wm_class)
       dicon->wm_class = wstrdup(wm_class);
     if (wm_instance)
       dicon->wm_instance = wstrdup(wm_instance);
 
-    path = wDefaultGetIconFile(scr, wm_instance, wm_class);
+    path = wDefaultGetIconFile(scr, wm_instance, wm_class, True);
+    if (!path && command) {
+	wApplicationExtractDirPackIcon(scr, command, wm_instance, wm_class);
+
+	path = wDefaultGetIconFile(scr, wm_instance, wm_class, False);
+    }
+
     if (path)
 	path = FindImage(wPreferences.icon_path, path);
-    dicon->icon = wIconCreateWithIconFile(scr, path);
+
+    dicon->icon = wIconCreateWithIconFile(scr, path, tile);
     if (path)
 	free(path);
+#ifdef XDND
+    wXDNDMakeAwareness(dicon->icon->core->window);
+#endif
 #ifdef REDUCE_APPICONS
     dicon->num_apps = 0;
 #endif
+#ifdef DEMATERIALIZE_ICON
+    {
+        XSetWindowAttributes attribs;
+        attribs.save_under = True;
+        XChangeWindowAttributes(dpy, dicon->icon->core->window,
+                CWSaveUnder, &attribs);
+    }
+#endif
 
     /* will be overriden by dock */
     dicon->icon->core->descriptor.handle_mousedown = appIconMouseDown;
@@ -146,7 +168,9 @@ wAppIconCreate(WWindow *leader_win)
     aicon->prev = NULL;
     aicon->next = scr->app_icon_list;
     if (scr->app_icon_list) {
-#ifdef REDUCE_APPICONS
+#ifndef REDUCE_APPICONS
+        scr->app_icon_list->prev = aicon;
+#else
 	/* If we aren't going to have a match, jump straight to new appicon */
 	if (leader_win->wm_class == NULL || leader_win->wm_class == NULL)
 	    atmp = NULL;
@@ -162,7 +186,7 @@ wAppIconCreate(WWindow *leader_win)
 		(strcmp(leader_win->wm_instance, tinstance) == 0))
 	    {
 		/* We have a winner */
-		wrelease(aicon);	
+		wrelease(aicon);
 		atmp->num_apps++;
 		applist->next = atmp->applist;
 		if (atmp->applist)
@@ -178,11 +202,9 @@ wAppIconCreate(WWindow *leader_win)
 	    atmp = atmp->next;
 	}
 	if (atmp == NULL) {
-#endif
-        scr->app_icon_list->prev = aicon;
-#ifdef REDUCE_APPICONS
+	    scr->app_icon_list->prev = aicon;
 	}
-#endif
+#endif /* REDUCE_APPICONS */
     }
     scr->app_icon_list = aicon;
 
@@ -195,6 +217,17 @@ wAppIconCreate(WWindow *leader_win)
 #endif
 
     aicon->icon = wIconCreate(leader_win);
+#ifdef DEMATERIALIZE_ICON
+    {
+        XSetWindowAttributes attribs;
+        attribs.save_under = True;
+        XChangeWindowAttributes(dpy, aicon->icon->core->window,
+                CWSaveUnder, &attribs);
+    }
+#endif
+#ifdef XDND
+    wXDNDMakeAwareness(aicon->icon->core->window);
+#endif
 
     /* will be overriden if docked */
     aicon->icon->core->descriptor.handle_mousedown = appIconMouseDown;
@@ -284,50 +317,76 @@ drawCorner(WIcon *icon, WWindow *wwin, int active)
 #endif /* NEWAPPICON */
 
 
+static void
+drawCorner(WIcon *icon)
+{
+    WScreen *scr = icon->core->screen_ptr;
+    XPoint points[3];
+
+    points[0].x = 1;
+    points[0].y = 1;
+    points[1].x = 12;
+    points[1].y = 1;
+    points[2].x = 1;
+    points[2].y = 12;
+    XFillPolygon(dpy, icon->core->window, scr->icon_title_texture->normal_gc,
+                 points, 3, Convex, CoordModeOrigin);
+    XDrawLine(dpy, icon->core->window, scr->icon_title_texture->light_gc,
+              0, 0, 0, 12);
+    XDrawLine(dpy, icon->core->window, scr->icon_title_texture->light_gc,
+              0, 0, 12, 0);
+    /* drawing the second line gives a weird concave look. -Dan */
+#if 0
+    XDrawLine(dpy, icon->core->window, scr->icon_title_texture->light_gc,
+              1, 1, 1, 11);
+    XDrawLine(dpy, icon->core->window, scr->icon_title_texture->light_gc,
+              1, 1, 11, 1);
+#endif
+}
+
+
+void
+wAppIconMove(WAppIcon *aicon, int x, int y)
+{
+    XMoveWindow(dpy, aicon->icon->core->window, x, y);
+    aicon->x_pos = x;
+    aicon->y_pos = y;
+}
+
+
 #ifdef WS_INDICATOR
 static void
 updateDockNumbers(WScreen *scr)
 {
-   int length;
-   char *ws_numbers;
-   GC numbers_gc;
-   XGCValues my_gc_values;
-   unsigned long my_v_mask = (GCForeground);
-   WAppIcon *dicon = scr->dock->icon_array[0];
+    int length;
+    char *ws_numbers;
+    GC numbers_gc;
+    XGCValues my_gc_values;
+    unsigned long my_v_mask = (GCForeground);
+    WAppIcon *dicon = scr->dock->icon_array[0];
+    
+    my_gc_values.foreground = scr->white_pixel;
+    numbers_gc = XCreateGC(dpy, dicon->icon->core->window,
+			   my_v_mask, &my_gc_values);
 
-   my_gc_values.foreground = scr->white_pixel;
-   numbers_gc = XCreateGC(dpy, dicon->icon->core->window,
-			  my_v_mask, &my_gc_values);
-   
-   ws_numbers = malloc(20);
-   sprintf(ws_numbers, "%i [ %i ]", scr->current_workspace+1, 
-	   ((scr->current_workspace/10)+1));
-   length = strlen(ws_numbers);
+    ws_numbers = malloc(20);
+    sprintf(ws_numbers, "%i [ %i ]", scr->current_workspace+1,
+	    ((scr->current_workspace/10)+1));
+    length = strlen(ws_numbers);
+    
+    XClearArea(dpy, dicon->icon->core->window, 2, 2, 50,
+	       scr->icon_title_font->y+1, False);
+    
+    XSetForeground(dpy, numbers_gc, scr->black_pixel);
+    WMDrawString(scr->wmscreen, dicon->icon->core->window, numbers_gc,
+		 scr->icon_title_font->font, 4, 3, ws_numbers, length);
 
-   XClearArea(dpy, dicon->icon->core->window, 2, 2, 50, 
-	      scr->icon_title_font->y+1, False);
+    XSetForeground(dpy, numbers_gc, scr->white_pixel);
+    WMDrawString(scr->wmscreen, dicon->icon->core->window, numbers_gc,
+		 scr->icon_title_font->font, 3, 2, ws_numbers, length);
 
-#ifndef I18N_MB
-   XSetFont(dpy, numbers_gc, scr->icon_title_font->font->fid);
-   XSetForeground(dpy, numbers_gc, scr->black_pixel);
-   XDrawString(dpy, dicon->icon->core->window,
-               numbers_gc, 4, scr->icon_title_font->y+3, ws_numbers, length);
-   XSetForeground(dpy, numbers_gc, scr->white_pixel);
-   XDrawString(dpy, dicon->icon->core->window,
-               numbers_gc, 3, scr->icon_title_font->y+2, ws_numbers, length);
-#else
-   XSetForeground(dpy, numbers_gc, scr->black_pixel);
-   XmbDrawString(dpy, dicon->icon->core->window,
-                 scr->icon_title_font->font, numbers_gc, 4,
-                 scr->icon_title_font->y+3, ws_numbers, length);
-   XSetForeground(dpy, numbers_gc, scr->white_pixel);
-   XmbDrawString(dpy, dicon->icon->core->window,
-                 scr->icon_title_font->font, numbers_gc, 3,
-                 scr->icon_title_font->y+2, ws_numbers, length);
-#endif /* I18N_MB */
-
-   XFreeGC(dpy, numbers_gc);
-   free(ws_numbers);
+    XFreeGC(dpy, numbers_gc);
+    free(ws_numbers);
 }
 #endif /* WS_INDICATOR */
 
@@ -348,11 +407,26 @@ wAppIconPaint(WAppIcon *aicon)
 	&& aicon->command!=NULL) {
 	XSetClipMask(dpy, scr->copy_gc, scr->dock_dots->mask);
 	XSetClipOrigin(dpy, scr->copy_gc, 0, 0);
-	XCopyArea(dpy, scr->dock_dots->image, aicon->icon->core->window, 
+	XCopyArea(dpy, scr->dock_dots->image, aicon->icon->core->window,
 		  scr->copy_gc, 0, 0, scr->dock_dots->width,
 		  scr->dock_dots->height, 0, 0);
     }
 
+#ifdef HIDDENDOT
+    {
+        WApplication *wapp;
+	wapp = wApplicationOf(aicon->main_window);
+	if (wapp && wapp->flags.hidden) {
+            XSetClipMask(dpy, scr->copy_gc, scr->dock_dots->mask);
+            XSetClipOrigin(dpy, scr->copy_gc, 0, 0);
+            XCopyArea(dpy, scr->dock_dots->image,
+                      aicon->icon->core->window,
+                      scr->copy_gc, 0, 0, 7,
+                      scr->dock_dots->height, 0, 0);
+        }
+    }
+#endif /* HIDDENDOT */
+
 #ifdef NEWAPPICON
     if (!wPreferences.strict_ns && aicon->icon->owner!=NULL) {
 	int active=0;
@@ -377,9 +451,12 @@ wAppIconPaint(WAppIcon *aicon)
     }
 #endif /* NEWAPPICON */
 
+    if (aicon->omnipresent)
+        drawCorner(aicon->icon);
+
     XSetClipMask(dpy, scr->copy_gc, None);
     if (aicon->launching) {
-	XFillRectangle(dpy, aicon->icon->core->window, scr->stipple_gc, 
+	XFillRectangle(dpy, aicon->icon->core->window, scr->stipple_gc,
 		       0, 0, wPreferences.icon_size, wPreferences.icon_size);
     }
 }
@@ -390,23 +467,23 @@ wAppIconPaint(WAppIcon *aicon)
 
 #ifdef REDUCE_APPICONS
 unsigned int
-wAppIconReduceAppCount(WApplication *wapp) 
+wAppIconReduceAppCount(WApplication *wapp)
 {
     WAppIconAppList *applist;
 
     if (wapp == NULL)
 	return 0;
 
-    if (wapp->app_icon == NULL) 
+    if (wapp->app_icon == NULL)
 	return 0;
 
-    /* If given a main window, check the applist 
-     * and remove the if it exists 
+    /* If given a main window, check the applist
+     * and remove the if it exists
      */
 	applist = wapp->app_icon->applist;
 	while (applist != NULL) {
 	        if (applist->wapp == wapp) {
-		    /* If this app owns the appicon, change the appicon's 
+		    /* If this app owns the appicon, change the appicon's
 		     * owner to the next app in the list or NULL
 		     */
 	    if (wapp->app_icon->icon->owner == applist->wapp->main_window_desc)
@@ -446,7 +523,7 @@ static void
 hideCallback(WMenu *menu, WMenuEntry *entry)
 {
     WApplication *wapp = (WApplication*)entry->clientdata;
-    
+
     if (wapp->flags.hidden) {
 	wWorkspaceChange(menu->menu->screen_ptr, wapp->last_workspace);
 	wUnhideApplication(wapp, False, False);
@@ -460,7 +537,7 @@ static void
 unhideHereCallback(WMenu *menu, WMenuEntry *entry)
 {
     WApplication *wapp = (WApplication*)entry->clientdata;
-    
+
     wUnhideApplication(wapp, False, True);
 }
 
@@ -479,11 +556,11 @@ setIconCallback(WMenu *menu, WMenuEntry *entry)
 	return;
     icon->editing = 1;
     scr = icon->icon->core->screen_ptr;
-    
+
     wretain(icon);
 
-    result = wIconChooserDialog(scr, &file);    
-    
+    result = wIconChooserDialog(scr, &file, icon->wm_instance, icon->wm_class);
+
     if (result && !icon->destroyed) {
 	if (file[0]==0) {
 	    free(file);
@@ -491,7 +568,8 @@ setIconCallback(WMenu *menu, WMenuEntry *entry)
 	}
 	if (!wIconChangeImageFile(icon->icon, file)) {
 	    wMessageDialog(scr, _("Error"),
-			   _("Could not open specified icon file"), WD_ERROR);
+			   _("Could not open specified icon file"),
+			   _("OK"), NULL, NULL);
 	} else {
 	    wDefaultChangeIcon(scr, icon->wm_instance, icon->wm_class, file);
 	    wAppIconPaint(icon);
@@ -508,17 +586,32 @@ static void
 killCallback(WMenu *menu, WMenuEntry *entry)
 {
     WApplication *wapp = (WApplication*)entry->clientdata;
+    char *buffer;
+
+    if (!WCHECK_STATE(WSTATE_NORMAL))
+	return;
+
+    WCHANGE_STATE(WSTATE_MODAL);
+
     assert(entry->clientdata!=NULL);
 
+    buffer = wstrappend(wapp->app_icon ? wapp->app_icon->wm_class : NULL,
+			_(" will be forcibly closed.\n"
+			  "Any unsaved changes will be lost.\n"
+			  "Please confirm."));
+
     wretain(wapp->main_window_desc);
     if (wPreferences.dont_confirm_kill
 	|| wMessageDialog(menu->frame->screen_ptr, _("Kill Application"),
-		       _("This will kill the application.\nAny unsaved changes will be lost.\nPlease confirm."),
-		       WD_YESNO)==WDB_YES) {
+			  buffer, _("Yes"), _("No"), NULL)==WAPRDefault) {
 	if (!wapp->main_window_desc->flags.destroyed)
 	    wClientKill(wapp->main_window_desc);
     }
     wrelease(wapp->main_window_desc);
+
+    free(buffer);
+
+    WCHANGE_STATE(WSTATE_NORMAL);
 }
 
 
@@ -526,13 +619,13 @@ static WMenu*
 createApplicationMenu(WScreen *scr)
 {
     WMenu *menu;
-    
+
     menu = wMenuCreate(scr, NULL, False);
     wMenuAddCallback(menu, _("Unhide Here"), unhideHereCallback, NULL);
-    wMenuAddCallback(menu, _("(Un)Hide"), hideCallback, NULL);
+    wMenuAddCallback(menu, _("Hide"), hideCallback, NULL);
     wMenuAddCallback(menu, _("Set Icon..."), setIconCallback, NULL);
     wMenuAddCallback(menu, _("Kill"), killCallback, NULL);
-    
+
     return menu;
 }
 
@@ -543,15 +636,23 @@ openApplicationMenu(WApplication *wapp, int x, int y)
     WMenu *menu;
     WScreen *scr = wapp->main_window_desc->screen_ptr;
     int i;
-    
+
     if (!scr->icon_menu) {
 	scr->icon_menu = createApplicationMenu(scr);
+	free(scr->icon_menu->entries[1]->text);
     }
 
     menu = scr->icon_menu;
-    if (!menu->flags.realized)
-	wMenuRealize(menu);
-    
+
+    if (wapp->flags.hidden) {
+	menu->entries[1]->text = _("Unhide");
+    } else {
+	menu->entries[1]->text = _("Hide");
+    }
+
+    menu->flags.realized = 0;
+    wMenuRealize(menu);
+
     x -= menu->frame->core->width/2;
     if (x + menu->frame->core->width > scr->scr_width)
 	x = scr->scr_width - menu->frame->core->width;
@@ -568,18 +669,20 @@ openApplicationMenu(WApplication *wapp, int x, int y)
 
 /******************************************************************/
 
-static void 
+static void
 iconExpose(WObjDescriptor *desc, XEvent *event)
 {
     wAppIconPaint(desc->parent);
 }
 
+
 static void
 iconDblClick(WObjDescriptor *desc, XEvent *event)
 {
     WAppIcon *aicon = desc->parent;
     WApplication *wapp;
     WScreen *scr = aicon->icon->core->screen_ptr;
+    int unhideHere;
 
 #ifndef REDUCE_APPICONS
     assert(aicon->icon->owner!=NULL);
@@ -588,7 +691,7 @@ iconDblClick(WObjDescriptor *desc, XEvent *event)
        fprintf(stderr, "Double-click disabled: missing main window.\n");
        return;
     }
-#endif 
+#endif
 
     wapp = wApplicationOf(aicon->icon->owner->main_window);
 #ifdef DEBUG0
@@ -603,22 +706,22 @@ iconDblClick(WObjDescriptor *desc, XEvent *event)
        return;
     }
 #endif /* REDUCE_APPICONS */
-    /* aproveita que ta aqui pra atualizar */
-    wapp->main_window = aicon->icon->owner->main_window;
-    
+
+    unhideHere = (event->xbutton.state & ShiftMask);
+
     /* go to the last workspace that the user worked on the app */
-    if (!(event->xbutton.state & ShiftMask))
+    if (!unhideHere && wapp->last_workspace != scr->current_workspace)
 	wWorkspaceChange(scr, wapp->last_workspace);
 
-    wUnhideApplication(wapp, event->xbutton.button==Button2,
-		       event->xbutton.state & ShiftMask);
+    wUnhideApplication(wapp, event->xbutton.button==Button2, unhideHere);
+
     if (event->xbutton.state & MOD_MASK) {
 	wHideOtherApplications(aicon->icon->owner);
     }
 }
 
 
-void 
+void
 appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 {
     WAppIcon *aicon = desc->parent;
@@ -633,25 +736,26 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
     WWorkspace *workspace = scr->workspaces[scr->current_workspace];
     int shad_x = 0, shad_y = 0, docking=0, dockable, collapsed = 0;
     int ix, iy;
+    int clickButton = event->xbutton.button;
     Pixmap ghost = None;
+    Window wins[2];
 
+    if (aicon->editing || WCHECK_STATE(WSTATE_MODAL))
+	return;
 
-    if ((desc->click_timestamp>0) &&
-	(event->xbutton.time-desc->click_timestamp <= wPreferences.dblclick_time)) {
-	if (event->xbutton.button == desc->click_button) {
- 	    desc->click_timestamp = -1;
- 	    iconDblClick(desc, event);
- 	}
+    if (IsDoubleClick(scr, event)) {
+	iconDblClick(desc, event);
 	return;
     }
-    desc->click_timestamp = event->xbutton.time;
-    desc->click_button = event->xbutton.button;
 
     if (event->xbutton.button == Button3) {
 	WObjDescriptor *desc;
 	WApplication *wapp = wApplicationOf(aicon->icon->owner->main_window);
 
-	openApplicationMenu(wapp, event->xbutton.x_root, 
+	if (!wapp)
+	    return;
+
+	openApplicationMenu(wapp, event->xbutton.x_root,
 			    event->xbutton.y_root);
 
 	/* allow drag select of menu */
@@ -669,7 +773,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
     else
 	wRaiseFrame(icon->core);
 
-    
+
     if (XGrabPointer(dpy, icon->core->window, True, ButtonMotionMask
 		     |ButtonReleaseMask|ButtonPressMask, GrabModeAsync,
 		     GrabModeAsync, None, None, CurrentTime) !=GrabSuccess) {
@@ -681,6 +785,18 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
     else
       dockable = canBeDocked(icon->owner);
 
+    wins[0] = icon->core->window;
+    wins[1] = scr->dock_shadow;
+    XRestackWindows(dpy, wins, 2);
+    if (superfluous) {
+        if (icon->pixmap!=None)
+            ghost = MakeGhostIcon(scr, icon->pixmap);
+        else
+            ghost = MakeGhostIcon(scr, icon->core->window);
+        XSetWindowBackgroundPixmap(dpy, scr->dock_shadow,
+                                   ghost);
+        XClearWindow(dpy, scr->dock_shadow);
+    }
 
     while (!done) {
 	WMMaskEvent(dpy, PointerMotionMask|ButtonReleaseMask|ButtonPressMask
@@ -695,7 +811,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 		if (abs(dx-ev.xmotion.x)>=MOVE_THRESHOLD
 		    || abs(dy-ev.xmotion.y)>=MOVE_THRESHOLD) {
 		    XChangeActivePointerGrab(dpy, ButtonMotionMask
-					    |ButtonReleaseMask|ButtonPressMask, 
+					    |ButtonReleaseMask|ButtonPressMask,
 					     wCursor[WCUR_MOVE], CurrentTime);
 		    grabbed=1;
 		} else {
@@ -722,24 +838,13 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
                         wDockShowIcons(scr->dock);
                     }
 
+                    if (scr->dock->auto_raise_lower)
+                        wDockRaise(scr->dock);
+
                     scr->last_dock = scr->dock;
 
 		    XMoveWindow(dpy, scr->dock_shadow, shad_x, shad_y);
 		    if (!docking) {
-			Window wins[2];
-
-			wins[0] = icon->core->window;
-			wins[1] = scr->dock_shadow;
-			XRestackWindows(dpy, wins, 2);
-                        if (superfluous) {
-			if (icon->pixmap!=None)
-			    ghost = MakeGhostIcon(scr, icon->pixmap);
-			else
-			    ghost = MakeGhostIcon(scr, icon->core->window);
-			XSetWindowBackgroundPixmap(dpy, scr->dock_shadow,
-						   ghost);
-			XClearWindow(dpy, scr->dock_shadow);
-                        }
                         XMapWindow(dpy, scr->dock_shadow);
 		    }
 		    docking = 1;
@@ -748,7 +853,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
                                          &ix, &iy, False)) {
                     shad_x = workspace->clip->x_pos + ix*wPreferences.icon_size;
 		    shad_y = workspace->clip->y_pos + iy*wPreferences.icon_size;
-		    
+
                     if (scr->last_dock != workspace->clip && collapsed) {
                         scr->last_dock->collapsed = 1;
                         wDockHideIcons(scr->last_dock);
@@ -759,24 +864,13 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
                         wDockShowIcons(workspace->clip);
                     }
 
+                    if (workspace->clip->auto_raise_lower)
+                        wDockRaise(workspace->clip);
+
                     scr->last_dock = workspace->clip;
 
 		    XMoveWindow(dpy, scr->dock_shadow, shad_x, shad_y);
 		    if (!docking) {
-			Window wins[2];
-
-			wins[0] = icon->core->window;
-			wins[1] = scr->dock_shadow;
-			XRestackWindows(dpy, wins, 2);
-                        if (superfluous) {
-			if (icon->pixmap!=None)
-			    ghost = MakeGhostIcon(scr, icon->pixmap);
-			else
-			    ghost = MakeGhostIcon(scr, icon->core->window);
-			XSetWindowBackgroundPixmap(dpy, scr->dock_shadow,
-						   ghost);
-			XClearWindow(dpy, scr->dock_shadow);
-                        }
 			XMapWindow(dpy, scr->dock_shadow);
 		    }
                     docking = 1;
@@ -792,7 +886,7 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 	    break;
 
 	 case ButtonRelease:
-	    if (ev.xbutton.button != desc->click_button)
+	    if (ev.xbutton.button != clickButton)
 		break;
 	    XUngrabPointer(dpy, CurrentTime);
 
@@ -806,17 +900,23 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
                 if (scr->last_dock->auto_collapse) {
                     collapsed = 0;
                 }
+                if (workspace->clip &&
+                    workspace->clip != scr->last_dock &&
+                    workspace->clip->auto_raise_lower)
+                    wDockLower(workspace->clip);
 
 		if (!docked) {
 		    /* If icon could not be docked, slide it back to the old
 		     * position */
-		    SlideWindow(icon->core->window, x, y, aicon->x_pos, 
+		    SlideWindow(icon->core->window, x, y, aicon->x_pos,
 				aicon->y_pos);
 		}
             } else {
 		XMoveWindow(dpy, icon->core->window, x, y);
 		aicon->x_pos = x;
 		aicon->y_pos = y;
+                if (workspace->clip && workspace->clip->auto_raise_lower)
+                    wDockLower(workspace->clip);
             }
 	    if (collapsed) {
 		scr->last_dock->collapsed = 1;
@@ -828,6 +928,10 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 		    XFreePixmap(dpy, ghost);
 		XSetWindowBackground(dpy, scr->dock_shadow, scr->white_pixel);
 	    }
+
+	    if (wPreferences.auto_arrange_icons)
+		wArrangeIcons(scr, True);
+
 	    done = 1;
 	    break;
 	}
@@ -835,5 +939,5 @@ appIconMouseDown(WObjDescriptor *desc, XEvent *event)
 #ifdef DEBUG
     puts("End icon move");
 #endif
-    
+
 }
