@@ -1,5 +1,5 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -43,6 +43,9 @@
 #include "stacking.h"
 #include "appicon.h"
 #include "appmenu.h"
+#ifdef KWM_HINTS
+#include "kwm.h"
+#endif
 
 /****** Global Variables ******/
 
@@ -58,6 +61,11 @@ extern Atom _XA_WINDOWMAKER_MENU;
 extern Atom _XA_GNUSTEP_WM_ATTR;
 extern Atom _XA_GNUSTEP_WM_RESIZEBAR;
 
+#ifdef SHAPE
+extern Bool wShapeSupported;
+#endif
+
+
 /*
  *--------------------------------------------------------------------
  * wClientRestore--
@@ -68,26 +76,26 @@ extern Atom _XA_GNUSTEP_WM_RESIZEBAR;
 void
 wClientRestore(WWindow *wwin)
 {
+#if 0
     int gx, gy;
-    
+
     wClientGetGravityOffsets(wwin, &gx, &gy);
     /* set the positio of the frame on screen */
-    wwin->frame_x -= gx * (wwin->old_border_width - FRAME_BORDER_WIDTH);
-    wwin->frame_y -= gy * (wwin->old_border_width - FRAME_BORDER_WIDTH);
+    wwin->frame_x -= gx * FRAME_BORDER_WIDTH;
+    wwin->frame_y -= gy * FRAME_BORDER_WIDTH;
     /* if gravity is to the south, account for the border sizes */
     if (gy > 0)
 	wwin->frame_y += (wwin->frame->top_width + wwin->frame->bottom_width);
-
-    XUnmapWindow(dpy, wwin->client_win);
+#endif
     XSetWindowBorderWidth(dpy, wwin->client_win, wwin->old_border_width);    
     XReparentWindow(dpy, wwin->client_win, wwin->screen_ptr->root_win,
 		    wwin->frame_x, wwin->frame_y);
-    if (wwin->flags.mapped)
-      XMapWindow(dpy, wwin->client_win);
     
     /* don't let the window get iconified after restart */
+    /*
     if (wwin->flags.shaded)
-      wClientSetState(wwin, NormalState, None);
+	wClientSetState(wwin, NormalState, None);
+     */
 }
 
 
@@ -105,7 +113,7 @@ wClientRestore(WWindow *wwin)
 void
 wClientSetState(WWindow *wwin, int state, Window icon_win)
 {
-    unsigned long data[2];
+    CARD32 data[2];
     
     wwin->state = state;
 
@@ -186,7 +194,7 @@ wClientConfigure(WWindow *wwin, XConfigureRequestEvent *xcre)
 	return;
     }
 #ifdef SHAPE
-    {
+    if (wShapeSupported) {
 	int junk;
 	unsigned int ujunk;
 	int b_shaped;
@@ -232,7 +240,8 @@ wClientConfigure(WWindow *wwin, XConfigureRequestEvent *xcre)
             nx = wwin->frame_x;
 
         if (xcre->value_mask & CWY)
-            ny = xcre->y - FRAME_BORDER_WIDTH - ((ofs_y < 0) ? 0 : wwin->frame->top_width);
+            ny = xcre->y - ((ofs_y < 0) ? 0 : wwin->frame->top_width)
+		- FRAME_BORDER_WIDTH;
         else
             ny = wwin->frame_y;
 
@@ -270,7 +279,7 @@ wClientSendProtocol(WWindow *wwin, Atom protocol, Time time)
     event.xclient.data.l[2] = 0;
     event.xclient.data.l[3] = 0;
     XSendEvent(dpy, wwin->client_win, False, NoEventMask, &event);
-    XFlush(dpy);
+    XSync(dpy, False);
 }
 
 
@@ -279,6 +288,7 @@ void
 wClientKill(WWindow *wwin)
 {
     XKillClient(dpy, wwin->client_win);
+
     XFlush(dpy);
 }
 
@@ -306,18 +316,31 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
     switch (event->atom) {
      case XA_WM_NAME:
 	/* window title was changed */
-
+	wwin->flags.wm_name_changed = 1;
 	if (wwin->frame) {
 	    if (!wFetchName(dpy, wwin->client_win, &tmp)) {
 		/* the hint was removed */
 		tmp = wstrdup(DEF_WINDOW_TITLE);
+#ifdef KWM_HINTS
+		wKWMSendEventMessage(wwin, WKWMChangedClient);
+#endif
 	    }
-	    wFrameWindowChangeTitle(wwin->frame, tmp);
-	    UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE);
+            if (wFrameWindowChangeTitle(wwin->frame, tmp)) {
+                /* only update the menu if the title has actually changed */
+                UpdateSwitchMenu(wwin->screen_ptr, wwin, ACTION_CHANGE);
+#ifdef KWM_HINTS
+		wKWMSendEventMessage(wwin, WKWMChangedClient);
+#endif
+            }
+	    if (tmp)
+		free(tmp);
 	}
 	break;
 
      case XA_WM_ICON_NAME:
+#ifdef KWM_HINTS
+	wKWMSendEventMessage(wwin, WKWMChangedClient);
+#endif
 	if (!wwin->icon) 
 	    break;
 	else {
@@ -334,13 +357,17 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	    WApplication *wapp = wApplicationOf(wwin->main_window);
 	    char **argv;
 	    int argc;
-	    
+
 	    if (wapp && wapp->app_icon) {
 		if (wapp->app_icon->command!=NULL)
-		  free(wapp->app_icon->command);
+		    free(wapp->app_icon->command);
 
 		if (XGetCommand(dpy, wwin->main_window, &argv, &argc)) {
-		    wapp->app_icon->command = FlattenStringList(argv, argc);
+		    if (argc > 0 && argv != NULL)
+			wapp->app_icon->command = FlattenStringList(argv,argc);
+		    if (argv) {
+			XFreeStringList(argv);
+		    }
 		}
 	    }
 	}
@@ -465,13 +492,16 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		|| (wwin->wm_hints->flags & IconWindowHint)) {
 		WApplication *wapp;
 
-		if (wwin->flags.miniaturized) {
+		if (wwin->flags.miniaturized && wwin->icon) {
 		    wIconUpdate(wwin->icon);
 		}
 		wapp = wApplicationOf(wwin->main_window);
 		if (wapp && wapp->app_icon) {
 		    wIconUpdate(wapp->app_icon->icon);
 		}
+#ifdef KWM_HINTS
+		wKWMSendEventMessage(wwin, WKWMIconChange);
+#endif
 	    }
 
 	    if (wwin->wm_hints->flags & UrgencyHint)
@@ -485,9 +515,16 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	
      case XA_WM_NORMAL_HINTS:
 	/* normal (geometry) hints */
-	
-	XGetWindowAttributes(dpy, wwin->client_win, &attribs);
-	GetNormalHints(wwin, &attribs, False);
+	{
+	    int foo;
+	    unsigned bar;
+
+	    XGetWindowAttributes(dpy, wwin->client_win, &attribs);
+	    wClientGetNormalHints(wwin, &attribs, False, &foo, &foo,
+				  &bar, &bar);
+	    /* TODO: should we check for consistency of the current
+	     * size against the new geometry hints? */
+	}
 	break;
 	
      case XA_WM_TRANSIENT_FOR:
@@ -523,15 +560,15 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		}
 		wwin->transient_for = new_owner;
 		if (new_owner==None) {
-		    if (wwin->window_flags.no_miniaturizable) {
-			wwin->window_flags.no_miniaturizable = 0;
-			wwin->window_flags.no_miniaturize_button = 0;
+		    if (WFLAGP(wwin, no_miniaturizable)) {
+			WSETUFLAG(wwin, no_miniaturizable, 0);
+			WSETUFLAG(wwin, no_miniaturize_button, 0);
 			if (wwin->frame)
 			    wWindowConfigureBorders(wwin);
 		    }
-		} else if (!wwin->window_flags.no_miniaturizable) {
-		    wwin->window_flags.no_miniaturizable = 1;
-		    wwin->window_flags.no_miniaturize_button = 1;
+		} else if (!WFLAGP(wwin, no_miniaturizable)) {
+		    WSETUFLAG(wwin, no_miniaturizable, 1);
+		    WSETUFLAG(wwin, no_miniaturize_button, 1);
 		    if (wwin->frame)
 			wWindowConfigureBorders(wwin);
 		}
@@ -541,20 +578,19 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 	
      default:
 	if (event->atom==_XA_WM_PROTOCOLS) {
+
 	    PropGetProtocols(wwin->client_win, &wwin->protocols);
-	    if (wwin->protocols.DELETE_WINDOW) {
-		wwin->window_flags.kill_close = 0;
-	    } else {
-		wwin->window_flags.kill_close = 1;		
-	    }
+
+	    WSETUFLAG(wwin, kill_close, !wwin->protocols.DELETE_WINDOW);
+
 	    if (wwin->frame)
 		wWindowUpdateButtonImages(wwin);
 	    
 	} else if (event->atom==_XA_WM_COLORMAP_WINDOWS) {
 	    
 	    GetColormapWindows(wwin);
-	    ReInstallActiveColormap();
-	    
+	    wColormapInstallForWindow(wwin->screen_ptr, wwin);
+
 	} else if (event->atom==_XA_WINDOWMAKER_MENU) {
 	    WApplication *wapp;
 
@@ -571,35 +607,43 @@ wClientCheckProperty(WWindow *wwin, XPropertyEvent *event)
 		wSetFocusTo(wwin->screen_ptr, 
 			    wwin->screen_ptr->focused_window);
 	    }
-	}
-#if 0
-	else if (event->atom==_XA_GNUSTEP_WM_ATTR) {
+	} else if (event->atom==_XA_GNUSTEP_WM_ATTR) {
 	    GNUstepWMAttributes *attr;
-	    
+
 	    PropGetGNUstepWMAttr(wwin->client_win, &attr);
 	    
+	    wWindowUpdateGNUstepAttr(wwin, attr);
+	    
+	    XFree(attr);
+	} else {
+#ifdef KWM_HINTS
+	    Bool done;
+
+	    done = wKWMCheckClientHintChange(wwin, event);
+#endif /* KWM_HINTS */
 	}
-#endif
     }
 }
 
 
 /*
  *---------------------------------------------------------------------- 
- * GetNormalHints--
+ * wClientGetNormalHints--
  * 	Get size (normal) hints and a default geometry for the client 
- * window. The hints are also checked for inconsistency.
+ * window. The hints are also checked for inconsistency. If geometry is
+ * True, the returned data will account for client specified initial
+ * geometry.
  * 
  * Side effects:
- * 	normal_hints is filled with valid data and geometry information 
- * about the client is also changed.
+ * 	normal_hints is filled with valid data.
  *---------------------------------------------------------------------- 
  */
 void
-GetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, int geometry)
+wClientGetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, Bool geometry,
+		      int *x, int *y, unsigned *width, unsigned *height)
 {
-    int pre_icccm=0;
-    
+    int pre_icccm = 0;		       /* not used */
+
     /* find a position for the window */
     if (!wwin->normal_hints)
         wwin->normal_hints = XAllocSizeHints();
@@ -607,11 +651,11 @@ GetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, int geometry)
     if (!PropGetNormalHints(wwin->client_win, wwin->normal_hints, &pre_icccm)) {
 	wwin->normal_hints->flags = 0;
     }
-    wwin->client.x = wattribs->x;
-    wwin->client.y = wattribs->y;
+    *x = wattribs->x;
+    *y = wattribs->y;
     
-    wwin->client.width = wattribs->width;
-    wwin->client.height = wattribs->height;
+    *width = wattribs->width;
+    *height = wattribs->height;
 
     if (!(wwin->normal_hints->flags & PWinGravity)) {
 	wwin->normal_hints->win_gravity = NorthWestGravity;
@@ -628,7 +672,14 @@ GetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, int geometry)
 	wwin->normal_hints->max_width = wwin->screen_ptr->scr_width*2;
 	wwin->normal_hints->max_height = wwin->screen_ptr->scr_height*2;
     }
-    
+
+    /* some buggy apps set weird hints.. */
+    if (wwin->normal_hints->max_width < wwin->normal_hints->min_width)
+	wwin->normal_hints->max_width = wwin->normal_hints->min_width;
+
+    if (wwin->normal_hints->max_height < wwin->normal_hints->min_height)
+	wwin->normal_hints->max_height = wwin->normal_hints->min_height;
+
     if (!(wwin->normal_hints->flags & PResizeInc)) {
 	wwin->normal_hints->width_inc = 1;
 	wwin->normal_hints->height_inc = 1;
@@ -664,18 +715,17 @@ GetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, int geometry)
 	wwin->normal_hints->min_width = wwin->normal_hints->max_width;
     }
 
-    /* pre ICCCM (old) client */
+#ifdef IGNORE_PPOSITION
+    wwin->normal_hints->flags &= ~PPosition;
+#endif
     if (pre_icccm && !wwin->screen_ptr->flags.startup && geometry) {
-#ifdef DEBUG
-	printf("PRE ICCCM\n");	
-#endif	
 	if (wwin->normal_hints->flags & (USPosition|PPosition)) {
-	    wwin->client.x = wwin->normal_hints->x;
-	    wwin->client.y = wwin->normal_hints->y;
+	    *x = wwin->normal_hints->x;
+	    *y = wwin->normal_hints->y;
 	}
 	if (wwin->normal_hints->flags & (USSize|PSize)) {
-	    wwin->client.width = wwin->normal_hints->width;
-	    wwin->client.height = wwin->normal_hints->height;
+	    *width = wwin->normal_hints->width;
+	    *height = wwin->normal_hints->height;
 	}
     }
 }
@@ -684,12 +734,19 @@ GetNormalHints(WWindow *wwin, XWindowAttributes *wattribs, int geometry)
 void 
 GetColormapWindows(WWindow *wwin)
 {
+#ifndef NO_CRASHES
     if (wwin->cmap_windows) {
 	XFree(wwin->cmap_windows);
     }
+
+    wwin->cmap_windows = NULL;
+    wwin->cmap_window_no = 0;
+
     if (!XGetWMColormapWindows(dpy, wwin->client_win, &(wwin->cmap_windows),
-			       &(wwin->cmap_window_no))) {
+			      &(wwin->cmap_window_no))
+	|| !wwin->cmap_windows) {
 	wwin->cmap_window_no = 0;
 	wwin->cmap_windows = NULL;
     }
+#endif
 }
