@@ -1,8 +1,9 @@
 /* dock.h- built-in Dock module for WindowMaker
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1998       Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,8 +44,13 @@ typedef struct WDock {
 #define EXTENDED_DOCK  WM_CLIP
     int type;
 
+    WMagicNumber auto_expand_magic;
     WMagicNumber auto_collapse_magic;
+    WMagicNumber auto_raise_magic;
+    WMagicNumber auto_lower_magic;
     unsigned int auto_collapse:1;      /* if clip auto-collapse itself */
+    unsigned int auto_raise_lower:1;   /* if clip should raise/lower when
+                                        * entered/leaved */
     unsigned int on_right_side:1;
     unsigned int collapsed:1;
     unsigned int mapped:1;
@@ -76,9 +82,9 @@ void wDockRaiseLower(WDock *dock);
 void wDockSaveState(WScreen *scr);
 
 Bool wDockAttachIcon(WDock *dock, WAppIcon *icon, int x, int y);
-int wDockSnapIcon(WDock *dock, WAppIcon *icon, int req_x, int req_y,
+Bool wDockSnapIcon(WDock *dock, WAppIcon *icon, int req_x, int req_y,
 		  int *ret_x, int *ret_y, int redocking);
-int wDockFindFreeSlot(WDock *dock, int *req_x, int *req_y);
+Bool wDockFindFreeSlot(WDock *dock, int *req_x, int *req_y);
 void wDockDetach(WDock *dock, WAppIcon *icon);
 
 void wDockTrackWindowLaunch(WDock *dock, Window window);
@@ -99,4 +105,14 @@ void wClipSaveState(WScreen *scr);
 proplist_t wClipSaveWorkspaceState(WScreen *scr, int workspace);
 WAppIcon* wClipRestoreState(WScreen *scr, proplist_t clip_state);
 
+void wClipUpdateForWorkspaceChange(WScreen *scr, int workspace);
+
+RImage *wClipMakeTile(WScreen *scr, RImage *normalTile);
+
+#define WO_FAILED          0
+#define WO_NOT_APPLICABLE  1
+#define WO_SUCCESS         2
+
+int wClipMakeIconOmnipresent(WAppIcon *aicon, int omnipresent);
+
 #endif
