@@ -1,5 +1,5 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -36,45 +36,69 @@
 #define WFF_RESIZEBAR	(1<<3)
 #define WFF_BORDER	(1<<4)
 #define WFF_SINGLE_STATE (1<<5)
+#ifdef XKB_BUTTON_HINT
+#define WFF_LANGUAGE_BUTTON	(1<<6)
+#endif
+
+
+#define WFF_IS_SHADED	(1<<16)
 
 typedef struct WFrameWindow {
     WScreen *screen_ptr;	       /* pointer to the screen structure */
-    short window_level;
-    short workspace;		       /* workspace that the window occupies */
     
     WCoreWindow *core;
     
     WCoreWindow *titlebar;	       /* the titlebar */
     WCoreWindow *left_button;	       /* miniaturize button */
+#ifdef XKB_BUTTON_HINT
+    WCoreWindow *language_button;
+#endif
     WCoreWindow *right_button;	       /* close button */
 
+    short workspace;		       /* workspace that the window occupies */
+
     short top_width;
     short bottom_width;
-    
+
+    short resizebar_corner_width;
+
     WCoreWindow *resizebar;	       /* bottom resizebar */
 
     Pixmap title_back[3];	       /* focused, unfocused, pfocused */
+    Pixmap resizebar_back[3];	       /* any, None, None */
     Pixmap lbutton_back[3];
     Pixmap rbutton_back[3];
+#ifdef XKB_BUTTON_HINT
+    Pixmap languagebutton_back[3];
+#endif
 
     WPixmap *lbutton_image;
     WPixmap *rbutton_image;
+#ifdef XKB_BUTTON_HINT
+    WPixmap *languagebutton_image;
+#endif
     
     union WTexture **title_texture;
     union WTexture **resizebar_texture;
     unsigned long *title_pixel;
     GC *title_gc;
-    WFont **font;
-
-    short resizebar_corner_width;
+    WMFont **font;
 
     char *title;		       /* window name (title) */
 
+#ifdef KEEP_XKB_LOCK_STATUS
+    int languagemode;
+    int last_languagemode;
+#endif /* KEEP_XKB_LOCK_STATUS */
+
     /* thing that uses this frame. passed as data to callbacks */
     void *child;
     
     /* callbacks */
     void (*on_click_left)(WCoreWindow *sender, void *data, XEvent *event);
+#ifdef XKB_BUTTON_HINT
+    void (*on_click_language)(WCoreWindow *sender, void *data, XEvent *event);
+#endif
     
     void (*on_click_right)(WCoreWindow *sender, void *data, XEvent *event);
     void (*on_dblclick_right)(WCoreWindow *sender, void *data, XEvent *event);
@@ -91,21 +115,38 @@ typedef struct WFrameWindow {
 	unsigned int resizebar:1;
 	unsigned int left_button:1;
 	unsigned int right_button:1;
+#ifdef XKB_BUTTON_HINT
+	unsigned int language_button:1;
+#endif
+
 	unsigned int need_texture_remake:1;
+
 	unsigned int single_texture:1;
 	
 	unsigned int hide_left_button:1;
 	unsigned int hide_right_button:1;
+#ifdef XKB_BUTTON_HINT
+	unsigned int hide_language_button:1;
+#endif
 	
 	unsigned int need_texture_change:1;
 	
 	unsigned int lbutton_dont_fit:1;
 	unsigned int rbutton_dont_fit:1;
+#ifdef XKB_BUTTON_HINT
+	unsigned int languagebutton_dont_fit:1;
+#endif
 
 	unsigned int repaint_only_titlebar:1;
 	unsigned int repaint_only_resizebar:1;
 
 	unsigned int is_client_window_frame:1;
+	
+	unsigned int incomplete_title:1;
+
+#ifdef OLWM_HINTS
+	unsigned int right_button_pushed_in:1;
+#endif
     } flags;
 } WFrameWindow;
 
@@ -115,7 +156,7 @@ wFrameWindowCreate(WScreen *scr, int wlevel, int x, int y,
 		   int width, int height, int flags,
 		   union WTexture **title_texture, 
 		   union WTexture **resize_texture,
-		   unsigned long *color, GC *gc, WFont **font);
+		   unsigned long *color, GC *gc, WMFont **font);
 
 void wFrameWindowUpdateBorders(WFrameWindow *fwin, int flags);
 
@@ -125,15 +166,22 @@ void wFrameWindowChangeState(WFrameWindow *fwin, int state);
 
 void wFrameWindowPaint(WFrameWindow *fwin);
 
-void wFrameWindowResize(WFrameWindow *fwin, int width, int height);
+void wFrameWindowConfigure(WFrameWindow *fwin, int x, int y, int width, int height);
 
-void wFrameWindowResizeInternal(WFrameWindow *fwin, int iwidth, int iheight);
+void wFrameWindowResize(WFrameWindow *fwin, int width, int height);
 
 void wFrameWindowShowButton(WFrameWindow *fwin, int flags);
 
 void wFrameWindowHideButton(WFrameWindow *fwin, int flags);
 
-void wFrameWindowChangeTitle(WFrameWindow *fwin, char *new_title);
+int wFrameWindowChangeTitle(WFrameWindow *fwin, char *new_title);
 
+#ifdef OLWM_HINTS
+void wFrameWindowUpdatePushButton(WFrameWindow *fwin, Bool pushed);
+#endif
+
+#ifdef XKB_BUTTON_HINT
+void wFrameWindowUpdateLanguageButton(WFrameWindow *fwin);
+#endif
 
 #endif
