@@ -1,5 +1,5 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -31,15 +31,19 @@ typedef void (WCallBack)(void *cdata);
 
 typedef void (WDeathHandler)(pid_t pid, unsigned int status, void *cdata);
 
-typedef void* WMagicNumber;
+void Shutdown(WShutdownMode mode);
 
 void RestoreDesktop(WScreen *scr);
 
-void Restart(char *manager);
+void Exit(int status);
+
+void Restart(char *manager, Bool abortOnFailure);
+
+void SetupEnvironment(WScreen *scr);
 
 void DispatchEvent(XEvent *event);
 
-void WipeDesktop(WScreen *scr);
+#ifndef LITE
 
 Bool wRootMenuPerformShortcut(XEvent *event);
 
@@ -49,35 +53,46 @@ void OpenRootMenu(WScreen *scr, int x, int y, int keyboard);
 
 void OpenSwitchMenu(WScreen *scr, int x, int y, int keyboard);
 
+#endif /* !LITE */
+
 void OpenWindowMenu(WWindow *wwin, int x, int y, int keyboard);
 
+void OpenMiniwindowMenu(WWindow *wwin, int x, int y);
+
 void OpenWorkspaceMenu(WScreen *scr, int x, int y);
 
 void CloseWindowMenu(WScreen *scr);
 
+#ifdef LITE
+
+#define UpdateSwitchMenu(a, b, c)
+
+#define UpdateSwitchMenuWorkspace(a, b)
+
+#else /*! LITE */
+
 void UpdateSwitchMenu(WScreen *scr, WWindow *wwin, int action);
 
 void UpdateSwitchMenuWorkspace(WScreen *scr, int workspace);
 
+#endif /* !LITE */
+
 WMagicNumber wAddDeathHandler(pid_t pid, WDeathHandler *callback, void *cdata);
 
-WMagicNumber wAddWindowSavedState(char *instance, char *class, char *command,
-                                  pid_t pid, WSavedState *state);
+void wColormapInstallForWindow(WScreen *scr, WWindow *wwin);
 
-WMagicNumber wGetWindowSavedState(Window win);
+void wColormapInstallRoot(WScreen *scr);
 
-void wDeleteWindowSavedState(WMagicNumber id);
+void wColormapUninstallRoot(WScreen *scr);
 
-
-void InstallColormap(WScreen *screen, struct WWindow *wwin);
-
-void ChangeColormap(struct WWindow *wwin, XEvent *event);
+void wColormapAllowClientInstallation(WScreen *scr, Bool starting);
 
 Pixmap LoadIcon(WScreen *scr, char *path, char *mask, int title_height);
 
 void PlaceIcon(WScreen *scr, int *x_ret, int *y_ret);
 
-void PlaceWindow(WWindow *wwin, int *x_ret, int *y_ret);
+void PlaceWindow(WWindow *wwin, int *x_ret, int *y_ret,
+                 unsigned int width, unsigned int height);
 
 #ifdef USECPP
 char *MakeCPPArgs(char *path);
@@ -85,20 +100,18 @@ char *MakeCPPArgs(char *path);
 
 char *ExpandOptions(WScreen *scr, char *cmdline);
 
-void ReInstallActiveColormap(void);
-void InstallWindowColormaps (WWindow *tmp);
-void InstallRootColormap();
-void UninstallRootColormap();
+void ExecuteShellCommand(WScreen *scr, char *command);
 
+Bool IsDoubleClick(WScreen *scr, XEvent *event);
 
 WWindow *NextFocusWindow(WScreen *scr);
 WWindow *PrevFocusWindow(WScreen *scr);
 
 void SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y);
 
-char *ShrinkString(WFont *font, char *string, int width);
+char *ShrinkString(WMFont *font, char *string, int width);
 
-char *FindImage(char **paths, char *file);
+char *FindImage(char *paths, char *file);
 
 RImage*wGetImageForWindowName(WScreen *scr, char *winstance, char *wclass);
 
@@ -111,21 +124,25 @@ char *FlattenStringList(char **list, int count);
 void ParseWindowName(proplist_t value, char **winstance, char **wclass,
                      char *where);
 
+void SendHelperMessage(WScreen *scr, char type, int workspace, char *msg);
+
 char *GetShortcutString(char *text);
 
+char *EscapeWM_CLASS(char *name, char *class);
+
+void UnescapeWM_CLASS(char *str, char **name, char **class);
+
 #ifdef NUMLOCK_HACK
 void wHackedGrabKey(int keycode, unsigned int modifiers,
 		    Window grab_window, Bool owner_events, int pointer_mode,
 		    int keyboard_mode);
+#endif
 
 void wHackedGrabButton(unsigned int button, unsigned int modifiers, 
 		       Window grab_window, Bool owner_events, 
 		       unsigned int event_mask, int pointer_mode, 
 		       int keyboard_mode, Window confine_to, Cursor cursor);
 
-void wHackedUngrabButton(unsigned int button, unsigned int modifiers,
-			 Window grab_window);
-#endif
 
 /* this function is in dock.c */
 void ParseCommand(char *command, char ***argv, int *argc);
@@ -134,16 +151,11 @@ void ParseCommand(char *command, char ***argv, int *argc);
 void wGetGeometryWindowSize(WScreen *scr, unsigned int *width, 
 			    unsigned int *height);
 
+void ExecExitScript();
+
 /****** I18N Wrapper for XFetchName,XGetIconName ******/
 
 Bool wFetchName(Display *dpy, Window win, char **winname);
 Bool wGetIconName(Display *dpy, Window win, char **iconname);
 
-#ifdef I18N_MB
-#define MyTextWidth(x,y,z) XmbTextEscapement(x,y,z)
-#else
-#define MyTextWidth(x,y,z) XTextWidth(x,y,z)  
-#endif
-
-
 #endif
