@@ -1,5 +1,5 @@
 /*
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -39,16 +39,19 @@
  */
 #include <X11/Xlocale.h>
 
+#define MAINFILE
+
 #include "WindowMaker.h"
 #include "window.h"
 #include "funcs.h"
 #include "menu.h"
 #include "keybind.h"
 #include "xmodifier.h"
+#include "defaults.h"
+#include "session.h"
 
 #include <proplist.h>
 
-
 /****** Global Variables ******/
 
 /* general info */
@@ -57,11 +60,12 @@ Display	*dpy;
 
 char *ProgName;
 
+unsigned int ValidModMask = 0xff;
+
 /* locale to use. NULL==POSIX or C */
 char *Locale=NULL;
 
-WScreen *wScreen=NULL;
-
+int wScreenCount=0;
 
 WPreferences wPreferences;
 
@@ -71,6 +75,11 @@ proplist_t wAttributeDomainName;
 
 WShortKey wKeyBindings[WKBD_LAST];
 
+/* defaults domains */
+WDDomain *WDWindowMaker = NULL;
+WDDomain *WDRootMenu = NULL;
+WDDomain *WDWindowAttributes = NULL;
+
 
 /* XContexts */
 XContext wWinContext;
@@ -86,28 +95,25 @@ Atom _XA_WM_DELETE_WINDOW;
 Atom _XA_WM_SAVE_YOURSELF;
 Atom _XA_WM_CLIENT_LEADER;
 Atom _XA_WM_COLORMAP_WINDOWS;
+Atom _XA_WM_COLORMAP_NOTIFY;
 
 Atom _XA_GNUSTEP_WM_ATTR;
-Atom _XA_WINDOWMAKER_WM_MINIATURIZE_WINDOW;
+Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
 Atom _XA_GNUSTEP_WM_RESIZEBAR;
 
-#ifdef MWM_HINTS
-/* MWM support */
-Atom _XA_MOTIF_WM_HINTS;
-#endif
-
 Atom _XA_WINDOWMAKER_MENU;
 Atom _XA_WINDOWMAKER_WM_PROTOCOLS;
 Atom _XA_WINDOWMAKER_STATE;
 
 Atom _XA_WINDOWMAKER_WM_FUNCTION;
+Atom _XA_WINDOWMAKER_NOTICEBOARD;
+Atom _XA_WINDOWMAKER_COMMAND;
 
 #ifdef OFFIX_DND
 Atom _XA_DND_PROTOCOL;
 Atom _XA_DND_SELECTION;
 #endif
 
-
 /* cursors */
 Cursor wCursor[WCUR_LAST];
 
@@ -117,13 +123,17 @@ Time LastTimestamp;
 Time LastFocusChange;
 
 #ifdef SHAPE
-int ShapeEventBase;
+Bool wShapeSupported;
+int wShapeEventBase;
 #endif
 
+#ifdef KEEP_XKB_LOCK_STATUS
+Bool wXkbSupported;
+int wXkbEventBase;
+#endif
 
 /* special flags */
-char WRestartASAP = 0;
-char WExitASAP = 0;
+char WProgramState = WSTATE_NORMAL;
 char WDelayedActionSet = 0;
 
 /* temporary stuff */
@@ -132,17 +142,31 @@ int wVisualID = -1;
 
 /******** End Global Variables *****/
 
+static char *DisplayName = NULL;
 
 static char **Arguments;
+
 static int ArgCount;
 
 extern void EventLoop();
 extern void StartUp();
 
 
+void
+Exit(int status)
+{
+#ifdef XSMP_ENABLED
+    wSessionDisconnectManager();
+#endif
+    if (dpy)
+	XCloseDisplay(dpy);
+
+    exit(status);
+}
+
 
 void
-Restart(char *manager)
+Restart(char *manager, Bool abortOnFailure)
 {
     char *prog=NULL;
     char *argv[MAX_RESTART_ARGS];
@@ -157,19 +181,49 @@ Restart(char *manager)
 	    }
 	}
     }
-
-    XCloseDisplay(dpy);
-    if (!prog)
-      execvp(Arguments[0], Arguments);
-    else {
-	execvp(prog, argv);
-	/* fallback */
-	execv(Arguments[0], Arguments);
+    if (dpy) {
+#ifdef XSMP_ENABLED
+	wSessionDisconnectManager();
+#endif
+	XCloseDisplay(dpy);
+	dpy = NULL;
     }
-    wfatal(_("Restart failed!!!"));
-    exit(-1);
+    if (!prog) {
+        execvp(Arguments[0], Arguments);
+        wfatal(_("failed to restart Window Maker."));
+    } else {
+        execvp(prog, argv);
+        wsyserror(_("could not exec %s"), prog);
+    }
+    if (abortOnFailure)
+        exit(-1);
 }
 
+
+
+void
+SetupEnvironment(WScreen *scr)
+{
+    char *tmp, *ptr;
+    char buf[16];
+
+    if (wScreenCount > 1) {
+    	tmp = wmalloc(strlen(DisplayName)+64);
+    	sprintf(tmp, "DISPLAY=%s", XDisplayName(DisplayName));
+    	ptr = strchr(strchr(tmp, ':'), '.');
+    	if (ptr)
+	    *ptr = 0;
+    	sprintf(buf, ".%i", scr->screen);
+    	strcat(tmp, buf);
+    	putenv(tmp);
+    }
+    tmp = wmalloc(60);
+    sprintf(tmp, "WRASTER_COLOR_RESOLUTION%i=%i", scr->screen, 
+		scr->rcontext->attribs->colors_per_channel);
+    putenv(tmp);
+}
+
+
 /*
  *---------------------------------------------------------------------
  * wAbort--
@@ -180,8 +234,14 @@ Restart(char *manager)
 void
 wAbort(Bool dumpCore)
 {
-    if (wScreen)
-	RestoreDesktop(wScreen);
+    int i;
+    WScreen *scr;
+
+    for (i=0; i<wScreenCount; i++) {
+	scr = wScreenWithNumber(i);
+	if (scr)
+	    RestoreDesktop(scr);
+    }
     printf(_("%s aborted.\n"), ProgName);
     if (dumpCore)
 	abort();
@@ -193,19 +253,25 @@ wAbort(Bool dumpCore)
 void
 print_help()
 {
-    printf(_("usage: %s [-options]\n"), ProgName);
-    puts(_("options:"));
-#ifdef USECPP
-    puts(_(" -nocpp 		disable preprocessing of configuration files"));
-#endif
-    puts(_(" -nodock		do not open the application Dock"));
-    puts(_(" -noclip		do not open the workspace Clip"));
-    /*
-    puts(_(" -locale locale		locale to use"));
-    */
-    puts(_(" -visualid visualid	visual id of visual to use"));
+    printf(_("Usage: %s [options]\n"), ProgName);
+    puts(_("The Window Maker window manager for the X window system"));
+    puts("");
     puts(_(" -display host:dpy	display to use"));
-    puts(_(" -version		print version and exit"));
+#ifdef USECPP
+    puts(_(" --no-cpp 		disable preprocessing of configuration files"));
+#endif
+    puts(_(" --no-dock		do not open the application Dock"));
+    puts(_(" --no-clip		do not open the workspace Clip"));
+    /*
+    puts(_(" --locale locale		locale to use"));
+    */
+    puts(_(" --visual-id visualid	visual id of visual to use"));
+    puts(_(" --static		do not update or save configurations"));
+#ifdef DEBUG
+    puts(_(" --synchronous		turn on synchronous display mode"));
+#endif
+    puts(_(" --version		print version and exit"));
+    puts(_(" --help			show this message"));
 }
 
 
@@ -217,8 +283,9 @@ check_defaults()
 
     path = wdefaultspathfordomain("");
     if (access(path, R_OK)!=0) {
-	wfatal(_("could not find user GNUstep directory.\n"
-		 "Make sure you have installed WindowMaker correctly and run wmaker.inst"));
+	wfatal(_("could not find user GNUstep directory (%s).\n"
+		 "Make sure you have installed Window Maker correctly and run wmaker.inst"),
+	       path);
 	exit(1);
     }
     
@@ -226,48 +293,87 @@ check_defaults()
 }
 
 
-
 static void
 execInitScript()
 {
     char *file;
-    
-    file = wfindfile(DEF_CONFIG_PATHS, DEF_INIT_SCRIPT);
+    char *paths = wstrappend(wusergnusteppath(), ":"DEF_CONFIG_PATHS);
+
+    file = wfindfile(paths, DEF_INIT_SCRIPT);
+    free(paths);
+
     if (file) {
+	if (system(file) != 0) {
+	    wsyserror(_("%s:could not execute initialization script"), file);
+	}
+#if 0
 	if (fork()==0) {
-	    close(ConnectionNumber(dpy));
-	    
 	    execl("/bin/sh", "/bin/sh", "-c", file, NULL);
 	    wsyserror(_("%s:could not execute initialization script"), file);
 	    exit(1);
 	}
+#endif
 	free(file);
     }
 }
 
 
+void
+ExecExitScript()
+{
+    char *file;
+    char *paths = wstrappend(wusergnusteppath(), ":"DEF_CONFIG_PATHS);
+
+    file = wfindfile(paths, DEF_EXIT_SCRIPT);
+    free(paths);
+
+    if (file) {
+	if (system(file) != 0) {
+	    wsyserror(_("%s:could not execute exit script"), file);
+	}
+#if 0
+	if (fork()==0) {
+	    execl("/bin/sh", "/bin/sh", "-c", file, NULL);
+	    wsyserror(_("%s:could not execute exit script"), file);
+	    exit(1);
+	}
+#endif
+	free(file);
+    }
+}
+
+
+
 int
 main(int argc, char **argv)
 {
     int i, restart=0;
-    char *display_name="";
-    char *tmp;
+    Bool multiHead = True;
+    char *str;
+    int d, s;
+#ifdef DEBUG
+    Bool doSync = False;
+#endif
+
+    wsetabort(wAbort);
+
+    /* for telling WPrefs what's the name of the wmaker binary being ran */
+    str = wstrappend("WMAKER_BIN_NAME=", argv[0]);
+    putenv(str);
 
     ArgCount = argc;
     Arguments = argv;
 
     WMInitializeApplication("WindowMaker", &argc, argv);
-    
+
+
     ProgName = strrchr(argv[0],'/');
     if (!ProgName)
       ProgName = argv[0];
     else
       ProgName++;
 
-    
-    /* check existence of Defaults DB directory */
-    check_defaults();
-    
+
     restart = 0;
     
     memset(&wPreferences, 0, sizeof(WPreferences));
@@ -275,21 +381,30 @@ main(int argc, char **argv)
     if (argc>1) {
 	for (i=1; i<argc; i++) {
 #ifdef USECPP
-	    if (strcmp(argv[i], "-nocpp")==0) {
+	    if (strcmp(argv[i], "-nocpp")==0
+		|| strcmp(argv[i], "--no-cpp")==0) {
 		wPreferences.flags.nocpp=1;
 	    } else
 #endif
-	    if (strcmp(argv[i], "-nodock")==0) {
+	    if (strcmp(argv[i], "-nodock")==0
+		|| strcmp(argv[i], "--no-dock")==0) {
 		wPreferences.flags.nodock=1;
-	    } else if (strcmp(argv[i], "-noclip")==0) {
+	    } else if (strcmp(argv[i], "-noclip")==0
+		       || strcmp(argv[i], "--no-clip")==0) {
 		wPreferences.flags.noclip=1;
-	    } else if (strcmp(argv[i], "-version")==0) {
-		printf("WindowMaker %s\n", VERSION);
+	    } else if (strcmp(argv[i], "-version")==0
+		       || strcmp(argv[i], "--version")==0) {
+		printf("Window Maker %s\n", VERSION);
 		exit(0);
-	    } else if (strcmp(argv[i], "-global_defaults_path")==0) {
-		printf("%s/Defaults/WindowMaker", PKGDATADIR);
+	    } else if (strcmp(argv[i], "--global_defaults_path")==0) {
+		puts(SYSCONFDIR);
 		exit(0);
-	    } else if (strcmp(argv[i], "-locale")==0) {
+#ifdef DEBUG
+            } else if (strcmp(argv[i], "--synchronous")==0) {
+                doSync = 1;
+#endif
+	    } else if (strcmp(argv[i], "-locale")==0
+		       || strcmp(argv[i], "--locale")==0) {
 		i++;
 		if (i>=argc) {
 		    wwarning(_("too few arguments for %s"), argv[i-1]);
@@ -302,8 +417,9 @@ main(int argc, char **argv)
 		    wwarning(_("too few arguments for %s"), argv[i-1]);
 		    exit(0);
 		}
-		display_name = argv[i];
-	    } else if (strcmp(argv[i], "-visualid")==0) {
+		DisplayName = argv[i];
+	    } else if (strcmp(argv[i], "-visualid")==0
+		       || strcmp(argv[i], "--visual-id")==0) {
 		i++;
 		if (i>=argc) {
 		    wwarning(_("too few arguments for %s"), argv[i-1]);
@@ -313,12 +429,35 @@ main(int argc, char **argv)
 		    wwarning(_("bad value for visualid: \"%s\""), argv[i]);
 		    exit(0);
 		}
-	    } else {
+	    } else if (strcmp(argv[i], "-static")==0
+		       || strcmp(argv[i], "--static")==0) {
+
+		wPreferences.flags.noupdates = 1;
+#ifdef XSMP_ENABLED
+	    } else if (strcmp(argv[i], "-clientid")==0
+		       || strcmp(argv[i], "-restore")==0) {
+		i++;
+		if (i>=argc) {
+		    wwarning(_("too few arguments for %s"), argv[i-1]);
+		    exit(0);
+		}
+#endif
+	    } else if (strcmp(argv[i], "--help")==0) {
 		print_help();
 		exit(0);
+	    } else {
+		printf(_("%s: invalid argument '%s'\n"), argv[0], argv[i]);
+		printf(_("Try '%s --help' for more information\n"), argv[0]);
+		exit(1);
 	    }
 	}
     }
+    
+    if (!wPreferences.flags.noupdates) {
+	/* check existence of Defaults DB directory */
+	check_defaults();
+    }
+
 #if 0
     tmp = getenv("LANG");
     if (tmp) {
@@ -338,22 +477,18 @@ main(int argc, char **argv)
     }
 #endif
     if (!Locale) {
-    	tmp = Locale = getenv("LANG");
-	setlocale(LC_ALL, "");
-    } else {
-	setlocale(LC_ALL, Locale);
+    	Locale = getenv("LANG");
     }
+    setlocale(LC_ALL, Locale);
     if (!Locale || strcmp(Locale, "C")==0 || strcmp(Locale, "POSIX")==0) 
       Locale = NULL;
 #ifdef I18N
     if (getenv("NLSPATH"))
       bindtextdomain("WindowMaker", getenv("NLSPATH"));
     else
-      bindtextdomain("WindowMaker", NLSDIR);
+      bindtextdomain("WindowMaker", LOCALEDIR);
     textdomain("WindowMaker");
-#endif
 
-#ifdef I18N_MB
     if (!XSupportsLocale()) {
 	wwarning(_("X server does not support locale"));
     }
@@ -361,31 +496,56 @@ main(int argc, char **argv)
  	wwarning(_("cannot set locale modifiers"));
     }
 #endif
-        
-    if (display_name && display_name[0]!=0) {
-	tmp = wmalloc(strlen(display_name)+64);
-	
-	sprintf(tmp, "DISPLAY=%s", display_name);
-	putenv(tmp);
-	free(tmp);
+
+    if (Locale) {
+	char *ptr;
+
+	Locale = wstrdup(Locale);
+	ptr = strchr(Locale, '.');
+	if (ptr)
+	    *ptr = 0;
     }
+
+
     /* open display */
-    dpy = XOpenDisplay(display_name);
+    dpy = XOpenDisplay(DisplayName);
     if (dpy == NULL) {
-	wfatal(_("could not open display \"%s\""), XDisplayName(display_name));
+	wfatal(_("could not open display \"%s\""), XDisplayName(DisplayName));
 	exit(1);
     }
 
+    if (fcntl(ConnectionNumber(dpy), F_SETFD, FD_CLOEXEC) < 0) {
+	wsyserror("error setting close-on-exec flag for X connection");
+	exit(1);
+    }
+
+    /* check if the user specified a complete display name (with screen).
+     * If so, only manage the specified screen */
+    if (DisplayName)
+	str = strchr(DisplayName, ':');
+    else
+	str = NULL;
+
+    if (str && sscanf(str, "%i.%i", &d, &s)==2)
+	multiHead = False;
+
+    DisplayName = XDisplayName(DisplayName);
+    str = wmalloc(strlen(DisplayName)+64);
+    sprintf(str, "DISPLAY=%s", DisplayName);
+    putenv(str);
+
 #ifdef DEBUG
-    XSynchronize(dpy, True);
+    if (doSync)
+        XSynchronize(dpy, True);
 #endif
 
     wXModifierInitialize();
 
-#ifdef SOUNDS
-    wSoundInitialize();
+#ifdef XSMP_ENABLED
+    wSessionConnectManager(argv, argc);
 #endif
-    StartUp();
+    
+    StartUp(!multiHead);
 
     execInitScript();
 
