@@ -1,8 +1,9 @@
 /* menu.c- generic menu, used for root menu, application menus etc.
  *
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1997, 1998 Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -51,8 +52,8 @@ extern WPreferences wPreferences;
 
 #define MOD_MASK wPreferences.modifier_mask
 
-#define MENU_SCROLL_STEP  menuScrollParameters[wPreferences.menu_scroll_speed].steps
-#define MENU_SCROLL_DELAY menuScrollParameters[wPreferences.menu_scroll_speed].delay
+#define MENU_SCROLL_STEP  menuScrollParameters[(int)wPreferences.menu_scroll_speed].steps
+#define MENU_SCROLL_DELAY menuScrollParameters[(int)wPreferences.menu_scroll_speed].delay
 
 
 
@@ -81,12 +82,57 @@ static void menuTitleMouseDown(WCoreWindow *sender, void *data, XEvent *event);
 
 static void menuCloseClick(WCoreWindow *sender, void *data, XEvent *event);
 
+static void updateTexture(WMenu *menu);
 
+#ifndef LITE
+static int saveMenuRecurs(proplist_t menus, WScreen *scr, WMenu *menu);
+static int restoreMenuRecurs(WScreen *scr, proplist_t menus, WMenu *menu, char *path);
+#endif /* !LITE */
 
 static void selectEntry(WMenu *menu, int entry_no);
 static void closeCascade(WMenu *menu);
 
 
+/****** Notification Observers ******/
+
+static void
+appearanceObserver(void *self, WMNotification *notif)
+{
+    WMenu *menu = (WMenu*)self;
+    int flags = (int)WMGetNotificationClientData(notif);
+
+    if (!menu->flags.realized)
+	return;
+    
+    if (WMGetNotificationName(notif) == WNMenuAppearanceSettingsChanged) {
+	if (flags & WFontSettings) {
+	    menu->flags.realized = 0;
+	    wMenuRealize(menu);
+	}
+	if (flags & WTextureSettings) {
+	    if (!menu->flags.brother)
+		updateTexture(menu);
+	}
+	if (flags & (WTextureSettings|WColorSettings)) {
+	    wMenuPaint(menu);
+	}
+    } else if (menu->flags.titled) {
+
+	if (flags & WFontSettings) {
+	    menu->flags.realized = 0;
+	    wMenuRealize(menu);
+	}
+	if (flags & WTextureSettings) {
+	    menu->frame->flags.need_texture_remake = 1;
+	}
+	if (flags & (WColorSettings|WTextureSettings)) {
+	    wFrameWindowPaint(menu->frame);
+	}
+    }
+}
+
+/************************************/
+
 
 /*
  *----------------------------------------------------------------------
@@ -100,7 +146,7 @@ static void closeCascade(WMenu *menu);
  * 	The created menu.
  *---------------------------------------------------------------------- 
  */
-WMenu *
+WMenu*
 wMenuCreate(WScreen *screen, char *title, int main_menu)
 {
     WMenu *menu;
@@ -112,9 +158,9 @@ wMenuCreate(WScreen *screen, char *title, int main_menu)
     memset(menu, 0, sizeof(WMenu));
 
 #ifdef SINGLE_MENULEVEL
-    tmp = WMSubmenuWindowLevel;
+    tmp = WMSubmenuLevel;
 #else
-    tmp = (main_menu ? WMMainMenuWindowLevel : WMSubmenuWindowLevel);
+    tmp = (main_menu ? WMMainMenuLevel : WMSubmenuLevel);
 #endif
 
     flags = WFF_SINGLE_STATE;
@@ -139,7 +185,7 @@ wMenuCreate(WScreen *screen, char *title, int main_menu)
     }
 
     menu->frame->flags.justification = WTJ_LEFT;
-    
+
     menu->frame->rbutton_image = screen->b_pixmaps[WBUT_CLOSE];
 
     menu->entry_no = 0;
@@ -185,7 +231,12 @@ wMenuCreate(WScreen *screen, char *title, int main_menu)
 	menu->brother->flags.brother = 1;
 	menu->brother->brother = menu;
     }
-    
+    WMAddNotificationObserver(appearanceObserver, menu,
+			      WNMenuAppearanceSettingsChanged, menu);
+
+    WMAddNotificationObserver(appearanceObserver, menu,
+			      WNMenuTitleAppearanceSettingsChanged, menu);
+
 
     return menu;
 }
@@ -214,48 +265,15 @@ insertEntry(WMenu *menu, WMenuEntry *entry, int index)
 {
     int i;
     
-    for (i=menu->entry_no; i>index; i--) {
+    for (i = menu->entry_no-1; i >= index; i--) {
 	menu->entries[i]->order++;
-	menu->entries[i+1]=menu->entries[i];
+	menu->entries[i+1] = menu->entries[i];
     }
     menu->entries[index] = entry;
 }
 
 
-void
-wMenuRefresh(WMenu *menu, int flags)
-{
-    int i;
-    
-    if (flags & MR_TEXT_BACK) {
-	menu->frame->flags.need_texture_remake = 1;
-    }
-    
-    if (flags & (MR_RESIZED|MR_TITLE_TEXT)) {
-	menu->flags.realized = 0;
-    }
-
-    wMenuRealize(menu);
-
-    if (menu->flags.titled)
-	wFrameWindowPaint(menu->frame);
-    
-    if (!menu->flags.brother) {
-	if (menu->brother)
-	    wMenuRefresh(menu->brother, flags);
-
-	for (i=0; i < menu->cascade_no; i++) {
-	    if (!menu->cascades[i]->flags.brother)
-		wMenuRefresh(menu->cascades[i], flags);
-	    else
-		wMenuRefresh(menu->cascades[i]->brother, flags);
-	}
-    }
-}
-
-
-
-WMenuEntry * 
+WMenuEntry*
 wMenuInsertCallback(WMenu *menu, int index, char *text,
 		    void (*callback)(WMenu *menu, WMenuEntry *entry), 
 		    void *clientdata)
@@ -359,10 +377,10 @@ wMenuEntrySetCascade(WMenu *menu, WMenuEntry *entry, WMenu *cascade)
     if (menu->flags.lowered) {
 	
 	cascade->flags.lowered = 1;
-	ChangeStackingLevel(cascade->frame->core, WMNormalWindowLevel);
+	ChangeStackingLevel(cascade->frame->core, WMNormalLevel);
 	
 	cascade->brother->flags.lowered = 1;
-	ChangeStackingLevel(cascade->brother->frame->core, WMNormalWindowLevel);
+	ChangeStackingLevel(cascade->brother->frame->core, WMNormalLevel);
     }
     
     if (!menu->flags.realized)
@@ -426,6 +444,90 @@ wMenuRemoveItem(WMenu *menu, int index)
 }
 
 
+static Pixmap
+renderTexture(WMenu *menu)
+{
+    RImage *img;
+    Pixmap pix;
+    int i;
+    RColor light;
+    RColor dark;
+    RColor mid;
+    WScreen *scr = menu->menu->screen_ptr;
+    WTexture *texture = scr->menu_item_texture;
+
+    if (wPreferences.menu_style == MS_NORMAL) {
+	img = wTextureRenderImage(texture, menu->menu->width, 
+				  menu->entry_height, WREL_MENUENTRY);
+    } else {
+	img = wTextureRenderImage(texture, menu->menu->width, 
+				  menu->menu->height+1, WREL_MENUENTRY);
+    }
+    if (!img) {
+	wwarning(_("could not render texture: %s"), 
+		 RMessageForError(RErrorCode));
+
+	return None;
+    }
+
+    if (wPreferences.menu_style == MS_SINGLE_TEXTURE) {
+	light.alpha = 0;
+	light.red = light.green = light.blue = 80;
+    
+	dark.alpha = 255;
+	dark.red = dark.green = dark.blue = 0;
+    
+	mid.alpha = 0;
+	mid.red = mid.green = mid.blue = 40;
+    
+	for (i = 1; i < menu->entry_no; i++) {	    
+	    ROperateLine(img, RSubtractOperation, 0, i*menu->entry_height-2,
+			 menu->menu->width-1, i*menu->entry_height-2, &mid);
+	    
+	    RDrawLine(img, 0, i*menu->entry_height-1,
+		      menu->menu->width-1, i*menu->entry_height-1, &dark);
+
+	    ROperateLine(img, RAddOperation, 0, i*menu->entry_height,
+			 menu->menu->width-1, i*menu->entry_height, 
+			 &light);
+	}
+    }
+    if (!RConvertImage(scr->rcontext, img, &pix)) {
+	wwarning(_("error rendering image:%s"), RMessageForError(RErrorCode));
+    }
+    RDestroyImage(img);
+
+    return pix;
+}
+
+
+static void
+updateTexture(WMenu *menu)
+{
+    WScreen *scr = menu->menu->screen_ptr;
+
+    /* setup background texture */
+    if (scr->menu_item_texture->any.type != WTEX_SOLID) {
+	if (!menu->flags.brother) {
+	    FREE_PIXMAP(menu->menu_texture_data);
+
+	    menu->menu_texture_data = renderTexture(menu);
+
+	    XSetWindowBackgroundPixmap(dpy, menu->menu->window,
+				       menu->menu_texture_data);
+	    XClearWindow(dpy, menu->menu->window);
+
+	    XSetWindowBackgroundPixmap(dpy, menu->brother->menu->window,
+				       menu->menu_texture_data);
+	    XClearWindow(dpy, menu->brother->menu->window);
+	}
+    } else {
+	XSetWindowBackground(dpy, menu->menu->window,
+			     scr->menu_item_texture->any.color.pixel);
+	XClearWindow(dpy, menu->menu->window);
+    }
+}
+
 
 void 
 wMenuRealize(WMenu *menu)
@@ -450,15 +552,15 @@ wMenuRealize(WMenu *menu)
     wFrameWindowUpdateBorders(menu->frame, flags);
 
     if (menu->flags.titled) {
-	twidth = MyTextWidth(scr->menu_title_font->font, menu->frame->title,
-			     strlen(menu->frame->title));
+	twidth = WMWidthOfString(scr->menu_title_font, menu->frame->title,
+				 strlen(menu->frame->title));
         theight = menu->frame->top_width;
-        twidth += theight + theight/2 + (wPreferences.new_style ? 8 : 0);
+        twidth += theight + (wPreferences.new_style ? 16 : 8);
     } else {
 	twidth = 0;
 	theight = 0;
     }
-    eheight = scr->menu_entry_font->height + 6;
+    eheight = WMFontHeight(scr->menu_entry_font) + 6;
     menu->entry_height = eheight;
     mrwidth = 0;
     mwidth = 0;
@@ -467,7 +569,7 @@ wMenuRealize(WMenu *menu)
 
 	/* search widest text */
 	text = menu->entries[i]->text;
-	width = MyTextWidth(scr->menu_entry_font->font, text, strlen(text))+12;
+	width = WMWidthOfString(scr->menu_entry_font, text, strlen(text))+10;
 
 	if (menu->entries[i]->flags.indicator) {
 	    width += MENU_INDICATOR_SPACE;
@@ -479,10 +581,10 @@ wMenuRealize(WMenu *menu)
 	/* search widest text on right */
 	text = menu->entries[i]->rtext;
 	if (text)
-	    rwidth = MyTextWidth(scr->menu_entry_font->font, text,
-				 strlen(text)) + 5;
+	    rwidth = WMWidthOfString(scr->menu_entry_font, text, strlen(text))
+		+ 5;
 	else if (menu->entries[i]->cascade>=0)
-	    rwidth = eheight;
+	    rwidth = 16;
 	else
 	    rwidth = 4;
 
@@ -494,41 +596,14 @@ wMenuRealize(WMenu *menu)
     if (mwidth < twidth)
 	mwidth = twidth;
 
+    
     wCoreConfigure(menu->menu, 0, theight, mwidth, menu->entry_no*eheight -1);
 
-    wFrameWindowResizeInternal(menu->frame, mwidth, menu->entry_no*eheight -1);
+    wFrameWindowResize(menu->frame, mwidth, menu->entry_no*eheight-1
+		       + menu->frame->top_width + menu->frame->bottom_width);
 
 
-    /* setup background texture */
-    switch (scr->menu_item_texture->any.type) {
-     case WTEX_DGRADIENT:
-     case WTEX_VGRADIENT:
-     case WTEX_HGRADIENT:
-     case WTEX_MHGRADIENT:
-     case WTEX_MVGRADIENT:
-     case WTEX_MDGRADIENT:
-     case WTEX_PIXMAP:
-	if (!menu->flags.brother) {
-	    FREE_PIXMAP(menu->menu_texture_data);
-
-	    wTextureRender(scr, scr->menu_item_texture,
-			   &menu->menu_texture_data, menu->menu->width,
-			   menu->entry_height, WREL_MENUENTRY);
-	    XSetWindowBackgroundPixmap(dpy, menu->menu->window,
-				       menu->menu_texture_data);
-	    XClearWindow(dpy, menu->menu->window);
-
-	    XSetWindowBackgroundPixmap(dpy, menu->brother->menu->window,
-				       menu->menu_texture_data);
-	    XClearWindow(dpy, menu->brother->menu->window);
-	}
-	break;
-	
-     default:
-	XSetWindowBackground(dpy, menu->menu->window,
-			     scr->menu_item_texture->any.color.pixel);
-	XClearWindow(dpy, menu->menu->window);
-    }
+    updateTexture(menu);
 
     menu->flags.realized = 1;
 
@@ -544,6 +619,8 @@ wMenuDestroy(WMenu *menu, int recurse)
 {
     int i;
 
+    WMRemoveNotificationObserver(menu);
+    
     /* remove any pending timers */
     if (menu->timer)
 	WMDeleteTimerHandler(menu->timer);
@@ -563,6 +640,12 @@ wMenuDestroy(WMenu *menu, int recurse)
 
 	    if (menu->entries[i]->rtext)
 	      free(menu->entries[i]->rtext);
+#ifdef USER_MENU
+
+	    if (menu->entries[i]->instances){
+		  PLRelease(menu->entries[i]->instances);
+		}
+#endif /* USER_MENU */
 
 	    if (menu->entries[i]->free_cdata && menu->entries[i]->clientdata) {
 		(*menu->entries[i]->free_cdata)(menu->entries[i]->clientdata);
@@ -602,31 +685,48 @@ wMenuDestroy(WMenu *menu, int recurse)
 }
 
 
+#define F_NORMAL	0
+#define F_TOP		1
+#define F_BOTTOM	2
+#define F_NONE		3
+
 static void
-drawFrame(WScreen *scr, Window win, int y, int w, int h)
+drawFrame(WScreen *scr, Window win, int y, int w, int h, int type)
 {
     XSegment segs[2];
+    int i;
 
-    segs[0].x1 = 0;
-    segs[0].y1 = y;
-    segs[0].x2 = w-1;
-    segs[0].y2 = y;
-    segs[1].x1 = 0;
-    segs[1].y1 = y;
-    segs[1].x2 = 0;
-    segs[1].y2 = y + h - 2;
-    XDrawSegments(dpy, win, scr->menu_item_auxtexture->light_gc, segs, 2);
+    i = 0;
+    segs[i].x1 = segs[i].x2 = w-1;
+    segs[i].y1 = y;
+    segs[i].y2 = y + h - 1;
+    i++;
+    if (type != F_TOP && type != F_NONE) {
+	segs[i].x1 = 1;
+	segs[i].y1 = segs[i].y2 = y + h-2;
+	segs[i].x2 = w-1;
+	i++;
+    }
+    XDrawSegments(dpy, win, scr->menu_item_auxtexture->dim_gc, segs, i);
 
-    XDrawLine(dpy, win, scr->menu_item_auxtexture->dark_gc, 0, y+h-1,
-	      w-1, y+h-1);
+    i = 0;
+    segs[i].x1 = 0;
+    segs[i].y1 = y;
+    segs[i].x2 = 0;
+    segs[i].y2 = y + h - 1;
+    i++;
+    if (type != F_BOTTOM && type != F_NONE) {
+	segs[i].x1 = 0;
+	segs[i].y1 = y;
+	segs[i].x2 = w-1;
+	segs[i].y2 = y;
+	i++;
+    }
+    XDrawSegments(dpy, win, scr->menu_item_auxtexture->light_gc, segs, i);
 
-    segs[0].x1 = 1;
-    segs[0].y1 = segs[0].y2 = y + h-2;
-    segs[0].x2 = w-1;
-    segs[1].x1 = segs[1].x2 = w-1;
-    segs[1].y1 = y + 1;
-    segs[1].y2 = y + h-2;
-    XDrawSegments(dpy, win, scr->menu_item_auxtexture->dim_gc, segs, 2);
+    if (type != F_TOP && type != F_NONE)
+	XDrawLine(dpy, win, scr->menu_item_auxtexture->dark_gc, 0, y+h-1,
+		  w-1, y+h-1);
 }
 
 
@@ -634,6 +734,7 @@ static void
 paintEntry(WMenu *menu, int index, int selected)
 {
     int x, y, w, h, tw;
+    int type;
     GC light, dim, dark, textGC;
     WScreen *scr=menu->frame->screen_ptr;
     Window win = menu->menu->window;
@@ -648,17 +749,28 @@ paintEntry(WMenu *menu, int index, int selected)
     dim = scr->menu_item_auxtexture->dim_gc;
     dark = scr->menu_item_auxtexture->dark_gc;
 
+    if (wPreferences.menu_style == MS_FLAT && menu->entry_no > 1) {
+	if (index == 0)
+	    type = F_TOP;
+	else if (index == menu->entry_no - 1)
+	    type = F_BOTTOM;
+	else
+	    type = F_NONE;
+    } else {
+	type = F_NORMAL;
+    }
+    
     /* paint background */
     if (selected) {
 	XSetForeground(dpy, scr->select_menu_gc, scr->select_pixel);
 	XFillRectangle(dpy, win, scr->select_menu_gc, 1, y+1, w-2, h-3);
 	if (scr->menu_item_texture->any.type == WTEX_SOLID)
-	    drawFrame(scr, win, y, w, h);
+	    drawFrame(scr, win, y, w, h, type);
     } else { 
 	if (scr->menu_item_texture->any.type == WTEX_SOLID) {
 	    XClearArea(dpy, win, 0, y + 1, w - 1, h - 3, False);
 	    /* draw the frame */
-	    drawFrame(scr, win, y, w, h);
+	    drawFrame(scr, win, y, w, h, type);
 	} else {
 	    XClearArea(dpy, win, 0, y, w, h, False);
 	}
@@ -680,14 +792,8 @@ paintEntry(WMenu *menu, int index, int selected)
     if (entry->flags.indicator)
 	x += MENU_INDICATOR_SPACE + 2;
 
-#ifdef I18N_MB
-    XmbDrawString(dpy, win, scr->menu_entry_font->font, 
-		  textGC, x, 3+y+scr->menu_entry_font->y, entry->text, 
-		  strlen(entry->text));
-#else
-    XDrawString(dpy, win, textGC, x, 3+y+scr->menu_entry_font->y, entry->text, 
-		strlen(entry->text));
-#endif
+    WMDrawString(scr->wmscreen, win, textGC, scr->menu_entry_font, 
+		 x, 3 + y, entry->text, strlen(entry->text));
 
     if (entry->cascade>=0) {
 	/* draw the cascade indicator */
@@ -712,6 +818,9 @@ paintEntry(WMenu *menu, int index, int selected)
 	 case MI_HIDDEN:
 	    indicator = scr->menu_hide_indicator;
 	    break;
+	 case MI_SHADED:
+	    indicator = scr->menu_shade_indicator;
+	    break;
 	 case MI_DIAMOND:
 	 default:
 	    indicator = scr->menu_radio_indicator;
@@ -737,19 +846,15 @@ paintEntry(WMenu *menu, int index, int selected)
     /* draw right text */
     
     if (entry->rtext && entry->cascade<0) {
-	tw = MyTextWidth(scr->menu_entry_font->font, entry->rtext,
-			 strlen(entry->rtext));
-#ifdef I18N_MB
-	XmbDrawString(dpy, win, scr->menu_entry_font->font, 
-		      textGC, w-6-tw, 3+y+scr->menu_entry_font->y, entry->rtext,
-		      strlen(entry->rtext));
-#else
-	XDrawString(dpy, win, textGC, w-6-tw,3+y+scr->menu_entry_font->y, 
-		    entry->rtext, strlen(entry->rtext));
-#endif
+	tw = WMWidthOfString(scr->menu_entry_font, entry->rtext,
+			     strlen(entry->rtext));
+
+	WMDrawString(scr->wmscreen, win, textGC, scr->menu_entry_font, w-6-tw, 
+		     y + 3, entry->rtext, strlen(entry->rtext));
     }
 }
 
+
 static void
 move_menus(WMenu *menu, int x, int y)
 {
@@ -788,7 +893,7 @@ makeVisible(WMenu *menu)
 	new_x = scr->scr_width - MENUW(menu) - 1;
 	move = 1;
     }
-    
+
     if (y1 < 0) {
 	new_y = 0;
 	move = 1;
@@ -796,11 +901,10 @@ makeVisible(WMenu *menu)
 	new_y = scr->scr_height - menu->entry_height - 1;
 	move = 1;
     }
-    
+
     new_y = new_y - menu->frame->top_width
 	- menu->selected_entry*menu->entry_height;
     move_menus(menu, new_x, new_y);
-
 }
 
 
@@ -849,6 +953,7 @@ keyboardMenu(WMenu *menu)
     if (menu->flags.editing)
 	return False;
 
+
     XGrabKeyboard(dpy, menu->frame->core->window, True, GrabModeAsync,
 		  GrabModeAsync, CurrentTime);
 
@@ -859,12 +964,13 @@ keyboardMenu(WMenu *menu)
 	new_x = scr_width-MENUW(menu)-1;
     
     move_menus(menu, new_x, new_y);
-    
-    while (!done) {
+
+    while (!done && menu->flags.mapped) {
 	XAllowEvents(dpy, AsyncKeyboard, CurrentTime);
 	WMMaskEvent(dpy, ExposureMask|ButtonMotionMask|ButtonPressMask
-		   |ButtonReleaseMask|KeyPressMask|KeyReleaseMask, &event);
-	
+		    |ButtonReleaseMask|KeyPressMask|KeyReleaseMask
+		    |SubstructureNotifyMask, &event);
+
 	switch (event.type) {
 	 case KeyPress:
 	    ksym = XLookupKeysym(&event.xkey, 0);
@@ -922,15 +1028,15 @@ keyboardMenu(WMenu *menu)
 
 		    if (entry->cascade >= 0 && menu->cascades
 			&& menu->cascades[entry->cascade]->entry_no > 0) {
-			
+
 			XUngrabKeyboard(dpy, CurrentTime);
 
 			selectEntry(menu->cascades[entry->cascade], 0);
 			if (!keyboardMenu(menu->cascades[entry->cascade]))
 			    done = 1;
-			
-			XGrabKeyboard(dpy, menu->frame->core->window, True, 
-				      GrabModeAsync, GrabModeAsync, 
+
+			XGrabKeyboard(dpy, menu->frame->core->window, True,
+				      GrabModeAsync, GrabModeAsync,
 				      CurrentTime);
 		    }
 		}
@@ -963,12 +1069,13 @@ keyboardMenu(WMenu *menu)
 	 default:
 	    if (event.type==ButtonPress)
 		done = 1;
+	    
 	    WMHandleEvent(&event);
 	}
     }
-    
+
     XUngrabKeyboard(dpy, CurrentTime);
-    
+
     if (done==2 && menu->selected_entry>=0) {
 	entry = menu->entries[menu->selected_entry];
     } else {
@@ -1175,6 +1282,7 @@ editEntry(WMenu *menu, WMenuEntry *entry)
 
 	     default:
 		WMHandleEvent(&event);
+		break;
 	    }
 	}
     }
@@ -1245,7 +1353,7 @@ selectEntry(WMenu *menu, int entry_no)
 	}
 	entry = menu->entries[entry_no];
 
-	if (entry->cascade>=0 && menu->cascades) {
+	if (entry->cascade>=0 && menu->cascades && entry->flags.enabled) {
 	    /* Callback for when the submenu is opened.
 	     */
 	    submenu = menu->cascades[entry->cascade];
@@ -1513,17 +1621,14 @@ getScrollAmount(WMenu *menu, int *hamount, int *vamount)
     
     getPointerPosition(scr, &xroot, &yroot);
 
-    
-    if (xroot <= 1 && menuX1 <= 0) {
+
+    if (xroot <= 1 && menuX1 < 0) {
 	/* scroll to the right */
 	*hamount = MIN(MENU_SCROLL_STEP, abs(menuX1));
-	
-	if (*hamount==0)
-	    *hamount = 1;
 
-    } else if (xroot >= screenW-2 && menuX2 > screenW-4) {
+    } else if (xroot >= screenW-2 && menuX2 > screenW-1) {
 	/* scroll to the left */
-	*hamount = MIN(MENU_SCROLL_STEP, abs(menuX2-screenW-4));
+	*hamount = MIN(MENU_SCROLL_STEP, abs(menuX2-screenW-1));
 	
 	if (*hamount==0)
 	    *hamount = 1;
@@ -1531,20 +1636,14 @@ getScrollAmount(WMenu *menu, int *hamount, int *vamount)
 	*hamount = -*hamount;
     }
     
-    if (yroot <= 1 && menuY1 <= 0) {
+    if (yroot <= 1 && menuY1 < 0) {
 	/* scroll down */
 	*vamount = MIN(MENU_SCROLL_STEP, abs(menuY1));
-	
-	if (*vamount==0)
-	    *vamount = 1;
-	
-    } else if (yroot >= screenH-2 && menuY2 > screenH-2) {
+        
+    } else if (yroot >= screenH-2 && menuY2 > screenH-1) {
 	/* scroll up */
 	*vamount = MIN(MENU_SCROLL_STEP, abs(menuY2-screenH-2));
-
-	if (*vamount==0)
-	    *vamount = 1;
-	
+        
 	*vamount = -*vamount;
     }
 }
@@ -1645,6 +1744,24 @@ isPointNearBoder(WMenu *menu, int x, int y)
     return flag;
 }
 
+ 
+typedef struct _delay {
+    WWindow *wwin;
+    WMenu *menu;
+    int ox,oy;
+} _delay;
+
+
+static void
+_leaving(_delay *dl)
+{
+    wMenuMove(dl->menu, dl->ox, dl->oy, True);
+    dl->menu->jump_back=NULL;
+    dl->menu->menu->screen_ptr->flags.jump_back_pending = 0;
+    free(dl);
+}
+
+
 void
 wMenuScroll(WMenu *menu, XEvent *event)
 {
@@ -1660,6 +1777,10 @@ wMenuScroll(WMenu *menu, XEvent *event)
 #ifdef DEBUG
     puts("Entering menu Scroll");
 #endif
+    
+    if (omenu->jump_back)
+        WMDeleteTimerWithClientData(omenu->jump_back);
+
 
     if ((/*omenu->flags.buttoned &&*/ !wPreferences.wrap_menus)
         || omenu->flags.app_menu) {
@@ -1679,9 +1800,9 @@ wMenuScroll(WMenu *menu, XEvent *event)
 
         WMNextEvent(dpy, &ev);
         switch (ev.type) {
-        case EnterNotify:
+	 case EnterNotify:
             WMHandleEvent(&ev);
-        case MotionNotify:
+	 case MotionNotify:
             x = (ev.type==MotionNotify) ? ev.xmotion.x_root : ev.xcrossing.x_root;
             y = (ev.type==MotionNotify) ? ev.xmotion.y_root : ev.xcrossing.y_root;
 
@@ -1744,8 +1865,20 @@ wMenuScroll(WMenu *menu, XEvent *event)
 	menu->timer = NULL;
     }
 
-    if (jump_back)
-        wMenuMove(omenu, old_frame_x, old_frame_y, True);
+    if (jump_back) {
+        _delay *delayer;
+        if (!omenu->jump_back) {
+            delayer=wmalloc(sizeof(_delay));
+            delayer->menu=omenu;
+            delayer->ox=old_frame_x;
+            delayer->oy=old_frame_y;
+            omenu->jump_back=delayer;
+            scr->flags.jump_back_pending = 1;
+        }
+        else delayer = omenu->jump_back;
+        WMAddTimerHandler(MENU_JUMP_BACK_DELAY,(WMCallback*)_leaving, delayer);
+    }
+
 
 #ifdef DEBUG
     puts("Leaving menu Scroll");
@@ -1763,7 +1896,7 @@ menuExpose(WObjDescriptor *desc, XEvent *event)
 typedef struct {
     int *delayed_select;
     WMenu *menu;
-    WMagicNumber magic;
+    WMHandlerID magic;
 } delay_data;
 
 
@@ -1772,14 +1905,17 @@ delaySelection(void *data)
 {
     delay_data *d = (delay_data*)data;
     int x, y, entry_no;
+    WMenu *menu;
 
     d->magic = NULL;
-    *(d->delayed_select) = 0;
 
-    if (findMenu(d->menu->menu->screen_ptr, &x, &y)) {
-        entry_no = getEntryAt(d->menu, x, y);
-        selectEntry(d->menu, entry_no);
+    menu = findMenu(d->menu->menu->screen_ptr, &x, &y);
+    if (menu && (d->menu == menu || d->delayed_select)) {
+        entry_no = getEntryAt(menu, x, y);
+        selectEntry(menu, entry_no);
     }
+    if (d->delayed_select)
+	*(d->delayed_select) = 0;
 }
 
 
@@ -1797,6 +1933,7 @@ menuMouseDown(WObjDescriptor *desc, XEvent *event)
     int delayed_select = 0;
     int entry_no;
     int x, y;
+    int prevx, prevy;
     int old_frame_x = 0;
     int old_frame_y = 0;
     delay_data d_data = {NULL, NULL, NULL};
@@ -1805,8 +1942,7 @@ menuMouseDown(WObjDescriptor *desc, XEvent *event)
         smenu = parentMenu(menu);
         old_frame_x = smenu->frame_x;
         old_frame_y = smenu->frame_y;
-    }
-    else if (event->xbutton.window == menu->frame->core->window) {
+    } else if (event->xbutton.window == menu->frame->core->window) {
         /* This is true if the menu was launched with right click on root window */
         delayed_select = 1;
         d_data.delayed_select = &delayed_select;
@@ -1864,27 +2000,110 @@ menuMouseDown(WObjDescriptor *desc, XEvent *event)
 		dragScrollMenuCallback(menu);
         }
     }
-
+    
+    prevx = bev->x_root;
+    prevy = bev->y_root;
     while (!done) {
-      int x, y;
+	int x, y;
+
+	XAllowEvents(dpy, SyncPointer, CurrentTime);
+
 	WMMaskEvent(dpy, ExposureMask|ButtonMotionMask|ButtonReleaseMask
 		    |ButtonPressMask, &ev);
 	switch (ev.type) {
 	 case MotionNotify:
             smenu = findMenu(scr, &x, &y);
+
 	    if (smenu == NULL) {
+		/* moved mouse out of menu */
+		
+		if (!delayed_select && d_data.magic) {
+		    WMDeleteTimerHandler(d_data.magic);
+		    d_data.magic = NULL;
+		}
 		if (menu==NULL
 		    || (menu->selected_entry>=0
-			&& menu->entries[menu->selected_entry]->cascade>=0))
+			&& menu->entries[menu->selected_entry]->cascade>=0)) {
+		    prevx = ev.xmotion.x_root;
+		    prevy = ev.xmotion.y_root;
+		    
 		    break;
+		}
 		selectEntry(menu, -1);
 		menu = smenu;
+		prevx = ev.xmotion.x_root;
+		prevy = ev.xmotion.y_root;
 		break;
 	    } else if (menu && menu!=smenu
 		       && (menu->selected_entry<0
 			   || menu->entries[menu->selected_entry]->cascade<0)) {
 		selectEntry(menu, -1);
+
+		if (!delayed_select && d_data.magic) {
+		    WMDeleteTimerHandler(d_data.magic);
+		    d_data.magic = NULL;
+		}
+	    } else {
+		
+		/* hysteresis for item selection */
+
+		/* check if the motion was to the side, indicating that 
+		 * the user may want to cross to a submenu */
+		if (!delayed_select && menu) {
+		    int dx;
+		    Bool moved_to_submenu;/* moved to direction of submenu */
+
+		    dx = abs(prevx - ev.xmotion.x_root);
+
+		    moved_to_submenu = False;
+		    if (dx > 0 /* if moved enough to the side */
+			/* maybe a open submenu */
+			&& menu->selected_entry>=0 
+			/* moving to the right direction */
+			&& (wPreferences.align_menus
+			    || ev.xmotion.y_root >= prevy)) {
+			int index;
+			
+			index = menu->entries[menu->selected_entry]->cascade;
+			if (index>=0) {
+			    if (menu->cascades[index]->frame_x>menu->frame_x) {
+				if (prevx < ev.xmotion.x_root)
+				    moved_to_submenu = True;
+			    } else {
+				if (prevx > ev.xmotion.x_root)
+				    moved_to_submenu = True;
+			    }
+			}
+		    }
+		    
+
+		    if (menu != smenu) {
+			if (d_data.magic) {
+			    WMDeleteTimerHandler(d_data.magic);
+			}
+			d_data.magic = NULL;
+		    } else if (moved_to_submenu) {
+			/* while we are moving, postpone the selection */
+			if (d_data.magic) {
+			    WMDeleteTimerHandler(d_data.magic);
+			}
+			d_data.delayed_select = NULL;
+			d_data.menu = menu;
+			d_data.magic = WMAddTimerHandler(MENU_SELECT_DELAY, 
+							 delaySelection,
+							 &d_data);
+			prevx = ev.xmotion.x_root;
+			prevy = ev.xmotion.y_root;
+			break;
+		    } else {
+			if (d_data.magic)
+			    WMDeleteTimerHandler(d_data.magic);
+			d_data.magic = NULL;
+		    }
+		}
 	    }
+	    prevx = ev.xmotion.x_root;
+	    prevy = ev.xmotion.y_root;
 	    if (menu!=smenu) {
 		/* pointer crossed menus */
 		if (menu && menu->timer) {
@@ -1979,8 +2198,7 @@ menuMouseDown(WObjDescriptor *desc, XEvent *event)
 	}
     }
 
-    if (((WMenu*)desc->parent)->flags.brother || close_on_exit
-	|| !smenu)
+    if (((WMenu*)desc->parent)->flags.brother || close_on_exit || !smenu)
 	closeCascade(desc->parent);
 
     /* close the cascade windows that should not remain opened */
@@ -2037,12 +2255,12 @@ changeMenuLevels(WMenu *menu, int lower)
     int i;
 
     if (!lower) {
-	ChangeStackingLevel(menu->frame->core, (!menu->parent ? WMMainMenuWindowLevel
-					  : WMSubmenuWindowLevel));
+	ChangeStackingLevel(menu->frame->core, (!menu->parent ? WMMainMenuLevel
+					  : WMSubmenuLevel));
 	wRaiseFrame(menu->frame->core);
 	menu->flags.lowered = 0;
     } else {
-	ChangeStackingLevel(menu->frame->core, WMNormalWindowLevel);
+	ChangeStackingLevel(menu->frame->core, WMNormalLevel);
 	wLowerFrame(menu->frame->core);
 	menu->flags.lowered = 1;
     }
@@ -2083,26 +2301,19 @@ menuTitleMouseDown(WCoreWindow *sender, void *data, XEvent *event)
     int x=menu->frame_x, y=menu->frame_y;
     int dx=event->xbutton.x_root, dy=event->xbutton.y_root;
     int i, lower;
-    
+    Bool started;
+
 #ifdef DEBUG
     printf("Moving menu\n");
 #endif
-    
+
     /* can't touch the menu copy */
     if (menu->flags.brother)
 	return;
-    
+
     if (event->xbutton.button != Button1 && event->xbutton.button != Button2)
       return;
-    if (XGrabPointer(dpy, menu->frame->titlebar->window, False, 
-		     ButtonMotionMask|ButtonReleaseMask|ButtonPressMask,
-		     GrabModeAsync, GrabModeAsync, None, 
-		     wCursor[WCUR_MOVE], CurrentTime)!=GrabSuccess) {
-#ifdef DEBUG0
-	wwarning("pointer grab failed for menu move\n");
-#endif
-	return;
-    }
+
     if (event->xbutton.state & MOD_MASK) {
 	wLowerFrame(menu->frame->core);
 	lower = 1;
@@ -2111,10 +2322,11 @@ menuTitleMouseDown(WCoreWindow *sender, void *data, XEvent *event)
 	lower = 0;
     }
     tmp = menu;
-    
+
     /* lower/raise all submenus */
     while (1) {
-	if (tmp->selected_entry>=0 && tmp->cascades) {
+	if (tmp->selected_entry>=0 && tmp->cascades
+	    && tmp->entries[tmp->selected_entry]->cascade>=0) {
 	    tmp = tmp->cascades[tmp->entries[tmp->selected_entry]->cascade];
 	    if (!tmp || !tmp->flags.mapped)
 	      break;
@@ -2126,6 +2338,7 @@ menuTitleMouseDown(WCoreWindow *sender, void *data, XEvent *event)
 	    break;
 	}
     }
+
     /* tear off the menu if it's a root menu or a cascade 
      application menu */
     if (!menu->flags.buttoned && !menu->flags.brother 
@@ -2145,17 +2358,30 @@ menuTitleMouseDown(WCoreWindow *sender, void *data, XEvent *event)
 	    }
 	}
     }
-    
+
+    started = False;
     while(1) {
 	WMMaskEvent(dpy, ButtonMotionMask|ButtonReleaseMask|ButtonPressMask
 		    |ExposureMask, &ev);
 	switch (ev.type) {
 	 case MotionNotify:
-	    x += ev.xmotion.x_root - dx;
-	    y += ev.xmotion.y_root - dy;
-	    dx = ev.xmotion.x_root;
-	    dy = ev.xmotion.y_root;
-	    wMenuMove(menu, x, y, True);
+	    if (started) {
+		x += ev.xmotion.x_root - dx;
+		y += ev.xmotion.y_root - dy;
+		dx = ev.xmotion.x_root;
+		dy = ev.xmotion.y_root;
+		wMenuMove(menu, x, y, True);
+	    } else {
+		if (abs(ev.xmotion.x_root - dx) > MOVE_THRESHOLD
+		    || abs(ev.xmotion.y_root - dy) > MOVE_THRESHOLD) {
+		    started = True;
+		    XGrabPointer(dpy, menu->frame->titlebar->window, False, 
+				 ButtonMotionMask|ButtonReleaseMask
+				 |ButtonPressMask,
+				 GrabModeAsync, GrabModeAsync, None, 
+				 wCursor[WCUR_MOVE], CurrentTime);
+		}
+	    }
 	    break;
 
 	 case ButtonPress:
@@ -2169,9 +2395,10 @@ menuTitleMouseDown(WCoreWindow *sender, void *data, XEvent *event)
 #endif
 	    XUngrabPointer(dpy, CurrentTime);
 	    return;
-	    
+
 	 default:
 	    WMHandleEvent(&ev);
+	    break;
 	}
     }
 }
@@ -2214,21 +2441,12 @@ wMenuSaveState(WScreen *scr)
 {
     proplist_t menus, key, value;
     int save_menus = 0;
+
     char buffer[256];
 
     menus = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
 
-    if (scr->root_menu && scr->root_menu->flags.buttoned) {
-        sprintf(buffer, "%i,%i", scr->root_menu->frame_x,
-                                 scr->root_menu->frame_y);
-        key = PLMakeString("RootMenu");
-        value = PLMakeString(buffer);
-        PLInsertDictionaryEntry(menus, key, value);
-        PLRelease(key);
-        PLRelease(value);
-        save_menus = 1;
-    }
-
+#ifndef LITE
     if (scr->switch_menu && scr->switch_menu->flags.buttoned) {
         sprintf(buffer, "%i,%i", scr->switch_menu->frame_x,
                                  scr->switch_menu->frame_y);
@@ -2239,7 +2457,11 @@ wMenuSaveState(WScreen *scr)
         PLRelease(value);
         save_menus = 1;
     }
-    
+
+    if (saveMenuRecurs(menus, scr, scr->root_menu))
+	save_menus = 1;
+
+#endif /* !LITE */
     if (scr->workspace_menu && scr->workspace_menu->flags.buttoned) {
         sprintf(buffer, "%i,%i", scr->workspace_menu->frame_x,
                                  scr->workspace_menu->frame_y);
@@ -2260,13 +2482,80 @@ wMenuSaveState(WScreen *scr)
 }
 
 
+#ifndef LITE
+
+static Bool
+getMenuPath(WMenu *menu, char *buffer, int bufSize)
+{
+    Bool ok = True;
+    int len = 0;
+
+    if (!menu->flags.titled || !menu->frame->title[0])
+	return False;
+
+    len = strlen(menu->frame->title);
+    if (len >= bufSize)
+	return False;
+
+    if (menu->parent) {
+	ok = getMenuPath(menu->parent, buffer, bufSize - len - 1);
+	if (!ok)
+	    return False;
+    }
+
+    strcat(buffer, "\\");
+    strcat(buffer, menu->frame->title);
+
+    return True;
+}
+
+static Bool
+saveMenuRecurs(proplist_t menus, WScreen *scr, WMenu *menu)
+{
+    proplist_t key, value;
+    int save_menus = 0, i;
+    char buffer[512];
+    Bool ok = True;
+
+    
+    if (menu->flags.brother)
+	menu = menu->brother;
+
+    if (menu->flags.buttoned && menu != scr->switch_menu) {
+
+	sprintf(buffer, "%i,%i", menu->frame_x, menu->frame_y);
+	value = PLMakeString(buffer);
+
+	buffer[0] = '\0';
+	ok = getMenuPath(menu, buffer, 510);
+
+	if (ok) {
+	    key = PLMakeString(buffer);
+	    PLInsertDictionaryEntry(menus, key, value);
+	    PLRelease(key);
+	    PLRelease(value);
+	    save_menus = 1;
+	}
+    }
+
+    if (ok) {
+	for (i = 0; i < menu->cascade_no; i++) {
+	    if (saveMenuRecurs(menus, scr, menu->cascades[i]))
+		save_menus = 1;
+	}
+    }
+    return save_menus;
+}
+#endif /* !LITE */
+
+
 #define COMPLAIN(key) wwarning(_("bad value in menus state info:%s"), key)
 
 
 static int
 restoreMenu(WScreen *scr, proplist_t menu, int which)
 {
-    int i, x, y;
+    int x, y;
     WMenu *pmenu = NULL;
     
     if (!menu)
@@ -2280,27 +2569,12 @@ restoreMenu(WScreen *scr, proplist_t menu, int which)
     if (sscanf(PLGetString(menu), "%i,%i", &x, &y)!=2)
         COMPLAIN("Position");
 
-    if (which & WSS_ROOTMENU) {
-        OpenRootMenu(scr, x, y, False);
-        pmenu = scr->root_menu;
-    }
-    else if (which & WSS_SWITCHMENU) {
+#ifndef LITE
+    if (which & WSS_SWITCHMENU) {
         OpenSwitchMenu(scr, x, y, False);
         pmenu = scr->switch_menu;
     }
-    else if (which & WSS_WSMENU) {
-        OpenWorkspaceMenu(scr, x, y);
-        pmenu = scr->workspace_menu;
-        if (pmenu->parent) {
-            /* make parent map the copy in place of the original */
-            for (i=0; i<pmenu->parent->cascade_no; i++) {
-                if (pmenu->parent->cascades[i] == pmenu) {
-                    pmenu->parent->cascades[i] = pmenu->brother;
-                    break;
-                }
-            }
-        }
-    }
+#endif /* !LITE */
 
     if (pmenu) {
         int width = MENUW(pmenu);
@@ -2319,10 +2593,69 @@ restoreMenu(WScreen *scr, proplist_t menu, int which)
 }
 
 
+#ifndef LITE
+static int
+restoreMenuRecurs(WScreen *scr, proplist_t menus, WMenu *menu, char *path)
+{
+    proplist_t key, entry;
+    char buffer[512];
+    int i, x, y, res;
+
+    if (strlen(path) + strlen(menu->frame->title) > 510)
+	return False;
+
+    sprintf(buffer, "%s\\%s", path, menu->frame->title);
+    key = PLMakeString(buffer);
+    entry = PLGetDictionaryEntry(menus, key);
+    res = False;
+
+    if (entry && PLIsString(entry)) {
+
+	if (sscanf(PLGetString(entry), "%i,%i", &x, &y) != 2)
+	    COMPLAIN("Position");
+
+	if (!menu->flags.mapped) {
+	    int width = MENUW(menu);
+	    int height = MENUH(menu);
+
+	    wMenuMapAt(menu, x, y, False);
+
+	    if (menu->parent) {
+		/* make parent map the copy in place of the original */
+		for (i=0; i<menu->parent->cascade_no; i++) {
+		    if (menu->parent->cascades[i] == menu) {
+			menu->parent->cascades[i] = menu->brother;
+			break;
+		    }
+		}
+	    }
+	    x = (x < -width) ? 0 : x;
+	    x = (x > scr->scr_width) ? scr->scr_width - width : x;
+	    y = (y < 0) ? 0 : y;
+	    y = (y > scr->scr_height) ? scr->scr_height - height : y;
+	    wMenuMove(menu, x, y, True);
+	    menu->flags.buttoned = 1;
+	    wFrameWindowShowButton(menu->frame, WFF_RIGHT_BUTTON);
+	    res = True;
+	}
+    }
+    
+    PLRelease(key);
+
+    for (i=0; i<menu->cascade_no; i++) {
+	if (restoreMenuRecurs(scr, menus, menu->cascades[i], buffer) != False)
+	    res = True;
+    }
+
+    return res;
+}
+#endif /* !LITE */
+
+
 void
 wMenuRestoreState(WScreen *scr)
 {
-    proplist_t menus, menu, key, rkey, skey, wkey;
+    proplist_t menus, menu, key, skey;
 
     key = PLMakeString("Menus");
     menus = PLGetDictionaryEntry(scr->session_state, key);
@@ -2333,19 +2666,19 @@ wMenuRestoreState(WScreen *scr)
 
     /* restore menus */
 
-    rkey = PLMakeString("RootMenu");
     skey = PLMakeString("SwitchMenu");
-    wkey = PLMakeString("WorkspaceMenu");
-    menu = PLGetDictionaryEntry(menus, rkey);
-    restoreMenu(scr, menu, WSS_ROOTMENU);
     menu = PLGetDictionaryEntry(menus, skey);
     restoreMenu(scr, menu, WSS_SWITCHMENU);
-    menu = PLGetDictionaryEntry(menus, wkey);
-    restoreMenu(scr, menu, WSS_WSMENU);
 
-    PLRelease(rkey);
+#ifndef LITE
+    if (!scr->root_menu) {
+        OpenRootMenu(scr, scr->scr_width*2, 0, False);
+        wMenuUnmap(scr->root_menu);
+    }
+    restoreMenuRecurs(scr, menus, scr->root_menu, "");
+#endif /* !LITE */
+
     PLRelease(skey);
-    PLRelease(wkey);
 }
 
 
@@ -2355,11 +2688,12 @@ OpenWorkspaceMenu(WScreen *scr, int x, int y)
     WMenu *menu, *parent;
     WMenuEntry *entry;
 
+#ifndef LITE
     if (!scr->root_menu) {
         OpenRootMenu(scr, scr->scr_width*2, 0, False);
         wMenuUnmap(scr->root_menu);
     }
-
+#endif
     menu = scr->workspace_menu;
     if (menu) {
         if (menu->flags.mapped) {
@@ -2377,8 +2711,7 @@ OpenWorkspaceMenu(WScreen *scr, int x, int y)
                 wRaiseFrame(menu->frame->core);
 		wMenuMapCopyAt(menu, x, y);
             }
-        }
-        else {
+        } else {
             wMenuMapAt(menu, x, y, False);
         }
     }
