@@ -1,5 +1,5 @@
 /* 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -30,9 +30,12 @@
 #include <unistd.h>
 #include <pwd.h>
 #include <math.h>
+#include <time.h>
 
+#include <WUtil.h>
 #include <wraster.h>
 
+
 #include "WindowMaker.h"
 #include "GNUstep.h"
 #include "screen.h"
@@ -55,8 +58,6 @@ extern WPreferences wPreferences;
 
 extern Time LastTimestamp;
 
-
-
 #ifdef OFFIX_DND
 extern Atom _XA_DND_SELECTION;
 #endif
@@ -190,17 +191,12 @@ MakeCPPArgs(char *path)
 #endif /* USECPP */
 
 
-#define canFocusShaded(win) (win->frame->workspace==scr->current_workspace \
-                             && win->flags.shaded && \
-                             !(win->flags.miniaturized || win->flags.hidden))
-
-
 WWindow*
 NextFocusWindow(WScreen *scr)
 {
     WWindow *tmp, *wwin, *closest, *min;
     Window d;
-    
+
     if (!(wwin = scr->focused_window))
         return NULL;
     tmp = wwin->prev;
@@ -208,9 +204,8 @@ NextFocusWindow(WScreen *scr)
     min = wwin;
     d = 0xffffffff;
     while (tmp) {
-        if ((tmp->flags.mapped || canFocusShaded(tmp))
-            && !tmp->window_flags.no_focusable &&
-            !tmp->window_flags.skip_window_list) {
+        if (wWindowCanReceiveFocus(tmp) 
+	    && (!WFLAGP(tmp, skip_window_list)|| tmp->flags.internal_window)) {
 	    if (min->client_win > tmp->client_win)
 	      min = tmp;
 	    if (tmp->client_win > wwin->client_win
@@ -241,9 +236,8 @@ PrevFocusWindow(WScreen *scr)
     max = wwin;
     d = 0xffffffff;
     while (tmp) {
-	if ((tmp->flags.mapped || canFocusShaded(tmp))
-            && !tmp->window_flags.no_focusable &&
-            !tmp->window_flags.skip_window_list) {
+	if (wWindowCanReceiveFocus(tmp) &&
+            (!WFLAGP(tmp, skip_window_list) || tmp->flags.internal_window)) {
 	    if (max->client_win < tmp->client_win)
 	      max = tmp;
 	    if (tmp->client_win < wwin->client_win
@@ -261,6 +255,38 @@ PrevFocusWindow(WScreen *scr)
 }
 
 
+
+#if 0
+/*
+ * Is win2 below win1?
+ */
+static Bool
+isBelow(WWindow *win1, WWindow *win2)
+{
+    int i;
+    WCoreWindow *tmp;
+    
+    tmp = win1->frame->core->stacking->under;
+    while (tmp) {
+	if (tmp == win2->frame->core)
+	    return True;
+	tmp = tmp->stacking->under;
+    }
+    
+    for (i=win1->frame->core->stacking->window_level-1; i>=0; i--) {
+	tmp = win1->screen_ptr->stacking_list[i];
+	while (tmp) {
+	    if (tmp == win2->frame->core)
+		return True;
+	    tmp = tmp->stacking->under;
+	}
+    }
+    return True;
+}
+#endif
+
+
+
 /*
  *  XFetchName Wrapper
  *
@@ -277,7 +303,8 @@ char **winname;
     if (XGetWMName(dpy, win, &text_prop)) {
 	if (text_prop.value && text_prop.nitems > 0) {
 	    if (text_prop.encoding == XA_STRING) {
-		*winname = (char *)text_prop.value;
+		*winname = wstrdup((char *)text_prop.value);
+		XFree(text_prop.value);
 	    } else {
 		text_prop.nitems = strlen((char *)text_prop.value);
 		if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) >=
@@ -286,7 +313,8 @@ char **winname;
 		    *winname = wstrdup(*list);
 		    XFreeStringList(list);
 		} else {
-		    *winname = (char *)text_prop.value;
+		    *winname = wstrdup((char *)text_prop.value);
+		    XFree(text_prop.value);
 		}
 	    }
 	} else {
@@ -337,7 +365,8 @@ char **iconname;
 }
 
 
-static void eatExpose()
+static void 
+eatExpose()
 {
     XEvent event, foo;
     
@@ -359,8 +388,10 @@ static void eatExpose()
 void
 SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 {
+    time_t time0 = time(NULL);
     float dx, dy, x=from_x, y=from_y, sx, sy, px, py;
     int dx_is_bigger=0;
+
     /* animation parameters */
     static struct {
 	int delay;
@@ -385,18 +416,18 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
     }
 
     if (dx_is_bigger) {
-        px = dx / apars[wPreferences.icon_slide_speed].slowdown;
-        if (px < apars[wPreferences.icon_slide_speed].steps && px > 0)
-            px = apars[wPreferences.icon_slide_speed].steps;
-        else if (px > -apars[wPreferences.icon_slide_speed].steps && px < 0)
-            px = -apars[wPreferences.icon_slide_speed].steps;
+        px = dx / apars[(int)wPreferences.icon_slide_speed].slowdown;
+        if (px < apars[(int)wPreferences.icon_slide_speed].steps && px > 0)
+            px = apars[(int)wPreferences.icon_slide_speed].steps;
+        else if (px > -apars[(int)wPreferences.icon_slide_speed].steps && px < 0)
+            px = -apars[(int)wPreferences.icon_slide_speed].steps;
         py = (sx == 0 ? 0 : px*dy/dx);
     } else {
-        py = dy / apars[wPreferences.icon_slide_speed].slowdown;
-        if (py < apars[wPreferences.icon_slide_speed].steps && py > 0)
-            py = apars[wPreferences.icon_slide_speed].steps;
-        else if (py > -apars[wPreferences.icon_slide_speed].steps && py < 0)
-            py = -apars[wPreferences.icon_slide_speed].steps;
+        py = dy / apars[(int)wPreferences.icon_slide_speed].slowdown;
+        if (py < apars[(int)wPreferences.icon_slide_speed].steps && py > 0)
+            py = apars[(int)wPreferences.icon_slide_speed].steps;
+        else if (py > -apars[(int)wPreferences.icon_slide_speed].steps && py < 0)
+            py = -apars[(int)wPreferences.icon_slide_speed].steps;
         px = (sy == 0 ? 0 : py*dx/dy);
     }
 
@@ -409,26 +440,28 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
             y = (float)to_y;
 
         if (dx_is_bigger) {
-            px = px * (1.0 - 1/(float)apars[wPreferences.icon_slide_speed].slowdown);
-            if (px < apars[wPreferences.icon_slide_speed].steps && px > 0)
-                px = apars[wPreferences.icon_slide_speed].steps;
-            else if (px > -apars[wPreferences.icon_slide_speed].steps && px < 0)
-                px = -apars[wPreferences.icon_slide_speed].steps;
+            px = px * (1.0 - 1/(float)apars[(int)wPreferences.icon_slide_speed].slowdown);
+            if (px < apars[(int)wPreferences.icon_slide_speed].steps && px > 0)
+                px = apars[(int)wPreferences.icon_slide_speed].steps;
+            else if (px > -apars[(int)wPreferences.icon_slide_speed].steps && px < 0)
+                px = -apars[(int)wPreferences.icon_slide_speed].steps;
             py = (sx == 0 ? 0 : px*dy/dx);
         } else {
-            py = py * (1.0 - 1/(float)apars[wPreferences.icon_slide_speed].slowdown);
-            if (py < apars[wPreferences.icon_slide_speed].steps && py > 0)
-                py = apars[wPreferences.icon_slide_speed].steps;
-            else if (py > -apars[wPreferences.icon_slide_speed].steps && py < 0)
-                py = -apars[wPreferences.icon_slide_speed].steps;
+            py = py * (1.0 - 1/(float)apars[(int)wPreferences.icon_slide_speed].slowdown);
+            if (py < apars[(int)wPreferences.icon_slide_speed].steps && py > 0)
+                py = apars[(int)wPreferences.icon_slide_speed].steps;
+            else if (py > -apars[(int)wPreferences.icon_slide_speed].steps && py < 0)
+                py = -apars[(int)wPreferences.icon_slide_speed].steps;
             px = (sy == 0 ? 0 : py*dx/dy);
         }
 
         XMoveWindow(dpy, win, (int)x, (int)y);
         XFlush(dpy);
-        if (apars[wPreferences.icon_slide_speed].delay > 0) {
-            wusleep(apars[wPreferences.icon_slide_speed].delay*1000L);
+        if (apars[(int)wPreferences.icon_slide_speed].delay > 0) {
+            wusleep(apars[(int)wPreferences.icon_slide_speed].delay*1000L);
         }
+	if (time(NULL) - time0 > MAX_ANIMATION_TIME)
+	    break;
     }
     XMoveWindow(dpy, win, to_x, to_y);
 
@@ -439,21 +472,19 @@ SlideWindow(Window win, int from_x, int from_y, int to_x, int to_y)
 
 
 char*
-ShrinkString(WFont *font, char *string, int width)
+ShrinkString(WMFont *font, char *string, int width)
 {
-#ifndef I18N_MB
     int w, w1=0;
     int p;
     char *pos;
     char *text;
     int p1, p2, t;
-#endif
 
-#ifdef I18N_MB
-    return wstrdup(string);
-#else
+    if (wPreferences.multi_byte_text)
+	return wstrdup(string);
+
     p = strlen(string);
-    w = MyTextWidth(font->font, string, p);
+    w = WMWidthOfString(font, string, p);
     text = wmalloc(strlen(string)+8);
     strcpy(text, string);
     if (w<=width)
@@ -466,8 +497,8 @@ ShrinkString(WFont *font, char *string, int width)
     if (pos) {
 	*pos = 0;
 	p = strlen(text);
-	w1=MyTextWidth(font->font, text, p);
-	if (w1>width) {
+	w1 = WMWidthOfString(font, text, p);
+	if (w1 > width) {
 	    w1 = 0;
 	    p = 0;
 	    *pos = ' ';
@@ -483,14 +514,14 @@ ShrinkString(WFont *font, char *string, int width)
 	*text=0;
     }
     strcat(text, "...");
-    width -= MyTextWidth(font->font, "...", 3);
+    width -= WMWidthOfString(font, "...", 3);
     
     pos = string;
     p1=0;
     p2=p;
     t = (p2-p1)/2;
     while (p2>p1 && p1!=t) {
-	w = MyTextWidth(font->font, &string[p-t], t);
+	w = WMWidthOfString(font, &string[p-t], t);
 	if (w>width) {
 	    p2 = t;
 	    t = p1+(p2-p1)/2;
@@ -501,25 +532,26 @@ ShrinkString(WFont *font, char *string, int width)
 	  p2=p1=t;
     }
     strcat(text, &string[p-p1]);
+
     return text;
-#endif /* I18N_MB */
 }
 
 
 char*
-FindImage(char **paths, char *file)
+FindImage(char *paths, char *file)
 {
     char *tmp, *path;
-    
+
     tmp = strrchr(file, ':');
     if (tmp) {
 	*tmp = 0;
-	path = wfindfileinlist(paths, file);
+	path = wfindfile(paths, file);
 	*tmp = ':';
     }
     if (!tmp || !path) {
-	path = wfindfileinlist(paths, file);
+	path = wfindfile(paths, file);
     }
+
     return path;
 }
 
@@ -544,10 +576,11 @@ FlattenStringList(char **list, int count)
 	return NULL;
     }
 
-    strcpy(flat_string, list[0]);
-    for (i=1; i<count; i++) {
+    *flat_string = 0;
+    for (i=0; i<count; i++) {
 	if (list[i]!=NULL && list[i][0]!=0) {
-            strcat(flat_string, " ");
+	    if (i>0)
+		strcat(flat_string, " ");
             wspace = strpbrk(list[i], " \t");
             if (wspace)
                 strcat(flat_string, "\"");
@@ -556,7 +589,7 @@ FlattenStringList(char **list, int count)
                 strcat(flat_string, "\"");
 	}
     }
-    
+
     return flat_string;
 }
 
@@ -671,16 +704,25 @@ ParseCommand(char *command, char ***argv, int *argc)
     *argc = count;
 }
 
-
+#if 0
 static void
 timeup(void *foo)
 {
     *(int*)foo=1;
 }
-
+#endif
 static char*
 getselection(WScreen *scr)
 {
+    char *tmp;
+    extern char *W_GetTextSelection(); /* in WINGs */
+    
+    tmp = W_GetTextSelection(scr->wmscreen, XA_PRIMARY);
+    if (!tmp)
+	tmp = W_GetTextSelection(scr->wmscreen, XA_CUT_BUFFER0);
+    return tmp;
+
+#if 0
     XEvent event;
     int timeover=0;
     WMHandlerID *id;
@@ -706,6 +748,7 @@ getselection(WScreen *scr)
     }
     wwarning(_("selection timed-out"));
     return NULL;
+#endif
 }
 
 
@@ -713,31 +756,86 @@ static char*
 getuserinput(WScreen *scr, char *line, int *ptr)
 {
     char *ret;
-    char *tmp;
-    int i;
-    char buffer[256];
-    
-    if (line[*ptr]!='(') {
-	tmp = _("Program Arguments");
-    } else {
-	i = 0;
-	while (line[*ptr]!=0 && line[*ptr]!=')') {
-	    (*ptr)++;
-	    if (line[*ptr]!='\\') {
-		buffer[i++] = line[*ptr];
-	    } else {
-		(*ptr)++;
-		if (line[*ptr]==0)
-		    break;
-	    }
-	}
-	if (i>0)
-	    buffer[i-1] = 0;
-	tmp = (char*)buffer;
-    }
+    char *title;
+    char *prompt;
+    int j, state;
+    int begin = 0;
+    char tbuffer[256], pbuffer[256];
 
+    title = _("Program Arguments");
+    prompt = _("Enter command arguments:");
     ret = NULL;
-    if (wInputDialog(scr, tmp, _("Enter command arguments:"), &ret)!= WDB_OK)
+
+#define _STARTING 0
+#define _TITLE 1
+#define _PROMPT 2
+#define _DONE 3
+
+    state = _STARTING;
+    j = 0;
+    for (; line[*ptr]!=0 && state!=_DONE; (*ptr)++) {
+	switch (state) {
+	 case _STARTING:
+	    if (line[*ptr]=='(') {
+		state = _TITLE;
+		begin = *ptr+1;
+	    } else {
+		state = _DONE;
+	    }
+	    break;
+
+	 case _TITLE:
+	    if (j <= 0 && line[*ptr]==',') {
+
+		j = 0;
+		if (*ptr > begin) {
+		    strncpy(tbuffer, &line[begin], WMIN(*ptr-begin, 255));
+		    tbuffer[WMIN(*ptr-begin, 255)] = 0;
+		    title = (char*)tbuffer;
+		}
+		begin = *ptr+1;
+		state = _PROMPT;
+
+	    } else if (j <= 0 && line[*ptr]==')') {
+
+		if (*ptr > begin) {
+		    strncpy(tbuffer, &line[begin], WMIN(*ptr-begin, 255));
+		    tbuffer[WMIN(*ptr-begin, 255)] = 0;
+		    title = (char*)tbuffer;
+		}
+		state = _DONE;
+
+	    } else if (line[*ptr]=='(') {
+		j++;
+	    } else if (line[*ptr]==')') {
+		j--;
+	    }
+
+	    break;
+
+	 case _PROMPT:
+	    if (line[*ptr]==')' && j==0) {
+
+		if (*ptr-begin > 1) {
+		    strncpy(pbuffer, &line[begin], WMIN(*ptr-begin, 255));
+		    pbuffer[WMIN(*ptr-begin, 255)] = 0;
+		    prompt = (char*)pbuffer;
+		}
+		state = _DONE;
+	    } else if (line[*ptr]=='(')
+		    j++;
+	    else if (line[*ptr]==')')
+		j--;
+	    break;
+	}
+    }
+    (*ptr)--;
+#undef _STARTING
+#undef _TITLE
+#undef _PROMPT
+#undef _DONE
+
+    if (!wInputDialog(scr, title, prompt, &ret))
 	return NULL;
     else
 	return ret;
@@ -796,6 +894,7 @@ get_dnd_selection(WScreen *scr)
  * OPTION	w	NORMAL		<selected window id>
  * OPTION	a	NORMAL		<input text>
  * OPTION	d	NORMAL		<OffiX DND selection object>
+ * OPTION	W	NORMAL		<current workspace>
  * OPTION	etc.	NORMAL		%<input>
  */
 #define TMPBUFSIZE 64
@@ -806,7 +905,7 @@ ExpandOptions(WScreen *scr, char *cmdline)
     char *out, *nout;
     char *selection=NULL;
     char *user_input=NULL;
-#ifdef OFFIX_DND
+#if defined(OFFIX_DND) || defined(XDND)
     char *dropped_thing=NULL;
 #endif
     char tmpbuf[TMPBUFSIZE];
@@ -881,6 +980,21 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		}
 		break;
 		
+	     case 'W':
+		sprintf(tmpbuf, "0x%x", 
+			(unsigned int)scr->current_workspace + 1);
+		slen = strlen(tmpbuf);
+		olen += slen;
+		nout = realloc(out,olen);
+		if (!nout) {
+		    wwarning(_("out of memory during expansion of \"%W\""));
+		    goto error;
+		}
+		out = nout;
+		strcat(out,tmpbuf);
+		optr+=slen;
+		break;
+		
 	     case 'a':
 		ptr++;
 		user_input = getuserinput(scr, cmdline, &ptr); 
@@ -895,11 +1009,20 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		    out = nout;
 		    strcat(out,user_input);
 		    optr+=slen;
+		} else {
+		    /* Not an error, but user has Canceled the dialog box.
+		     * This will make the command to not be performed. */
+		    goto error;
 		}
 		break;
 
-#ifdef OFFIX_DND
+#if defined(OFFIX_DND) || defined(XDND)
 	     case 'd':
+#ifdef XDND
+        if(scr->xdestring) {
+            dropped_thing = wstrdup(scr->xdestring);
+        }
+#endif
 		if (!dropped_thing) {
 		    dropped_thing = get_dnd_selection(scr);
 		}
@@ -922,13 +1045,10 @@ ExpandOptions(WScreen *scr, char *cmdline)
 		
 	     case 's':
 		if (!selection) {
-		    if (!XGetSelectionOwner(dpy, XA_PRIMARY)) {
-			wwarning(_("selection not available"));
-			goto error;
-		    }
 		    selection = getselection(scr);
 		}
 		if (!selection) {
+		    wwarning(_("selection not available"));
 		    goto error;
 		}
 		slen = strlen(selection);
@@ -996,7 +1116,6 @@ void
 ParseWindowName(proplist_t value, char **winstance, char **wclass, char *where)
 {
     char *name;
-    char *dot;
 
     *winstance = *wclass = NULL;
 
@@ -1011,38 +1130,11 @@ ParseWindowName(proplist_t value, char **winstance, char **wclass, char *where)
 	return;
     }
 
-    dot = strchr(name, '.');
-    if (dot==name) {
-	*wclass = wstrdup(&name[1]);
-	*winstance = NULL;
-    } else if (!dot) {
-	*winstance = wstrdup(name);
-	*wclass = NULL;
-    } else {
-	*dot = 0;
-	*winstance = wstrdup(name);
-	*dot = '.'; /* restore old string */
-	*wclass = wstrdup(&dot[1]);
-    }
-}
-
-
-static char*
-catenate(char *text, char *str)
-{
-    int i;
-    
-    if (!text)
-	return wstrdup(str);
-    else {
-	i = strlen(text)+strlen(str)+2;
-	text = wrealloc(text, i);
-	strcat(text, str);
-	return text;
-    }
+    UnescapeWM_CLASS(name, winstance, wclass);
 }
 
 
+#if 0
 static char*
 keysymToString(KeySym keysym, unsigned int state)
 {
@@ -1066,18 +1158,20 @@ keysymToString(KeySym keysym, unsigned int state)
 
     return buf;
 }
-
+#endif
 
 char*
 GetShortcutString(char *text)
 {
     char *buffer = NULL;
-/*    char *tmp;*/
     char *k;
     int modmask = 0;
 /*    KeySym ksym;*/
     int control = 0;
+    char *tmp;
     
+    tmp = text = wstrdup(text);
+
     /* get modifiers */
     while ((k = strchr(text, '+'))!=NULL) {
 	int mod;
@@ -1091,40 +1185,238 @@ GetShortcutString(char *text)
 	modmask |= mod;
 	
 	if (strcasecmp(text, "Meta")==0) {
-	    buffer = catenate(buffer, "M+");
+	    buffer = wstrappend(buffer, "M+");
 	} else if (strcasecmp(text, "Alt")==0) {
-	    buffer = catenate(buffer, "A+");
+	    buffer = wstrappend(buffer, "A+");
 	} else if (strcasecmp(text, "Shift")==0) {
-	    buffer = catenate(buffer, "Sh+");
+	    buffer = wstrappend(buffer, "Sh+");
 	} else if (strcasecmp(text, "Mod1")==0) {
-	    buffer = catenate(buffer, "M1+"); 
+	    buffer = wstrappend(buffer, "M1+"); 
 	} else if (strcasecmp(text, "Mod2")==0) {
-	    buffer = catenate(buffer, "M2+"); 
+	    buffer = wstrappend(buffer, "M2+"); 
 	} else if (strcasecmp(text, "Mod3")==0) {
-	    buffer = catenate(buffer, "M3+");
+	    buffer = wstrappend(buffer, "M3+");
 	} else if (strcasecmp(text, "Mod4")==0) {
-	    buffer = catenate(buffer, "M4+"); 
+	    buffer = wstrappend(buffer, "M4+"); 
 	} else if (strcasecmp(text, "Mod5")==0) {
-	    buffer = catenate(buffer, "M5+"); 
+	    buffer = wstrappend(buffer, "M5+"); 
 	} else if (strcasecmp(text, "Control")==0) {
 	    control = 1;
 	} else {
-	    buffer = catenate(buffer, text);
+	    buffer = wstrappend(buffer, text);
 	}
 	text = k+1;
     }
 
     if (control) {
-	buffer = catenate(buffer, "^");
+	buffer = wstrappend(buffer, "^");
     }
-    buffer = catenate(buffer, text);
+    buffer = wstrappend(buffer, text);
     
     /* get key */
 /*    ksym = XStringToKeysym(text);
     tmp = keysymToString(ksym, modmask);
     puts(tmp);
-    buffer = catenate(buffer, tmp);
-    free(tmp);
+    buffer = wstrappend(buffer, tmp);
 */
+    free(tmp);
+
     return buffer;
 }
+
+
+char*
+EscapeWM_CLASS(char *name, char *class)
+{
+    char *ret;
+    char *ename = NULL, *eclass = NULL;
+    int i, j, l;
+
+    if (!name && !class)
+	return NULL;
+
+    if (name) {
+	l = strlen(name);
+	ename = wmalloc(l*2);
+	j = 0;
+	for (i=0; i<l; i++) {
+	    if (name[i]=='\\') {
+		ename[j++] = '\\';
+	    } else if (name[i]=='.') {
+		ename[j++] = '\\';
+	    }
+	    ename[j++] = name[i];
+	}
+	ename[j] = 0;
+    }
+    if (class) {
+	l = strlen(class);
+	eclass = wmalloc(l*2);
+	j = 0;
+	for (i=0; i<l; i++) {
+	    if (class[i]=='\\') {
+		eclass[j++] = '\\';
+	    } else if (class[i]=='.') {
+		eclass[j++] = '\\';
+	    }
+	    eclass[j++] = class[i];
+	}
+	eclass[j] = 0;
+    }
+
+    if (ename && eclass) {
+	ret = wmalloc(strlen(ename)+strlen(eclass)+4);
+	sprintf(ret, "%s.%s", ename, eclass);
+	free(ename);
+	free(eclass);
+    } else if (ename) {
+	ret = wstrdup(ename);
+	free(ename);
+    } else {
+	ret = wstrdup(eclass);
+	free(eclass);
+    }
+
+    return ret;
+}
+
+
+void
+UnescapeWM_CLASS(char *str, char **name, char **class)
+{
+    int i, j, k, dot;
+    Bool esc;
+
+    j = strlen(str);
+    *name = wmalloc(j);
+    **name = 0;
+    *class = wmalloc(j);
+    **class = 0;
+
+    /* separate string in 2 parts */
+    esc = False;
+    dot = 0;
+    for (i = 0; i < j; i++) {
+	if (!esc) {
+	    if (str[i]=='\\') {
+		esc = True;
+	    } else if (str[i]=='.') {
+		dot = i;
+		break;
+	    }
+	} else {
+	    esc = False;
+	}
+    }
+
+    /* unescape strings */
+    esc = False;
+    k = 0;
+    for (i = 0; i < dot; i++) {
+	if (!esc) {
+	    if (str[i]=='\\') {
+		esc = True;
+	    } else {
+		(*name)[k++] = str[i];
+	    }
+	} else {
+	    (*name)[k++] = str[i];
+	    esc = False;
+	}
+    }
+    (*name)[k] = 0;
+
+    esc = False;
+    k = 0;
+    for (i = dot+1; i<j; i++) {
+	if (!esc) {
+	    if (str[i]=='\\') {
+		esc = True;
+	    } else {
+		(*class)[k++] = str[i];
+	    }
+	} else {
+	    esc = False;
+	}
+    }
+    (*class)[k] = 0;
+    
+    if (!*name) {
+	free(*name);
+	*name = NULL;
+    }
+    if (!*class) {
+	free(*class);
+	*class = NULL;
+    }
+}
+
+
+
+void
+SendHelperMessage(WScreen *scr, char type, int workspace, char *msg)
+{
+    unsigned char *buffer;
+    int len;
+    int i;
+    char buf[16];
+
+    if (!scr->flags.backimage_helper_launched) {
+	return;
+    }
+    
+    len = (msg ? strlen(msg) : 0) + (workspace >=0 ? 4 : 0) + 1 ;
+    buffer = wmalloc(len+5);
+    sprintf(buf, "%4i", len);
+    memcpy(buffer, buf, 4);
+    buffer[4] = type;
+    i = 5;
+    if (workspace >= 0) {
+	sprintf(buf, "%4i", workspace);
+	memcpy(&buffer[i], buf, 4);
+	i += 4;
+	buffer[i] = 0;
+    }
+    if (msg)
+	strcpy(&buffer[i], msg);
+
+    if (write(scr->helper_fd, buffer, len+4) < 0) {
+	wsyserror(_("could not send message to background image helper"));
+    }
+    free(buffer);
+}
+
+
+void
+ExecuteShellCommand(WScreen *scr, char *command)
+{
+    static char *shell = NULL;
+    pid_t pid;
+
+    /*
+     * This have a problem: if the shell is tcsh (not sure about others)
+     * and ~/.tcshrc have /bin/stty erase ^H somewhere on it, the shell
+     * will block and the command will not be executed.
+    if (!shell) {
+	shell = getenv("SHELL");
+	if (!shell)
+	    shell = "/bin/sh";
+    }
+     */
+    shell = "/bin/sh";
+
+    pid = fork();
+    if (pid==0) {
+
+	SetupEnvironment(scr);
+
+#ifdef HAVE_SETPGID
+	setpgid(0, 0);
+#endif
+	execl(shell, shell, "-c", command, NULL);
+	wsyserror("could not execute %s -c %s", shell, command);
+	Exit(-1);
+    } else if (pid < 0) {
+	wsyserror("cannot fork a new process");
+    }
+}
