@@ -1,8 +1,9 @@
 /* rootmenu.c- user defined menu
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
+ *  Copyright (c) 1998       Dan Pascu
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +23,8 @@
 
 #include "wconfig.h"
 
+#ifndef LITE
+
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,12 +60,15 @@
 
 extern char *Locale;
 
-extern wCursor[WCUR_LAST];
+extern WDDomain *WDRootMenu;
+
+extern Cursor wCursor[WCUR_LAST];
 
 extern Time LastTimestamp;
 
 extern WPreferences wPreferences;
 
+extern int wScreenCount;
 
 static WMenu *readMenuPipe(WScreen *scr, char **file_name);
 static WMenu *readMenuFile(WScreen *scr, char *file_name);
@@ -108,6 +114,7 @@ static Shortcut *shortcutList = NULL;
  * REFRESH - forces the desktop to be repainted
  * EXIT [QUICK] - exit the window manager [without confirmation]
  * EXEC <program> - execute an external program
+ * SHEXEC <command> - execute a shell command
  * WORKSPACE_MENU - places the workspace submenu
  * ARRANGE_ICONS
  * RESTART [<window manager>] - restarts the window manager
@@ -132,12 +139,15 @@ static Shortcut *shortcutList = NULL;
  *		  CLEAR_SESSION is used. If SaveSessionOnExit = Yes; in
  *		  WindowMaker domain file, then saving is automatically
  *		  done on every windowmaker exit, overwriting any
- *		  SAVE_SESSION or CLEAR_SESSION (see below).
+ *		  SAVE_SESSION or CLEAR_SESSION (see below). Also save
+ *		  dock state now.
  * CLEAR_SESSION - clears any previous saved session. This will not have
  *		  any effect if SaveSessionOnExit is True.
  *							     
  */
 
+#define MAX(a,b)	((a)>(b) ? (a) : (b))
+
 
 #define M_QUICK		1
 
@@ -147,49 +157,16 @@ static void
 execCommand(WMenu *menu, WMenuEntry *entry)
 {
     char *cmdline;
-#if 0
-    char *path, *chr;
-#endif
+
     cmdline = ExpandOptions(menu->frame->screen_ptr, (char*)entry->clientdata);
-#if 0
-    path = wstrdup(cmdline);
-    chr = strchr(path, ' ');
-    if (chr)
-	*chr = 0;
-    
-    if (access(path, X_OK)!=0) {
-	path = realloc(path, strlen(cmdline)+128);
-	sprintf(path, _("Program \"%s\" not found or cannot be executed."),
-		cmdline);
-	wMessageDialog(menu->frame->screen_ptr, _("Error"), path, WD_ERROR);
-	free(path);
-	free(cmdline);
-	return;
-    }
-    
-    free(path);
-#endif
+
     XGrabPointer(dpy, menu->frame->screen_ptr->root_win, True, 0,
 		 GrabModeAsync, GrabModeAsync, None, wCursor[WCUR_WAIT],
 		 CurrentTime);
     XSync(dpy, 0);
 
     if (cmdline) {
-        /* We should not use ParseCommand() here, because it does not handle
-         * correctly '~/' expansion and '>' redirection.
-         * While we can do '~/' expansion ourselves, we can do nothing about
-         * '>' redirection.
-         * So we better let /bin/sh to do that for us. Dan.
-	 * Ok. -Alfredo
-         */
-        if (fork()==0) {
-	    close(ConnectionNumber(dpy));
-#ifdef HAVE_SETPGID
-            setpgid(0, 0);
-#endif
-            execl("/bin/sh", "/bin/sh", "-c", cmdline, NULL);
-            exit(-1);
-        }
+	ExecuteShellCommand(menu->frame->screen_ptr, cmdline);
 	free(cmdline);
     }
     XUngrabPointer(dpy, CurrentTime);
@@ -200,51 +177,97 @@ execCommand(WMenu *menu, WMenuEntry *entry)
 static void
 exitCommand(WMenu *menu, WMenuEntry *entry)
 {
+    static int inside = 0;
+
     /* prevent reentrant calls */
-    wMenuSetEnabled(menu, entry->order, False);
-    if ((int)entry->clientdata==M_QUICK
+    if (inside)
+	return;
+    inside = 1;
+
+    if ((long)entry->clientdata==M_QUICK
 	|| wMessageDialog(menu->frame->screen_ptr, _("Exit"),
-		_("Exit window manager?"), WD_EXIT_CONFIRM)==WDB_EXIT) {
+			  _("Exit window manager?"), 
+			  _("Exit"), _("Cancel"), NULL)==WAPRDefault) {
 #ifdef DEBUG
 	printf("Exiting WindowMaker.\n");
 #endif
-
-	wScreenSaveState(menu->frame->screen_ptr);
-	
-	RestoreDesktop(menu->frame->screen_ptr);
-	exit(0);
+	Shutdown(WSExitMode);
     }
-    wMenuSetEnabled(menu, entry->order, True);
+    inside = 0;
 }
 
 
 static void
 shutdownCommand(WMenu *menu, WMenuEntry *entry)
 {
-    /* prevent reentrant calls */
-    wMenuSetEnabled(menu, entry->order, False);
-    if ((int)entry->clientdata==M_QUICK
-	|| wMessageDialog(menu->frame->screen_ptr, _("Close X session"), 
-	  _("Close Window System session?\n(all applications will be closed)"),
-			  WD_EXIT_CONFIRM)==WDB_EXIT) {
-	printf(_("Exiting...\n"));
-	
-	wScreenSaveState(menu->frame->screen_ptr);
+    static int inside = 0;
+    int result;
 
- 	WipeDesktop(menu->frame->screen_ptr);
-	exit(0);
+    /* prevent reentrant calls */
+    if (inside)
+	return;
+    inside = 1;
+
+#define R_CANCEL 0
+#define R_CLOSE 1
+#define R_KILL 2
+
+    
+    result = R_CANCEL;
+    if ((long)entry->clientdata==M_QUICK)
+	result = R_CLOSE;
+    else {
+#ifdef XSMP_ENABLED
+	if (wSessionIsManaged()) {
+	    int r;
+	    
+	    r = wMessageDialog(menu->frame->screen_ptr, 
+			       _("Close X session"), 
+			       _("Close Window System session?\n"
+				 "Kill might close applications with unsaved data."),
+			       _("Close"), _("Kill"), _("Cancel"));
+	    if (r==WAPRDefault)
+		result = R_CLOSE;
+	    else if (r==WAPRAlternate)
+		result = R_KILL;
+	} else
+#endif
+	{
+	    int r;
+
+	    r = wMessageDialog(menu->frame->screen_ptr, 
+			       _("Kill X session"), 
+			       _("Kill Window System session?\n"
+				 "(all applications will be closed)"),
+			       _("Kill"), _("Cancel"), NULL);
+	    if (r==WAPRDefault)
+		result = R_KILL;
+	}
+    } 
+    
+    if (result!=R_CANCEL) {
+#ifdef XSMP_ENABLED
+	if (result == R_CLOSE) {
+	    Shutdown(WSLogoutMode);
+	} else 
+#endif /* XSMP_ENABLED */
+	{
+	    Shutdown(WSKillMode);
+	}
     }
-    wMenuSetEnabled(menu, entry->order, True);
+#undef R_CLOSE
+#undef R_CANCEL
+#undef R_KILL
+    inside = 0;
 }
 
 
 static void
 restartCommand(WMenu *menu, WMenuEntry *entry)
 {
-    wScreenSaveState(menu->frame->screen_ptr);
-
-    RestoreDesktop(menu->frame->screen_ptr);
-    Restart((char*)entry->clientdata);
+    Shutdown(WSRestartPreparationMode);
+    Restart((char*)entry->clientdata, False);
+    Restart(NULL, True);
 }
 
 
@@ -277,7 +300,10 @@ hideOthersCommand(WMenu *menu, WMenuEntry *entry)
 static void
 saveSessionCommand(WMenu *menu, WMenuEntry *entry)
 {
-    wSessionSaveState(menu->frame->screen_ptr);
+    if (!wPreferences.save_session_on_exit)
+	wSessionSaveState(menu->frame->screen_ptr);
+
+    wScreenSaveState(menu->frame->screen_ptr);
 }
 
 
@@ -291,24 +317,57 @@ clearSessionCommand(WMenu *menu, WMenuEntry *entry)
 static void
 infoPanelCommand(WMenu *menu, WMenuEntry *entry)
 {
-    wMenuSetEnabled(menu, entry->order, False);
     wShowInfoPanel(menu->frame->screen_ptr);
-    wMenuSetEnabled(menu, entry->order, True);
 }
 
 
 static void
 legalPanelCommand(WMenu *menu, WMenuEntry *entry)
 {
-    wMenuSetEnabled(menu, entry->order, False);
     wShowLegalPanel(menu->frame->screen_ptr);
-    wMenuSetEnabled(menu, entry->order, True);
 }
 
 
-
 /********************************************************************/
 
+
+static char*
+getLocalizedMenuFile(char *menu)
+{
+    char *buffer;
+    char *ptr;
+
+    if (!Locale)
+	return NULL;
+
+    buffer = wmalloc(strlen(menu)+32);
+
+    /* try menu.locale_name */
+    sprintf(buffer, "%s.%s", menu, Locale);
+    if (access(buffer, F_OK)==0) {
+	return buffer;
+    }
+    /* check if it is in the form aa_bb.encoding and check for aa_bb */
+    ptr = strchr(Locale, '.');
+    if (ptr) {
+	*ptr = 0;
+	if (access(buffer, F_OK)==0) {
+	    return buffer;
+	}
+    }
+    /* now check for aa */
+    ptr = strchr(buffer, '_');
+    if (ptr) {
+	*ptr = 0;
+	if (access(buffer, F_OK)==0) {
+	    return buffer;
+	}
+    }
+
+    return NULL;
+}
+
+
 static void
 raiseMenus(WMenu *menu)
 {
@@ -333,7 +392,7 @@ wRootMenuPerformShortcut(XEvent *event)
     int done = 0;
     
     /* ignore CapsLock */
-    modifiers = event->xkey.state & VALID_MOD_MASK;
+    modifiers = event->xkey.state & ValidModMask;
     
     for (ptr = shortcutList; ptr!=NULL; ptr = ptr->next) {
 	if (ptr->keycode==0) 
@@ -380,10 +439,10 @@ rebindKeygrabs(WScreen *scr)
     while (wwin!=NULL) {
 	XUngrabKey(dpy, AnyKey, AnyModifier, wwin->frame->core->window);
 
-	if (!wwin->window_flags.no_bind_keys) {
+	if (!WFLAGP(wwin, no_bind_keys)) {
 	    wWindowSetKeyGrabs(wwin);
 	}
-	wwin = wwin->next;
+	wwin = wwin->prev;
     }
 }
 
@@ -432,8 +491,7 @@ addShortcut(char *file, char *shortcutDefinition, WMenu *menu,
 	*k = 0;
 	mod = wXModifierFromKey(b);
 	if (mod<0) {
-	    wwarning(_("%s:invalid key modifier \"%s\""), file,
-		     shortcutDefinition);
+	    wwarning(_("%s:invalid key modifier \"%s\""), file, b);
 	    free(ptr);
 	    return False;
 	}
@@ -540,6 +598,7 @@ next_token(char *line, char **next)
     return ret;
 }
 
+
 static void
 separateCommand(char *line, char ***file, char **command)
 {
@@ -615,38 +674,21 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
         while(path[i] != NULL) {
 	    char *tmp;
 
+	    if (strcmp(path[i], "-noext")==0) {
+		i++;
+		continue;
+	    }
+	    
 	    tmp = wexpandpath(path[i]);
 	    free(path[i]);
-	    path[i] = tmp;
-
-            if (Locale) {
-                lpath = wmalloc(strlen(path[i])+32);
-
-                strcpy(lpath, path[i]);
-                strcat(lpath, ".");
-                strcat(lpath, Locale);
-                if (stat(lpath, &stat_buf)<0) {
-                    int i;
-                    i = strlen(Locale);
-                    if (i>2) {
-                        lpath[strlen(lpath)-(i-2)]=0;
-                        if (stat(lpath, &stat_buf)==0) {
-                            free(path[i]);
-                            path[i] = lpath;
-                            lpath = NULL;
-                        }
-                    }
-                } else {
-                    free(path[i]);
-                    path[i] = lpath;
-                    lpath = NULL;
-                }
-            }
-
-            if (lpath) {
-                free(lpath);
-                lpath = NULL;
-            }
+	    lpath = getLocalizedMenuFile(tmp);
+	    if (lpath) {
+		free(tmp);
+		path[i] = lpath;
+		lpath = NULL;
+	    } else {
+		path[i] = tmp;
+	    }
 
             if (stat(path[i], &stat_buf)==0) {
                 if (last < stat_buf.st_mtime)
@@ -662,7 +704,7 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
         }
 
         if (first < 0) {
-            wsyserror(_("%s:could not stat menu :%s"), "OPEN_MENU",
+            wsyserror(_("%s:could not stat menu:%s"), "OPEN_MENU",
                       (char*)entry->clientdata);
             goto finish;
         }
@@ -673,7 +715,7 @@ constructMenu(WMenu *menu, WMenuEntry *entry)
 	    if (S_ISDIR(stat_buf.st_mode)) {
 		/* menu directory */
 		submenu = readMenuDirectory(menu->frame->screen_ptr,
-                                            entry->text, &path[first], cmd);
+                                            entry->text, path, cmd);
 		if (submenu)
 		    submenu->timestamp = last;
 	    } else if (S_ISREG(stat_buf.st_mode)) {
@@ -723,7 +765,7 @@ addWorkspaceMenu(WScreen *scr, WMenu *menu, char *title)
 	
 	wsmenu = wWorkspaceMenuMake(scr, True);
 	scr->workspace_menu = wsmenu;
-	entry=wMenuAddCallback(menu, title, NULL, NULL);
+	entry = wMenuAddCallback(menu, title, NULL, NULL);
 	wMenuEntrySetCascade(menu, entry, wsmenu);
 	
 	wWorkspaceMenuUpdate(scr, wsmenu);
@@ -766,7 +808,18 @@ addMenuEntry(WMenu *menu, char *title, char *shortcut, char *command,
 	    wwarning(_("%s:missing parameter for menu command \"%s\""),
 		     file_name, command);
 	else {
-	    entry = wMenuAddCallback(menu, title, execCommand, wstrdup(params));
+	    entry = wMenuAddCallback(menu, title, execCommand, 
+				     wstrappend("exec ", params));
+	    entry->free_cdata = free;
+	    shortcutOk = True;
+	}
+    } else if (strcmp(command, "SHEXEC")==0) {
+	if (!params)
+	    wwarning(_("%s:missing parameter for menu command \"%s\""),
+		     file_name, command);
+	else {
+	    entry = wMenuAddCallback(menu, title, execCommand,
+				     wstrdup(params));
 	    entry->free_cdata = free;
 	    shortcutOk = True;
 	}
@@ -1224,6 +1277,25 @@ myCompare(dir_data *d1, dir_data *d2)
 
 
 /************  Menu Configuration From Directory   *************/
+
+static Bool
+isFilePackage(char *file)
+{
+    int l;
+
+    /* check if the extension indicates this file is a 
+     * file package. For now, only recognize .themed */
+
+    l = strlen(file);
+
+    if (l > 7 && strcmp(&(file[l-7]), ".themed")==0) {
+	return True;
+    } else {
+	return False;
+    }
+}
+
+
 static WMenu*
 readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
 {
@@ -1235,9 +1307,16 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
     LinkedList *dirs = NULL, *files = NULL;
     int length, i, have_space=0;
     dir_data *data;
+    int stripExtension = 0;
 
     i=0;
     while (path[i]!=NULL) {
+	if (strcmp(path[i], "-noext")==0) {
+	    stripExtension = 1;
+	    i++;
+	    continue;
+	}
+
         dir = opendir(path[i]);
         if (!dir) {
             i++;
@@ -1250,7 +1329,10 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
                 strcmp(dentry->d_name, "..")==0)
                 continue;
 
-            buffer = wmalloc(strlen(path[i])+strlen(dentry->d_name)+4);
+	    if (dentry->d_name[0] == '.')
+		continue;
+
+            buffer = malloc(strlen(path[i])+strlen(dentry->d_name)+4);
             if (!buffer) {
                 wsyserror(_("out of memory while constructing directory menu %s"),
                          path[i]);
@@ -1265,32 +1347,36 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
                 wsyserror(_("%s:could not stat file \"%s\" in menu directory"),
                           path[i], dentry->d_name);
             } else {
-                data = (dir_data*) wmalloc(sizeof(dir_data));
-                data->name = wstrdup(dentry->d_name);
-                data->index = i;
-                if (S_ISDIR(stat_buf.st_mode)) {
+		Bool isFilePack = False;
+
+		data = NULL;
+                if (S_ISDIR(stat_buf.st_mode)
+		    && !(isFilePack = isFilePackage(dentry->d_name))) {
+
                     /* access always returns success for user root */
                     if (access(buffer, X_OK)==0) {
                         /* Directory is accesible. Add to directory list */
+
+			data = (dir_data*) wmalloc(sizeof(dir_data));
+			data->name = wstrdup(dentry->d_name);
+			data->index = i;
+
                         list_insert_sorted(data, &dirs, (int(*)())myCompare);
-                        data = NULL;
                     }
-                } else if (S_ISREG(stat_buf.st_mode)) {
+                } else if (S_ISREG(stat_buf.st_mode) || isFilePack) {
                     /* Hack because access always returns X_OK success for user root */
 #define S_IXANY (S_IXUSR | S_IXGRP | S_IXOTH)
                     if ((command!=NULL && access(buffer, R_OK)==0) ||
                         (command==NULL && access(buffer, X_OK)==0 &&
                          (stat_buf.st_mode & S_IXANY))) {
+
+			data = (dir_data*) wmalloc(sizeof(dir_data));
+			data->name = wstrdup(dentry->d_name);
+			data->index = i;
+
                         list_insert_sorted(data, &files, (int(*)())myCompare);
-                        data = NULL;
                     }
                 }
-                if (data!=NULL) {
-                    if (data->name)
-                        free(data->name);
-                    free(data);
-                    data = NULL;
-                }
             }
             free(buffer);
         }
@@ -1313,7 +1399,7 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
         length = strlen(path[d->index])+strlen(d->name)+6;
         if (command)
             length += strlen(command) + 6;
-        buffer = wmalloc(length);
+        buffer = malloc(length);
         if (!buffer) {
             wsyserror(_("out of memory while constructing directory menu %s"),
                      path[d->index]);
@@ -1386,7 +1472,12 @@ readMenuDirectory(WScreen *scr, char *title, char **path, char *command)
         if (have_space)
             strcat(buffer, "\"");
 
-        addMenuEntry(menu, f->name, NULL, "EXEC", buffer, path[f->index]);
+	if (stripExtension) {
+	    char *ptr = strrchr(f->name, '.');
+	    if (ptr && ptr!=f->name)
+		*ptr = 0;
+	}
+        addMenuEntry(menu, f->name, NULL, "SHEXEC", buffer, path[f->index]);
 
         free(buffer);
         if (files->head) {
@@ -1410,11 +1501,16 @@ makeDefaultMenu(WScreen *scr)
     
     menu = wMenuCreate(scr, _("Commands"), True);
     wMenuAddCallback(menu, "XTerm", execCommand, "xterm");
+    wMenuAddCallback(menu, "rxvt", execCommand, "rxvt");
+    wMenuAddCallback(menu, _("Restart"), restartCommand, NULL);
     wMenuAddCallback(menu, _("Exit..."), exitCommand, NULL);
     return menu;
 }
 
 
+
+
+
 /*
  *---------------------------------------------------------------------- 
  * configureMenu--
@@ -1435,59 +1531,48 @@ configureMenu(WScreen *scr, proplist_t definition)
     if (PLIsString(definition)) {
 	struct stat stat_buf;
 	char *path = NULL;
+	Bool menu_is_default = False;
 
 	/* menu definition is a string. Probably a path, so parse the file */
-	
+
 	tmp = wexpandpath(PLGetString(definition));
 
-	if (Locale) {
-	    path = wmalloc(strlen(tmp)+32);
-
-	    strcpy(path, tmp);
-	    strcat(path, ".");
-	    strcat(path, Locale);
-
-	    /* look for menu.xy */
-	    if (stat(path, &stat_buf)<0) {		
-		int i;
-		i = strlen(Locale);
-		if (i>2) {
-		    path[strlen(path)-(i-2)]=0;
-		    /* look for menu.xy_zw */
-		    if (stat(path, &stat_buf)<0) {
-			free(path);
-			/* If did not find any localized menus, try
-			 * only menu. This can also mean that
-			 * the path in WMRootMenu was already the
-			 * path for the localized menu (eg: menu = "menu.ab")
-			 */
-			path = NULL;
-		    }
-		}
-	    }
-	}
+	path = getLocalizedMenuFile(tmp);
 
 	if (!path)
 	    path = wfindfile(DEF_CONFIG_PATHS, tmp);
 
 	if (!path) {
-	    wsyserror(_("%s:could not find menu file \"%s\""), "WMRootMenu",
+	    path = wfindfile(DEF_CONFIG_PATHS, DEF_MENU_FILE);
+	    menu_is_default = True;
+	}
+
+	if (!path) {
+	    wsyserror(_("could not find menu file \"%s\" referenced in WMRootMenu"),
 		     tmp);
 	    free(tmp);
 	    return NULL;
 	}
-	
+
 	if (stat(path, &stat_buf)<0) {
-	    wsyserror(_("%s:could not access menu \"%s\""), "WMRootMenu", path);
+	    wsyserror(_("could not access menu \"%s\" referenced in WMRootMenu"), path);
 	    free(path);
 	    free(tmp);
 	    return NULL;
 	}
 
-	if (!scr->root_menu || stat_buf.st_mtime > scr->root_menu->timestamp) {
+	if (!scr->root_menu || stat_buf.st_mtime > scr->root_menu->timestamp
+	    /* if the pointer in WMRootMenu has changed */
+	    || WDRootMenu->timestamp > scr->root_menu->timestamp) {
+	    
+	    if (menu_is_default) {
+		wwarning(_("using default menu file \"%s\" as the menu referenced in WMRootMenu could not be found "),
+			 path);
+	    }
+
 	    menu = readMenuFile(scr, path);
 	    if (menu)
-		menu->timestamp = stat_buf.st_mtime;
+		menu->timestamp = MAX(stat_buf.st_mtime, WDRootMenu->timestamp);
 	} else {
 	    menu = NULL;
 	}
@@ -1514,9 +1599,30 @@ configureMenu(WScreen *scr, proplist_t definition)
     menu = wMenuCreate(scr, mtitle, False);
     menu->on_destroy = removeShortcutsForMenu;
 
+#ifdef GLOBAL_SUBMENU_FILE
+    {
+	WMenu *submenu;
+	WMenuEntry *mentry;
+
+	submenu = readMenuFile(scr, GLOBAL_SUBMENU_FILE);
+
+	if (submenu) {
+	    mentry = wMenuAddCallback(menu, submenu->frame->title, NULL, NULL);
+	    wMenuEntrySetCascade(menu, mentry, submenu);
+	}
+    }
+#endif
+
     for (i=1; i<count; i++) {
 	elem = PLGetArrayElement(definition, i);
-
+#if 0
+	if (PLIsString(elem)) {
+	    char *file;
+	    
+	    file = PLGetString(elem);
+	    
+	}
+#endif	
 	if (!PLIsArray(elem) || PLGetNumberOfElements(elem) < 2)
 	    goto error;
 	
@@ -1616,14 +1722,13 @@ OpenRootMenu(WScreen *scr, int x, int y, int keyboard)
 	    if (keyboard)
 		wMenuMapAt(menu, 0, 0, True);
 	    else
-		wMenuMapCopyAt(menu, x-menu->frame->core->width/2, 
-			       y-menu->frame->top_width/2);
+		wMenuMapCopyAt(menu, x-menu->frame->core->width/2, y);
 	}
 	return;
     }
     
     
-    definition = scr->droot_menu->dictionary;
+    definition = WDRootMenu->dictionary;
 
 /*
     definition = PLGetDomain(domain);
@@ -1631,10 +1736,11 @@ OpenRootMenu(WScreen *scr, int x, int y, int keyboard)
     if (definition) {
 	if (PLIsArray(definition)) {
 	    if (!scr->root_menu 
-		|| scr->droot_menu->timestamp > scr->root_menu->timestamp) {
+		|| WDRootMenu->timestamp > scr->root_menu->timestamp) {
 		menu = configureMenu(scr, definition);
 		if (menu)
-		    menu->timestamp = scr->droot_menu->timestamp;
+		    menu->timestamp = WDRootMenu->timestamp;
+
 	    } else
 		menu = NULL;
 	} else {
@@ -1645,6 +1751,12 @@ OpenRootMenu(WScreen *scr, int x, int y, int keyboard)
     if (!menu) {
 	/* menu hasn't changed or could not be read */
 	if (!scr->root_menu) {
+	    wMessageDialog(scr, _("Error"),
+			  _("The applications menu could not be loaded. "
+			    "Look at the console output for a detailed "
+			    "description of the errors."),
+			   _("OK"), NULL, NULL);
+
 	    menu = makeDefaultMenu(scr);
 	    scr->root_menu = menu;
 	}
@@ -1656,11 +1768,11 @@ OpenRootMenu(WScreen *scr, int x, int y, int keyboard)
 	scr->root_menu = menu;
     }
     if (menu) {
-	wMenuMapAt(menu, x-menu->frame->core->width/2, y-menu->frame->top_width/2,
-		   keyboard);
+	wMenuMapAt(menu, x-menu->frame->core->width/2, y, keyboard);
     }
     
     if (scr->flags.root_menu_changed_shortcuts)
 	rebindKeygrabs(scr);
 }
 
+#endif /* !LITE */
