@@ -1,6 +1,6 @@
 /* screen.c - screen management
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -28,9 +28,13 @@
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include <X11/Xatom.h>
 #ifdef SHAPE
 #include <X11/extensions/shape.h>
 #endif
+#ifdef KEEP_XKB_LOCK_STATUS     
+#include <X11/XKBlib.h>         
+#endif /* KEEP_XKB_LOCK_STATUS */
 
 #include <wraster.h>
 
@@ -47,34 +51,62 @@
 #include "resources.h"
 #include "workspace.h"
 #include "session.h"
+#include "balloon.h"
+#ifdef KWM_HINTS
+# include "kwm.h"
+#endif
+#ifdef GNOME_STUFF
+# include "gnome.h"
+#endif
+#ifdef OLWM_HINTS
+# include "openlook.h"
+#endif
 
 #include <proplist.h>
 
 #include "defaults.h"
 
 
+#ifdef LITE
+#define EVENT_MASK (LeaveWindowMask|EnterWindowMask|PropertyChangeMask\
+		|SubstructureNotifyMask|PointerMotionMask \
+		|SubstructureRedirectMask|KeyPressMask|KeyReleaseMask)
+#else
 #define EVENT_MASK (LeaveWindowMask|EnterWindowMask|PropertyChangeMask\
 		|SubstructureNotifyMask|PointerMotionMask \
 		|SubstructureRedirectMask|ButtonPressMask|ButtonReleaseMask\
 		|KeyPressMask|KeyReleaseMask)
-
-
-#define STIPPLE_WIDTH 2
-#define STIPPLE_HEIGHT 2
-static char STIPPLE_DATA[] = {0x02, 0x01};
-
-
-static proplist_t dApplications=NULL;
-static proplist_t dWorkspace;
-static proplist_t dDock;
-static proplist_t dClip;
-
+#endif
 
 /**** Global variables ****/
 
 extern Cursor wCursor[WCUR_LAST];
 extern WPreferences wPreferences;
 extern Atom _XA_WINDOWMAKER_STATE;
+extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
+
+
+extern int wScreenCount;
+
+#ifdef KEEP_XKB_LOCK_STATUS     
+extern int wXkbSupported;
+#endif
+
+extern WDDomain *WDWindowMaker;
+
+
+/**** Local ****/
+
+#define STIPPLE_WIDTH 2
+#define STIPPLE_HEIGHT 2
+static char STIPPLE_DATA[] = {0x02, 0x01};
+
+static int CantManageScreen = 0;
+
+static proplist_t dApplications = NULL;
+static proplist_t dWorkspace;
+static proplist_t dDock;
+static proplist_t dClip;
 
 
 static void
@@ -101,14 +133,13 @@ make_keys()
  * 	Nothing, unless something really evil happens...
  * 
  * Side effects:
- * 	The program is aborted.
+ * 	CantManageScreen is set to 1;
  *---------------------------------------------------------------------- 
  */
 static int 
 alreadyRunningError(Display *dpy, XErrorEvent *error)
 {
-    wfatal(_("it seems that there already is a window manager running"));
-    exit(1);
+    CantManageScreen = 1;
     return -1;
 }
 
@@ -148,6 +179,25 @@ allocButtonPixmaps(WScreen *scr)
     if (pix)
       pix->shared = 1;
     scr->b_pixmaps[WBUT_ICONIFY] = pix;
+#ifdef XKB_BUTTON_HINT
+    pix = wPixmapCreateFromXPMData(scr, PRED_XKBGROUP1_XPM);
+    if (pix)
+      pix->shared = 1;
+    scr->b_pixmaps[WBUT_XKBGROUP1] = pix;
+    pix = wPixmapCreateFromXPMData(scr, PRED_XKBGROUP2_XPM);
+    if (pix)
+      pix->shared = 1;
+    scr->b_pixmaps[WBUT_XKBGROUP2] = pix;
+    pix = wPixmapCreateFromXPMData(scr, PRED_XKBGROUP3_XPM);
+    if (pix)
+      pix->shared = 1;
+    scr->b_pixmaps[WBUT_XKBGROUP3] = pix;
+    pix = wPixmapCreateFromXPMData(scr, PRED_XKBGROUP4_XPM);
+    if (pix)
+      pix->shared = 1;
+    scr->b_pixmaps[WBUT_XKBGROUP4] = pix;
+#endif
+
     
     pix = wPixmapCreateFromXPMData(scr, PRED_KILL_XPM);
     if (pix)
@@ -156,7 +206,6 @@ allocButtonPixmaps(WScreen *scr)
 }
 
 
-
 static void
 draw_dot(WScreen *scr, Drawable d, int x, int y, GC gc)
 {
@@ -168,6 +217,7 @@ draw_dot(WScreen *scr, Drawable d, int x, int y, GC gc)
     XDrawPoint(dpy, d, gc, x+1, y+1);
 }
 
+
 static WPixmap*
 make3Dots(WScreen *scr)
 {
@@ -274,14 +324,23 @@ allocGCs(WScreen *scr)
 
     /* frame GC */
     wGetColor(scr, DEF_FRAME_COLOR, &color);
-
-    gcv.foreground = color.pixel;
     gcv.function = GXxor;
+    /* this will raise the probability of the XORed color being different
+     * of the original color in PseudoColor when not all color cells are
+     * initialized */
+    if (DefaultVisual(dpy, scr->screen)->class==PseudoColor)
+    	gcv.plane_mask = (1<<(scr->depth-1))|1;
+    else
+	gcv.plane_mask = AllPlanes;
+    gcv.foreground = color.pixel;
+    if (gcv.foreground == 0)
+	gcv.foreground = 1;
     gcv.line_width = DEF_FRAME_THICKNESS;
     gcv.subwindow_mode = IncludeInferiors;
     gcv.graphics_exposures = False;
     scr->frame_gc = XCreateGC(dpy, scr->root_win, GCForeground|GCGraphicsExposures
-			      |GCFunction|GCSubwindowMode|GCLineWidth, &gcv);
+			      |GCFunction|GCSubwindowMode|GCLineWidth
+			      |GCPlaneMask, &gcv);
     
     /* line GC */
     gcv.foreground = color.pixel;
@@ -309,13 +368,6 @@ allocGCs(WScreen *scr)
     scr->copy_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground
 			     |GCGraphicsExposures, &gcv);
 
-    /* clip GC */
-    gcv.foreground = scr->white_pixel;
-    gcv.background = scr->black_pixel;
-    gcv.graphics_exposures = False;
-    scr->clip_gc = XCreateGC(dpy, scr->w_win, GCForeground|GCBackground
-			     |GCGraphicsExposures, &gcv);
-
     /* window title text GC */
     gcv.graphics_exposures = False;
     scr->window_title_gc = XCreateGC(dpy, scr->w_win,GCGraphicsExposures,&gcv);
@@ -329,14 +381,22 @@ allocGCs(WScreen *scr)
     /* move/size display GC */
     gcv.graphics_exposures = False;
     gcm = GCGraphicsExposures;
-    scr->info_text_gc = XCreateGC(dpy, scr->root_win, gcm, &gcv);
+    scr->info_text_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
+    
+    /* misc drawing GC */
+    gcv.graphics_exposures = False;
+    gcm = GCGraphicsExposures;
+    scr->draw_gc = XCreateGC(dpy, scr->w_win, gcm, &gcv);
 
-    /* frame boder color */
-    wGetColor(scr, FRAME_BORDER_COLOR, &color);
-    scr->frame_border_pixel = color.pixel;
+    assert (scr->stipple_bitmap!=None);
+    
+	
+    /* mono GC */
+    scr->mono_gc = XCreateGC(dpy, scr->stipple_bitmap, gcm, &gcv);
 }
 
 
+
 static void
 createPixmaps(WScreen *scr)
 {
@@ -355,7 +415,7 @@ createPixmaps(WScreen *scr)
       pix->shared = 1;
     scr->menu_radio_indicator = pix;
 
-    
+
     pix = wPixmapCreateFromXBMData(scr, (char*)MENU_CHECK_INDICATOR_XBM_DATA,
 				   (char*)MENU_CHECK_INDICATOR_XBM_DATA,
 				   MENU_CHECK_INDICATOR_XBM_SIZE,
@@ -382,15 +442,25 @@ createPixmaps(WScreen *scr)
     if (pix!=NULL)
       pix->shared = 1;
     scr->menu_hide_indicator = pix;
-    
+
+    pix = wPixmapCreateFromXBMData(scr, (char*)MENU_SHADE_INDICATOR_XBM_DATA,
+                                  (char*)MENU_SHADE_INDICATOR_XBM_DATA,
+                                  MENU_SHADE_INDICATOR_XBM_SIZE,
+                                  MENU_SHADE_INDICATOR_XBM_SIZE,
+                                  scr->black_pixel, scr->white_pixel);
+    if (pix!=NULL)
+      pix->shared = 1;
+    scr->menu_shade_indicator = pix;
+
 
     image = wDefaultGetImage(scr, "Logo", "WMPanel");
 
     if (!image) {
-	wwarning(_("could not load logo image for panels"));
+	wwarning(_("could not load logo image for panels: %s"),
+		 RMessageForError(RErrorCode));
     } else {
 	if (!RConvertImageMask(scr->rcontext, image, &p, &m, 128)) {
-	    wwarning(_("error making logo image for panel:%s"), RErrorString);
+	    wwarning(_("error making logo image for panel:%s"), RMessageForError(RErrorCode));
 	} else {
 	    wmpix = WMCreatePixmapFromXPixmaps(scr->wmscreen, p, m,
 					       image->width, image->height, 
@@ -400,11 +470,9 @@ createPixmaps(WScreen *scr)
 	}
 	RDestroyImage(image);
     }
-    
-    if (!wPreferences.flags.nodock || !wPreferences.flags.noclip) {
-	scr->dock_dots = make3Dots(scr);
-    }
-    
+
+    scr->dock_dots = make3Dots(scr);
+
     /* titlebar button pixmaps */
     allocButtonPixmaps(scr);
 }
@@ -432,13 +500,14 @@ createInternalWindows(WScreen *scr)
     XSetWindowAttributes attribs;
 
     /* window for displaying geometry information during resizes and moves */
-    vmask = CWBackPixmap|CWBackPixel|CWCursor|CWSaveUnder|CWOverrideRedirect;
+    vmask = CWBorderPixel|CWBackPixmap|CWBackPixel|CWCursor|CWSaveUnder|CWOverrideRedirect;
+    attribs.border_pixel = scr->black_pixel;
     attribs.save_under = True;
     attribs.override_redirect = True;    
     attribs.cursor = wCursor[WCUR_DEFAULT];
     attribs.background_pixmap = None;
-    if (scr->resizebar_texture[0])
-	attribs.background_pixel = scr->resizebar_texture[0]->normal.pixel;
+    if (scr->widget_texture)
+	attribs.background_pixel = scr->widget_texture->normal.pixel;
     else
 	attribs.background_pixel = scr->light_pixel;
     vmask |= CWColormap;
@@ -462,10 +531,11 @@ createInternalWindows(WScreen *scr)
     XSelectInput(dpy, scr->no_focus_win, KeyPressMask|KeyReleaseMask);
     XMapWindow(dpy, scr->no_focus_win);
 
-    XSetInputFocus (dpy, scr->no_focus_win, RevertToParent, CurrentTime);
+    XSetInputFocus(dpy, scr->no_focus_win, RevertToParent, CurrentTime);
  
     /* shadow window for dock buttons */
-    vmask = CWBackPixmap|CWBackPixel|CWCursor|CWSaveUnder|CWOverrideRedirect;
+    vmask = CWBorderPixel|CWBackPixmap|CWBackPixel|CWCursor|CWSaveUnder|CWOverrideRedirect;
+    attribs.border_pixel = scr->black_pixel;
     attribs.save_under = True;
     attribs.override_redirect = True;
     attribs.background_pixmap = None;
@@ -476,9 +546,110 @@ createInternalWindows(WScreen *scr)
       XCreateWindow(dpy, scr->root_win, 0, 0, wPreferences.icon_size, 
 		    wPreferences.icon_size, 0, scr->w_depth, CopyFromParent, 
 		    scr->w_visual, vmask, &attribs);
+
+    /* workspace name balloon for clip */
+    vmask = CWBackPixel|CWSaveUnder|CWOverrideRedirect|CWColormap
+	|CWBorderPixel;
+    attribs.save_under = True;
+    attribs.override_redirect = True;
+    attribs.colormap = scr->w_colormap;
+    attribs.background_pixel = scr->icon_back_texture->normal.pixel;
+    attribs.border_pixel = 0; /* do not care */
+    scr->clip_balloon =
+      XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, scr->w_depth,
+		    CopyFromParent, scr->w_visual, vmask, &attribs);
+
+    
+    /* workspace name */
+    vmask = CWBackPixel|CWSaveUnder|CWOverrideRedirect|CWColormap
+	|CWBorderPixel;
+    attribs.save_under = True;
+    attribs.override_redirect = True;
+    attribs.colormap = scr->w_colormap;
+    attribs.background_pixel = scr->icon_back_texture->normal.pixel;
+    attribs.border_pixel = 0; /* do not care */
+    scr->workspace_name =
+      XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, scr->w_depth,
+		    CopyFromParent, scr->w_visual, vmask, &attribs);
+
+    
+    /* for our window manager info notice board */
+    scr->info_window =
+      XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, CopyFromParent,
+		    CopyFromParent, CopyFromParent, CWOverrideRedirect,
+		    &attribs);
+
+    /*
+     * If the window is clicked without having ButtonPress selected, the
+     * resulting event will have event.xbutton.window == root.
+     */
+    XSelectInput(dpy, scr->clip_balloon, ButtonPressMask);
 }
 
 
+#if 0
+static Bool
+aquireManagerSelection(WScreen *scr)
+{
+    char buffer[32];
+    XEvent ev;
+    Time timestamp;
+
+    sprintf(buffer, "WM_S%i", scr->screen);
+    scr->managerAtom = XInternAtom(dpy, buffer, False);
+
+    /* for race-conditions... */
+    XGrabServer(dpy);
+
+    /* if there is another manager running, don't try to replace it
+     * (for now, at least) */
+    if (XGetSelectionOwner(dpy, scr->managerAtom) != None) {
+	XUngrabServer(dpy);
+	return False;
+    }
+
+    /* become the manager for this screen */
+
+    scr->managerWindow = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 1, 1,
+					     0, 0, 0);
+
+    XSelectInput(dpy, scr->managerWindow, PropertyChangeMask);
+    /* get a timestamp */
+    XChangeProperty(dpy, scr->managerWindow, scr->managerAtom,
+		    XA_INTEGER, 32, PropModeAppend, NULL, 0);
+    while (1) {
+	XWindowEvent(dpy, scr->managerWindow, &ev);
+	if (ev.type == PropertyNotify) {
+	    timestamp = ev.xproperty.time;
+	    break;
+	}
+    }
+    XSelectInput(dpy, scr->managerWindow, NoEvents);
+    XDeleteProperty(dpy, scr->managerWindow, scr->managerAtom);
+
+    XSetSelectionOwner(dpy, scr->managerAtom, scr->managerWindow, CurrentTime);
+
+    XUngrabServer(dpy);
+
+    /* announce our arrival */
+
+    ev.xclient.type = ClientMessage;
+    ev.xclient.message_type = XInternAtom(dpy, "MANAGER", False);
+    ev.xclient.destination = scr->root_win;
+    ev.xclient.format = 32;
+    ev.xclient.data.l[0] = timestamp;
+    ev.xclient.data.l[1] = scr->managerAtom;
+    ev.xclient.data.l[2] = scr->managerWindow;
+    ev.xclient.data.l[3] = 0;
+    ev.xclient.data.l[4] = 0;
+
+    XSendEvent(dpy, scr->root_win, False, StructureNotify, &ev);
+    XSync(dpy, False);
+
+    return True;
+}
+#endif
+
 /*
  *----------------------------------------------------------------------
  * wScreenInit--
@@ -497,34 +668,17 @@ createInternalWindows(WScreen *scr)
  * TODO: User specifiable visual.
  *---------------------------------------------------------------------- 
  */
-WScreen *
+WScreen*
 wScreenInit(int screen_number)
 {
     WScreen *scr;
     XIconSize icon_size[1];
     RContextAttributes rattr;
     extern int wVisualID;
-    WDDomain *domain;
     long event_mask;
     WMColor *color;
     XErrorHandler oldHandler;
-    
-    domain = wDefaultsInitDomain("WindowMaker", True);
-    if (!domain->dictionary) {
-	wwarning(_("could not read domain \"%s\" from defaults database"),
-		 "WindowMaker");
-    }
-    
-    /* read defaults that don't change until a restart */
-    wReadStaticDefaults(domain ? domain->dictionary : NULL);
 
-    /* check sanity of some values */
-    if (wPreferences.icon_size < 16) {
-	wwarning(_("icon size is configured to %i, but it's too small. Using 16, instead\n"),
-		 wPreferences.icon_size);
-	wPreferences.icon_size = 16;
-    }
-    
     scr = wmalloc(sizeof(WScreen));
     memset(scr, 0, sizeof(WScreen));
 
@@ -537,22 +691,53 @@ wScreenInit(int screen_number)
     scr->scr_width = WidthOfScreen(ScreenOfDisplay(dpy, screen_number));
     scr->scr_height = HeightOfScreen(ScreenOfDisplay(dpy, screen_number));
 
+    scr->usableArea.x2 = scr->scr_width;
+    scr->usableArea.y2 = scr->scr_height;
+    scr->totalUsableArea.x2 = scr->scr_width;
+    scr->totalUsableArea.y2 = scr->scr_height;
+
+#if 0
+    if (!aquireManagerSelection(scr)) {
+	free(scr);
+
+	return NULL;
+    }
+#endif
+    CantManageScreen = 0;
     oldHandler = XSetErrorHandler((XErrorHandler)alreadyRunningError);
 
     event_mask = EVENT_MASK;
-    
-    if (wPreferences.disable_root_mouse)
+
+    if (wPreferences.disable_root_mouse) {
 	event_mask &= ~(ButtonPressMask|ButtonReleaseMask);
-	
+    }
+    
     XSelectInput(dpy, scr->root_win, event_mask);
+
+#ifdef KEEP_XKB_LOCK_STATUS     
+    if (wXkbSupported) {
+        XkbSelectEvents(dpy,XkbUseCoreKbd,
+                XkbStateNotifyMask,
+                XkbStateNotifyMask);
+    }
+#endif /* KEEP_XKB_LOCK_STATUS */
+
     XSync(dpy, False);
     XSetErrorHandler(oldHandler);
+    
+    if (CantManageScreen) {
+	free(scr);
+	return NULL;
+    }
 
     XDefineCursor(dpy, scr->root_win, wCursor[WCUR_DEFAULT]);
 
     /* screen descriptor for raster graphic library */
     rattr.flags = RC_RenderMode | RC_ColorsPerChannel;
-    rattr.render_mode = wPreferences.no_dithering?RM_MATCH:RM_DITHER;
+    rattr.render_mode = wPreferences.no_dithering 
+				? RBestMatchRendering
+				: RDitheredRendering;
+
     rattr.colors_per_channel = wPreferences.cmap_size;
     if (rattr.colors_per_channel<2)
 	rattr.colors_per_channel = 2;
@@ -564,7 +749,7 @@ wScreenInit(int screen_number)
     scr->rcontext = RCreateContext(dpy, screen_number, &rattr);
     if (!scr->rcontext) {
 	wwarning(_("could not initialize graphics library context: %s"),
-		 RErrorString);
+		 RMessageForError(RErrorCode));
 	wAbort(False);
     } else {
 	char **formats;
@@ -578,7 +763,6 @@ wScreenInit(int screen_number)
 		    break;
 		}
 	    }
-	    RFreeStringList(formats);
 	}
     }
     
@@ -591,9 +775,15 @@ wScreenInit(int screen_number)
     scr->white_pixel = scr->rcontext->white;    
 
     /* create screen descriptor for WINGs */
-    scr->wmscreen = WMCreateScreenWithRContext(dpy, screen_number, 
+    scr->wmscreen = WMCreateScreenWithRContext(dpy, screen_number,
 					       scr->rcontext);
 
+    if (!scr->wmscreen) {
+	wfatal(_("could not do initialization of WINGs widget set"));
+
+	return NULL;
+    }
+
     color = WMGrayColor(scr->wmscreen);
     scr->light_pixel = WMColorPixel(color);
     WMReleaseColor(color);
@@ -602,44 +792,44 @@ wScreenInit(int screen_number)
     scr->dark_pixel = WMColorPixel(color);
     WMReleaseColor(color);
 
+    {
+	XColor xcol;
+	/* frame boder color */
+	wGetColor(scr, FRAME_BORDER_COLOR, &xcol);
+	scr->frame_border_pixel = xcol.pixel;
+    }
+
     /* create GCs with default values */
     allocGCs(scr);
-    
-    /* read defaults */
-    scr->dmain = domain;
-    wReadDefaults(scr, domain->dictionary);
 
-    /* init other domains */
-    scr->droot_menu = wDefaultsInitDomain("WMRootMenu", False);
-    if (!scr->droot_menu->dictionary) {
-	wwarning(_("could not read domain \"%s\" from defaults database"),
-		 "WMRootMenu");
-    }
+    /* read defaults for this screen */
+    wReadDefaults(scr, WDWindowMaker->dictionary);
 
-    scr->dwindow = wDefaultsInitDomain("WMWindowAttributes", True);
-    if (!scr->dwindow->dictionary) {
-	wwarning(_("could not read domain \"%s\" from defaults database"),
-		 "WMWindowAttributes");
-    }
+    createInternalWindows(scr);
+
+#ifdef KWM_HINTS
+    wKWMInitStuff(scr);
+#endif
+
+#ifdef GNOME_STUFF
+    wGNOMEInitStuff(scr);
+#endif
+
+#ifdef OLWM_HINTS
+    wOLWMInitStuff(scr);
+#endif
 
     /* create initial workspace */
     wWorkspaceNew(scr);
 
-    
-    /* setup defaults file polling */
-    WMAddTimerHandler(3000, wDefaultsCheckDomains, scr);
-    
-    
-    createInternalWindows(scr);
-
     /* create shared pixmaps */
     createPixmaps(scr);
 
     /* set icon sizes we can accept from clients */
-    icon_size[0].min_width = 1;
-    icon_size[0].min_height = 1;
-    icon_size[0].max_width = wPreferences.icon_size;
-    icon_size[0].max_height = wPreferences.icon_size;
+    icon_size[0].min_width = 8;
+    icon_size[0].min_height = 8;
+    icon_size[0].max_width = wPreferences.icon_size-4;
+    icon_size[0].max_height = wPreferences.icon_size-4;
     icon_size[0].width_inc = 1;
     icon_size[0].height_inc = 1;
     XSetIconSizes(dpy, scr->root_win, icon_size, 1);
@@ -647,23 +837,177 @@ wScreenInit(int screen_number)
     /* setup WindowMaker protocols property in the root window*/
     PropSetWMakerProtocols(scr->root_win);
 
-    
+    /* setup our noticeboard */
+    XChangeProperty(dpy, scr->info_window, _XA_WINDOWMAKER_NOTICEBOARD,
+		    XA_WINDOW, 32, PropModeReplace, 
+		    (unsigned char*)&scr->info_window, 1);
+    XChangeProperty(dpy, scr->root_win, _XA_WINDOWMAKER_NOTICEBOARD,
+		    XA_WINDOW, 32, PropModeReplace, 
+		    (unsigned char*)&scr->info_window, 1);
+
+
+#ifdef BALLOON_TEXT
+    /* initialize balloon text stuff */
+    wBalloonInitialize(scr);
+#endif
+
+    wScreenUpdateUsableArea(scr);
+
+#ifndef LITE
     /* kluge to load menu configurations at startup */
     OpenRootMenu(scr, -10000, -10000, False);
     wMenuUnmap(scr->root_menu);
-    
+#endif
+
     return scr;
 }
 
 
+void
+wScreenUpdateUsableArea(WScreen *scr)
+{
+#ifdef GNOME_STUFF
+    WReservedArea *area;
+#endif
+
+    scr->totalUsableArea = scr->usableArea;
+
+
+    if (scr->dock && (!scr->dock->lowered 
+		      || wPreferences.no_window_over_dock)) {
+
+	int offset = wPreferences.icon_size + DOCK_EXTRA_SPACE;
+
+	if (scr->dock->on_right_side) {
+	    scr->totalUsableArea.x2 = WMIN(scr->totalUsableArea.x2,
+					  scr->scr_width - offset);
+	} else {
+	    scr->totalUsableArea.x1 = WMAX(scr->totalUsableArea.x1, offset);
+	}
+    }
+    
+    if (wPreferences.no_window_over_icons) {
+	if (wPreferences.icon_yard & IY_VERT) {
+
+	    if (!(wPreferences.icon_yard & IY_RIGHT)) {
+		scr->totalUsableArea.x1 += wPreferences.icon_size;
+	    } else {
+		scr->totalUsableArea.x2 -= wPreferences.icon_size;
+	    }
+	} else {
+
+	    if (wPreferences.icon_yard & IY_TOP) {
+		scr->totalUsableArea.y1 += wPreferences.icon_size;
+	    } else {
+		scr->totalUsableArea.y2 -= wPreferences.icon_size;
+	    }
+	}
+    }
+
+#ifdef KWM_HINTS
+    {
+	WArea area;
+
+	if (wKWMGetUsableArea(scr, &area)) {
+	    scr->totalUsableArea.x1 = WMAX(scr->totalUsableArea.x1, area.x1);
+	    scr->totalUsableArea.y1 = WMAX(scr->totalUsableArea.y1, area.y1);
+	    scr->totalUsableArea.x2 = WMIN(scr->totalUsableArea.x2, area.x2);
+	    scr->totalUsableArea.y2 = WMIN(scr->totalUsableArea.y2, area.y2);
+	}
+    }
+#endif
+
+#ifdef GNOME_STUFF
+    area = scr->reservedAreas;    
+
+    while (area) {
+	int th, bh;
+	int lw, rw;
+	int w, h;
+
+	w = area->area.x2 - area->area.x1;
+	h = area->area.y2 - area->area.y1;
+
+	th = area->area.y1;
+	bh = scr->scr_height - area->area.y2;
+	lw = area->area.x1;
+	rw = scr->scr_width - area->area.x2;
+
+	if (WMIN(th, bh) < WMIN(lw, rw)) {
+	    /* horizontal */
+	    if (th < bh) {
+		/* on top */
+		if (scr->totalUsableArea.y1 < area->area.y2)
+		    scr->totalUsableArea.y1 = area->area.y2;
+	    } else {
+		/* on bottom */
+		if (scr->totalUsableArea.y2 > area->area.y1)
+		    scr->totalUsableArea.y2 = area->area.y1;
+	    }
+	} else {
+	    /* vertical */
+	    if (lw < rw) {
+		/* on left */
+		if (scr->totalUsableArea.x1 < area->area.x2)
+		    scr->totalUsableArea.x1 = area->area.x2;
+	    } else {
+		/* on right */
+		if (scr->totalUsableArea.x2 > area->area.x1)
+		    scr->totalUsableArea.x2 = area->area.x1;
+	    }
+	}
+
+	area = area->next;
+    }
+#endif /* GNOME_STUFF */
+
+    if (scr->totalUsableArea.x2 - scr->totalUsableArea.x1 < scr->scr_width/2) {
+	scr->totalUsableArea.x2 = scr->usableArea.x2;
+	scr->totalUsableArea.x1 = scr->usableArea.x1;
+    }
+    if (scr->totalUsableArea.y2 - scr->totalUsableArea.y1 < scr->scr_height/2) {
+	scr->totalUsableArea.y2 = scr->usableArea.y2;
+	scr->totalUsableArea.y1 = scr->usableArea.y1;
+    }
+
+#ifdef not_used
+#ifdef KWM_HINTS
+    {
+	int i;
+
+	for (i = 0; i < scr->workspace_count; i++) {
+	    wKWMSetUsableAreaHint(scr, i);
+	}
+    }
+#endif
+#endif
+}
+
+
 void
 wScreenRestoreState(WScreen *scr)
 {
     proplist_t state;
+    char *path;
 
     make_keys();
 
-    scr->session_state = PLGetProplistWithPath(wdefaultspathfordomain("WMState"));
+    if (wScreenCount == 1)
+	path = wdefaultspathfordomain("WMState");
+    else {
+	char buf[16];
+	sprintf(buf, "WMState.%i", scr->screen);
+	path = wdefaultspathfordomain(buf);
+    }
+    scr->session_state = PLGetProplistWithPath(path);
+    free(path);
+    if (!scr->session_state && wScreenCount>1) {
+	char buf[16];
+	sprintf(buf, "WMState.%i", scr->screen);
+	path = wdefaultspathfordomain(buf);
+	scr->session_state = PLGetProplistWithPath(path);
+	free(path);
+    }
 
     if (!wPreferences.flags.noclip) {
         state = PLGetDictionaryEntry(scr->session_state, dClip);
@@ -677,6 +1021,7 @@ wScreenRestoreState(WScreen *scr)
         scr->dock = wDockRestoreState(scr, state, WM_DOCK);
     }
 
+    wScreenUpdateUsableArea(scr);
 }
 
 
@@ -685,7 +1030,9 @@ wScreenSaveState(WScreen *scr)
 {
     WWorkspaceState wstate;
     WWindow *wwin;
+    char *str;
     proplist_t path, old_state, foo;
+    CARD32 data[2];
 
 
     make_keys();
@@ -695,11 +1042,12 @@ wScreenSaveState(WScreen *scr)
  */
     wstate.workspace = scr->current_workspace;
 
+    data[0] = wstate.flags;
+    data[1] = wstate.workspace;
 
     XChangeProperty(dpy, scr->root_win, _XA_WINDOWMAKER_STATE,
                     _XA_WINDOWMAKER_STATE, 32, PropModeReplace,
-                    (unsigned char *) &wstate, 
-		    sizeof(WWorkspaceState)/sizeof(int));
+                    (unsigned char *) data, 2);
     
     /* save state of windows */
     wwin = scr->focused_window;
@@ -708,6 +1056,11 @@ wScreenSaveState(WScreen *scr)
 	wwin = wwin->prev;
     }
 
+
+    if (wPreferences.flags.noupdates)
+	return;
+
+
     old_state = scr->session_state;
     scr->session_state = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
 
@@ -747,9 +1100,19 @@ wScreenSaveState(WScreen *scr)
 
     wMenuSaveState(scr);
 
-    path = PLMakeString(wdefaultspathfordomain("WMState"));
+    if (wScreenCount == 1)
+	str = wdefaultspathfordomain("WMState");
+    else {
+	char buf[16];
+	sprintf(buf, "WMState.%i", scr->screen);
+	str = wdefaultspathfordomain(buf);
+    }
+    path = PLMakeString(str);
+    free(str);
     PLSetFilename(scr->session_state, path);
-    PLSave(scr->session_state, YES);
+    if (!PLSave(scr->session_state, YES)) {
+	wsyserror(_("could not save session state in %s"), PLGetString(path));
+    }
     PLRelease(path);
     PLRelease(old_state);
 }
@@ -784,3 +1147,5 @@ wScreenBringInside(WScreen *scr, int *x, int *y, int width, int height)
 
     return moved;
 }
+
+
