@@ -1,8 +1,9 @@
 /*
- *  WindowMaker window manager
- * 
+ *  Window Maker window manager
+ *
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
- * 
+ *  Copyright (c) 1999       Dan Pascu
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -36,12 +37,12 @@
 #include <X11/cursorfont.h>
 #include <X11/Xproto.h>
 #include <X11/keysym.h>
+#ifdef SHAPE
+#include <X11/extensions/shape.h>
+#endif
 
 #include "WindowMaker.h"
 #include "GNUstep.h"
-#ifdef MWM_HINTS
-# include "motif.h"
-#endif
 #include "texture.h"
 #include "screen.h"
 #include "window.h"
@@ -53,25 +54,53 @@
 #include "keybind.h"
 #include "framewin.h"
 #include "session.h"
+#include "defaults.h"
+#include "properties.h"
+#include "dialog.h"
+#ifdef XDND
+#include "xdnd.h"
+#endif
 
 #include "xutil.h"
 
-#ifdef WMSOUND
-#include "wmsound.h"
+#ifdef KWM_HINTS
+#include "kwm.h"
 #endif
 
-
+#if 0
 #ifdef SYS_SIGLIST_DECLARED
 extern const char * const sys_siglist[];
 #endif
+#endif
+
+/* for SunOS */
+#ifndef SA_RESTART
+# define SA_RESTART 0
+#endif
+
+
 /****** Global Variables ******/
 
 extern WPreferences wPreferences;
 
+extern WDDomain *WDWindowMaker;
+extern WDDomain *WDRootMenu;
+extern WDDomain *WDWindowAttributes;
+
 extern WShortKey wKeyBindings[WKBD_LAST];
 
-/* currently only single screen support */
-extern WScreen *wScreen;
+extern int wScreenCount;
+
+
+#ifdef SHAPE
+extern Bool wShapeSupported;
+extern int wShapeEventBase;
+#endif
+
+#ifdef KEEP_XKB_LOCK_STATUS
+extern Bool wXkbSupported;
+extern int wXkbEventBase;
+#endif
 
 /* contexts */
 extern XContext wWinContext;
@@ -87,6 +116,7 @@ extern Atom _XA_WM_DELETE_WINDOW;
 extern Atom _XA_WM_SAVE_YOURSELF;
 extern Atom _XA_WM_CLIENT_LEADER;
 extern Atom _XA_WM_COLORMAP_WINDOWS;
+extern Atom _XA_WM_COLORMAP_NOTIFY;
 
 extern Atom _XA_GNUSTEP_WM_ATTR;
 
@@ -94,29 +124,30 @@ extern Atom _XA_WINDOWMAKER_MENU;
 extern Atom _XA_WINDOWMAKER_WM_PROTOCOLS;
 extern Atom _XA_WINDOWMAKER_STATE;
 extern Atom _XA_WINDOWMAKER_WM_FUNCTION;
+extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
+extern Atom _XA_WINDOWMAKER_COMMAND;
 
-extern Atom _XA_WINDOWMAKER_WM_MINIATURIZE_WINDOW;
-extern Atom _XA_GNUSTEP_WM_RESIZEBAR;
+extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
 
 #ifdef OFFIX_DND
 extern Atom _XA_DND_PROTOCOL;
 extern Atom _XA_DND_SELECTION;
 #endif
 
+
 /* cursors */
 extern Cursor wCursor[WCUR_LAST];
 
 /* special flags */
-extern char WRestartASAP;
-extern char WExitASAP;
 extern char WDelayedActionSet;
 
+/***** Local *****/
+
+static WScreen **wScreen = NULL;
 
 
-#ifdef NUMLOCK_HACK
 static unsigned int _NumLockMask = 0;
 static unsigned int _ScrollLockMask = 0;
-#endif
 
 
 
@@ -125,7 +156,6 @@ static void manageAllWindows();
 extern void NotifyDeadProcess(pid_t pid, unsigned char status);
 
 
-
 static int 
 catchXError(Display *dpy, XErrorEvent *error)
 {
@@ -152,6 +182,7 @@ catchXError(Display *dpy, XErrorEvent *error)
 	     */
 	    || (error->request_code == X_InstallColormap))) {
 #ifndef DEBUG
+
 	return 0;
 #else
 	printf("got X error %x %x %x\n", error->request_code,
@@ -159,7 +190,7 @@ catchXError(Display *dpy, XErrorEvent *error)
 	return 0;
 #endif
     }
-    FormatXError(dpy, error, buffer, MAXLINE); 
+    FormatXError(dpy, error, buffer, MAXLINE);
     wwarning(_("internal X error: %s\n"), buffer);
     return -1;
 }
@@ -172,13 +203,14 @@ catchXError(Display *dpy, XErrorEvent *error)
  *---------------------------------------------------------------------- 
  */
 static int
-handleXIO(Display *dpy)
+handleXIO(Display *xio_dpy)
 {
-    exit(0);
+    dpy = NULL;
+    Exit(0);
+    return 0;
 }
 
 
-
 /*
  *----------------------------------------------------------------------
  * delayedAction-
@@ -209,7 +241,9 @@ static RETSIGTYPE
 handleSig(int sig)
 {
     static int already_crashed = 0;
+    int dumpcore = 0;
 #ifndef NO_EMERGENCY_AUTORESTART
+    int crashAction;
     char *argv[2];
     
     argv[1] = NULL;
@@ -217,17 +251,17 @@ handleSig(int sig)
     
     /* 
      * No functions that potentially do Xlib calls should be called from
-     * here. Xlib calls are not atomic so, so the logical integrity of
-     * Xlib is not guaranteed if a Xlib call is made from a signal handler.
+     * here. Xlib calls are not reentrant so the integrity of Xlib is
+     * not guaranteed if a Xlib call is made from a signal handler.
      */
-    if (sig == SIGHUP) {
+    if (sig == SIGUSR1) {
 #ifdef SYS_SIGLIST_DECLARED
         wwarning(_("got signal %i (%s) - restarting\n"), sig, sys_siglist[sig]);
 #else
         wwarning(_("got signal %i - restarting\n"), sig);
 #endif
 
-	WRestartASAP = 1;
+	WCHANGE_STATE(WSTATE_NEED_RESTART);
 	/* setup idle handler, so that this will be handled when
 	 * the select() is returned becaused of the signal, even if
 	 * there are no X events in the queue */
@@ -236,10 +270,15 @@ handleSig(int sig)
 	    WMAddIdleHandler(delayedAction, NULL);
 	}
         return;
-    } else if (sig==SIGTERM) {
-	printf(_("%s: Received signal SIGTERM. Exiting..."), ProgName);
-	
-	WExitASAP = 1;
+    } else if (sig == SIGTERM || sig == SIGHUP) {
+#ifdef SYS_SIGLIST_DECLARED
+        wwarning(_("got signal %i (%s) - exiting...\n"), sig, sys_siglist[sig]);
+#else
+        wwarning(_("got signal %i - exiting...\n"), sig);
+#endif
+
+	WCHANGE_STATE(WSTATE_NEED_EXIT);
+
 	if (!WDelayedActionSet) {
 	    WDelayedActionSet = 1;
 	    WMAddIdleHandler(delayedAction, NULL);
@@ -252,35 +291,72 @@ handleSig(int sig)
 #else
     wfatal(_("got signal %i\n"), sig);
 #endif
-    
-    if (sig==SIGSEGV || sig==SIGFPE || sig==SIGBUS) {
+
+    /* Setting the signal behaviour back to default and then reraising the
+     * signal is a cleaner way to make program exit and core dump than calling
+     * abort(), since it correctly returns from the signal handler and sets
+     * the flags accordingly. -Dan
+     */
+    if (sig==SIGSEGV || sig==SIGFPE || sig==SIGBUS || sig==SIGILL
+	|| sig==SIGABRT) {
 	if (already_crashed) {
 	    wfatal(_("crashed while trying to do some post-crash cleanup. Aborting immediatelly."));
-	    abort();
+            signal(sig, SIG_DFL);
+            kill(getpid(), sig);
+            return;
 	}
 	already_crashed = 1;
-	
+
+	dumpcore = 1;
+
 #ifndef NO_EMERGENCY_AUTORESTART
-    	/* restart another window manager so that the X session doesn't
-	 * go to space */
+        /* Close the X connection and open a new one. This is to avoid messing
+         * Xlib because we call to Xlib functions in a signal handler.
+         */
+        if (dpy)
+            XCloseDisplay(dpy);
+        dpy = XOpenDisplay("");
+        if (dpy) {
+	    XGrabServer(dpy);
+            crashAction = wShowCrashingDialogPanel(sig);
+            XCloseDisplay(dpy);
+            dpy = NULL;
+        } else {
+            wsyserror(_("cannot open connection for crashing dialog panel. Aborting."));
+            crashAction = WMAbort;
+        }
 
-    	wwarning(_("trying to start alternative window manager..."));
-    	if (dpy)
-	    XCloseDisplay(dpy);
-    	dpy = NULL;
-    
-    	argv[0] = FALLBACK_WINDOWMANAGER;
-    	execvp(FALLBACK_WINDOWMANAGER, argv);
+        if (crashAction == WMAbort) {
+            signal(sig, SIG_DFL);
+            kill(getpid(), sig);
+            return;
+        }
 
-    	argv[0] = "fvwm";
-    	execvp("fvwm", argv);
+        if (crashAction == WMRestart) {
+            /* we try to restart Window Maker */
+            wwarning(_("trying to restart Window Maker..."));
+            Restart(NULL, False);
+            /* fallback to alternate window manager then */
+        }
 
-    	argv[0] = "twm";
-    	execvp("twm", argv);
+    	wwarning(_("trying to start alternate window manager..."));
+
+        Restart(FALLBACK_WINDOWMANAGER, False);
+        Restart("fvwm", False);
+        Restart("twm", False);
+        wfatal(_("failed to start alternate window manager. Aborting."));
+#else
+        wfatal(_("a fatal error has occured, probably due to a bug. "
+                 "Please fill the included BUGFORM and report it."));
 #endif /* !NO_EMERGENCY_AUTORESTART */
+
+        signal(sig, SIG_DFL);
+        kill(getpid(), sig);
+        return;
+
     }
-    
-    wAbort(sig==SIGSEGV);
+
+    wAbort(dumpcore);
 }
 
 
@@ -320,28 +396,35 @@ getWorkspaceState(Window root, WWorkspaceState **state)
     int fmt_ret;    
     unsigned long nitems_ret;
     unsigned long bytes_after_ret;
+    CARD32 *data;
 
-    if (XGetWindowProperty(dpy, root, _XA_WINDOWMAKER_STATE, 0,
-                           sizeof(WWorkspaceState),
+    if (XGetWindowProperty(dpy, root, _XA_WINDOWMAKER_STATE, 0, 2,
                            True, _XA_WINDOWMAKER_STATE,
                            &type_ret, &fmt_ret, &nitems_ret, &bytes_after_ret,
-                           (unsigned char **)state)!=Success)
+                           (unsigned char **)&data)!=Success || !data)
       return 0;
-    if (type_ret==_XA_WINDOWMAKER_STATE)
+
+    *state = malloc(sizeof(WWorkspaceState));
+    if (*state) {
+	(*state)->flags = data[0];
+	(*state)->workspace = data[1];
+    }
+    XFree(data);
+
+    if (*state && type_ret==_XA_WINDOWMAKER_STATE)
       return 1;
     else
       return 0;
 }
 
 
-#ifdef NUMLOCK_HACK
 static void
 getOffendingModifiers()
 {
     int i;
     XModifierKeymap *modmap;
     KeyCode nlock, slock;
-    static mask_table[8] = {
+    static int mask_table[8] = {
 	ShiftMask,LockMask,ControlMask,Mod1Mask,
 	    Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
     };
@@ -357,9 +440,9 @@ getOffendingModifiers()
     
     if (modmap!=NULL && modmap->max_keypermod>0) {
 	for (i=0; i<8*modmap->max_keypermod; i++) {
-	    if (modmap->modifiermap[i]==nlock)
+	    if (modmap->modifiermap[i]==nlock && nlock!=0)
 		_NumLockMask = mask_table[i/modmap->max_keypermod];
-	    else if (modmap->modifiermap[i]==slock)
+	    else if (modmap->modifiermap[i]==slock && slock!=0)
 		_ScrollLockMask = mask_table[i/modmap->max_keypermod];
 	}
     }
@@ -369,6 +452,8 @@ getOffendingModifiers()
 }
 
 
+
+#ifdef NUMLOCK_HACK
 void 
 wHackedGrabKey(int keycode, unsigned int modifiers,
 	       Window grab_window, Bool owner_events, int pointer_mode,
@@ -381,21 +466,27 @@ wHackedGrabKey(int keycode, unsigned int modifiers,
      * ScrollLock. How much memory/CPU does such a monstrosity consume
      * in the server?
      */
-    XGrabKey(dpy, keycode, modifiers|_NumLockMask, 
-	     grab_window, owner_events, pointer_mode, keyboard_mode);
-    XGrabKey(dpy, keycode, modifiers|_ScrollLockMask,
-	     grab_window, owner_events, pointer_mode, keyboard_mode);
-    XGrabKey(dpy, keycode, modifiers|_NumLockMask|_ScrollLockMask,
-	     grab_window, owner_events, pointer_mode, keyboard_mode);
-    XGrabKey(dpy, keycode, modifiers|_NumLockMask|LockMask,
-	     grab_window, owner_events, pointer_mode, keyboard_mode);
-    XGrabKey(dpy, keycode, modifiers|_ScrollLockMask|LockMask,
-	     grab_window, owner_events, pointer_mode, keyboard_mode);
-    XGrabKey(dpy, keycode, modifiers|_NumLockMask|_ScrollLockMask|LockMask,
-	     grab_window, owner_events, pointer_mode, keyboard_mode);
+    if (_NumLockMask)
+	XGrabKey(dpy, keycode, modifiers|_NumLockMask, 
+		 grab_window, owner_events, pointer_mode, keyboard_mode);
+    if (_ScrollLockMask)
+	XGrabKey(dpy, keycode, modifiers|_ScrollLockMask,
+		 grab_window, owner_events, pointer_mode, keyboard_mode);
+    if (_NumLockMask && _ScrollLockMask)
+	XGrabKey(dpy, keycode, modifiers|_NumLockMask|_ScrollLockMask,
+		 grab_window, owner_events, pointer_mode, keyboard_mode);
+    if (_NumLockMask)
+	XGrabKey(dpy, keycode, modifiers|_NumLockMask|LockMask,
+		 grab_window, owner_events, pointer_mode, keyboard_mode);
+    if (_ScrollLockMask)
+	XGrabKey(dpy, keycode, modifiers|_ScrollLockMask|LockMask,
+		 grab_window, owner_events, pointer_mode, keyboard_mode);
+    if (_NumLockMask && _ScrollLockMask)
+	XGrabKey(dpy, keycode, modifiers|_NumLockMask|_ScrollLockMask|LockMask,
+		 grab_window, owner_events, pointer_mode, keyboard_mode);
     /* phew, I guess that's all, right? */
 }
-
+#endif
 
 void 
 wHackedGrabButton(unsigned int button, unsigned int modifiers, 
@@ -403,31 +494,45 @@ wHackedGrabButton(unsigned int button, unsigned int modifiers,
 		  unsigned int event_mask, int pointer_mode, 
 		  int keyboard_mode, Window confine_to, Cursor cursor)
 {
+    XGrabButton(dpy, button, modifiers, grab_window, owner_events,
+		event_mask, pointer_mode, keyboard_mode, confine_to, cursor);
+
     if (modifiers==AnyModifier)
 	return;
+
+    XGrabButton(dpy, button, modifiers|LockMask, grab_window, owner_events,
+		event_mask, pointer_mode, keyboard_mode, confine_to, cursor);
     
+#ifdef NUMLOCK_HACK    
     /* same as above, but for mouse buttons */
-    XGrabButton(dpy, button, modifiers|_NumLockMask,
-		grab_window, owner_events, event_mask, pointer_mode,
+    if (_NumLockMask)
+	XGrabButton(dpy, button, modifiers|_NumLockMask,
+		    grab_window, owner_events, event_mask, pointer_mode,
 		keyboard_mode, confine_to, cursor);
-    XGrabButton(dpy, button, modifiers|_ScrollLockMask,
-		grab_window, owner_events, event_mask, pointer_mode,
-		keyboard_mode, confine_to, cursor);
-    XGrabButton(dpy, button, modifiers|_ScrollLockMask|_NumLockMask,
-		grab_window, owner_events, event_mask, pointer_mode,
-		keyboard_mode, confine_to, cursor);
-    XGrabButton(dpy, button, modifiers|_NumLockMask|LockMask,
-		grab_window, owner_events, event_mask, pointer_mode,
-		keyboard_mode, confine_to, cursor);
-    XGrabButton(dpy, button, modifiers|_ScrollLockMask|LockMask,
-		grab_window, owner_events, event_mask, pointer_mode,
-		keyboard_mode, confine_to, cursor);
-    XGrabButton(dpy, button, modifiers|_ScrollLockMask|_NumLockMask|LockMask,
-		grab_window, owner_events, event_mask, pointer_mode,
+    if (_ScrollLockMask)
+	XGrabButton(dpy, button, modifiers|_ScrollLockMask,
+		    grab_window, owner_events, event_mask, pointer_mode,
+		    keyboard_mode, confine_to, cursor);
+    if (_NumLockMask && _ScrollLockMask)
+	XGrabButton(dpy, button, modifiers|_ScrollLockMask|_NumLockMask,
+		    grab_window, owner_events, event_mask, pointer_mode,
+		    keyboard_mode, confine_to, cursor);
+    if (_NumLockMask)
+	XGrabButton(dpy, button, modifiers|_NumLockMask|LockMask,
+		    grab_window, owner_events, event_mask, pointer_mode,
 		keyboard_mode, confine_to, cursor);
+    if (_ScrollLockMask)
+	XGrabButton(dpy, button, modifiers|_ScrollLockMask|LockMask,
+		    grab_window, owner_events, event_mask, pointer_mode,
+		    keyboard_mode, confine_to, cursor);
+    if (_NumLockMask && _ScrollLockMask)
+	XGrabButton(dpy, button, modifiers|_ScrollLockMask|_NumLockMask|LockMask,
+		    grab_window, owner_events, event_mask, pointer_mode,
+		    keyboard_mode, confine_to, cursor);
+#endif /* NUMLOCK_HACK */
 }
 
-
+#ifdef notused
 void
 wHackedUngrabButton(unsigned int button, unsigned int modifiers, 
 		    Window grab_window)
@@ -445,12 +550,85 @@ wHackedUngrabButton(unsigned int button, unsigned int modifiers,
     XUngrabButton(dpy, button, modifiers|_NumLockMask|_ScrollLockMask|LockMask,
 		  grab_window);
 }
+#endif
+
+
+
+WScreen*
+wScreenWithNumber(int i)
+{
+    assert(i < wScreenCount);
+
+    return wScreen[i];
+}
+
+
+WScreen*
+wScreenForRootWindow(Window window)
+{
+    int i;
+
+    if (wScreenCount==1)
+	return wScreen[0];
+
+    /*
+     * Since the number of heads will probably be small (normally 2),
+     * it should be faster to use this than a hash table, because
+     * of the overhead.
+     */
+    for (i=0; i<wScreenCount; i++) {
+	if (wScreen[i]->root_win == window) {
+	    return wScreen[i];
+	}
+    }
+
+    assert("bad_root_window" && 0);
+    
+    return NULL;
+}
+
+
+WScreen*
+wScreenSearchForRootWindow(Window window)
+{
+    int i;
+
+    if (wScreenCount==1)
+	return wScreen[0];
+
+    /*
+     * Since the number of heads will probably be small (normally 2),
+     * it should be faster to use this than a hash table, because
+     * of the overhead.
+     */
+    for (i=0; i<wScreenCount; i++) {
+	if (wScreen[i]->root_win == window) {
+	    return wScreen[i];
+	}
+    }
+    
+    return NULL;
+}
+
+
+WScreen*
+wScreenForWindow(Window window)
+{
+    XWindowAttributes attr;
+
+    if (wScreenCount==1)
+	return wScreen[0];
+
+    if (XGetWindowAttributes(dpy, window, &attr)) {
+	return wScreenForRootWindow(attr.root);
+    }
+    return NULL;
+}
+
 
-#endif /* NUMLOCK_HACK */
 
-       
 /*
- *---------------------------------------------------------- 
+ *----------------------------------------------------------
  * StartUp--
  * 	starts the window manager and setup global data.
  * Called from main() at startup.
@@ -459,22 +637,33 @@ wHackedUngrabButton(unsigned int button, unsigned int modifiers,
  * global data declared in main.c is initialized
  *----------------------------------------------------------
  */
-void 
-StartUp()
+void
+StartUp(Bool defaultScreenOnly)
 {
     WWorkspaceState *ws_state;
     struct sigaction sig_action;
-    int i;
+    int j, max;
+
+    /*
+     * Ignore CapsLock in modifiers
+     */
+    ValidModMask = 0xff & ~LockMask;
 
-#ifdef NUMLOCK_HACK
     getOffendingModifiers();
-#endif
-    
+    /*
+     * Ignore NumLock and ScrollLock too
+     */
+    ValidModMask &= ~(_NumLockMask|_ScrollLockMask);
+
+
     memset(&wKeyBindings, 0, sizeof(wKeyBindings));
 
     wWinContext = XUniqueContext();
     wAppWinContext = XUniqueContext();
     wStackContext = XUniqueContext();
+
+/*    _XA_VERSION = XInternAtom(dpy, "VERSION", False);*/
+
     _XA_WM_STATE = XInternAtom(dpy, "WM_STATE", False);
     _XA_WM_CHANGE_STATE = XInternAtom(dpy, "WM_CHANGE_STATE", False);
     _XA_WM_PROTOCOLS = XInternAtom(dpy, "WM_PROTOCOLS", False);
@@ -483,32 +672,35 @@ StartUp()
     _XA_WM_SAVE_YOURSELF = XInternAtom(dpy, "WM_SAVE_YOURSELF", False);    
     _XA_WM_CLIENT_LEADER = XInternAtom(dpy, "WM_CLIENT_LEADER", False);
     _XA_WM_COLORMAP_WINDOWS = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);
-    
+    _XA_WM_COLORMAP_NOTIFY = XInternAtom(dpy, "WM_COLORMAP_NOTIFY", False);
+
     _XA_GNUSTEP_WM_ATTR = XInternAtom(dpy, GNUSTEP_WM_ATTR_NAME, False);
-#ifdef MWM_HINTS
-    _XA_MOTIF_WM_HINTS = XInternAtom(dpy, "_MOTIF_WM_HINTS", False);
-#endif
 
     _XA_WINDOWMAKER_MENU = XInternAtom(dpy, "_WINDOWMAKER_MENU", False);
     _XA_WINDOWMAKER_STATE = XInternAtom(dpy, "_WINDOWMAKER_STATE", False);
 
     _XA_WINDOWMAKER_WM_PROTOCOLS =
-      XInternAtom(dpy, "_WINDOWMAKER_WM_PROTOCOLS", False);
+	XInternAtom(dpy, "_WINDOWMAKER_WM_PROTOCOLS", False);
 
-    _XA_WINDOWMAKER_WM_MINIATURIZE_WINDOW = 
-      XInternAtom(dpy, WINDOWMAKER_WM_MINIATURIZE_WINDOW, False);
+    _XA_GNUSTEP_WM_MINIATURIZE_WINDOW = 
+	XInternAtom(dpy, GNUSTEP_WM_MINIATURIZE_WINDOW, False);
     
-    _XA_GNUSTEP_WM_RESIZEBAR =
-      XInternAtom(dpy, GNUSTEP_WM_RESIZEBAR, False);
-
     _XA_WINDOWMAKER_WM_FUNCTION = XInternAtom(dpy, "_WINDOWMAKER_WM_FUNCTION",
-					  False);
+					      False);
 
+    _XA_WINDOWMAKER_NOTICEBOARD = XInternAtom(dpy, "_WINDOWMAKER_NOTICEBOARD",
+					      False);
+    
+    _XA_WINDOWMAKER_COMMAND = XInternAtom(dpy, "_WINDOWMAKER_COMMAND", False);
 
 #ifdef OFFIX_DND
     _XA_DND_SELECTION = XInternAtom(dpy, "DndSelection", False);
     _XA_DND_PROTOCOL = XInternAtom(dpy, "DndProtocol", False);
 #endif
+#ifdef XDND
+    wXDNDInitializeAtoms();
+#endif
+
 
     /* cursors */
     wCursor[WCUR_NORMAL] = XCreateFontCursor(dpy, XC_left_ptr);    
@@ -529,16 +721,18 @@ StartUp()
     sig_action.sa_flags = 0;
     sigaction(SIGINT, &sig_action, NULL);
     sigaction(SIGTERM, &sig_action, NULL);
+    sigaction(SIGHUP, &sig_action, NULL);
     sigaction(SIGQUIT, &sig_action, NULL);
     sigaction(SIGSEGV, &sig_action, NULL);
     sigaction(SIGBUS, &sig_action, NULL);
     sigaction(SIGFPE, &sig_action, NULL);
+    sigaction(SIGABRT, &sig_action, NULL);
 
-    /* Here we set SA_RESTART for safety, because SIGHUP may not be handled
+    /* Here we set SA_RESTART for safety, because SIGUSR1 may not be handled
      * immediately.
      * -Dan */
     sig_action.sa_flags = SA_RESTART;
-    sigaction(SIGHUP, &sig_action, NULL);
+    sigaction(SIGUSR1, &sig_action, NULL);
 
     /* ignore dead pipe */
     sig_action.sa_handler = ignoreSig;
@@ -550,92 +744,172 @@ StartUp()
     sig_action.sa_flags = SA_NOCLDSTOP|SA_RESTART;
     sigaction(SIGCHLD, &sig_action, NULL);
 
+    /* Now we unblock all signals, that may have been blocked by the parent
+     * who exec()-ed us. This can happen for example if Window Maker crashes
+     * and restarts itself or another window manager from the signal handler.
+     * In this case, the new proccess inherits the blocked signal mask and
+     * will no longer react to that signal, until unblocked.
+     * This is because the signal handler of the proccess who crashed (parent)
+     * didn't return, and the signal remained blocked. -Dan
+     */
+    sigfillset(&sig_action.sa_mask);
+    sigprocmask(SIG_UNBLOCK, &sig_action.sa_mask, NULL);
+
     /* handle X shutdowns a such */
     XSetIOErrorHandler(handleXIO);
 
     /* set hook for out event dispatcher in WINGs event dispatcher */
     WMHookEventHandler(DispatchEvent);
 
-    /* Sound init */
-#ifdef WMSOUND
-    wSoundInit(dpy);
-#endif
+    /* initialize defaults stuff */
+    WDWindowMaker = wDefaultsInitDomain("WindowMaker", True);
+    if (!WDWindowMaker->dictionary) {
+	wwarning(_("could not read domain \"%s\" from defaults database"),
+		 "WindowMaker");
+    }
+
+    /* read defaults that don't change until a restart and are
+     * screen independent */
+    wReadStaticDefaults(WDWindowMaker ? WDWindowMaker->dictionary : NULL);
+
+    /* check sanity of some values */
+    if (wPreferences.icon_size < 16) {
+	wwarning(_("icon size is configured to %i, but it's too small. Using 16, instead\n"),
+		 wPreferences.icon_size);
+	wPreferences.icon_size = 16;
+    }
+
+    /* init other domains */
+    WDRootMenu = wDefaultsInitDomain("WMRootMenu", False);
+    if (!WDRootMenu->dictionary) {
+	wwarning(_("could not read domain \"%s\" from defaults database"),
+		 "WMRootMenu");
+    }
+
+    WDWindowAttributes = wDefaultsInitDomain("WMWindowAttributes", True);
+    if (!WDWindowAttributes->dictionary) {
+	wwarning(_("could not read domain \"%s\" from defaults database"),
+		 "WMWindowAttributes");
+    }
 
     XSetErrorHandler((XErrorHandler)catchXError);
 
-    /* right now, only works for the default screen */
-    wScreen = wScreenInit(DefaultScreen(dpy));
-
-    /* restore workspace state */
-    if (!getWorkspaceState(wScreen->root_win, &ws_state)) {
-        ws_state = NULL;
+#ifdef SHAPE
+    /* ignore j */
+    wShapeSupported = XShapeQueryExtension(dpy, &wShapeEventBase, &j);
+#endif
+    
+#ifdef KEEP_XKB_LOCK_STATUS
+    wXkbSupported = XkbQueryExtension(dpy, NULL, &wXkbEventBase, NULL, NULL, NULL);
+    if(wPreferences.modelock && !wXkbSupported) {
+        wwarning(_("XKB is not supported. KbdModeLock is automatically disabled."));
+        wPreferences.modelock = 0;
     }
+#endif
 
-    wScreenRestoreState(wScreen);
-
-    /* manage all windows that were already 
-     * here before us */
-    if (!wPreferences.flags.nodock && wScreen->dock)
-        wScreen->last_dock = wScreen->dock;
-
-    manageAllWindows(wScreen);
-
-    /* restore saved menus */
-    wMenuRestoreState(wScreen);
-
-    /* If we're not restarting restore session */
-    if (ws_state == NULL)
-        wSessionRestoreState(wScreen);
-
-
-    /* auto-launch apps */
-    if (!wPreferences.flags.nodock && wScreen->dock) {
-        wScreen->last_dock = wScreen->dock;
-	wDockDoAutoLaunch(wScreen->dock, 0);
-    }
-    /* auto-launch apps in clip */
-    if (!wPreferences.flags.noclip) {
-        for(i=0; i<wScreen->workspace_count; i++) {
-            if (wScreen->workspaces[i]->clip) {
-                wScreen->last_dock = wScreen->workspaces[i]->clip;
-                wDockDoAutoLaunch(wScreen->workspaces[i]->clip, i);
-            }
-        }
-    }
-
-    /* go to workspace where we were before restart */
-    if (ws_state) { 
-	wWorkspaceForceChange(wScreen, ws_state->workspace);
-        XFree(ws_state);
+    if (defaultScreenOnly) {
+	max = 1;
     } else {
-        wSessionRestoreLastWorkspace(wScreen);
+	max = ScreenCount(dpy);
     }
-}
+    wScreen = wmalloc(sizeof(WScreen*)*max);
 
+    wScreenCount = 0;
 
-
-static int
-getState(Window window)
-{
-    Atom type;
-    int form;
-    unsigned long nitems, bytes_rem;
-    unsigned char *data;
-    long ret;
-
-    if (XGetWindowProperty(dpy, window, _XA_WM_STATE, 0, 3, False,
-			   _XA_WM_STATE, &type,&form,&nitems,&bytes_rem,
-			   &data)==Success) {
-	if (data != NULL) {
-	    ret = *(long*)data;
-	    free(data);
-	    return ret;
+    /* manage the screens */
+    for (j = 0; j < max; j++) {
+	if (defaultScreenOnly || max==1) {
+	    wScreen[wScreenCount] = wScreenInit(DefaultScreen(dpy));
+	    if (!wScreen[wScreenCount]) {
+		wfatal(_("it seems that there is already a window manager running"));
+		Exit(1);
+	    }
+	} else {
+	    wScreen[wScreenCount] = wScreenInit(j);
+	    if (!wScreen[wScreenCount]) {
+		wwarning(_("could not manage screen %i"), j);
+		continue;
+	    }
 	}
+	wScreenCount++;
+    }
+
+    /* initialize/restore state for the screens */
+    for (j = 0; j < wScreenCount; j++) {
+	/* restore workspace state */
+	if (!getWorkspaceState(wScreen[j]->root_win, &ws_state)) {
+	    ws_state = NULL;
+	}
+
+	wScreenRestoreState(wScreen[j]);
+
+	/* manage all windows that were already here before us */
+	if (!wPreferences.flags.nodock && wScreen[j]->dock)
+	    wScreen[j]->last_dock = wScreen[j]->dock;
+	
+	manageAllWindows(wScreen[j]);
+	
+	/* restore saved menus */
+	wMenuRestoreState(wScreen[j]);
+	
+	/* If we're not restarting restore session */
+	if (ws_state == NULL)
+	    wSessionRestoreState(wScreen[j]);
+	
+	
+	/* auto-launch apps */
+	if (!wPreferences.flags.nodock && wScreen[j]->dock) {
+	    wScreen[j]->last_dock = wScreen[j]->dock;
+	    wDockDoAutoLaunch(wScreen[j]->dock, 0);
+	}
+	/* auto-launch apps in clip */
+	if (!wPreferences.flags.noclip) {
+	    int i;
+	    for(i=0; i<wScreen[j]->workspace_count; i++) {
+		if (wScreen[j]->workspaces[i]->clip) {
+		    wScreen[j]->last_dock = wScreen[j]->workspaces[i]->clip;
+		    wDockDoAutoLaunch(wScreen[j]->workspaces[i]->clip, i);
+		}
+	    }
+	}
+
+	/* go to workspace where we were before restart */
+	if (ws_state) { 
+	    wWorkspaceForceChange(wScreen[j], ws_state->workspace);
+	    free(ws_state);
+	} else {
+	    wSessionRestoreLastWorkspace(wScreen[j]);
+	}
+
+#ifdef KWM_HINTS
+	wKWMSetInitializedHint(wScreen[j]);
+#endif
+    }
+    
+    if (wScreenCount == 0) {
+	wfatal(_("could not manage any screen"));
+	Exit(1);
+    }
+
+    if (!wPreferences.flags.noupdates) {
+	/* setup defaults file polling */
+	WMAddTimerHandler(3000, wDefaultsCheckDomains, NULL);
     }
-    return -1;
 }
 
 
+
+
+static Bool
+windowInList(Window window, Window *list, int count)
+{
+    for (; count>=0; count--) {
+	if (window == list[count])
+	    return True;
+    }
+    return False;
+}
+
 /*
  *-----------------------------------------------------------------------
  * manageAllWindows--
@@ -653,21 +927,20 @@ manageAllWindows(WScreen *scr)
     Window root, parent;
     Window *children;
     unsigned int nchildren;
-    XWindowAttributes wattribs;
     unsigned int i, j;
-    int state;
     WWindow *wwin;
-    XWMHints *wmhints;
-    
+
     XGrabServer(dpy);
     XQueryTree(dpy, scr->root_win, &root, &parent, &children, &nchildren);
 
     scr->flags.startup = 1;
 
     /* first remove all icon windows */
-    for (i=0; i<nchildren; i++) {
+    for (i = 0; i < nchildren; i++) {
+	XWMHints *wmhints;
+
 	if (children[i]==None) 
-	  continue;
+	    continue;
 
     	wmhints = XGetWMHints(dpy, children[i]);
 	if (wmhints && (wmhints->flags & IconWindowHint)) {
@@ -684,69 +957,59 @@ manageAllWindows(WScreen *scr)
 	    XFree(wmhints);
 	}
     }
-    /* map all windows without OverrideRedirect */
-    for (i=0; i<nchildren; i++) {
-	if (children[i]==None) 
-	  continue;
 
-	XGetWindowAttributes(dpy, children[i], &wattribs);
 
-	state = getState(children[i]);
-	if (!wattribs.override_redirect 
-	    && (state>=0 || wattribs.map_state!=IsUnmapped)) {
-	    XUnmapWindow(dpy, children[i]);
-	    
-	    if (state==WithdrawnState) {
-		/* move the window far away so that it doesn't flash */
-		XMoveWindow(dpy, children[i], scr->scr_width+10,
-			    scr->scr_height+10);
+    for (i = 0; i < nchildren; i++) {
+	if (children[i] == None)
+	    continue;
+
+#ifdef KWM_HINTS
+	wKWMCheckModule(scr, children[i]);
+#endif
+	wwin = wManageWindow(scr, children[i]);
+	if (wwin) {
+	    /* apply states got from WSavedState */
+	    /* shaded + minimized is not restored correctly */
+	    if (wwin->flags.shaded) {
+		wwin->flags.shaded = 0;
+		wShadeWindow(wwin);
 	    }
+	    if (wwin->flags.miniaturized
+		&& (wwin->transient_for == None
+		    || wwin->transient_for == scr->root_win
+		    || !windowInList(wwin->transient_for, children, 
+				     nchildren))) {
 
-	    wwin = wManageWindow(scr, children[i]);
-	    if (wwin) {
-		if (state==WithdrawnState) {
-		    wwin->flags.mapped = 0;
-		    wClientSetState(wwin, WithdrawnState, None);
-		    XSelectInput(dpy, wwin->client_win, NoEventMask);
-		    XRemoveFromSaveSet(dpy, wwin->client_win);
-		    wUnmanageWindow(wwin, True);
-                } else {
-                    /* apply states got from WSavedState */
-                    /* shaded + minimized is not restored correctly */
-                    if (wwin->flags.shaded) {
-			wwin->flags.shaded = 0;
-                        wShadeWindow(wwin);
-                    }
-                    if (wwin->flags.hidden) {
-                        WApplication *wapp = wApplicationOf(wwin->main_window);
-                        wwin->flags.hidden = 0;
-                        if (wapp) {
-                            wHideApplication(wapp);
-                        }
-                        wwin->flags.ignore_next_unmap=1;
-                    } else {
-                        if (wwin->wm_hints &&
-                            (wwin->wm_hints->flags & StateHint) && state<0)
-                            state=wwin->wm_hints->initial_state;
-                        if (state==IconicState) {
-                            wIconifyWindow(wwin);
-                            wwin->flags.ignore_next_unmap=1;
-                        } else {
-                            wClientSetState(wwin, NormalState, None);
-                        }
-                    }
-                }
-            }
-	    if (state==WithdrawnState) {
-		/* move the window back to it's old position */
-		XMoveWindow(dpy, children[i], wattribs.x, wattribs.y);
+		wwin->flags.skip_next_animation = 1;
+		wwin->flags.miniaturized = 0;
+		wIconifyWindow(wwin);
+	    } else {
+		wClientSetState(wwin, NormalState, None);
 	    }
 	}
     }
     XUngrabServer(dpy);
+
+    /* hide apps */
+    wwin = scr->focused_window;
+    while (wwin) {
+	if (wwin->flags.hidden) {
+	    WApplication *wapp = wApplicationOf(wwin->main_window);
+
+	    if (wapp) {
+		wwin->flags.hidden = 0;
+		wHideApplication(wapp);
+	    } else {
+		wwin->flags.hidden = 0;
+	    }
+	}
+	wwin = wwin->prev;
+    }
+
     XFree(children);
     scr->flags.startup = 0;
     scr->flags.startup2 = 1;
+
     while (XPending(dpy)) {
 	XEvent ev;
 	WMNextEvent(dpy, &ev);
