@@ -1,6 +1,6 @@
 /* wdefaults.c - window specific defaults
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -50,6 +50,9 @@ extern WPreferences wPreferences;
 
 extern proplist_t wAttributeDomainName;
 
+extern WDDomain *WDWindowAttributes;
+
+
 /* Local stuff */
 
 
@@ -63,19 +66,26 @@ static proplist_t ANoTitlebar = NULL;
 static proplist_t ANoResizebar;
 static proplist_t ANoMiniaturizeButton;
 static proplist_t ANoCloseButton;
-static proplist_t ANoHideOthers;       /* app */
+static proplist_t ANoHideOthers;
 static proplist_t ANoMouseBindings;
 static proplist_t ANoKeyBindings;
 static proplist_t ANoAppIcon;	       /* app */
 static proplist_t AKeepOnTop;
+static proplist_t AKeepOnBottom;
 static proplist_t AOmnipresent;
 static proplist_t ASkipWindowList;
 static proplist_t AKeepInsideScreen;
 static proplist_t AUnfocusable;
 static proplist_t AAlwaysUserIcon;
 static proplist_t AStartMiniaturized;
+static proplist_t AStartMaximized;
 static proplist_t AStartHidden;	       /* app */
 static proplist_t ADontSaveSession;    /* app */
+static proplist_t AEmulateAppIcon;
+static proplist_t AFullMaximize;
+#ifdef XKB_BUTTON_HINT
+static proplist_t ANoLanguageButton;
+#endif
 
 static proplist_t AStartWorkspace;
 
@@ -100,6 +110,7 @@ init_wdefaults(WScreen *scr)
     ANoKeyBindings = PLMakeString("NoKeyBindings");
     ANoAppIcon = PLMakeString("NoAppIcon");
     AKeepOnTop = PLMakeString("KeepOnTop");
+    AKeepOnBottom = PLMakeString("KeepOnBottom");
     AOmnipresent = PLMakeString("Omnipresent");
     ASkipWindowList = PLMakeString("SkipWindowList");
     AKeepInsideScreen = PLMakeString("KeepInsideScreen");
@@ -107,8 +118,14 @@ init_wdefaults(WScreen *scr)
     AAlwaysUserIcon = PLMakeString("AlwaysUserIcon");
     AStartMiniaturized = PLMakeString("StartMiniaturized");
     AStartHidden = PLMakeString("StartHidden");
+    AStartMaximized = PLMakeString("StartMaximized");
     ADontSaveSession = PLMakeString("DontSaveSession");
-    
+    AEmulateAppIcon = PLMakeString("EmulateAppIcon");
+    AFullMaximize = PLMakeString("FullMaximize");
+#ifdef XKB_BUTTON_HINT
+    ANoLanguageButton = PLMakeString("NoLanguageButton");
+#endif
+
     AStartWorkspace = PLMakeString("StartWorkspace");
     
     AnyWindow = PLMakeString("*");
@@ -160,9 +177,22 @@ get_value(proplist_t dict_win, proplist_t dict_class, proplist_t dict_name,
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ * wDefaultFillAttributes--
+ * 	Retrieves attributes for the specified instance/class and
+ * fills attr with it. Values that are actually defined are also
+ * set in mask. If useGlobalDefault is True, the default for
+ * all windows ("*") will be used for when no values are found
+ * for that instance/class. 
+ * 
+ *----------------------------------------------------------------------
+ */
 void
 wDefaultFillAttributes(WScreen *scr, char *instance, char *class, 
-		       WWindowAttributes *attr, Bool useGlobalDefault)
+		       WWindowAttributes *attr,
+		       WWindowAttributes *mask,
+		       Bool useGlobalDefault)
 {
     proplist_t value;
     proplist_t key1, key2, key3;
@@ -174,12 +204,12 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
       key1 = PLMakeString(strcat(strcat(strcpy(buffer,instance),"."),class));
     else
       key1 = NULL;
-    
+
     if (instance)
       key2 = PLMakeString(instance);
     else
       key2 = NULL;
-    
+
     if (class)
       key3 = PLMakeString(class);
     else
@@ -191,12 +221,12 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
 
     PLSetStringCmpHook(NULL);
 
-    if (scr->dwindow->dictionary) {
-	dw = key1 ? PLGetDictionaryEntry(scr->dwindow->dictionary, key1) : NULL;
-	dn = key2 ? PLGetDictionaryEntry(scr->dwindow->dictionary, key2) : NULL;
-	dc = key3 ? PLGetDictionaryEntry(scr->dwindow->dictionary, key3) : NULL;
+    if (WDWindowAttributes->dictionary) {
+	dw = key1 ? PLGetDictionaryEntry(WDWindowAttributes->dictionary, key1) : NULL;
+	dn = key2 ? PLGetDictionaryEntry(WDWindowAttributes->dictionary, key2) : NULL;
+	dc = key3 ? PLGetDictionaryEntry(WDWindowAttributes->dictionary, key3) : NULL;
 	if (useGlobalDefault)
-	    da = PLGetDictionaryEntry(scr->dwindow->dictionary, AnyWindow);
+	    da = PLGetDictionaryEntry(WDWindowAttributes->dictionary, AnyWindow);
 	else
 	    da = NULL;
     } else {
@@ -212,74 +242,78 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
     if (key3)
       PLRelease(key3);
 
+#define APPLY_VAL(value, flag, attrib)	\
+	if (value) {attr->flag = getBool(attrib, value); \
+			if (mask) mask->flag = 1;}
+
     /* get the data */        
     value = get_value(dw, dc, dn, da, ANoTitlebar, No, useGlobalDefault);
-    if (value)
-	attr->no_titlebar = getBool(ANoTitlebar, value);
+    APPLY_VAL(value, no_titlebar, ANoTitlebar);
         
     value = get_value(dw, dc, dn, da, ANoResizebar, No, useGlobalDefault);
-    if (value)
-	attr->no_resizebar = getBool(ANoResizebar, value);
+    APPLY_VAL(value, no_resizebar, ANoResizebar);
 
     value = get_value(dw, dc, dn, da, ANoMiniaturizeButton, No, useGlobalDefault);
-    if (value)
-	attr->no_miniaturize_button = getBool(ANoMiniaturizeButton, value);
+    APPLY_VAL(value, no_miniaturize_button, ANoMiniaturizeButton);
     
     value = get_value(dw, dc, dn, da, ANoCloseButton, No, useGlobalDefault);
-    if (value)
-	attr->no_close_button = getBool(ANoCloseButton, value);
+    APPLY_VAL(value, no_close_button, ANoCloseButton);
 
     value = get_value(dw, dc, dn, da, ANoHideOthers, No, useGlobalDefault);
-    if (value)
-	attr->no_hide_others = getBool(ANoHideOthers, value);
+    APPLY_VAL(value, no_hide_others, ANoHideOthers);
     
     value = get_value(dw, dc, dn, da, ANoMouseBindings, No, useGlobalDefault);
-    if (value)
-	attr->no_bind_mouse = getBool(ANoMouseBindings, value);
+    APPLY_VAL(value, no_bind_mouse, ANoMouseBindings);
     
     value = get_value(dw, dc, dn, da, ANoKeyBindings, No, useGlobalDefault);
-    if (value)
-	attr->no_bind_keys = getBool(ANoKeyBindings, value);
+    APPLY_VAL(value, no_bind_keys, ANoKeyBindings);
     
     value = get_value(dw, dc, dn, da, ANoAppIcon, No, useGlobalDefault);
-    if (value)
-	attr->no_appicon = getBool(ANoAppIcon, value);
+    APPLY_VAL(value, no_appicon, ANoAppIcon);
 
     value = get_value(dw, dc, dn, da, AKeepOnTop, No, useGlobalDefault);
-    if (value)
-	attr->floating = getBool(AKeepOnTop, value);
+    APPLY_VAL(value, floating, AKeepOnTop);
+
+    value = get_value(dw, dc, dn, da, AKeepOnBottom, No, useGlobalDefault);
+    APPLY_VAL(value, sunken, AKeepOnBottom);
 
     value = get_value(dw, dc, dn, da, AOmnipresent, No, useGlobalDefault);
-    if (value)
-	attr->omnipresent = getBool(AOmnipresent, value);
+    APPLY_VAL(value, omnipresent, AOmnipresent);
 
     value = get_value(dw, dc, dn, da, ASkipWindowList, No, useGlobalDefault);
-    if (value)
-	attr->skip_window_list = getBool(ASkipWindowList, value);
+    APPLY_VAL(value, skip_window_list, ASkipWindowList);
     
     value = get_value(dw, dc, dn, da, AKeepInsideScreen, No, useGlobalDefault);
-    if (value)
-	attr->dont_move_off = getBool(AKeepInsideScreen, value);
+    APPLY_VAL(value, dont_move_off, AKeepInsideScreen);
 
     value = get_value(dw, dc, dn, da, AUnfocusable, No, useGlobalDefault);
-    if (value)
-	attr->no_focusable = getBool(AUnfocusable, value);
+    APPLY_VAL(value, no_focusable, AUnfocusable);
 
     value = get_value(dw, dc, dn, da, AAlwaysUserIcon, No, useGlobalDefault);
-    if (value)
-	attr->always_user_icon = getBool(AAlwaysUserIcon, value);
+    APPLY_VAL(value, always_user_icon, AAlwaysUserIcon);
 
     value = get_value(dw, dc, dn, da, AStartMiniaturized, No, useGlobalDefault);
-    if (value)
-	attr->start_miniaturized = getBool(AStartMiniaturized, value);
+    APPLY_VAL(value, start_miniaturized, AStartMiniaturized);
     
     value = get_value(dw, dc, dn, da, AStartHidden, No, useGlobalDefault);
-    if (value)
-	attr->start_hidden = getBool(AStartHidden, value);
+    APPLY_VAL(value, start_hidden, AStartHidden);
+
+    value = get_value(dw, dc, dn, da, AStartMaximized, No, useGlobalDefault);
+    APPLY_VAL(value, start_maximized, AStartMaximized);
 
     value = get_value(dw, dc, dn, da, ADontSaveSession, No, useGlobalDefault);
-    if (value)
-	attr->dont_save_session = getBool(ADontSaveSession, value);
+    APPLY_VAL(value, dont_save_session, ADontSaveSession);
+
+    value = get_value(dw, dc, dn, da, AEmulateAppIcon, No, useGlobalDefault);
+    APPLY_VAL(value, emulate_appicon, AEmulateAppIcon);
+
+    value = get_value(dw, dc, dn, da, AFullMaximize, No, useGlobalDefault);
+    APPLY_VAL(value, full_maximize, AFullMaximize);
+
+#ifdef XKB_BUTTON_HINT
+    value = get_value(dw, dc, dn, da, ANoLanguageButton, No, useGlobalDefault);
+    APPLY_VAL(value, no_language_button, ANoLanguageButton);
+#endif
 
     /* clean up */
     PLSetStringCmpHook(StringCompareHook);
@@ -288,7 +322,8 @@ wDefaultFillAttributes(WScreen *scr, char *instance, char *class,
 
 
 proplist_t
-get_generic_value(WScreen *scr, char *instance, char *class, proplist_t option)
+get_generic_value(WScreen *scr, char *instance, char *class, proplist_t option,
+		  Bool noDefault)
 {
     proplist_t value, key, dict;
     char buffer[256];
@@ -300,7 +335,7 @@ get_generic_value(WScreen *scr, char *instance, char *class, proplist_t option)
     if (class && instance) {
 	key = PLMakeString(strcat(strcat(strcpy(buffer,instance),"."),class));
 	
-	dict = PLGetDictionaryEntry(scr->dwindow->dictionary, key);
+	dict = PLGetDictionaryEntry(WDWindowAttributes->dictionary, key);
 	PLRelease(key);
 
 	if (dict) {
@@ -311,7 +346,7 @@ get_generic_value(WScreen *scr, char *instance, char *class, proplist_t option)
     if (!value && instance) {
 	key = PLMakeString(instance);
 	
-	dict = PLGetDictionaryEntry(scr->dwindow->dictionary, key);
+	dict = PLGetDictionaryEntry(WDWindowAttributes->dictionary, key);
 	PLRelease(key);
 
 	if (dict) {
@@ -322,7 +357,7 @@ get_generic_value(WScreen *scr, char *instance, char *class, proplist_t option)
     if (!value && class) {
 	key = PLMakeString(class);
 	
-	dict = PLGetDictionaryEntry(scr->dwindow->dictionary, key);
+	dict = PLGetDictionaryEntry(WDWindowAttributes->dictionary, key);
 	PLRelease(key);
 
 	if (dict) {
@@ -330,8 +365,8 @@ get_generic_value(WScreen *scr, char *instance, char *class, proplist_t option)
 	}
     }
 
-    if (!value) {
-	dict = PLGetDictionaryEntry(scr->dwindow->dictionary, AnyWindow);
+    if (!value && !noDefault) {
+	dict = PLGetDictionaryEntry(WDWindowAttributes->dictionary, AnyWindow);
 	
 	if (dict) {
 	    value = PLGetDictionaryEntry(dict, option);
@@ -345,7 +380,8 @@ get_generic_value(WScreen *scr, char *instance, char *class, proplist_t option)
 
 
 char*
-wDefaultGetIconFile(WScreen *scr, char *instance, char *class)
+wDefaultGetIconFile(WScreen *scr, char *instance, char *class, 
+		    Bool noDefault)
 {
     proplist_t value;
     char *tmp;
@@ -354,11 +390,11 @@ wDefaultGetIconFile(WScreen *scr, char *instance, char *class)
 	init_wdefaults(scr);
     }
 
-    if (!scr->dwindow->dictionary)
+    if (!WDWindowAttributes->dictionary)
 	return NULL;
 
-    value = get_generic_value(scr, instance, class, AIcon);
-    
+    value = get_generic_value(scr, instance, class, AIcon, noDefault);
+
     if (!value)
 	return NULL;
 
@@ -375,7 +411,7 @@ wDefaultGetImage(WScreen *scr, char *winstance, char *wclass)
     char *path;
     RImage *image;
 
-    file_name = wDefaultGetIconFile(scr, winstance, wclass);
+    file_name = wDefaultGetIconFile(scr, winstance, wclass, False);
     if (!file_name)
 	return NULL;
     
@@ -388,7 +424,7 @@ wDefaultGetImage(WScreen *scr, char *winstance, char *wclass)
     
     image = RLoadImage(scr->rcontext, path, 0);
     if (!image) {
-	wwarning(_("error loading image file \"%s\""), path, RErrorString);
+	wwarning(_("error loading image file \"%s\""), path, RMessageForError(RErrorCode));
     }
     free(path);
 
@@ -409,10 +445,11 @@ wDefaultGetStartWorkspace(WScreen *scr, char *instance, char *class)
 	init_wdefaults(scr);
     }
 
-    if (!scr->dwindow->dictionary)
+    if (!WDWindowAttributes->dictionary)
 	return -1;
 
-    value = get_generic_value(scr, instance, class, AStartWorkspace);
+    value = get_generic_value(scr, instance, class, AStartWorkspace, 
+			      False);
     
     if (!value)
 	return -1;
@@ -421,14 +458,14 @@ wDefaultGetStartWorkspace(WScreen *scr, char *instance, char *class)
 
     if (!tmp || strlen(tmp)==0)
 	return -1;
-    
+
     if (sscanf(tmp, "%i", &w)!=1) {
 	w = -1;
 	for (i=0; i < scr->workspace_count; i++) {
 	    if (strcmp(scr->workspaces[i]->name, tmp)==0) {
 		w = i;
 		break;
-	    }	
+	    }
 	}
     } else {
 	w--;
@@ -441,7 +478,7 @@ wDefaultGetStartWorkspace(WScreen *scr, char *instance, char *class)
 void
 wDefaultChangeIcon(WScreen *scr, char *instance, char* class, char *file)
 {
-    WDDomain *db = scr->dwindow;
+    WDDomain *db = WDWindowAttributes;
     proplist_t icon_value=NULL, value, attr, key, def_win, def_icon=NULL;
     proplist_t dict = db->dictionary;
     char *buffer;
@@ -465,18 +502,6 @@ wDefaultChangeIcon(WScreen *scr, char *instance, char* class, char *file)
         buffer = wmalloc(strlen(instance) + strlen(class) + 2);
         strcat(strcat(strcpy(buffer, instance), "."), class);
         key = PLMakeString(buffer);
-        if (PLGetDictionaryEntry(dict, key)==NULL) {
-            PLRelease(key);
-            key = PLMakeString(instance);
-            if (PLGetDictionaryEntry(dict, key)==NULL) {
-                PLRelease(key);
-                key = PLMakeString(class);
-                if (PLGetDictionaryEntry(dict, key)==NULL) {
-                    PLRelease(key);
-                    key = PLMakeString(buffer);
-                }
-            }
-        }
         free(buffer);
     } else if (instance) {
         key = PLMakeString(instance);
@@ -499,18 +524,18 @@ wDefaultChangeIcon(WScreen *scr, char *instance, char* class, char *file)
             same = 1;
     }
 
-    if((attr = PLGetDictionaryEntry(dict, key)) != NULL) {
+    if ((attr = PLGetDictionaryEntry(dict, key)) != NULL) {
         if (PLIsDictionary(attr)) {
             if (icon_value!=NULL && !same)
                 PLMergeDictionaries(attr, icon_value);
             else
                 PLRemoveDictionaryEntry(attr, AIcon);
         }
-    }
-    else if (icon_value!=NULL && !same) {
+    } else if (icon_value!=NULL && !same) {
         PLInsertDictionaryEntry(dict, key, icon_value);
     }
-    PLSave(dict, YES);
+    if (!wPreferences.flags.noupdates)
+	PLSave(dict, YES);
 
     PLRelease(key);
     if(icon_value)
