@@ -1,6 +1,6 @@
 /* workspace.c- Workspace management
  * 
- *  WindowMaker window manager
+ *  Window Maker window manager
  * 
  *  Copyright (c) 1997, 1998 Alfredo K. Kojima
  * 
@@ -23,11 +23,16 @@
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#ifdef SHAPE
+#include <X11/extensions/shape.h>
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <string.h>
+#include <time.h>
 
 #include "WindowMaker.h"
 #include "wcore.h"
@@ -41,6 +46,12 @@
 #include "actions.h"
 #include "workspace.h"
 #include "appicon.h"
+#ifdef GNOME_STUFF
+#include "gnome.h"
+#endif
+#ifdef KWM_HINTS
+#include "kwm.h"
+#endif
 
 #include <proplist.h>
 
@@ -85,27 +96,50 @@ wWorkspaceNew(WScreen *scr)
 	scr->workspace_count++;
 
 	wspace = wmalloc(sizeof(WWorkspace));
+	wspace->name = NULL;
 
-        wspace->name = wmalloc(strlen(_("Workspace %i"))+8);
-        sprintf(wspace->name, _("Workspace %i"), scr->workspace_count);
-        if (!wPreferences.flags.noclip)
+#ifdef KWM_HINTS
+	if (scr->flags.kwm_syncing_count) {
+	    wspace->name = wKWMGetWorkspaceName(scr, scr->workspace_count-1);
+	}
+#endif
+	if (!wspace->name) {
+	    wspace->name = wmalloc(strlen(_("Workspace %i"))+8);
+	    sprintf(wspace->name, _("Workspace %i"), scr->workspace_count);
+	}
+
+
+        if (!wPreferences.flags.noclip) {
             wspace->clip = wDockCreate(scr, WM_CLIP);
-        else
+	} else
             wspace->clip = NULL;
 
-	
 	list = wmalloc(sizeof(WWorkspace*)*scr->workspace_count);
-	
+
 	for (i=0; i<scr->workspace_count-1; i++) {
 	    list[i] = scr->workspaces[i];
 	}
 	list[i] = wspace;
-	free(scr->workspaces);
+	if (scr->workspaces)
+	    free(scr->workspaces);
 	scr->workspaces = list;
 
 	wWorkspaceMenuUpdate(scr, scr->workspace_menu);
 	wWorkspaceMenuUpdate(scr, scr->clip_ws_menu);
-	
+#ifdef GNOME_STUFF
+	wGNOMEUpdateWorkspaceHints(scr);
+#endif
+#ifdef KWM_HINTS
+	if (!scr->flags.kwm_syncing_count) {
+	    wKWMUpdateWorkspaceCountHint(scr);
+	    wKWMUpdateWorkspaceNameHint(scr, scr->workspace_count-1);
+	}
+#ifdef not_used
+	wKWMSetUsableAreaHint(scr, scr->workspace_count-1);
+#endif
+#endif
+	XFlush(dpy);
+
 	return scr->workspace_count-1;
     }
     return -1;
@@ -113,24 +147,25 @@ wWorkspaceNew(WScreen *scr)
 
 
 
-void
+Bool
 wWorkspaceDelete(WScreen *scr, int workspace)
 {
     WWindow *tmp;
     WWorkspace **list;
     int i, j;
 
+
     if (workspace<=0)
-	return;
+	return False;
 
     /* verify if workspace is in use by some window */
     tmp = scr->focused_window;
     while (tmp) {
-	if (tmp->frame->workspace==workspace)
-	    return;
+	if (!IS_OMNIPRESENT(tmp) && tmp->frame->workspace==workspace)
+	    return False;
 	tmp = tmp->prev;
     }
-    
+
     if (!wPreferences.flags.noclip) {
         wDockDestroy(scr->workspaces[workspace]->clip);
         scr->workspaces[workspace]->clip = NULL;
@@ -176,71 +211,324 @@ wWorkspaceDelete(WScreen *scr, int workspace)
             wMenuRemoveItem(menu, --i);
 	wMenuRealize(menu);
     }
+
+#ifdef GNOME_STUFF
+    wGNOMEUpdateWorkspaceHints(scr);
+#endif
+#ifdef KWM_HINTS
+    wKWMUpdateWorkspaceCountHint(scr);
+#endif
+
+    if (scr->current_workspace >= scr->workspace_count)
+	wWorkspaceChange(scr, scr->workspace_count-1);
+
+    return True;
+}
+
+
+typedef struct WorkspaceNameData {
+    int count;
+    RImage *back;
+    RImage *text;
+    time_t timeout;
+} WorkspaceNameData;
+
+
+
+static void
+hideWorkpaceName(void *data)
+{
+    WScreen *scr = (WScreen*)data;
+
+    if (!scr->workspace_name_data || scr->workspace_name_data->count == 0
+	|| time(NULL) > scr->workspace_name_data->timeout) {
+	XUnmapWindow(dpy, scr->workspace_name);
+
+	if (scr->workspace_name_data) {
+	    RDestroyImage(scr->workspace_name_data->back);
+	    RDestroyImage(scr->workspace_name_data->text);
+	    free(scr->workspace_name_data);
+
+	    scr->workspace_name_data = NULL;
+	}
+	scr->workspace_name_timer = NULL;
+    } else {
+	RImage *img = RCloneImage(scr->workspace_name_data->back);
+	Pixmap pix;
+
+	scr->workspace_name_timer = 
+	    WMAddTimerHandler(WORKSPACE_NAME_FADE_DELAY, hideWorkpaceName,
+			      scr);
+
+	RCombineImagesWithOpaqueness(img, scr->workspace_name_data->text,
+				     scr->workspace_name_data->count*255/10);
+
+	RConvertImage(scr->rcontext, img, &pix);
+
+	RDestroyImage(img);
+
+	XSetWindowBackgroundPixmap(dpy, scr->workspace_name, pix);
+	XClearWindow(dpy, scr->workspace_name);
+	XFreePixmap(dpy, pix);
+	XFlush(dpy);
+
+	scr->workspace_name_data->count--;
+    }
+}
+
+
+
+static void
+showWorkspaceName(WScreen *scr, int workspace)
+{
+    WorkspaceNameData *data;
+    RXImage *ximg;
+    Pixmap text, mask;
+    int w, h;
+    int px, py;
+    char *name = scr->workspaces[workspace]->name;
+    int len = strlen(name);
+    int x, y;
+
+    if (wPreferences.workspace_name_display_position == WD_NONE
+	|| scr->workspace_count < 2)
+	return;
+
+    if (scr->workspace_name_timer) {
+	WMDeleteTimerHandler(scr->workspace_name_timer);
+	XUnmapWindow(dpy, scr->workspace_name);
+	XFlush(dpy);
+    }
+    scr->workspace_name_timer = WMAddTimerHandler(WORKSPACE_NAME_DELAY,
+						  hideWorkpaceName, scr);
+
+    if (scr->workspace_name_data) {
+	RDestroyImage(scr->workspace_name_data->back);
+	RDestroyImage(scr->workspace_name_data->text);
+	free(scr->workspace_name_data);
+    }
+
+    data = wmalloc(sizeof(WorkspaceNameData));
+
+    w = WMWidthOfString(scr->workspace_name_font, name, len);
+    h = WMFontHeight(scr->workspace_name_font);
+
+    switch (wPreferences.workspace_name_display_position) {
+     case WD_CENTER:
+	px = (scr->scr_width - (w+4))/2;
+	py = (scr->scr_height - (h+4))/2;
+	break;
+     case WD_TOP:
+	px = (scr->scr_width - (w+4))/2;
+	py = 0;
+	break;
+     case WD_BOTTOM:
+	px = (scr->scr_width - (w+4))/2;
+	py = scr->scr_height - h;
+	break;
+     case WD_TOPLEFT:
+	px = 0;
+	py = 0;
+	break;
+     case WD_TOPRIGHT:
+	px = scr->scr_width - w;
+	py = 0;
+	break;
+     case WD_BOTTOMLEFT:
+	px = 0;
+	py = scr->scr_height - h;
+	break;
+     case WD_BOTTOMRIGHT:
+	px = scr->scr_width - w;
+	py = scr->scr_height - h;
+	break;
+    }
+    XResizeWindow(dpy, scr->workspace_name, w+4, h+4);
+    XMoveWindow(dpy, scr->workspace_name, px, py);
+
+    text = XCreatePixmap(dpy, scr->w_win, w+4, h+4, scr->w_depth);
+    mask = XCreatePixmap(dpy, scr->w_win, w+4, h+4, 1);
+
+    XSetForeground(dpy, scr->draw_gc, scr->black_pixel);
+    XFillRectangle(dpy, text, scr->draw_gc, 0, 0, w+4, h+4);
+
+    XSetForeground(dpy, scr->mono_gc, 0);
+    XFillRectangle(dpy, mask, scr->mono_gc, 0, 0, w+4, h+4);
+
+    XSetForeground(dpy, scr->mono_gc, 1);
+    for (x = 0; x <= 4; x++) {
+	for (y = 0; y <= 4; y++) {
+	    WMDrawString(scr->wmscreen, mask, scr->mono_gc,
+			 scr->workspace_name_font, x, y, name, len);
+	}
+    }
+
+    XSetForeground(dpy, scr->draw_gc, scr->white_pixel);
+    WMDrawString(scr->wmscreen, text, scr->draw_gc, scr->workspace_name_font, 
+		 2, 2, scr->workspaces[workspace]->name,
+		 strlen(scr->workspaces[workspace]->name));
+#ifdef SHAPE
+    XShapeCombineMask(dpy, scr->workspace_name, ShapeBounding, 0, 0, mask,
+		      ShapeSet);
+#endif
+    XSetWindowBackgroundPixmap(dpy, scr->workspace_name, text);
+    XClearWindow(dpy, scr->workspace_name);
+
+    data->text = RCreateImageFromDrawable(scr->rcontext, text, None);
+
+    XFreePixmap(dpy, text);
+    XFreePixmap(dpy, mask);
+
+    if (!data->text) {
+	XMapRaised(dpy, scr->workspace_name);
+	XFlush(dpy);
+
+	goto erro;
+    }
+
+    ximg = RGetXImage(scr->rcontext, scr->root_win, px, py,
+		      data->text->width, data->text->height);
+
+    if (!ximg) {
+	goto erro;
+    }
+
+    XMapRaised(dpy, scr->workspace_name);
+    XFlush(dpy);
+
+    data->back = RCreateImageFromXImage(scr->rcontext, ximg->image, NULL);
+    RDestroyXImage(scr->rcontext, ximg);
+
+    if (!data->back) {
+	goto erro;
+    }
+
+    data->count = 10;
+
+    /* set a timeout for the effect */
+    data->timeout = time(NULL) + 2 +
+	(WORKSPACE_NAME_DELAY + WORKSPACE_NAME_FADE_DELAY*data->count)/1000;
+
+    scr->workspace_name_data = data;
+
+
+    return;
+
+erro:
+    if (scr->workspace_name_timer)
+	WMDeleteTimerHandler(scr->workspace_name_timer);
+
+    if (data->text)
+	RDestroyImage(data->text);
+    if (data->back)
+	RDestroyImage(data->back);
+    free(data);
+
+    scr->workspace_name_data = NULL;
+
+    scr->workspace_name_timer = WMAddTimerHandler(WORKSPACE_NAME_DELAY +
+						  10*WORKSPACE_NAME_FADE_DELAY,
+						  hideWorkpaceName, scr);
 }
 
 
 void
 wWorkspaceChange(WScreen *scr, int workspace)
 {
+    if (scr->flags.startup || scr->flags.startup2) {
+	return;
+    }
+
     if (workspace != scr->current_workspace) {
         wWorkspaceForceChange(scr, workspace);
+    } /*else {
+	showWorkspaceName(scr, workspace);
+    }*/
+}
+
+
+void
+wWorkspaceRelativeChange(WScreen *scr, int amount)
+{
+    int w;
+
+    w = scr->current_workspace + amount;
+
+    if (amount < 0) {
+
+	if (w >= 0)
+	    wWorkspaceChange(scr, w);
+	else if (wPreferences.ws_cycle)
+	    wWorkspaceChange(scr, scr->workspace_count + w);
+
+    } else if (amount > 0) {
+
+	if (w < scr->workspace_count)
+	    wWorkspaceChange(scr, w);
+	else if (wPreferences.ws_advance)
+	    wWorkspaceChange(scr, WMIN(w, MAX_WORKSPACES-1));
+	else if (wPreferences.ws_cycle)
+	    wWorkspaceChange(scr, w % scr->workspace_count);
     }
 }
 
+
+
 void
 wWorkspaceForceChange(WScreen *scr, int workspace)
 {
-    WWindow *tmp, *foc=NULL;
+    WWindow *tmp, *foc=NULL, *foc2=NULL;
     
     if (workspace >= MAX_WORKSPACES || workspace < 0)
 	return;
 
+    SendHelperMessage(scr, 'C', workspace+1, NULL);
+
     if (workspace > scr->workspace_count-1) {
 	wWorkspaceMake(scr, workspace - scr->workspace_count + 1);
     }
 
-    if (!wPreferences.flags.noclip) {
-        scr->clip_icon->dock = scr->workspaces[workspace]->clip;
-        if (scr->current_workspace != workspace) {
-            WDock *old_clip = scr->workspaces[scr->current_workspace]->clip;
-
-            wDockHideIcons(old_clip);
-	    if (old_clip->auto_collapse && !old_clip->collapsed)
-                old_clip->collapsed = 1;
-            wDockShowIcons(scr->workspaces[workspace]->clip);
-        }
-    }
+    wClipUpdateForWorkspaceChange(scr, workspace);
 
     scr->current_workspace = workspace;
 
     wWorkspaceMenuUpdate(scr, scr->workspace_menu);
 
     wWorkspaceMenuUpdate(scr, scr->clip_ws_menu);
-    
-    if ((tmp = scr->focused_window)!= NULL) {	
+
+    if ((tmp = scr->focused_window)!= NULL) {
+	if (IS_OMNIPRESENT(tmp) || tmp->flags.changing_workspace)
+	    foc = tmp;
+		  /* foc2 = tmp; will fix annoyance with gnome panel
+		   * but will create annoyance for every other application 
+		   */
+
 	while (tmp) {
 	    if (tmp->frame->workspace!=workspace && !tmp->flags.selected) {
 		/* unmap windows not on this workspace */
 		if ((tmp->flags.mapped||tmp->flags.shaded)
-		    && !tmp->window_flags.omnipresent
+		    && !IS_OMNIPRESENT(tmp)
 		    && !tmp->flags.changing_workspace) {
-		    XUnmapWindow(dpy, tmp->frame->core->window);
-		    tmp->flags.mapped = 0;
+		    
+		    wWindowUnmap(tmp);
 		}
                 /* also unmap miniwindows not on this workspace */
-                if (tmp->flags.miniaturized &&
-                    !tmp->window_flags.omnipresent && tmp->icon) {
+                if (tmp->flags.miniaturized && !IS_OMNIPRESENT(tmp) 
+		    && tmp->icon) {
                     if (!wPreferences.sticky_icons) {
                         XUnmapWindow(dpy, tmp->icon->core->window);
 			tmp->icon->mapped = 0;
-		    } else {
+		    }
+#if 0
+		    else {
 			tmp->icon->mapped = 1;
 			/* Why is this here? -Alfredo */
                         XMapWindow(dpy, tmp->icon->core->window);
 		    }
+#endif
                 }
 		/* update current workspace of omnipresent windows */
-		if (tmp->window_flags.omnipresent) {		    
+		if (IS_OMNIPRESENT(tmp)) {
 		    WApplication *wapp = wApplicationOf(tmp->main_window);
 
 		    tmp->frame->workspace = workspace;
@@ -248,28 +536,28 @@ wWorkspaceForceChange(WScreen *scr, int workspace)
 		    if (wapp) {
 			wapp->last_workspace = workspace;
 		    }
+		    if (!foc2)
+			foc2 = tmp;
 		}
             } else {
 		/* change selected windows' workspace */
 		if (tmp->flags.selected) {
 		    wWindowChangeWorkspace(tmp, workspace);
-                    if (!tmp->flags.miniaturized) {
+                    if (!tmp->flags.miniaturized && !foc) {
                         foc = tmp;
                     }
                 } else {
 		    if (!tmp->flags.hidden) {
 			if (!(tmp->flags.mapped || tmp->flags.miniaturized)) {
 			    /* remap windows that are on this workspace */
-			    XMapWindow(dpy, tmp->frame->core->window);
+			    wWindowMap(tmp);
 			    if (!foc)
 				foc = tmp;
-			    if (!tmp->flags.shaded)
-				tmp->flags.mapped = 1;
 			}
 			/* Also map miniwindow if not omnipresent */
 			if (!wPreferences.sticky_icons &&
 			    tmp->flags.miniaturized &&
-			    !tmp->window_flags.omnipresent && tmp->icon) {
+			    !IS_OMNIPRESENT(tmp) && tmp->icon) {
 			    tmp->icon->mapped = 1;
 			    XMapWindow(dpy, tmp->icon->core->window);
 			}
@@ -279,9 +567,12 @@ wWorkspaceForceChange(WScreen *scr, int workspace)
 	    tmp = tmp->prev;
 	}
 
-	if (scr->focused_window->flags.mapped)
-	    foc = scr->focused_window;
+	if (!foc)
+	    foc = foc2;
 
+	if (scr->focused_window->flags.mapped && !foc) {
+	    foc = scr->focused_window;
+	}
 	if (wPreferences.focus_mode == WKF_CLICK) {
 	    wSetFocusTo(scr, foc);
 	} else {
@@ -307,16 +598,16 @@ wWorkspaceForceChange(WScreen *scr, int workspace)
      * no autoarrange icons, because else the icons in different workspaces
      * can be superposed.
      * This can be avoided if appicons are also workspace specific.
-     * 
-     * But this completely breakes AutoArrangeIcons=NO. 
      */
     if (!wPreferences.sticky_icons)
         wArrangeIcons(scr, False);
 
     if (scr->dock)
         wAppIconPaint(scr->dock->icon_array[0]);
+
     if (scr->clip_icon) {
-        if (scr->workspaces[workspace]->clip->auto_collapse) {
+        if (scr->workspaces[workspace]->clip->auto_collapse ||
+            scr->workspaces[workspace]->clip->auto_raise_lower) {
             /* to handle enter notify. This will also */
             XUnmapWindow(dpy, scr->clip_icon->icon->core->window);
             XMapWindow(dpy, scr->clip_icon->icon->core->window);
@@ -324,13 +615,23 @@ wWorkspaceForceChange(WScreen *scr, int workspace)
             wClipIconPaint(scr->clip_icon);
         }
     }
+
+    showWorkspaceName(scr, workspace);
+
+#ifdef GNOME_STUFF
+    wGNOMEUpdateCurrentWorkspaceHint(scr);
+#endif
+#ifdef KWM_HINTS
+    wKWMUpdateCurrentWorkspaceHint(scr);
+#endif
+/*   XSync(dpy, False); */
 }
 
 
 static void
 switchWSCommand(WMenu *menu, WMenuEntry *entry)
 {
-    wWorkspaceChange(menu->frame->screen_ptr, (int)entry->clientdata);
+    wWorkspaceChange(menu->frame->screen_ptr, (long)entry->clientdata);
 }
 
 
@@ -386,49 +687,68 @@ cropline(char *line)
 }
 
 
+void
+wWorkspaceRename(WScreen *scr, int workspace, char *name)
+{
+    char buf[MAX_WORKSPACENAME_WIDTH+1];
+    char *tmp;
+
+    if (workspace >= scr->workspace_count)
+	return;
+
+    /* trim white spaces */
+    tmp = cropline(name);
+
+    if (strlen(tmp)==0) {
+	sprintf(buf, _("Workspace %i"), workspace+1);
+    } else {
+	strncpy(buf, tmp, MAX_WORKSPACENAME_WIDTH);
+    }
+    buf[MAX_WORKSPACENAME_WIDTH] = 0;
+
+    /* update workspace */
+    free(scr->workspaces[workspace]->name);
+    scr->workspaces[workspace]->name = wstrdup(buf);
+
+    if (scr->clip_ws_menu) {
+	if (strcmp(scr->clip_ws_menu->entries[workspace+2]->text, buf)!=0) {
+	    free(scr->clip_ws_menu->entries[workspace+2]->text);
+	    scr->clip_ws_menu->entries[workspace+2]->text = wstrdup(buf);
+	    wMenuRealize(scr->clip_ws_menu);
+	}
+    }
+    if (scr->workspace_menu) {
+	if (strcmp(scr->workspace_menu->entries[workspace+2]->text, buf)!=0) {
+	    free(scr->workspace_menu->entries[workspace+2]->text);
+	    scr->workspace_menu->entries[workspace+2]->text = wstrdup(buf);
+	    wMenuRealize(scr->workspace_menu);
+	}
+    }
+
+    UpdateSwitchMenuWorkspace(scr, workspace);
+
+    if (scr->clip_icon)
+        wClipIconPaint(scr->clip_icon);
+    
+#ifdef GNOME_STUFF
+    wGNOMEUpdateWorkspaceNamesHint(scr);
+#endif
+#ifdef KWM_HINTS
+    wKWMUpdateWorkspaceNameHint(scr, workspace);
+#endif
+}
+
+
+
+
 /* callback for when menu entry is edited */
 static void
 onMenuEntryEdited(WMenu *menu, WMenuEntry *entry)
 {
-    int wspace = (int)entry->clientdata;
-    WScreen *scr = menu->frame->screen_ptr;
-    char buf[MAX_WORKSPACENAME_WIDTH+1];
     char *tmp;
 
-    /* trim white spaces */
-    tmp = cropline(entry->text);
-    
-    if (strlen(tmp)==0) {
-	sprintf(buf, _("Workspace %i"), wspace+1);
-	free(entry->text);
-    } else {
-	strncpy(buf, tmp, MAX_WORKSPACENAME_WIDTH);
-	free(entry->text);
-    }
-    buf[MAX_WORKSPACENAME_WIDTH] = 0;
-    entry->text = wstrdup(buf);
-    
-    /* update workspace */
-    free(scr->workspaces[wspace]->name);
-    scr->workspaces[wspace]->name = wstrdup(entry->text);
-   
-    if (menu == scr->workspace_menu) {
-	if (scr->clip_ws_menu) {
-	    free(scr->clip_ws_menu->entries[entry->order]->text);
-	    scr->clip_ws_menu->entries[entry->order]->text = wstrdup(buf);
-	    wMenuRealize(scr->clip_ws_menu);
-	}
-    } else {
-	if (scr->workspace_menu) {
-	    free(scr->workspace_menu->entries[entry->order]->text);
-	    scr->workspace_menu->entries[entry->order]->text = wstrdup(buf);
-	    wMenuRealize(scr->workspace_menu);
-	}
-    }
-    
-    UpdateSwitchMenuWorkspace(scr, wspace);
-    if (scr->clip_icon)
-        wClipIconPaint(scr->clip_icon);
+    tmp = entry->text;
+    wWorkspaceRename(menu->frame->screen_ptr, (long)entry->clientdata, tmp);
 }
 
 
@@ -457,7 +777,8 @@ wWorkspaceMenuMake(WScreen *scr, Bool titled)
 void
 wWorkspaceMenuUpdate(WScreen *scr, WMenu *menu)
 {
-    int i, ws;
+    int i;
+    long ws;
     char title[MAX_WORKSPACENAME_WIDTH+1];
     WMenuEntry *entry;
     int tmp;
@@ -548,7 +869,7 @@ wWorkspaceRestoreState(WScreen *scr)
 {
     proplist_t parr, pstr, wks_state;
     proplist_t clip_state;
-    int i, wscount;
+    int i, j, wscount;
 
     make_keys();
 
@@ -578,11 +899,32 @@ wWorkspaceRestoreState(WScreen *scr)
             if (scr->workspaces[i]->clip)
                 wDockDestroy(scr->workspaces[i]->clip);
             scr->workspaces[i]->clip = wDockRestoreState(scr, clip_state,
-                                                          WM_CLIP);
+                                                         WM_CLIP);
             if (i>0)
                 wDockHideIcons(scr->workspaces[i]->clip);
+
+            /* We set the global icons here, because scr->workspaces[i]->clip
+             * was not valid in wDockRestoreState().
+             * There we only set icon->omnipresent to know which icons we
+             * need to set here.
+             */
+            for (j=0; j<scr->workspaces[i]->clip->max_icons; j++) {
+                WAppIcon *aicon = scr->workspaces[i]->clip->icon_array[j];
+
+                if (aicon && aicon->omnipresent) {
+                    aicon->omnipresent = 0;
+                    wClipMakeIconOmnipresent(aicon, True);
+                    XMapWindow(dpy, aicon->icon->core->window);
+                }
+            }
         }
+#ifdef KWM_HINTS
+	wKWMUpdateWorkspaceNameHint(scr, i);
+#endif
     }
+#ifdef GNOME_STUFF
+    wGNOMEUpdateWorkspaceNamesHint(scr);
+#endif
 }
 
 
